/*  File: anytoany.c
 *  Author: Fred Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1995
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: reads data from any project to update any other 
 * SCCS: $Id: anytoany.c,v 1.1 2001/02/14 12:44:35 edgrif Exp $
 * HISTORY:
 * Last edited: Mar 17 13:41 1995 (fw)
 * * Mar  7 10:52 1995 (fw): converted from VAX code..
 * Created: Tue Mar  7 10:52:21 1995 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;		/* .nam - in */
extern FILE *unit_2 ;		/* .cor - in */

extern FILE *unit_21 ;		/* .nam - out */
extern FILE *unit_22 ;		/* .cor - out */
extern FILE *unit_23 ;		/* .cnt  - out*/
extern FILE *unit_29 ;		/* .flg  - out*/

void main (void) /* anytoany */
{
  /* Local variables */
  long int nbands1, nclone1, nbands3, nclone3,
           nband, igel, lband, line1, lband1, ix, itemp;
  char cnt[1], tra[1], tname[8];
  char inproject[128]="", outproject[128]="" ;
  long int zero = 0 ;
  short int iband ;
  
  *tra = 'C';
  *cnt = 'Y';
  
  printf ("***************************************************\n"
	  "*                  ANYtoANY                       *\n"
	  "*                  --------                       *\n"
	  "*  **WARNING**  This program alters  **WARNING**  *\n"
	  "*      database files of the output project.      *\n"
	  "*                                                 *\n"
	  "*  Input Project : data is copied to output       *\n"
	  "*      project, losing remarks and contigs.       *\n"
	  "*                                                 *\n"
	  "*  Output Project : all clone data from input     *\n"
	  "*      project is added -> Permenently modified ! *\n"
	  "***************************************************\n\n") ;
  
  printf ("   input project name = ") ;
  readinput ("", inproject, 128) ;
  if (strlength(inproject, 128) == 0) { printf ("--STOP-- (no in-project)\n") ; exit (1) ;}
  
  if (!(unit_1 = FILopen (inproject, "nam", "r"))) exit (-1) ;
  if (!(unit_2 = FILopen (inproject, "cor", "r"))) exit (-1) ;

  /* get info on input project */
  seek_1 (1) ;
  fread (&nclone3, sizeof(long int), 1, unit_1);
  fread (&nbands3, sizeof(long int), 1, unit_1);
  printf ("   input database %10s : number of clones = %6ld, "
	  "number of bands = %8ld\n\n", inproject, nclone3, nbands3) ;



  printf ("  output project name = ") ;
  readinput ("", outproject, 128) ;
  if (strlength(outproject, 128) == 0) { printf ("--STOP-- (no out-project)\n") ; exit (1) ;}
  if (!(unit_21 = FILopen (outproject, "nam", "r+"))) exit (-1) ;
  if (!(unit_22 = FILopen (outproject, "cor", "r+"))) exit (-1) ;
  
  /* get info on output project */
  seek_21 (1) ;
  fread (&nclone1, sizeof(long int), 1, unit_21);
  fread (&nbands1, sizeof(long int), 1, unit_21);
  printf ("  output database %10s : number of clones = %6ld, "
	  "number of bands = %8ld\n\n", outproject, nclone1, nbands1) ;


  line1 = 2;
  if (*cnt == 'Y' || *cnt == 'y')
    {
      if (!(unit_23 = FILopen (outproject, "cnt", "r+"))) exit (-1) ;
      if (!(unit_29 = FILopen (outproject, "flg", "r+"))) exit (-1) ;
    }
  seek_1 (line1) ;
  fread (tname, sizeof(char), 8, unit_1);
  fread (&nband, sizeof(long int), 1, unit_1);
  fread (&lband1, sizeof(long int), 1, unit_1);
  fread (&igel, sizeof(long int), 1, unit_1);

  

  for (ix = line1; ix <= nclone3 + 1; ++ix)
    {
      seek_1 (ix) ;
      fread (tname, sizeof(char), 8, unit_1);
      fread (&nband, sizeof(long int), 1, unit_1);
      fread (&lband, sizeof(long int), 1, unit_1);
      fread (&igel, sizeof(long int), 1, unit_1);
      
      itemp = lband+nbands1-lband1+1 ;
      seek_21(nclone1+ix-line1+2) ;
      fwrite (tname, sizeof(char), 8, unit_21);
      fwrite (&nband, sizeof(long int), 1, unit_21);
      fwrite (&itemp, sizeof(long int), 1, unit_21);
      fwrite (&igel, sizeof(long int), 1, unit_21);

      if (*cnt == 'Y' || *cnt == 'y')
        {
	  seek_23 (nclone1+ix-line1+2) ;
	  fwrite (&zero, sizeof(long int), 1, unit_23);
	  fwrite (&nband, sizeof(long int), 1, unit_23);
	  fwrite (&zero, sizeof(long int), 1, unit_23);
	  fwrite (&zero, sizeof(long int), 1, unit_23);

	  seek_29 (nclone1+ix-line1+2) ;
	  fwrite (&zero, sizeof(long int), 1, unit_29);
	}
    }

  for (ix = lband1; ix <= nbands3; ++ix)
    {
      fseek (unit_2, ix * sizeof(short int), SEEK_SET) ;
      fread (&iband, sizeof(short int), 1, unit_2);

      fseek (unit_22, (nbands1+ix-lband1+1) * sizeof(short int), SEEK_SET) ;
      fwrite (&iband, sizeof(short int), 1, unit_22);
    }
  seek_21(1) ;
  itemp = nclone1+nclone3+2-line1 ;
  fwrite (&itemp, sizeof(long int), 1, unit_21);
  itemp = nbands1+nbands3+1-lband1 ;
  fwrite (&itemp, sizeof(long int), 1, unit_21);

  if (*tra == 'T' || *tra == 't')
    {
      seek_1(1) ;
      itemp = line1 - 2 ;
      fwrite (&itemp, sizeof(long int), 1, unit_1);
      itemp = lband1 - 1 ;
      fwrite (&itemp, sizeof(long int), 1, unit_1);
    }

  printf ("Data from %s added to %s\n\n", inproject, outproject) ;
  /* give final statistic */
  seek_21 (1) ;
  fread (&nclone1, sizeof(long int), 1, unit_21);
  fread (&nbands1, sizeof(long int), 1, unit_21);
  printf ("extended database %10s : number of clones = %6ld, "
	  "number of bands = %8ld\n\n", outproject, nclone1, nbands1) ;

  return;
} /* anytoany */

