/*  File: auto.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description:
 * SCCS: $Id: auto.c,v 1.1 2001/02/14 12:44:35 edgrif Exp $
 * HISTORY:
 * Last edited: Feb 13 16:30 1995 (fw)
 * Created: Thu May 19 15:43:33 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;			/* .nam */
extern FILE *unit_4 ;			/* .cnt */
extern FILE *unit_12 ;			/* map.out */

/* Common Block Declarations */

/* cint CB */
extern long int nbanda[30000], lbanda[30000], igela[30000], iconta[30000],
         iposa[30000], ilena[30000], icana[30000], irema[30000];
/* cass CB */
extern long int gpos[1000], glen[1000], gmat[1000], gclst[1000], ngrp;

/* chars CB */
extern char namea[240000], chrom[65];

/* matdat CB */
extern long int itolmn, itoldf;
extern float logfact[201];

/* Table of constant values */

static long int c__1 = 1;
static long int c__0 = 0;
static long int c__50 = 50;
static long int c__2 = 2;
static long int c__10 = 10;

/* global variables */
static long int zeropos ;

/* position filepointer at beginning of current line */
void startmapout (FILE *unit)
{
  int line ;

  line = (ftell(unit)-zeropos)/102 ;
  if ((ftell(unit)-zeropos) % 102 == 0) --line ;
  fseek (unit, zeropos + line*102, SEEK_SET) ;
  printf ("(start) line : %ld (%ld)\n", (ftell(unit)-zeropos)/102, ftell(unit)-zeropos) ;
}

/* position filepointer at beginning of previous line */
void backmapout (FILE *unit)
{
  printf ("1:(back) line : %ld (%ld)\n", (ftell(unit)-zeropos)/102, ftell(unit)-zeropos) ;
  startmapout (unit) ;
  printf ("2:(back) line : %ld (%ld)\n", (ftell(unit)-zeropos)/102, ftell(unit)-zeropos) ;
  startmapout (unit) ;
  printf ("3:(back) line : %ld (%ld)\n", (ftell(unit)-zeropos)/102, ftell(unit)-zeropos) ;
}

void read2003 (char *name)
{
  char lbuf[200] ;

  readline (unit_12, lbuf) ;
  strtcpy (name, lbuf+5, 8) ;
  name[strlength(name, 8)] = 0 ;
}
void read2012 (char *name, float *prob,
	       long int *iint, long int *ndiff)
{
  char lbuf[200] ;
  readline (unit_12, lbuf) ;

  strtcpy (name, lbuf+46, 8) ;
  name[strlength(name, 8)] = 0 ;
  printf ("2012: \"%s\" ", name) ;
  strtcpy (stemp, lbuf+46+8+12, 9) ; *prob = a2flt (stemp) ;
  printf ("\"%s\"  ", stemp) ;
  strtcpy (stemp, lbuf+46+8+12+9, 6) ; *iint = a2int (stemp) ;
  printf ("\"%s\"  ", stemp) ;
  strtcpy (stemp, lbuf+46+8+12+9+6, 6) ; *ndiff = a2int (stemp) ;
  printf ("\"%s\"\n", stemp) ;
}


/* aut	allows return to auto mode after excursion through  manual */
/* j	clone currently being matched */
/* iamat	best canonical in same contig as best match */
/* ictrl	1(def): on return to auto, get next clone */
/* 	2: on return to auto, reconsider if contig 0 */
/* 	3: on return to auto, offer reconsideration (after trial positioning) */
int auto_(BOOL *aut, long int *j, long int *iamat,
	  long int *ictrl, long int *nclone)
{
    /* Local variables */
  static long int ican, igel, ilen, irem;
  static long int iint, ipos, k, lband, nband, ndiff;
  static long int icont, itemp, ncomp, iq, icontt, iposav;
  static long int icnuma[4000];
  static float prob, probc;
  static char tname[9], tname1[9], names[9], cname[9], sname[9];
  static char remark[50], ans[1], ans1[1];

  probc = 1e-8f;
/* first check whether already in auto */
  strcpy(cname, "");
  if (*aut)
    {
/* reenter auto mode, so set pointer to start of line */
      backmapout (unit_12) ;

      if (*iamat != 0)
	{
	  contloss_(&iconta[*iamat - 1], iconta, nclone);
	}
/* protect backspace after name change */
      fprintf (stdout, "\n (old) sname: %s\n", sname) ;
      strtcpy(tname, namea + ((*j - 1) << 3), 8);
      if (strncmp(tname, sname, strlength(tname, 8)) != 0)
	{
	  gprintf (stdout, "  clone name has been changed from %s to %s :\n"
		  "    accept displayed situation (A), "
		  "(def = return to manual) : ", sname, tname) ;
	  readinput ("", stemp, 2) ; *ans = *stemp ;
	  if (*ans == 'A' || *ans == 'a')
	    {
	      *ictrl = 1;
	      goto L202;
	    }
	  else
	    {
	      /* NB not via 220, so keeps old sname */
	      return 0;
	    }
	}
      if (*ictrl == 2)
	{
	  *ictrl = 1;
	  if (iconta[*j - 1] == 0)
	    {
	      /* go back to tname */
	      while(strncmp(cname, tname, strlength (tname, 8)) != 0)
		{
		  /* considered replacing backspaces, but
		     then vulnerable if two clones have same name */
		  backmapout (unit_12) ;
		  if (ftell(unit_12) == zeropos)
		    { break ; }

		  read2003 (cname) ;
		}
	      goto L213;
	    }
	}
      else if (*ictrl == 3)
	{
	  readline (unit_12, 0) ;
	  /* go back to tname */
	  while(strncmp(cname, tname, strlength (tname, 8)) != 0)
	    {
	      backmapout (unit_12) ;
	      if (ftell(unit_12) == zeropos)
		{ break ; }

	      read2003 (cname) ;
	    }
	  gprintf (stdout, "%4d-%-8s(%3db,%4d) still being considered\n"
		  " accept displayed situation(A) (def = reconsider) : ",
		  igela[*j - 1], cname, nbanda[*j - 1],
		  iconta[*j - 1]);
	  readinput ("", stemp, 2) ; *ans = *stemp ;
	  if (*ans == 'A' || *ans == 'a')
	    {
	      gprintf (stdout, " type N if flag is Not required : ") ;
	      readinput ("", stemp, 2) ; *ans = *stemp ;
	      if (*ans == 'n' || *ans == 'N')
		{
		  wrflag_(j, &c__1, &c__0);
		}
	      seek_4 (*j) ;
	      fread (&itemp, sizeof(long int), 1, unit_4);
	      fread (&ilen, sizeof(long int), 1, unit_4);
	      fread (&ican, sizeof(long int), 1, unit_4);
	      fread (&irem, sizeof(long int), 1, unit_4);

	      seek_4 (*j) ;
	      fwrite (&iposa[*j - 1], sizeof(long int), 1, unit_4);
	      fwrite (&ilen, sizeof(long int), 1, unit_4);
	      fwrite (&ican, sizeof(long int), 1, unit_4);
	      fwrite (&irem, sizeof(long int), 1, unit_4);
	      fwrt_(unit_4);
	      goto L202;
	      /* ensure data not filed */
	    }
	  else
	    {
	      seek_4 (*j) ;
	      fread (&ipos, sizeof(long int), 1, unit_4);
	      fread (&ilen, sizeof(long int), 1, unit_4);
	      fread (&ican, sizeof(long int), 1, unit_4);
	      fread (&irem, sizeof(long int), 1, unit_4);
	      if (ican != 0)
		{
		  gprintf (stdout, "\n clone is now canonical or buried;\n"
			  " returning to contig to sort it out\n") ;
		  *ictrl = 3;
		  goto L209;
		}
	      iposa[*j - 1] = 0;
	      seek_4 (*j) ;
	      fwrite (&c__0, sizeof(long int), 1, unit_4);
	      fwrite (&ilen, sizeof(long int), 1, unit_4);
	      fwrite (&ican, sizeof(long int), 1, unit_4);
	      fwrite (&irem, sizeof(long int), 1, unit_4);
	      fwrt_(unit_4);
	      wrflag_(j, &c__1, &c__0);
	      icontt = iconta[*j - 1];
	      iconta[*j - 1] = 0;
	      
	      seek_1 (*j) ;
	      fread (tname1, sizeof(char), 8, unit_1);
	      fread (&nband, sizeof(long int), 1, unit_1);
	      fread (&lband, sizeof(long int), 1, unit_1);
	      fread (&igel, sizeof(long int), 1, unit_1);
	      fread (&itemp, sizeof(long int), 1, unit_1);

	      seek_1 (*j) ;
	      fwrite (tname1, sizeof(char), 8, unit_1);
	      fwrite (&nband, sizeof(long int), 1, unit_1);
	      fwrite (&lband, sizeof(long int), 1, unit_1);
	      fwrite (&igel, sizeof(long int), 1, unit_1);
	      fwrite (&c__0, sizeof(long int), 1, unit_1);
	      fwrt_(unit_1);
	      contloss_(&icontt, iconta, nclone);
	      goto L213;
	    }
	}
      goto L202;
      /* if not yet in auto, initiate auto mode */
    }
  *ictrl = 1;
L201:
  gprintf (stdout, "  while in AUTO mode: blank on display contig goes from\n"
	  "  auto to manual; blank on global control goes "
	  "from manual to auto\n") ;
L215:
  gprintf (stdout, "  first clone to consider (F gets new file) = ") ;
  readinput ("", tname, 8) ;
  if (strcasecmp(tname, "f") == 0)
    {
      gprintf (stdout, " new output file name = ") ;
      readinput ("map.out", remark, 40) ;
      if (strlength(remark, 40) > 0)
        {
	  fclose (unit_12) ;
	  unit_12 = FILopen (remark, "", "r") ;
	  if (!unit_12) goto L214 ;
	}
      goto L215;
    }

  rewind (unit_12) ;
  while (!feof(unit_12) && fgetc (unit_12) != '@') ;fgetc(unit_12) ;
  zeropos = ftell (unit_12) ;
  printf ("zeropos : %ld\n", zeropos) ;

  if (strlength(tname, 8) == 0)
    {
/* stay at zero position for first clone */
      if (feof(unit_12)) goto L207 ;
      if (ferror(unit_12)) goto L214 ;
      *aut = TRUE;
      goto L202;
    }
  else
    {
/* find selected clone */
L204:
      if (feof(unit_12)) goto L205 ;
      if (ferror(unit_12)) goto L214 ;
      read2003 (cname) ;

      if (strncmp(cname, tname, strlength(tname, 8)) != 0)
        {
          goto L204;
        }
      if (feof(unit_12)) goto L207 ;
      read2012 (names, &prob, &iint, &ndiff) ;
      *aut = TRUE;
      goto L206;
L205:
      gprintf (stdout, "%s not in map.out\n", tname) ;
      goto L201;
    }
/* find next clone */
L202:
  if (feof(unit_12)) goto L207 ;
  read2003 (cname) ;
  if (strcmp(cname, tname) == 0)
    {
      goto L202;
    }
  if (strlength(cname, 8) == 0)
    {
      goto L202;
    }
L213:
  if (feof(unit_12)) goto L207 ;
  read2012 (names, &prob, &iint, &ndiff) ;
  strtcpy(tname, cname, 8);
L206:
/* clone found; get line number (= j) */
  for (*j = 2; *j <= *nclone + 1; ++(*j))
    {
      if (strncmp(namea + ((*j - 1) << 3), cname, strlength(cname, 8)) == 0)
	{
	  goto L211;
	}
    }
  gprintf (stdout, " error in map.out : \"%s\" not found in database\n",
	   cname);
  goto L201;
L211:
  initvt640_();
  selectvt100_();
  gprintf (stdout, "  next clone = %4d-%-8s(%3db,%4d)\n",
	  igela[*j - 1], cname, nbanda[*j - 1], iconta[*j - 1]) ;

  /* get k = line number of highest matching clone */
  for (k = 2; k <= *nclone + 1; ++k)
    {
      if (strncmp(namea + ((k - 1) << 3), names, strlength(names, 8)) == 0)
        {
          goto L210;
	}
    }
  gprintf (stdout, " error in map.out : \"%s\" not found in database\n",
	   names);
  goto L201;
L210:
  icont = iconta[k - 1];
  ican = abs(icana[k - 1]) % 10000000;
  if (ican > 1)
    {
      asscont_(nclone, &icont, iconta, iposa, &ncomp, icnuma, 
               &iposav, icana);
      bestmatch_(j, iamat, &ncomp, icnuma, itoldf);
    }
  else
    {
      *iamat = k;
    }
  if (iconta[*j - 1] != 0)
    {
      gprintf (stdout, "%s already in contig \n", cname) ;
      *ictrl = 1;
      goto L208;
    }
  else if (icont == 0)
    {
      gprintf (stdout, " best match is unattached \n");
      *ictrl = 1;
      goto L208;
    }
  else if (prob > probc)
    {
      gprintf (stdout, " probability not low enough for autoburial \n") ;
      *ictrl = 1;
      goto L2015;
    }
  else if (icana[*j - 1] != 0)
    {
      gprintf (stdout, "%s already has CAN number\n", cname) ;
      goto L208;
    }
  else if (irema[*j - 1] != 0)
    {
      gprintf (stdout, "%s has remark: can't bury it\n", cname);
      goto L2015;
    }
  strtcpy (ntemp1, cname, 8) ;
  strtcpy (ntemp2, namea + ((*iamat - 1) << 3), 8) ;
  gprintf (stdout, " best canonical for %-8s is %4d-%-8s(%3db,%4d)\n",
	  ntemp1, igela[*iamat - 1], ntemp2, nbanda[k - 1], icont);
/* shouldnt be possible floatly! */
  if (iconta[*iamat - 1] != icont)
    {
      gprintf (stdout, " best canonical not in same contig as best match\n") ;
    }
  if (iint >= ndiff)
    {
      gprintf (stdout, " INTERNAL\n");
    }
/* *********** burial menu *********** */
  gprintf (stdout, " accept(Y[D]),reject(N),limbo(L[D]),control(C)"
	  " trial(T)  (def = display) : ") ;
  strcpy (stemp, "    ") ;
  readinput ("", stemp, 2) ; *ans = stemp[0] ; *ans1 = stemp[1] ;
  if (*ans == 'C' || *ans == 'c')
    {
      goto L208;
    }
  else if (*ans == 'N' || *ans == 'n')
    {
      goto L202;
    }
  else if (*ans == 'T' || *ans =='t')
    {
      goto L2014;
    }
/* bury clone */
  else if (*ans == 'Y' || *ans == 'y' || *ans == 'L' || *ans == 'l')
    {
      iconta[*j - 1] = iconta[*iamat - 1];
      seek_1 (*j) ;
      fread (tname1, sizeof(char), 8, unit_1);
      fread (&nband, sizeof(long int), 1, unit_1);
      fread (&lband, sizeof(long int), 1, unit_1);
      fread (&igel, sizeof(long int), 1, unit_1);
      fread (&itemp, sizeof(long int), 1, unit_1);
      if (strncmp(tname1, cname, strlength(cname, 8)) != 0)
        {
	  gprintf (stdout, "%s  %s file/array discrepancy \n", tname1, cname) ;
          goto L208;
	}
      seek_1 (*j) ;
      fwrite (tname1, sizeof(char), 8, unit_1);
      fwrite (&nband, sizeof(long int), 1, unit_1);
      fwrite (&lband, sizeof(long int), 1, unit_1);
      fwrite (&igel, sizeof(long int), 1, unit_1);
      fwrite (&icont, sizeof(long int), 1, unit_1);
      fwrt_(unit_1);
      if (*ans == 'y' || *ans == 'Y')
        {
          ican = -(*iamat);
        }
      else
        {
          ican = *iamat + 10000000;
        }
      icana[*iamat - 1] = -1;
      seek_4 (*iamat) ;
      fread (&ipos, sizeof(long int), 1, unit_4);
      fread (&ilen, sizeof(long int), 1, unit_4);
      fread (&itemp, sizeof(long int), 1, unit_4);
      fread (&irem, sizeof(long int), 1, unit_4);

      seek_4 (*iamat) ;
      fwrite (&ipos, sizeof(long int), 1, unit_4);
      fwrite (&ilen, sizeof(long int), 1, unit_4);
      fwrite (&icana[*iamat - 1], sizeof(long int), 1, unit_4);
      fwrite (&irem, sizeof(long int), 1, unit_4);

      iposa[*j - 1] = ipos;
      ilena[*j - 1] = nband;
      icana[*j - 1] = ican;
      
      seek_4 (*j) ;
      fread (&itemp, sizeof(long int), 1, unit_4);
      fread (&itemp, sizeof(long int), 1, unit_4);
      fread (&itemp, sizeof(long int), 1, unit_4);
      fread (&irem, sizeof(long int), 1, unit_4);
      seek_4 (*j) ;
      fwrite (&ipos, sizeof(long int), 1, unit_4);
      fwrite (&ilen, sizeof(long int), 1, unit_4);
      fwrite (&ican, sizeof(long int), 1, unit_4);
      fwrite (&irem, sizeof(long int), 1, unit_4);
      fwrt_(unit_4);

      if (*ans1 == 'd' || *ans1 == 'D')
        {
          *ictrl = 1;
          goto L220;
	}
      goto L202;
    }
  else
    {
      *ictrl = 2;
      goto L220;
    }
/* *********** non-burial menu *********** */
L2015:
  strcpy (stemp, "    ") ;
  gprintf (stdout, " control(C),trial(T),next(N)  (def = display) : ") ;
  readinput ("", stemp, 2) ; *ans = stemp[0] ; *ans1 = stemp[1] ;
  if (*ans == 'C' || *ans == 'c')
    {
      goto L208;
    }
  else if (*ans == 'T' || *ans == 't')
    {
      goto L2014;
    }
  else if (*ans == 'N' || *ans == 'n')
    {
      goto L202;
    } 
  else
    {
      *ictrl = 2;
      goto L220;
    }
/* *********** control menu *********** */
L208:
  gprintf (stdout, " manual(M),previous(P),next(N)  (def = display) : ") ;
  readinput ("", stemp, 2) ; *ans = *stemp ;
  if (*ans == 'M' || *ans == 'm')
    {
      *aut = FALSE;
      goto L220;
    }
  else if (*ans == 'P' || *ans == 'p')
    {
      while(strncmp(cname, tname, strlength(tname, 8)) == 0 || strlength(cname, 8) == 0)
        {
	  backmapout (unit_12) ;
	  if (ftell(unit_12) <= zeropos)
	    { fseek (unit_12, zeropos+102, SEEK_SET) ; break ; }

	  read2003 (cname) ;
	  printf ("cname: \"%s\" , tname : \"%s\" , len: %d\n", cname, tname, strlength(cname, 8) ) ;
        }

      goto L213;
    }
  else if (*ans == 'N' || *ans == 'n')
    {
      goto L202;
    } 
  else
    {
      *ictrl = 2;
      goto L220;
    }
  /* exit after trial positioning */
L2014:
  *ictrl = 3;
  iconta[*j - 1] = icont;
  seek_1 (*j) ;
  fread (tname1, sizeof(char), 8, unit_1);
  fread (&nband, sizeof(long int), 1, unit_1);
  fread (&lband, sizeof(long int), 1, unit_1);
  fread (&igel, sizeof(long int), 1, unit_1);
  fread (&itemp, sizeof(long int), 1, unit_1);

  seek_1 (*j) ;
  fwrite (tname1, sizeof(char), 8, unit_1);
  fwrite (&nband, sizeof(long int), 1, unit_1);
  fwrite (&lband, sizeof(long int), 1, unit_1);
  fwrite (&igel, sizeof(long int), 1, unit_1);
  fwrite (&icont, sizeof(long int), 1, unit_1);
  fwrt_(unit_1);

  iposa[*j - 1] = iposa[*iamat - 1] + (ilena[*iamat - 1] - nband) / 2 ;
  asscont_(nclone, &icont, iconta, iposa, &ncomp, icnuma, &iposav, icana);

  for (iq = 1; iq <= ncomp; ++iq)
    {
      if (icnuma[iq - 1] == *j)
        {
          goto L2016;
	}
    }
L2016:
  posclone_(&iq, &ncomp, icnuma, &iposa[*j - 1], &c__50); /* range 50 */
  wrflag_(j, &c__1, &c__1);
  goto L220;
L214:
  /* exit on no output file */
  gprintf (stdout, " output file not available or not connected\n"
	  " returning to manual mode\n") ;
  goto L209;
L207:
  /* exit at end of map.out; close AUTO */
  gprintf (stdout, " end of map.out; returning to manual mode\n") ;
  *aut = FALSE;
L209:
  /* exit after keying */
  waitForMore (RETURN_KEY) ;
 L220:
  /* set sname before exit */
  if (*j != 0)
    {
      strtcpy(sname, namea + ((*j - 1) << 3), 8);
    }
  return 0;
} /* auto_ */

#undef tname1
#undef tname
#undef names


/* to position clone (ICTP IN ICNUMA) in contig automatically */
/* ISPAN = no of clones to search either way from current ICTP */
/* position; 0 means all */
int posclone_(long int *ictp, long int *ncomp,
	      long int *icnuma, long int *ipos, long int *ispan)
{
  /* Local variables */
  static long int iend, ilen, jctp, irng, nrnk, ncln1, j, icmax, ibest, 
         nband1, iq, iy, imatch, coords[200], itolmn, istart, ibg, ipl, ipr;

  /* Parameter adjustments */
  --icnuma;
  
  /* Function Body */
  if (*ictp < 1 || *ictp > *ncomp || nbanda[icnuma[*ictp] - 1] <= 0) 
    {
      gprintf (stdout, "  can't position this clone\n") ;
      waitForMore (RETURN_KEY) ;

      return 0;
    }
/* NUMBER IN PROBABILITY RANK */
  nrnk = 5;
/* MAX NUMBER OF CLONES FOR ONE CONTIG */
  icmax = 5;
  itolmn = 10;
  ncln1 = 0;

/* get tolerance and probability limits */
  ngrp = 0;
  jctp = 0;
  istart = 1;
  iend = *ncomp;
  if (*ispan > 0)
    {
      istart = max(1,*ictp - *ispan);
      iend = min(*ncomp,*ictp + *ispan);
    }
  for (iq = istart; iq <= iend; ++iq)
    {
      iy = icnuma[iq];
      if (nbanda[iy - 1] > 0)
        {
          ++ngrp;
          if (iq == *ictp)
            {
              jctp = ngrp;
	    }
          gclst[ngrp - 1] = iy;
          glen[ngrp - 1] = ilena[iy - 1];
          gpos[ngrp - 1] = iposa[iy - 1];
          gmat[ngrp - 1] = 0;
	}
    }
/* get match matrix for group */
  getbnds_(&gclst[jctp - 1], &nband1, coords);
  ilen = glen[jctp - 1];
  for (j = 1; j <= ngrp; ++j)
    {
      if (j == jctp)
        {
          goto L16;
	}
      matchs_(gclst[j - 1], nband1, coords, itolmn, &imatch);
      ibg = min(glen[j - 1],ilen) / 10;
      imatch = max(0,imatch - ibg);
      gmat[j - 1] = imatch;
L16:
	;
    }
/* position clone */
  ibest = -1000000;
  ipl = gpos[0];
  ipr = gpos[ngrp - 1] + glen[ngrp - 1];
  gpos[jctp - 1] = (ipl + ipr) / 2;
  irng = (ipr - ipl) / 2 + 50;
  repos2_(&jctp, &ibest, &irng, &c__2);
  repos2_(&jctp, &ibest, &c__10, &c__1);
  *ipos = gpos[jctp - 1];
  return 0;
} /* posclone_ */

int repos2_(long int *icln, long int *ibest, long int *irng, long int *istep)
{
  /* Local variables */
  static long int isav, itsc, ibest1;
  static long int iq, ishift;
  
  ishift = 0;
  isav = gpos[*icln - 1];
  ibest1 = -1000000;
  for (iq = -(*irng); *istep < 0 ? iq >= *irng : iq <= *irng; iq += *istep)
    {
      gpos[*icln - 1] = isav + iq;
      score2_(&itsc, icln);
      if (itsc > ibest1)
        {
          ibest1 = itsc;
          ishift = iq;
	}
    }
  gpos[*icln - 1] = isav + ishift;
  return 0;
} /* repos2_ */

/* score -1 for match undrawn or nonmatch drawn, */
/* +1 for match drawn */
/* for 1 clone only */
int score2_(long int *iscore, long int *icln)
{
  /* System generated locals */
  long int i__3, i__4;
  
  /* Local variables */
  static long int imat, ivlp, ipos1, ipos2, ipos3, ipos4, iq;
  
  *iscore = 0;
  ipos2 = gpos[*icln - 1];
  ipos4 = ipos2 + glen[*icln - 1];
  for (iq = 1; iq <= *icln - 1; ++iq)
    {
      imat = gmat[iq - 1];
      ipos1 = gpos[iq - 1];
      ipos3 = ipos1 + glen[iq - 1];
      ivlp = max(0,min(ipos3,ipos4) - max(ipos1,ipos2));
/* Computing 2nd power */
      i__3 = min(imat,ivlp);
/* Computing 2nd power */
      i__4 = abs(imat - ivlp);
      *iscore = *iscore + i__3 * i__3 - i__4 * i__4;
    }
  for (iq = *icln + 1; iq <= ngrp; ++iq)
    {
      imat = gmat[iq - 1];
      ipos1 = gpos[iq - 1];
      ipos3 = ipos1 + glen[iq - 1];
      ivlp = max(0,min(ipos3,ipos4) - max(ipos1,ipos2));
/* Computing 2nd power */
      i__3 = min(imat,ivlp);
/* Computing 2nd power */
      i__4 = abs(imat - ivlp);
      *iscore = *iscore + i__3 * i__3 - i__4 * i__4;
    }
  return 0;
} /* score2_ */

/* emphasises clone IEM; if IEM -ve, deletes old emphasis */
int emphclone_(long int *iem, long int *ixs, long int *iyst, 
	       long int *ncomp, long int *icnuma,
	       long int *iscale, long int *imode)
{
  /* Local variables */
  static long int ican;
  static long int ipye, ipye1, ipye2, ipxfe, itemp, ixmin, ixmax, ipxte, ix, 
  iy, ihspac, imy, iys;
  
  /* Parameter adjustments */
  --icnuma;
  
  /* Function Body */
  ixmin = 10;
  ixmax = 1022;

  if (*iem == -1)
    {
      /* if line number -ve, delete */
      ipxfe = 9999999;
      return 0;
    }
  if (*iem < 0)
    {
      *iem = -(*iem);
      if (ipxfe != 9999999)
        {
          emph_(ipxfe, ipxte, ipye1, ipye2);
	}
    }
  ican = abs(icana[*iem - 1]) % 10000000;
  if (ican > 1)
    {
      iy = ican;
    }
  else
    {
      iy = *iem;
    }
  iys = *iyst;
  ihspac = 15;
  itemp = *imode / 100 % 10;
  if (itemp >= 2 && itemp <= 3)
    {
      ihspac <<= 1;
      iys = 200;
    }
  for (ix = 1; ix <= *ncomp; ++ix)
    {
      if (icnuma[ix] == iy)
        {
          ipxfe = (*ixs + iposa[iy - 1]) * *iscale + 500;
          ipxte = ipxfe + ilena[iy - 1] * *iscale;
          if (ipxfe > ixmax || ipxte < ixmin)
            {
              ipxfe = 9999999;
              return 0;
	    }
          ipxfe = max(ipxfe,ixmin);
          ipxte = min(ipxte,ixmax);
          ipye = iys + (ihspac << 1) * ((ix - 1) % 8) +
            ihspac * ((ix - 1) / 8 % 2);
          imy = ipye % 13;
          if (imy == 1 || imy == 4 || imy == 9)
            {
              ipye1 = ipye - 1;
            }
          else
            {
              ipye1 = ipye - 2;
	    }
          if (imy == 2 || imy == 7 || imy == 12)
            {
              ipye2 = ipye + 1;
	    }
          else
            {
              ipye2 = ipye + 2;
	    }
          emph_(ipxfe, ipxte, ipye1, ipye2);
          return 0;
	}
    }
  return 0;
} /* emphclone_ */

int emph_(long int ipxfe, long int ipxte, long int ipye1, long int ipye2)
{ /* contained calls to dotscompvt640
     but that was hard to import to our X-graphlib,
     so we just redraw the whole deal, and emphasize a clone RED */
  colorvt640_(RED) ;

  movevt640_(ipxfe, ipye1);
  rectvt640_(ipxte, 3) ;
/*
  drawvt640_(ipxte, ipye1);

  movevt640_(ipxfe, ipye2);
  drawvt640_(ipxte, ipye2);
*/
  colorvt640_(BLACK) ;	/* set color back to BLACK */
  return 0;
} /* emph_ */

/* buries clone i under canonical with lowest prob */
int bury_(long int *i, long int *iret, long int *ncomp,
                           long int *icnuma)
{
  /* Local variables */
  static long int ican, ilen, irem, ipos, itemp, ibury;
  static BOOL limbo;
  
  /* Parameter adjustments */
  --icnuma;
  
  /* Function Body */
  if (*iret == 2)
    {
      limbo = TRUE;
    }
  else
    {
      limbo = FALSE;
    }
  /* check validity of burial */
  /* return flag */
  *iret = 35;
  if (icana[*i - 1] == -1)
    {
      gprintf (stdout, "can't bury canonical\n") ;
      return 0;
    }
  if (icana[*i - 1] != 0)
    {
      gprintf (stdout, "reveal clone before reassigning canonical\n") ;
      return 0;
    }
  if (irema[*i - 1] != 0)
    {
      gprintf (stdout, "can't bury clone with remark\n") ;
      return 0;
    }
  bestmatch_(i, &ibury, ncomp, &icnuma[1], itoldf);
  if (ibury == 0)
    {
      return 0;
    }
  if (iconta[ibury - 1] != iconta[*i - 1])
    {
      return 0;
    }
  if (limbo)
    {
      ican = ibury + 10000000;
    }
  else
    {
      ican = -ibury;
    }
  icana[ibury - 1] = -1;

  seek_4 (ibury) ;
  fread (&ipos, sizeof(long int), 1, unit_4);
  fread (&ilen, sizeof(long int), 1, unit_4);
  fread (&itemp, sizeof(long int), 1, unit_4);
  fread (&irem, sizeof(long int), 1, unit_4);
  seek_4 (ibury) ;
  fwrite (&ipos, sizeof(long int), 1, unit_4);
  fwrite (&ilen, sizeof(long int), 1, unit_4);
  fwrite (&icana[ibury - 1], sizeof(long int), 1, unit_4);
  fwrite (&irem, sizeof(long int), 1, unit_4);
  iposa[*i - 1] = ipos;
  icana[*i - 1] = ican;

  seek_4 (*i) ;
  fread (&itemp, sizeof(long int), 1, unit_4);
  fread (&ilen, sizeof(long int), 1, unit_4);
  fread (&itemp, sizeof(long int), 1, unit_4);
  fread (&irem, sizeof(long int), 1, unit_4);
  seek_4 (*i) ;
  fwrite (&ipos, sizeof(long int), 1, unit_4);
  fwrite (&ilen, sizeof(long int), 1, unit_4);
  fwrite (&ican, sizeof(long int), 1, unit_4);
  fwrite (&irem, sizeof(long int), 1, unit_4);
  fwrt_(unit_4);
  wrflag_(i, &c__1, &c__0);
  *i = ibury;
  *iret = 65;
  return 0;
} /* bury_ */

/* finds best match for clone i in icnuma */
int bestmatch_(long int *i, long int *mbest, long int *ncomp, 
	       long int *icnuma, long int itol)
{
  /* Local variables */
  static long int nbandh, nbandl, nbandm, ix, iy, imatch;
  static long int coords[200];
  static float prob, probmin, logfact[201];
  
  /* Parameter adjustments */
  --icnuma;
  
  /* Function Body */
  *mbest = 0;
  /* initialise match prob */
  probmin = 1.f;
  getbnds_(i, &nbandm, coords);
  for (ix = 1; ix <= *ncomp; ++ix)
    {
      iy = icnuma[ix];
      if (iy == *i)
        {
          goto L30;
        }
      matchs_(iy, nbandm, coords, itol, &imatch);
      if (nbandm < nbanda[iy - 1])
        {
          nbandl = nbandm;
          nbandh = nbanda[iy - 1];
	}
      else
        {
          nbandl = nbanda[iy - 1];
          nbandh = nbandm;
	}
      probcoinc_(imatch, nbandl, nbandh, &prob, itol, logfact);
      if (prob < probmin)
        {
          probmin = prob;
          *mbest = iy;
	}
    L30:
      ;
    }
  return 0;
} /* bestmatch_ */

