/*  File: cc2ace.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: dumps internal data from database
                into ace file.
 * SCCS: $Id: cc2ace.c,v 1.1 2001/02/14 12:44:36 edgrif Exp $
 * HISTORY:
 * Last edited: Nov 24 11:15 1999 (fw)
 * * Jun 24 16:35 1994 (fw): changed to work inside ContigC
 * * Jan 29 23:11 1992 (rd): reads vaxmap entries
 * Created: Thu May 19 15:42:43 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_11 ;			/* .rem */
extern FILE *unit_13 ;			/* .flg */

extern long int nbanda[30000], lbanda[30000], igela[30000], iconta[30000],
         iposa[30000], ilena[30000], icana[30000], irema[30000];
extern char namea[240000];

/*****************************************************************/

void dumpAce (char *projectbase, char *aceproject, long int nclone, long int nbands)
{
  FILE  *aceout ;
  static char  *chromName[] = {"","I","II","III","IV","V","X"} ;
  char  clone[9], gene[9], remark[41] ;
  long int flag, itemp ;
  short int hchrom, hflag, hsitu ;
  float pchr ;
  int ix, remrec, i ;

  seek_13 (1) ;
  fread (&itemp, sizeof(long int), 1, unit_13);

  if (!(aceout = FILopen (aceproject, "", "w")))
    return ;

  fprintf (aceout, "// dump from ContigC project %s\n", projectbase) ;
  fprintf (aceout, "// version : %ld   date : %s\n", itemp, getdatetime()) ;
  fprintf (aceout, "// %ld clones, %ld bands\n", nclone, nbands) ;

  for (ix = 1 ; ix <= nclone ; ++ix)
    {
      strtcpy (clone, namea + (ix << 3), 8) ;

      /*Show the result*/
      if (!(ix%1000))
	{
	  gprintf (stdout, "\n%d : %s", ix, clone) ;
	  fflush (aceout) ;
	}
      else if (!(ix%20))
	gprintf (stdout, ".") ;
 
              /* Write to ace files */
      if  (*clone != '!')          /* deleted clone */
        { fprintf (aceout, "\nClone \"%s\"\n", clone) ;
 
          if (nbanda[ix])
            { fprintf (aceout,"Gel_number %ld\n", igela[ix]) ;
              fprintf (aceout, "Bands %ld %ld\n", lbanda[ix], nbanda[ix]) ;
            }
 
          if (icana[ix] > 10000000)
	    {
	      strtcpy (ntemp1, namea + (((icana[ix]-10000000)-1) << 3), 8) ;
	      fprintf (aceout,"Funny_match_to %s\n", ntemp1) ;
	    }
          else if (icana[ix] > 0)
	    {
	      strtcpy (ntemp1, namea + ((icana[ix]-1) << 3), 8) ;
	      fprintf (aceout,"Exact_match_to %s\n", ntemp1) ;
	    }
          else if (icana[ix] < -1)
	    {
	      strtcpy (ntemp1, namea + ((-icana[ix]-1) << 3), 8) ;
	      fprintf (aceout,"Approximate_match_to %s\n", ntemp1) ;
	    }
          else if (iconta[ix])
	    fprintf (aceout, "pMap ctg%ld %ld %ld\n", iconta[ix],
		     iposa[ix], iposa[ix] + ilena[ix]) ;

	  flag = 0 ;
	  seek_13 (ix+1) ;	/* because, the index in this loop
				 is by 1 smaller, than normal indexing */
	  fread (&flag, sizeof(long int), 1, unit_13) ;
	  if (FILerror(unit_13))
	    {
	      fprintf(stderr,
		      "FATAL ERROR: failed reading the flag for clone %d\n"
		      "Something is corrupted ... abort ...", ix) ;
	      exit (-1);
	    }
          if (flag)
            { if (flag & 1) fprintf (aceout,"Autopos\n") ;
              if (flag & 16) fprintf (aceout,"Cosmid_grid\n") ;
              if (flag & 32) fprintf (aceout,"Canon_for_cosmid\n") ;
              if (flag & 64) fprintf (aceout,"Gridded POLY1\n") ;
              if (flag & 2 || flag & 4)
                fprintf (aceout,"Flag %ld\n",flag) ;
            }

	  for (remrec = irema[ix] ; remrec ; 
	       remrec = (hflag >= 0) ? hflag : 32767 - hflag)
	    {
	      seek_11 (remrec) ;
	      itemp = 0 ; *gene = 0 ; *remark = 0 ;  hchrom = 0 ;
	      hflag = 0 ; pchr = 0 ; hsitu = 0 ;
	      fread (&itemp, sizeof(long int), 1, unit_11);
	      fread (gene, sizeof(char), 8, unit_11);
	      gene[strlength(gene, 8)] = 0 ;
	      fread (remark, sizeof(char), 40, unit_11);
	      remark[strlength(remark, 40)] = 0 ;
	      fread (&hchrom, sizeof(short int), 1, unit_11);
	      fread (&hflag, sizeof(short int), 1, unit_11);
	      fread (&pchr, sizeof(float), 1, unit_11) ;
	      fread (&hsitu, sizeof(short int), 1, unit_11) ;

	      if (itemp != (ix+1))
		continue ;

	      for (i = 0; i < strlength(remark, 40); ++i)
		if (remark[i] == '\"') remark[i] = '\'' ;

	      if (strlength(gene, 8) > 0)
                fprintf (aceout,"Positive_locus \"%s\"\n", gene) ;

              if (strlength(remark, 40) > 0)
		if (strstr (remark, "PCR"))
		  fprintf (aceout,"PCR_remark \"%s\"\n", remark) ;
		else if (remark[0] == 'Y')
		  fprintf (aceout,"Y_remark \"%s\"\n", remark) ;
		else
		  fprintf (aceout,"General_remark \"%s\"\n", remark) ;
 
              if (hchrom)
                fprintf (aceout,"Chromosome %s\n", chromName[hchrom]) ;
 
              if (hsitu)
                fprintf (aceout,"In_situ %d %d\n", hsitu/128, hsitu%128) ;

	      if (pchr)
		fprintf (aceout,"Vaxmap %6.2f\n", pchr) ;

	      remrec = hflag ;
	      if (remrec < 0) remrec = 32767 - remrec ;
            }
        }
    }
  fclose (aceout) ;
}
