/*  File: checkdb.c
 *  Author: Fred Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1995
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: check database for correctness & consistency
 * SCCS: $Id: checkdb.c,v 1.1 2001/02/14 12:44:36 edgrif Exp $
 * HISTORY:
 * Last edited: Mar 28 17:29 1995 (fw)
 * * Mar 28 13:12 1995 (fw): first requirement is to check
 increasing band values in .cor
 * Created: Tue Mar 28 13:12:29 1995 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

extern FILE *unit_1 ;           /* .nam */
extern FILE *unit_2 ;           /* .cor */

/************************************************************/
void printclone (char *cname, long int nband, long int lband, 
		 long int igel, long int icont, short int *coords)
{
  int ix, itemp ;

  printf ("%4ld-%-8s(%3ldb,%4ld) -- ",
	  igel, cname, nband, icont) ;
  for (ix = 0, itemp = 1; ix < nband ; ++ix, ++itemp)
    { printf ("%6d ", coords[ix]);
      if (!(itemp%10)) printf ("\n                            ") ;
    }
  if ((itemp-1)%10) printf ("\n") ;
  printf ("\n") ;
	      
}

void bubblesort (short int *coords, int nband)
{
  BOOL Sorted ;
  int k, numItems ;
  short int temp ;


  numItems = nband;
  do
    {
      Sorted = TRUE;
      for (k = 1; k < numItems - 1; ++k)
	{
	  if (coords[k] < coords[k-1])
	    {
	      temp = coords[k-1] ;
	      coords[k-1] = coords[k] ;
	      coords[k] = temp ;
	      
              Sorted = FALSE;
	    }
	}
      --numItems ;
    }
  while (!Sorted) ;
}

void main (int argc, char **argv)
{
  char projectbase[128]="" ;
  long int ic, nbands, nclone, nband, lband, ix, igel, icont ;
  char name [8], cname[9] ;
  short int coords[100], lastb, lastcloneb = 0;
  BOOL ERROR, WRITE = FALSE ;

  if (argc > 1)
    {
      strcpy (projectbase, argv[1]) ;
      printf ("  project name = %s\n", projectbase) ;
    }
  else
    {
      printf ("****************************************************\n"
	      "*                   checkDB                        *\n"
	      "*                   -------                        *\n"
	      "* this program checks the database for correctness *\n"
	      "* and consistency, and may *ALTER* the files !!!   *\n"     
	      "****************************************************\n\n") ;
      printf ("   project name = ") ;
      readinput ("", projectbase, 128) ;
   }
  if (argc > 2)
    strcpy (stemp, argv[2]) ;
  else
    {
      printf ("   correct files (y/n) ? ") ;
      readinput ("", stemp, 1) ;
    }
  
  if (strcasecmp(stemp, "y") == 0)
    WRITE = TRUE ;

  if (strlength(projectbase, 128) == 0) { printf ("--STOP-- (no in-project)\n") ; exit (1) ;}
  
  if (!(unit_1 = FILopen (projectbase, "nam", "r+"))) exit (-1) ;
  if (!(unit_2 = FILopen (projectbase, "cor", "r+"))) exit (-1) ;

  /* get info on input project */
  seek_1 (1) ;
  fread (&nclone, sizeof(long int), 1, unit_1);
  fread (&nbands, sizeof(long int), 1, unit_1);
  printf ("   database %10s : number of clones = %6ld, "
          "number of bands = %8ld\n\n", projectbase, nclone, nbands) ;

  /** check the .cor file for consistency of band pointers **/
  /**********************************************************/

  rewind (unit_2) ;

  for (ic = 2; ic < nclone + 2; ic++)
    {
      seek_1 (ic) ;
      fread (name, sizeof(char), 8, unit_1) ;
      strtcpy (cname, name, 8) ;
      fread (&nband, sizeof(long int), 1, unit_1) ;
      fread (&lband, sizeof(long int), 1, unit_1) ;
      fread (&igel,  sizeof(long int), 1, unit_1) ;
      fread (&icont, sizeof(long int), 1, unit_1) ;

      if (nband < 2) continue ;

      fseek (unit_2, sizeof(short int) * (lband - 1), SEEK_SET) ;
      fread (&coords, sizeof(short int), nband, unit_2) ;

      lastb = coords[0] ; ERROR = FALSE ;
      for (ix = 1; ix < nband ; ++ix)
	{
	  if (coords[ix] < lastb)
	    ERROR = TRUE ;
	  lastb = coords[ix] ;
	}

      if (ERROR == TRUE)	  /* not in order */
	{
	  if (coords[0] >= lastcloneb) /* first band belongs to last clone */
	    {
	      printf (" bandpointer error           (%d)\n", lastcloneb) ;
	      printclone (cname, nband, lband, igel, icont, coords) ;

	      if (WRITE)
		{
		  /* write correct data back */
		  seek_1 (ic) ;
		  fwrite (name, sizeof(char), 8, unit_1) ;
		  fwrite (&nband, sizeof(long int), 1, unit_1) ;
		  
		  lband ++ ;
		  
		  fwrite (&lband, sizeof(long int), 1, unit_1) ;
		}
	    }
	  else if (abs(coords[0] - coords[nband-1]) < 500 && coords[0] > 1800)
	    {
	      printf (" discard first band          (%d)\n", lastcloneb) ;
	      printclone (cname, nband, lband, igel, icont, coords) ;

	      if (WRITE)
		{
		  /* write correct data back */
		  seek_1 (ic) ;
		  fwrite (name, sizeof(char), 8, unit_1) ;
		  fwrite (&nband, sizeof(long int), 1, unit_1) ;
		  
		  lband ++ ;
		  
		  fwrite (&lband, sizeof(long int), 1, unit_1) ;
		}
	    }
	  else   		/* bands must be out of order */
	    {
	      printf (" bands not in order          (%d)\n", lastcloneb) ;
	      printclone (cname, nband, lband, igel, icont, coords) ;
#ifdef DOESN_T_WORK
	      if (WRITE)
		{
		  bubblesort (coords, nband) ;
		  printf ("sort bands or input manually (s/m) ? ") ;
		  readinput ("", stemp, 1) ;
		  if (strcasecmp(stemp, "s") == 0)
		    {
		      printf ("sorted bands :\n") ;
		      printclone (cname, nband, lband, igel, icont, coords) ;
		      printf ("modify .cor file (y/n) ? ") ;
		      readinput ("", stemp, 1) ;
		      if (strcasecmp(stemp, "y") == 0)
			{
			  fseek (unit_2, sizeof(short int) * (lband - 1), SEEK_SET) ;
			  fwrite (&coords, sizeof(short int), nband, unit_2) ;
			}
		    }
		  else if (strcasecmp(stemp, "m") == 0)
		    {
		      char bandinput[1000] ;
		      short int band, newcoords[100] ;
		      int numinput = 0;

		      printf ("input new band positions seperated by spaces\n: ") ;
		      readinput ("", bandinput, 1000) ;

		      /* read values from band input into newcoords */

		      if ((numinput + 1) != nband)
			{
			  printf ("Warning : you changed the number of bands to %d !!\n", numinput + 1) ;
			  nband = numinput + 1 ;
			}
		      printf ("the new bands :\n") ;
		      printclone (cname, nband, lband, igel, icont, newcoords) ;

		      printf ("modify .nam and .cor file (y/n) ? ") ;
		      readinput ("", stemp, 1) ;
		      if (strcasecmp(stemp, "y") == 0)
			{
			  fseek (unit_2, sizeof(short int) * (lband - 1), SEEK_SET) ;
			  fwrite (&coords, sizeof(short int), nband, unit_2) ;

			  seek_1 (ic) ;
			  fwrite (name, sizeof(char), 8, unit_1) ;
			  fwrite (&nband, sizeof(long int), 1, unit_1) ;
			}
	      
		    }
		}
#endif
	    }
	}
      lastcloneb = coords[nband-1] ;
    }
}
