/*  File: clib.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description:
 * SCCS: $Id: clib.c,v 1.1 2001/02/14 12:44:36 edgrif Exp $
 * HISTORY:
 * Last edited: Oct 30 00:29 1995 (rd)
 * Created: Thu May 19 15:44:13 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;			/* .nam */
extern FILE *unit_4 ;			/* .cnt */
extern FILE *unit_10 ;			/* .stk */
extern FILE *unit_11 ;			/* .rem */
extern FILE *unit_13 ;			/* .flg */

/* Common Block Declarations */

/* csec CB */
extern short int coorda[BANDMAX];

/* cint CB */
extern long int nbanda[30000], lbanda[30000], igela[30000], iconta[30000],
         iposa[30000], ilena[30000], icana[30000], irema[30000];

/* chars CB */
extern char namea[240000], chrom[65];

/* matdat CB */
extern long int itolmn, itoldf;
extern float logfact[201];

/* chr1 CB */
extern float xzero1, scale1;
extern long int ichrgp, maxlev1, levgap1;
extern short int genlev1[6000]	/* was [6][1000] */;

/* Table of constant values */

static long int c__1 = 1;
static long int c__2 = 2;
static long int c__0 = 0;
static long int c__4 = 4;
#ifndef NO_GRAPHICS
static long int c__6 = 6;
static short int sh__1 = 1;
static long int c__15 = 15;
static long int c_n1 = -1;
static long int c__380 = 380;
#endif

/* CLIB */
/* collection of subroutines used in contig programmes */
#ifndef NO_GRAPHICS
int marker_(long int *coordm, long int *nbandm, long int *id, 
	    float *y, float *xmax, float *xmin, float *ymax,
	    float *ymin, long int *margxl, long int *margxr,
	    long int *margyl, long int *margyr, long int *
	    isxmax, long int *isymax)
{
  /* Local variables */
  static long int iband;
  static float xf, yf;
  static float xt, yt;
  
  /* Parameter adjustments */
  --coordm;
  
  /* Function Body */
  *y = 38.f;
  *id = 0;
  /* plot marker lane */
  selectvt640_();
  yf = *y;
  yt = *y + 1.5;
  *y += -1.5;
  for (iband = 1; iband <= *nbandm; ++iband)
    {
      xf = (float) coordm[iband];
      xt = xf;
      line1_(&xf, &xt, &yf, &yt, xmax, xmin, ymax, ymin, margxl, margxr, 
	     margyl, margyr, isxmax, isymax);
    }
  return 0;
} /* marker_ */
#endif /* ! NO_GRAPHICS */

int analyse_(char *namea, long int *nbanda, long int *lbanda, 
	     long int *igela, long int *iconta,
	     long int *nclone)
{
  double log(double);

  /* Local variables */
  static char name1[9], name2[9];
  static long int i, j, k, jband, toler, i1, i2, i3, nband4, coord1[200], 
  coord2[200], coord3[200], coord4[200], icont2;
  static float logfact[201];
  
  --iconta;
  --igela;
  --lbanda;
  --nbanda;
  namea -= 8;
  
/* calc log factorials */
  logfact[0] = 0.f;
  logfact[1] = 0.f;
  for (i = 2; i <= 200; ++i)
    {
      logfact[i] = logfact[i - 1] + log((float) i);
    }
/* get tolerance */
L2:
  gprintf (stdout, "\n  tolerance in tenths of mm = ") ;
  readinput ("", stemp, 10) ; if (nonint(stemp)) goto L2 ;
  toler = a2int (stemp) ;
  if (toler == 0)
    {
      return 0;
    }
L4:
/* get clones to match */
  gprintf (stdout, "  name of new clone = ") ;
  readinput ("", name1, 8) ;
  if (strlength (name1, 8) == 0)
    {
      goto L2;
    }
  getclone_(&i1, name1, coord1, namea + 8, &nbanda[1], &lbanda[1], nclone);
  if (strlength (name1, 8) == 0)
    {
      goto L4;
    }
L6:
  gprintf (stdout, "  name of existing clone = ") ;
  readinput ("", name2, 8) ;
  if (strlength (name2, 8) == 0)
    {
      goto L4;
    }
  getclone_(&i2, name2, coord2, namea + 8, &nbanda[1], &lbanda[1], nclone);
/*  first match given clones */
  if (strlength (name2, 8) == 0)
    {
      goto L6;
    }
  i3 = 0;
/* now find clones in same contig as existing clone */
  matchd_(&i1, &i2, &i3, coord1, &nbanda[i1], coord2, &nbanda[i2],
	  namea + 8, &nbanda[1], &lbanda[1],
	  &igela[1], &iconta[1], toler, logfact);
  icont2 = iconta[i2];
  if (icont2 == 0)
    {
      goto L6;
    }
  for (i3 = 2; i3 <= *nclone + 1; ++i3)
    {
      if (i3 == i1 || i3 == i2)
	{
	  goto L30;
	}
      if (iconta[i3] == icont2)
	{
	  nband4 = 0;
	  for (j = 1; j <= nbanda[i3]; ++j)
	    {
	      coord3[j - 1] = coorda[lbanda[i3] + j - 2];
	    }
	  for (j = 1; j <= nbanda[i2]; ++j)
	    {
	      jband = coord2[j - 1];
	      for (k = 1; k <= nbanda[i3]; ++k)
		{
		  if (abs(jband - coord3[k - 1]) < toler)
		    {
		      coord3[k - 1] = 9999;
		      goto L31;
		    }
		}
/* if we get this far, band from i2 has no match in i3 */
/* so add it to coord4 */
	      ++nband4;
	      coord4[nband4 - 1] = jband;
L31:
	      ;
	    }
	  matchd_(&i1, &i2, &i3, coord1, &nbanda[i1], coord4,
		  &nband4, namea + 8, &nbanda[1], &lbanda[1],
		  &igela[1], &iconta[1], toler, logfact);
	}
L30:
      ;
    }
  goto L6;
} /* analyse_ */

int getclone_(long int *i, char *gname, long int *gcoord, char 
	      *namea, long int *nbanda, long int *lbanda,
	      long int *nclone)
{
  /* Local variables */
  static long int j;
  
  /* Parameter adjustments */
  --lbanda;
  --nbanda;
  namea -= 8;
  --gcoord;
  
  for (*i = 2; *i <= *nclone + 1; ++(*i))
    {
      if (strncmp(gname, namea + (*i << 3), 8) == 0)
	{
	  for (j = 1; j <= nbanda[*i]; ++j)
	    {
	      gcoord[j] = coorda[j + lbanda[*i] - 2];
	    }
	  return 0;
	}
    }
  strcpy(gname, "");
  return 0;
} /* getclone_ */

int matchd_(long int *i1, long int *i2, long int *i3, long int *
	    coord1, long int *nband1, long int *coord2,
	    long int *nband2, char *namea, long int *nbanda,
	    long int *lbanda, long int *igela, long int *
	    iconta, long int toler, float *logfact)
{
  /* Local variables */
  static float prob;
  static long int j, k, l;
  static long int coord5[200], nbandh, nbandl, imatch;
  
  /* Parameter adjustments */
  --iconta;
  --igela;
  --lbanda;
  --nbanda;
  namea -= 8;
  --coord2;
  --coord1;
  
  /* Function Body */
  nbandl = min(*nband1,*nband2);
  nbandh = max(*nband1,*nband2);

/* prevent coord2 corruption! */
  for (j = 1; j <= *nband2; ++j)
    {
      coord5[j - 1] = coord2[j];
/* loop for each band of coord1 */
    }
  imatch = 0;
/* loop for each band of coord5 */
  for (k = 1; k <= *nband1; ++k)
    {
      for (l = 1; l <= *nband2; ++l)
	{
	  if (abs(coord1[k] - coord5[l - 1]) < toler)
	    {
	      ++imatch;
/* prevent either band being retested for this clone pair */
	      coord5[l - 1] = 5000;
	      goto L100;
	    }
	}
L100:
      ;
    }
  probcoinc_(imatch, nbandl, nbandh, &prob, toler, logfact);
  if (*i3 == 0)
    {
      strtcpy (ntemp1, namea + (*i1 << 3), 8) ;
      strtcpy (ntemp2, namea + (*i2 << 3), 8) ;
      gprintf (stdout, "\n %4d matches  %4d-%-8s(%3db,%4d)   "
	      "%4d-%-8s(%3db,%4d)  %8.2e\n\n",
	      imatch, igela[*i1], ntemp1, (*nband1),
	      iconta[*i1],  igela[*i2], ntemp2, *nband2,
	      iconta[*i2], prob) ;
      return 0;
    }
  else
    {
      strtcpy (ntemp1, namea + (*i1 << 3), 8) ;
      strtcpy (ntemp2, namea + (*i2 << 3), 8) ;
      strtcpy (ntemp3, namea + (*i3 << 3), 8) ;
      gprintf (stdout, " %4d matches  %4d-%-8s    %4d-%-8s minus "
	      "%4d-%-8s   %8.2e\n",
	      imatch, igela[*i1], ntemp1, 
	      igela[*i2], ntemp2, igela[*i3], 
	      ntemp3, prob);
    }
  return 0;
} /* matchd_ */

int probcoinc_(long int imatch, long int nbandl,
	       long int nbandh, float *prob,
	       long int toler, float *logfact)
{
  /* Local variables */
  static float a, c;
  static long int n;
  static float p, pa, pb;
  
  if (imatch > nbandl)
    {
      *prob = 1.f;
      return 0;
    }
/* calculate binomial probability of imatch or more for this clone pair */
  p = pow_ri(1 - (float) (toler << 1) / 3300.f, nbandh);
  pa = log(p);
  pb = log(1 - p);
/* minimum to avoid underflow */
  *prob = 5e-30f;
  for (n = imatch; n <= nbandl; ++n)
    {
      c = logfact[nbandl] - logfact[nbandl - n] - logfact[n];
      a = c + n * pb + (nbandl - n) * pa;
      if (a < -88.f)
	{
	  goto L11;
	}
      *prob += exp(a);
L11:
      ;
    }
  return 0;
} /* probcoinc_ */

/* compares single clone with rest of database. */
int map_(long int *nclone, long int *nbands, char *namea, 
	 long int *nbanda, long int *lbanda, long int *igela,
	 long int *iconta)
{
  /* Local variables */
  static char name1[9];
  static long int i, k, toler, lband1, nband1, coords[200];
  static float probm, ps;
  
  /* Parameter adjustments */
  --iconta;
  --igela;
  --lbanda;
  --nbanda;
  namea -= 8;
  
#ifndef NO_GRAPHICS
  clearvt640_() ;
#endif

  gprintf (stdout, "Match analysis of one clone "
	   "against the database\n\n") ;

L2:
/* get tolerance and probability limit */
  gprintf (stdout, "  tolerance in tenths of mm = ") ;
  readinput ("", stemp, 5) ; if (nonint(stemp)) goto L2 ;
  toler = a2int (stemp) ;
  if (toler == 0)
    {
      return 0;
    }
L3:
  gprintf (stdout, "  maximum probability to report = ") ;
  readinput ("", stemp, 10) ; if (nonflt(stemp)) goto L3 ;
  probm = a2flt (stemp) ;
  if (probm == 0.f)
    {
      goto L2;
    }
  ps = 1 - (float) (toler << 1) / 3300.f;
L4:
  gprintf (stdout, "  name of clone = ") ;
  readinput ("", name1, 8) ;
  if (strlength (name1, 8) == 0)
    {
      goto L2;
    }
  for (i = 2; i <= *nclone + 1; ++i)
    {
      if (strncmp(name1, namea + (i << 3), 8) == 0)
	{
	  goto L17;
	}
    }
  goto L4;
L17:
/* read in data for subclone i */
  lband1 = lbanda[i];
  nband1 = nbanda[i];
  for (k = 1; k <= nband1; ++k)
    {
      coords[k - 1] = coorda[lband1 + k - 2];
    }

/* loop for each clone before subclone i */
  patch6_(i, 2, i - 1, nband1, coords, toler, probm, ps);

/* loop for each clone after subclone ito */
  patch6_(i, i + 1, *nclone + 1, nband1, coords, toler, probm, ps);

  goto L4;
} /* map_ */

int patch2_(long int *i, long int *ifromp, long int *itop, 
	    long int *coords, long int *nbanda, long int *lbanda,
	    long int *igela, long int *iconta, long int toler,
	    long int *match, char *namea, float *logfact)
{
  /* Local variables */
  static float prob;
  static long int j, k, lband2, nband1, nband2, nbandh, nbandl,
                  imatch, minmat, coordt[200];
  
  /* Parameter adjustments */
  namea -= 8;
  match -= 201;
  --iconta;
  --igela;
  --lbanda;
  --nbanda;
  --coords;
  
  /* Function Body */
  for (j = *ifromp; j <= *itop; ++j)
    {
      nband1 = nbanda[*i];
      nband2 = nbanda[j];
/* need to calc number of matches that correspond to required level
   of probability */
      lband2 = lbanda[j];
      if (nband2 == 0)
	{
	  goto L150;
	}
      nbandl = min(nband1,nband2);
      nbandh = max(nband1,nband2);
/* if 99 then out of range */
      minmat = match[nbandl + nbandh * 200];
/* read in data for clone j */
      if (minmat == 99)
	{
	  goto L150;
	}
      for (k = 1; k <= nband2; ++k)
	{
	  coordt[k - 1] = coorda[lband2 + k - 2];
	}
/* enough matches to output? */
      getmatch_(nband1, &coords[1], nband2, coordt, toler, &imatch);
      if (imatch >= minmat)
	{
	  probcoinc_(imatch, nbandl, nbandh, &prob, toler, logfact);
	  strtcpy (ntemp1, namea + (*i << 3), 8) ;
	  strtcpy (ntemp2, namea + (j << 3), 8) ;
	  gprintf (stdout, " %4d matches  %4d-%-8s(%3db,%4d)   "
		  "%4d-%-8s(%3db,%4d)  %8.2e\n",
		  imatch, igela[*i], ntemp1,
		  nband1, iconta[*i], igela[j], ntemp2,
		  nband2, iconta[j], prob);
	}
L150:
      ;
    }
  return 0;
} /* patch2_ */

int patch3_(long int ifromp, long int itop, long int nband1, 
	    long int *coords, long int toler, float *logfact,
	    float *rank, long int *rnkdir, float *rankt,
	    long int *rktdir, float probm, float ps,
	    long int *irnkpt, long int nrnk,
	    long int *mat, long int *matt)
{
  /* System generated locals */
  float r__1, r__2;
  
  /* Local variables */
  static float prob;
  static long int irnk;
  static float c;
  static long int j, k;
  static float p;
  static long int ishuf;
  static long int lband2, nband2, nbandh, nbandl, imatch, coordt[200];
  static float fbt;
  
  /* Parameter adjustments */
  --matt;
  --mat;
  --rktdir;
  --rankt;
  --rnkdir;
  --rank;
  --coords;
  
  /* Function Body */
  for (j = ifromp; j <= itop; ++j)
    {
      nband2 = nbanda[j - 1];
      lband2 = lbanda[j - 1];
      if (nband2 == 0)
	{
	  goto L150;
	}

/* read in data for clone j */
      nbandl = min(nband1,nband2);
      nbandh = max(nband1,nband2);
      for (k = 1; k <= nband2; ++k)
	{
	  coordt[k - 1] = coorda[lband2 + k - 2];
	}
/* calculate first binomial term */
      getmatch_(nband1, &coords[1], nband2, coordt, toler, &imatch);
      p = pow_ri(ps, nbandh);
      c = logfact[nbandl] - logfact[nbandl - imatch] - logfact[imatch];
/* worth keeping? */
/* Computing MAX */
      r__1 = -88.f, r__2 = c + imatch * log(1 - p) +
	(nbandl - imatch) * log(p);
      fbt = exp((dmax(r__1,r__2)));
      if (fbt > probm)
	{
	  if (fbt > rankt[nrnk])
	    {
	      goto L150;
	    }
	  probcoinc_(imatch, nbandl, nbandh, &prob, toler, logfact);
	  for (irnk = 1; irnk <= nrnk; ++irnk)
	    {
	      if (prob < rankt[irnk])
		{
		  for (ishuf = nrnk; ishuf >= irnk; --ishuf)
		    {
		      rankt[ishuf + 1] = rankt[ishuf];
		      rktdir[ishuf + 1] = rktdir[ishuf];
		      matt[ishuf + 1] = matt[ishuf];
		    }
		  rankt[irnk] = prob;
		  rktdir[irnk] = j;
		  matt[irnk] = imatch;
		  goto L150;
		}
	    }
	}
      else
	{
	  probcoinc_(imatch, nbandl, nbandh, &prob, toler, logfact);
	  *irnkpt = min(99,*irnkpt + 1);
	  for (irnk = 1; irnk <= *irnkpt; ++irnk)
	    {
	      if (prob < rank[irnk])
		{
		  for (ishuf = *irnkpt; ishuf >= irnk; --ishuf)
		    {
		      rank[ishuf + 1] = rank[ishuf];
		      rnkdir[ishuf + 1] = rnkdir[ishuf];
		      mat[ishuf + 1] = mat[ishuf];
		    }
		  rank[irnk] = prob;
		  rnkdir[irnk] = j;
		  mat[irnk] = imatch;
		  goto L150;
		}
	    }
	}
L150:
      ;
    }
  return 0;
} /* patch3_ */

int patch6_(long int i, long int ifromp, long int itop, 
	    long int nband1, long int *coords, long int toler,
	    float probm, float ps)
{
  /* System generated locals */
  float r__1, r__2;
  
  /* Local variables */
  static float prob;
  static float c;
  static long int j, k;
  static float p;
  static long int lband2, nband2, nbandh, nbandl, imatch, coordt[200];
  static float fbt;
  
  /* Parameter adjustments */
  --coords;

  /* Function Body */
  for (j = ifromp; j <= itop; ++j)
    {
      nband2 = nbanda[j - 1];
      lband2 = lbanda[j - 1];
      if (nband2 == 0)
	{
	  goto L150;
	}

/* read in data for clone j */
      nbandl = min(nband1,nband2);
      nbandh = max(nband1,nband2);
      for (k = 1; k <= nband2; ++k)
	{
	  coordt[k - 1] = coorda[lband2 + k - 2];
	}

/* calculate first binomial term */
      getmatch_(nband1, &coords[1], nband2, coordt, toler, &imatch);
      p = pow_ri(ps, nbandh);
      c = logfact[nbandl] - logfact[nbandl - imatch] - 
	logfact[imatch];
/* worth keeping? */
      r__1 = -88.f, r__2 = c + imatch * log(1 - p) + (nbandl - imatch) * 
	log(p);
      fbt = exp((dmax(r__1,r__2)));
      if (fbt > probm)
	{
	  goto L150;
	}
      probcoinc_(imatch, nbandl, nbandh, &prob, toler, logfact);
      if (prob > probm)
	{
	  goto L150;
	}
      strtcpy (ntemp1, namea + ((i - 1) << 3), 8) ;
      strtcpy (ntemp2, namea + ((j - 1) << 3), 8) ;
      gprintf (stdout, " %4d matches  %4d-%-8s(%3db,%4d)     "
	      "%4d-%-8s(%3db,%4d)  %8.2e\n",
	      imatch,
	      igela[i - 1], ntemp1,
	      nband1, iconta[i - 1], igela[j - 1],
	      ntemp2, nband2, 
	      iconta[j - 1], prob);
L150:
      ;
    }
  return 0;
} /* patch6_ */

/* gives best nrnk matches for coords */
int patch4_(long int ifromp, long int itop, long int nband1, 
	    long int *coords, long int toler, float *logfact,
	    float *rankt, long int *rktdir, float ps,
	    long int nrnk, long int *matt)
{
  /* System generated locals */
  float r__1, r__2;
  
  /* Local variables */
  static float prob;
  static long int irnk;
  static float c;
  static long int j, k;
  static float p;
  static long int ishuf;
  static long int lband2, nband2, nbandh, nbandl, imatch, coordt[200];
  static float fbt;
  
  /* Parameter adjustments */
  --matt;
  --rktdir;
  --rankt;
  --coords;
  
  /* Function Body */
  for (j = ifromp; j <= itop; ++j)
    {
      nband2 = nbanda[j - 1];
      lband2 = lbanda[j - 1];
      if (nband2 == 0)
	{
	  goto L150;
	}

/* read in data for clone j */
      nbandl = min(nband1,nband2);
      nbandh = max(nband1,nband2);
      for (k = 1; k <= nband2; ++k)
	{
	  coordt[k - 1] = coorda[lband2 + k - 2];
	}

      getmatch_(nband1, &coords[1], nband2, coordt, toler, &imatch);

/* calculate first binomial term */
      p = pow_ri(ps, nbandh);
      c = logfact[nbandl] - logfact[nbandl - imatch] - logfact[imatch];
/* worth keeping? */
/* Computing MAX */
      r__1 = -88.f, r__2 = c + imatch * log(1 - p) + (nbandl - imatch) * 
	log(p);
      fbt = exp((dmax(r__1,r__2)));
      if (fbt > rankt[nrnk])
	{
	  goto L150;
	}
      probcoinc_(imatch, nbandl, nbandh, &prob, toler, logfact);
      for (irnk = 1; irnk <= nrnk; ++irnk)
	{
	  if (prob < rankt[irnk])
	    {
	      for (ishuf = nrnk; ishuf >= irnk; --ishuf)
		{
		  rankt[ishuf + 1] = rankt[ishuf];
		  rktdir[ishuf + 1] = rktdir[ishuf];
		  matt[ishuf + 1] = matt[ishuf];
		}
	      rankt[irnk] = prob;
	      rktdir[irnk] = j;
	      matt[irnk] = imatch;
	      goto L150;
	    }
	}
L150:
      ;
    }
  return 0;
} /* patch4_ */

/* match clone j with coords containig nband1; output imatch */
int matchs_(long int j, long int nband1, long int *coords, 
	    long int toler, long int *imatch)
{
  /* Local variables */
  static long int k, lband2, nband2, nbandh, nbandl, coordt[200];
  
  /* Parameter adjustments */
  --coords;
  
  /* Function Body */
  *imatch = 0;
  nband2 = nbanda[j - 1];
  lband2 = lbanda[j - 1];
  if (nband2 == 0)
    {
      return 0;
    }

/* read in data for clone j */
  nbandl = min(nband1,nband2);
  nbandh = max(nband1,nband2);
  for (k = 1; k <= nband2; ++k)
    {
      coordt[k - 1] = coorda[lband2 + k - 2];
    }

  getmatch_(nband1, &coords[1], nband2, coordt, toler, imatch);
  return 0;
} /* matchs_ */

/* subtract i3 from i2, output result in coord4(nband4) */
int clondif_(long int *i2, long int *i3, long int *itol, 
	     long int *nband4, long int *coord4)
{
  /* Local variables */
  static long int j, k, jband, idiff, coord3[200], kstart;
  
    /* Parameter adjustments */
  --coord4;
  
  /* Function Body */
  *nband4 = 0;
  if (*i2 == 0 || *i3 == 0)
    {
      return 0;
    }
  for (j = 1; j <= nbanda[*i3 - 1]; ++j)
    {
      coord3[j - 1] = coorda[lbanda[*i3 - 1] + j - 2];
    }
  kstart = 1;
  for (j = 1; j <= nbanda[*i2 - 1]; ++j)
    {
      jband = coorda[lbanda[*i2 - 1] + j - 2];
      for (k = kstart; k <= nbanda[*i3 - 1]; ++k)
	{
	  idiff = jband - coord3[k - 1];
	  if (abs(idiff) < *itol)
	    {
	      kstart = k + 1;
	      goto L31;
	    }
	  else if (idiff < 0)
	    {
	      kstart = k;
	      goto L34;
	    }
/* if we get this far, band from i2 has no match in i3 */
/* so add it to coord4 */
	}
L34:
      ++(*nband4);
      coord4[*nband4] = jband;
L31:
      ;
    }
  return 0;
} /* clondif_ */

/* subtract 3 from 2, output result in coord4(nband4) */
int bnddif_(long int *nband2, long int *coord2, long int *
	    nband3, long int *coord3, long int *itol,
	    long int *nband4, long int *coord4)
{
  /* Local variables */
  static long int j, k, jband, idiff, kstart;
  
  /* Parameter adjustments */
  --coord4;
  --coord3;
  --coord2;
  
  /* Function Body */
  *nband4 = 0;
  kstart = 1;
  for (j = 1; j <= *nband2; ++j)
    {
      jband = coord2[j];
      for (k = kstart; k <= *nband3; ++k)
	{
	  idiff = jband - coord3[k];
	  if (abs(idiff) < *itol)
	    {
	      kstart = k + 1;
	      goto L31;
	    }
	  else if (idiff < 0)
	    {
	      kstart = k;
	      goto L34;
	    }
/* if we get this far, band from i2 has no match in i3 */
/* so add it to coord4 */
	}
L34:
      ++(*nband4);
      coord4[*nband4] = jband;
L31:
      ;
    }
  return 0;
} /* bnddif_ */

int getmatch_(long int nband1, long int *coords,
	      long int nband2, long int *coordt,
	      long int toler, long int *imatch)
{
  /* Local variables */
  static long int kbnd, k, l, idiff, lstart;
  
  /* Parameter adjustments */
  --coordt;
  --coords;
  
  /* Function Body */
  *imatch = 0;
  lstart = 1;
  for (k = 1; k <= nband1; ++k)
    {
/* loop for each band of clone j */
      kbnd = coords[k];
      for (l = lstart; l <= nband2; ++l)
	{
	  idiff = kbnd - coordt[l];
	  if (abs(idiff) < toler)
	    {
/* prevent either band being retested for this clone pair */
	      ++(*imatch);
	      lstart = l + 1;
	      goto L100;
	    }
	  else if (idiff < 0)
	    {
	      lstart = l;
	      goto L100;
	    }
	}
L100:
      ;
    }
  return 0;
} /* getmatch_ */

int mapstatsl_(long int *match, float *pmax, float *tol, float *logfact)
{
  /* System generated locals */
  float r__1, r__2;

  /* Local variables */
  static float a, c;
  static long int n;
  static float p, pa, pb;
  static long int nh, nl;

/* need to calc for pairs of clones, each having up to PLAN bands */
/* the number of matches required to reach a certain level of */
/* probability (probm). store these in a PLANxPLAN array (match) */
/* let the clones be h and l and numbers of bands nh and nl */
/* tolerance is tol and max resolution is 3300. units so prob */
/* of a single match between two particular bands is tol/3300. */
/* loop for clone h */
  /* Parameter adjustments */
  match -= 201;

  /* Function Body */
  for (nh = 1; nh <= 200; ++nh)
    {
      p = pow_ri(1 - *tol * 2 / 3300.f, nh);
      pa = log(p);
/* loop for l clone */
      pb = log(1 - p);
/* loop for all possible numbers of matches */
      for (nl = 1; nl <= nh; ++nl)
	{
/* calc number of combinations of nl from n */
	  for (n = 1; n <= nl; ++n)
	    {
/*  calc binomial */
	      c = logfact[nl] - logfact[nl - n] - logfact[n];
/* check if prob high enough */
/* Computing MAX */
	      r__1 = -88.f, r__2 = c + n * pb + (nl - n) * pa;
	      a = dmax(r__1,r__2);
/* n matches are sufficiently improbable, so save value of n */
	      if (exp(a) > *pmax)
		{
		  goto L50;
		}
/* dont look further for this pair of band numbers */
	      match[nl + nh * 200] = n;
	      goto L90;
/* if we get this far there is no number of matches between this */
/* number of bands that is sufficiently improbable, so set to 99 */
L50:
	      ;
	    }
	  match[nl + nh * 200] = 99;
L90:
	  ;
	}
    }
  return 0;
} /* mapstatsl_ */

int asscont_(long int *nclone, long int *icont, long int *
	     iconta, long int *iposa, long int *ncomp,
	     long int *icnuma, long int *iposav, long int *icana)
{
  /* Local variables */
  static long int ipos, ix, iy;
  
  /* Parameter adjustments */
  --icana;
  --icnuma;
  --iposa;
  --iconta;
  
  /* Function Body */
  *ncomp = 0;
  if (*icont == 0)
    {
      return 0;
    }
  for (iy = 2; iy <= *nclone + 1; ++iy)
    {
      if (iconta[iy] == *icont)
	{
	  if (abs(icana[iy]) > 1)
	    {
	      goto L30;
	    }
	  ++(*ncomp);
	  ipos = iposa[iy];
	  for (ix = *ncomp; ix >= 2; --ix)
	    {
	      if (ipos > iposa[icnuma[ix - 1]])
		{
		  icnuma[ix] = iy;
		  goto L30;
		}
	      icnuma[ix] = icnuma[ix - 1];
	    }
	  icnuma[1] = iy;
	}
L30:
      ;
    }
  if (*ncomp == 0)
    {
      return 0;
    }
  *iposav = (iposa[icnuma[1]] + iposa[icnuma[*ncomp]]) / 2;
  return 0;
} /* asscont_ */

/* assembles contig, includes burieds, finds true right end clone */
int asscont3_(long int *nclone, long int *icont, long int *
	      ncomp, long int *icnuma, long int *iposav, long int *icrt)
{
  /* Local variables */
  static long int ican, ipos;
  static char tname[9];
  static long int ix, iy, irt;

/* protect against contig 0 */
/* initialise right end hunt */
    /* Parameter adjustments */
  --icnuma;
  
  /* Function Body */
  if (*icont == 0)
    {
      return 0;
    }
  irt = -100000;
/* loop for each clone */
  *ncomp = 0;
  for (iy = 2; iy <= *nclone + 1; ++iy)
    {
/* omit cancellations */
      if (iconta[iy - 1] == *icont)
	{
	  strtcpy(tname, namea + ((iy - 1) << 3), 8);
/* correct dimensions for buried clones */
	  if (*tname == '!')
	    {
	      goto L30;
	    }
	  ican = abs(icana[iy - 1]) % 10000000;
	  if (ican > 1)
	    {
	      iposa[iy - 1] = iposa[ican - 1];
	      ilena[iy - 1] = ilena[ican - 1];
	    }
/* compute right end */
	  ipos = iposa[iy - 1];
	  if (ipos + ilena[iy - 1] > irt)
	    {
	      *icrt = iy;
	      irt = ipos + ilena[iy - 1];
/* place clone in rank of left ends */
	    }
	  ++(*ncomp);
	  for (ix = *ncomp; ix >= 2; --ix)
	    {
	      if (ipos > iposa[icnuma[ix - 1] - 1])
		{
		  icnuma[ix] = iy;
		  goto L30;
		}
		icnuma[ix] = icnuma[ix - 1];
	    }
	  icnuma[1] = iy;
	}
L30:
      ;
    }
  if (*ncomp == 0)
    {
      return 0;
    }
  *iposav = (iposa[icnuma[1] - 1] + iposa[icnuma[*ncomp] - 1])/ 2;
  return 0;
} /* asscont3_ */

/* finds size of contig */
int sizcont_(long int *nclone, long int *icont, long int *ibnd)
{
  /* Local variables */
  static long int ilft, icrt, ipos, iclft;
  static char tname[9];
  static long int iy, irt;
  
/* protect against contig 0 */
/* initialise end hunts */
  if (*icont == 0)
    {
      return 0;
    }
  ilft = 1000000;
/* loop for each clone */
  irt = -1000000;
  for (iy = 2; iy <= *nclone + 1; ++iy)
    {
/* omit cancellations */
      if (iconta[iy - 1] == *icont)
	{
	  strtcpy(tname, namea + ((iy - 1) << 3), 8);
/* omit burieds */
	  if (*tname == '!')
	    {
	      goto L30;
	    }
	  if (abs(icana[iy - 1]) > 1)
	    {
	      goto L30;
	    }
/* compute ends */
	  ipos = iposa[iy - 1];
	  if (ipos < ilft)
	    {
	      iclft = iy;
	      ilft = ipos;
	    }
	  if (ipos + ilena[iy - 1] > irt)
	    {
	      icrt = iy;
	      irt = ipos + ilena[iy - 1];
	    }
	}
L30:
      ;
    }
  *ibnd = irt - ilft;
  return 0;
} /* sizcont_ */

/* finds size of contig */
int sizcont1_(long int *nclone, long int *icont, long int *ilft,
	      long int *irt)
{
  /* Local variables */
  static long int ibnd, icrt, ipos, iclft;
  static char tname[9];
  static long int iy;
  
/* protect against contig 0 */
/* initialise end hunts */
  if (*icont == 0)
    {
      return 0;
    }
  *ilft = 1000000;
/* loop for each clone */
  *irt = -1000000;
  for (iy = 2; iy <= *nclone + 1; ++iy)
    {
/* omit cancellations */
      if (iconta[iy - 1] == *icont)
	{
	  strtcpy(tname, namea + ((iy - 1) << 3), 8);
/* omit burieds */
	  if (*tname == '!')
	    {
	      goto L30;
	    }
	  if (abs(icana[iy - 1]) > 1)
	    {
	      goto L30;
	    }
/* compute ends */
	  ipos = iposa[iy - 1];
	  if (ipos < *ilft)
	    {
	      iclft = iy;
	      *ilft = ipos;
	    }
	  if (ipos + ilena[iy - 1] > *irt)
	    {
	      icrt = iy;
	      *irt = ipos + ilena[iy - 1];
	    }
	}
L30:
      ;
    }
  ibnd = *irt - *ilft;
  return 0;
} /* sizcont1_ */

/* puts true right end clone in icnuma(PCNT), true left end */
/* in icnuma(PCNT-1); chromosome number in (PCNT-2); */
/* uses shell sort for ordering */
int asscont4_(long int *nclone, long int *icont, long int *ncomp,
	      long int *icnuma, long int *iposav)
{
  /* Local variables */
  static long int ilen, ilft, icrt, ipos, iclft, ictra[4000];
  static char tname[9];
  static long int itemp, iy;
  static short int hchrom;
  static long int ichrom;
  static long int irt;
  static char tremark[41];
  
/* initialise end hunts and chrom number hunt */
  /* Parameter adjustments */
  --icnuma;
  
  /* Function Body */
  irt = -1000000;
  ilft = 1000000;
/* assemble contig */
  ichrom = 0;
  *ncomp = 0;
  if (*icont == 0)
    {
      return 0;
    }
  for (iy = 2; iy <= *nclone + 1; ++iy)
    {
      if (iconta[iy - 1] == *icont)
	{
	  if (abs(icana[iy - 1]) > 1)
	    {
	      goto L30;
	    }
	  if (*ncomp >= 3997)
	    {
	      gprintf (stdout, "  ICNUMA array space exceeded !!!\n") ;
	      waitForMore (RETURN_KEY) ;
	      return 0;
	    }
	  ++(*ncomp);
	  ipos = iposa[iy - 1];
	  if (ipos < ilft)
	    {
	      iclft = iy;
	      ilft = ipos;
	    }
	  ilen = ilena[iy - 1];
	  if (ipos + ilen > irt)
	    {
	      icrt = iy;
	      irt = ipos + ilen;
	    }
	  icnuma[*ncomp] = iy;
	  ictra[*ncomp - 1] = ipos + ilen / 2;
	  if (ichrom == 0 && irema[iy - 1] > 0)
	    {
	      seek_11 (irema[iy - 1]) ;
	      fread (&itemp, sizeof(long int), 1, unit_11);
	      fread (tname, sizeof(char), 8, unit_11);
	      fread (tremark, sizeof(char), 40, unit_11);
	      fread (&hchrom, sizeof(short int), 1, unit_11);
	      if (hchrom > ichrom)
		{
		  ichrom = hchrom;
		}
	    }
	}
L30:
      ;
    }
  if (*ncomp == 0)
    {
      return 0;
    }
  srtctr_(ncomp, ictra, &icnuma[1]);
  *iposav = (iposa[iclft - 1] + iposa[icrt - 1] + 
	     ilena[icrt - 1]) / 2;
  icnuma[4000] = icrt;
  icnuma[3999] = iclft;
  icnuma[3998] = ichrom;
  return 0;
} /* asscont4_ */

/* sort on ictra */
int srtctr_(long int *ncomp, long int *ictra, long int *icnuma)
{
  /* System generated locals */
  long int i__2, i__3;
  
  /* Local variables */
  static long int m;
  static long int iq, jq;

  /* Parameter adjustments */
  --icnuma;
  --ictra;
  
  /* Function Body */
  m = *ncomp;
  while(m != 1)
    {
      m = (m + 2) / 3;
      for (iq = m + 1; iq <= *ncomp; ++iq)
	{
	  i__2 = m + 1;
	  i__3 = -m;
	  for (jq = iq; i__3 < 0 ? jq >= i__2 : jq <= i__2; jq += i__3)
	    {
	      if (ictra[jq] >= ictra[jq - m])
		{
		  goto L101;
		}
	      exchg_(&ictra[jq], &ictra[jq - m]);
	      exchg_(&icnuma[jq], &icnuma[jq - m]);
	    }
L101:
	  ;
	}
    }
  return 0;
} /* srtctr_ */

int exchg_(long int *i, long int *j)
{
  static long int itemp;
  
  itemp = *i;
  *i = *j;
  *j = itemp;
  return 0;
} /* exchg_ */

#ifndef NO_GRAPHICS
/*  [plotcont2_() deleted] */

int plotcont3_(long int *ixs, long int *iys, long int *iscale, 
	       long int *ncomp, long int *icnuma)
{
  /* Local variables */
  char tname3[11], chplt[6];
  static long int ilen;
  static char pgen[9];
  static char prem[41];
  static long int ipxf;
  static long int ipos, ipxt, itxr, ichar;
  static long int itemp, ixmin, ixmax;
  static long int iv, ix, iy, ncharg;
  static long int ncharr;
  static short int hchrom;
  static long int nchars;
  static long int ipy, itx, ity;
  
  /* Parameter adjustments */
  --icnuma;
  
  /* Function Body */
  ixmin = 10;
  ixmax = 1022;
  ichar = 9 ;
  strtcpy(chplt, "     ", 5);
  for (ix = 1; ix <= *ncomp; ++ix)
    {
      iy = icnuma[ix];
      memset (tname3, 0, 10) ;
      strtcpy(tname3, namea + ((iy - 1) << 3), 8);
/* clone is canonical */
      if (icana[iy - 1] == -1)
	{
	  strcat (tname3, " *") ;
	}
      ipos = iposa[iy - 1];
      ilen = ilena[iy - 1];
      itx = (*ixs + ipos + ilen / 2 - 5) * *iscale + 500;
      itxr = itx;
/* clone has remark */
      if (irema[iy - 1] > 0)
	{
	  seek_11 (irema[iy - 1]) ;
	  fread (&itemp, sizeof(long int), 1, unit_11);
	  fread (pgen, sizeof(char), 8, unit_11);
	  fread (prem, sizeof(char) , 40, unit_11);
	  fread (&hchrom, sizeof(short int), 1, unit_11);
	  if (hchrom != 0)
	    {
	      strtcpy(chplt, chrom + hchrom * 5, 5);
	    }
/* clip */
	  ity = *iys - 30;
	  ncharg = max(0,min(8,(ixmax - itx) / ichar));
	  ncharr = max(0,min(40,(ixmax - itx) / ichar));
	  if (itx < ixmin)
	    {
	      ncharg = max(0,8 - (ixmin - itx) / ichar);
	      ncharr = max(0,40 - (ixmin - itx) / ichar);
	      itxr = ixmin;
	      for (iv = 1; iv <= ncharg; ++iv)
		{
		  pgen[iv - 1] = pgen[iv + 8 - ncharg - 1];
		}
	      for (iv = 1; iv <= ncharr; ++iv)
		{
		  prem[iv - 1] = prem[iv + 40 - ncharr - 1];
		}
	    }
	  movevt640_(itxr, ity - 20);
	  textvt640_(pgen, ncharg);
	  movevt640_(itxr, ity);
	  textvt640_(prem, ncharr);
	}
      ipy = *iys + (ix - 1) % 8 * 30 + (ix - 1) / 8 % 2 * 15;
      ity = ipy + 8;
      ipxf = (*ixs + ipos) * *iscale + 500;
      ipxt = ipxf + ilen * *iscale;
      if (ipxf > ixmax)
	{
	  goto L35;
	}
      if (ipxt < ixmin)
	{
	  goto L32;
	}
      if (ipxf < ixmin)
	{
	  ipxf = ixmin;
	}
      if (ipxt > ixmax)
	{
	  ipxt = ixmax;
	}
      nchars = max(0,min(10,(ixmax - itx) / ichar));
      if (itx < ixmin)
	{
	  nchars = max(0,10 - (ixmin - itx) / ichar);
	  itx = ixmin;
	  for (iv = 1; iv <= nchars; ++iv)
	    {
	      tname3[iv - 1] = tname3[iv + 10 - nchars - 1];
	    }
	}
      movevt640_(ipxf, ipy);
      drawvt640_(ipxt, ipy);
      movevt640_(itx, ity);
      textvt640_(tname3, nchars);
L32:
      ;
    }
L35:
  movevt640_(10, *iys + 220);
  textvt640_(chplt, 5);
  return 0;
} /* plotcont3_ */





/* plots PSS links unless project empty; prevents REM fields tangling */
/* imode used for communication:- */
/* mod(imode,10)=1:  VT640 */
/* mod(imode,10)=2:  Plessey */
/* mod(imode/10,10)=0:  plot genes and first level remarks */
/* mod(imode/10,10)=1:  omit remarks, but plot genes */
/* mod(imode/10,10)=2:  plot genes and all remarks */
/* mod(imode/100,10) gives inmsl, allows name selection */
/* mod(imode/1000,100)=0:  single contig */
/* mod(imode/1000,100)=1:  double contig */
int plotcont4_(long int *ixs, long int *iyst, long int *iscale, 
	       long int *ncomp, long int *icnuma, char *projectbase,
	       long int *nclone, long int *imode)
{
  /* Local variables */
  char tname3[13], chplt[6] ;
  static long int ilen;
  static char pgen[9];
  static long int icrt;
  static char prem[41];
  static long int ipxf;
  static long int ivir, ipos, itxg, iply;
  static long int ipry, ipxt;
  static short int hflag;
  static long int ichar;
  static char tname[9];
  static long int itemp, ipcrt, inmsl, ixmin, ixmax;
  static long int itemp1, iq, iv, ix, iy, ncharg, ihspac;
  static short int hchrom;
  static long int ipclft;
  static long int nchars, lnkmin, remlev[15], lnkmax, lnktol;
  static long int itlstg, ixumin;
  static long int ilg, ias, lev;
  static long int ipy, iys, itx, ity;
  static BOOL vir;
  static BOOL bmpg;
  static BOOL npro;
  static long int remnot;
  static long int remall;
  
/* set switches and terminal characteristics */
  /* Parameter adjustments */
  --icnuma;

  /* Function Body */
  if (*ncomp == 0)
    {
      return 0;
    }
  iys = *iyst;
/*  npro=(projectbase.ne.' ') */
  if (strlength (projectbase, 128) == 0)
    npro = FALSE ;
  else
    npro = TRUE ;
  
  inmsl = *imode / 100 % 10;
  remnot = *imode / 10 % 10 == 1;
  remall = *imode / 10 % 10 == 2;

  ixmin = 10;
  ixmax = 1022;
  ichar = 9 ;
  ixumin = ixmin - ichar * 40;
  strtcpy (chplt, "     ", 5);
  if (icnuma[3998] != 0)
    {
      strtcpy(chplt, chrom + icnuma[3998] * 5, 5);
    }
  itlstg = -1000000;
  for (lev = 1; lev <= 15; ++lev)
    {
      remlev[lev - 1] = -1000000;
    }
  lnkmin = 182;
  lnkmax = 840;
  lnktol = 7;
  ipclft = iposa[icnuma[3999] - 1];
  icrt = icnuma[4000];
  ipcrt = iposa[icrt - 1] + ilena[icrt - 1];
  iply = iys + 100;
  ipry = iply;
  ivir = 10;
  ihspac = 15;
  if (inmsl >= 2 && inmsl <= 3)
    {
      ihspac <<= 1;
      iys = 200;
    }
  for (ix = 1; ix <= *ncomp; ++ix)
    {
      iy = icnuma[ix];
      ipos = iposa[iy - 1];
      ilen = ilena[iy - 1];
      ipxf = (*ixs + ipos) * *iscale + 500;
      ipxt = ipxf + ilen * *iscale;
/* fast reject for off screen left */
      if (ipxt < ixumin)
	{
	  goto L32;
	}
/* fast reject for off screen right */
      if (ipxf > ixmax)
	{
	  goto L32;
	}
      if (iflag_(iy, 4) == 1)
	{
	  vir = TRUE;
	}
      else
	{
	  vir = FALSE;
	}
/* allow name selection */
      memset (tname3, 0, 11) ;	/* clear the string */
      if (inmsl > 0)
	{
	  namsel_(tname3, &inmsl, nclone, &iy);
	}
      else
	{
	  strtcpy(tname3, namea + ((iy - 1) << 3), 8);
	}
/* get length of name */
      ias = strlength (tname3, 8) ;
      if (inmsl == 0)
	{
/* canonical */
	  if (icana[iy - 1] == -1)
	    {
	      strcat (tname3, " *") ;
	    }
	  if (vir)
	    {
	      for (iq = ias; iq >= 1; --iq)
		{
		  tname3[iq] = tname3[iq - 1];
		}
	      tname3[0] = '(';
	      strcat (tname3, ")") ;
	    }
	}
/*      itx = (*ixs + ipos + ilen / 2) * *iscale + 500 - ias / 2 * ichar; */
      itx = (ipxf + ipxt) / 2 - ias / 2 * ichar ;
/* clone has remark */
      if (irema[iy - 1] > 0)
	{
	  seek_11 (irema[iy - 1]) ;
	  fread (&itemp, sizeof(long int), 1, unit_11);
	  fread (pgen, sizeof(char), 8, unit_11);
	  fread (prem, sizeof(char), 40, unit_11);
	  fread (&hchrom, sizeof(short int), 1, unit_11);
	  fread (&hflag, sizeof(short int), 1, unit_11);
	  ity = iys - 30;
/* get length of gene */
	  ilg = strlength (pgen, 8) ;
	  if (ilg == 0)
	    {
	      goto L37;
	    }
/* bump gene to right if necessary */
	  if (itlstg > itx)
	    {
	      itxg = itlstg;
	      bmpg = TRUE;
	    }
	  else
	    {
	      itxg = itx;
	      bmpg = FALSE;
	    }
/* clip gene */
	  itlstg = itxg + (ilg + 1) * ichar;
	  ncharg = max(0,min(8,(ixmax - itxg) / ichar));
	  if (itxg < ixmin)
	    {
	      ncharg = max(0,8 - (ixmin - itxg) / ichar - 1);
	      itxg += (8 - ncharg) * ichar;
	      for (iv = 1; iv <= ncharg; ++iv)
		{
		  pgen[iv - 1] = pgen[iv + 8 - ncharg - 1];
		}
	    }
/* plot gene, indicate bump with comma */
	  if (bmpg && itxg - ichar > ixmin)
	    {
	      movevt640_(itxg - ichar, ity);
	      textvt640_(",", 1);
	    }
	  movevt640_(itxg, ity);
	  textvt640_(pgen, ncharg);
L37:
/* skip if remarks not wanted */
	  if (remnot)
	    {
	      goto L36;
	    }
	  remplot_(prem, &ichar, remlev, &c__15, &ixmin, &ixmax, &itx, &ity);
/* plot secondary remarks */
	  if (remall)
	    {
	      while(hflag != 0)
		{
		  itemp1 = hflag;
/* not yet implemented */
		  if (itemp1 < 0)
		    {
		      itemp1 = 32767 - itemp1;
		    }
		  seek_11 (itemp1) ;
		  fread (&itemp, sizeof(long int), 1 , unit_11);
		  fread (tname, sizeof(char), 8, unit_11);
		  fread (prem, sizeof(char), 40, unit_11);
		  fread (&hchrom, sizeof(short int), 1, unit_11);
		  fread (&hflag, sizeof(short int), 1, unit_11);
		  remplot_(prem, &ichar, remlev, &c__15, &ixmin, &ixmax, &
			   itx, &ity);
		}
	    }
	}
L36:
      ipy = iys + (ihspac << 1) * ((ix - 1) % 8) +
	ihspac * ((ix - 1) / 8 % 2);

      if (vir)
	{
	  ipy += 4;
	}
      ity = ipy + 8;
      if (ipxt < ixmin)
	{
	  goto L32;
	}
      if (npro && ipos - ipclft < lnktol)
	{
	  if (ipxf > lnkmin)
	    {
	      dislnk_(&iy, &c_n1, &ipxf, &iply, projectbase, nclone);
	    }
	}
      if (npro && ipcrt - ipos - ilen < lnktol)
	{
	  if (ipxt < lnkmax)
	    {
	      dislnk_(&iy, &c__1, &ipxt, &ipry, projectbase, nclone);
	    }
	}
      if (ipxf < ixmin)
	{
	  ipxf = ixmin;
	}
      if (ipxt > ixmax)
	{
	  ipxt = ixmax;
	}
      nchars = max(0, min(10, (ixmax - itx) / ichar));
      if (itx < ixmin)
	{
	  nchars = max(0, 10 - (ixmin - itx) / ichar - 1);
	  itx += (10 - nchars) * ichar;
	  for (iv = 1; iv <= nchars; ++iv)
	    {
	      tname3[iv - 1] = tname3[iv + 10 - nchars - 1];
	    }
	}
      movevt640_(ipxf, ipy);
      drawvt640_(ipxt, ipy);
/* emphasise clones if inmsl eq 2 or 3 */
      if (inmsl >= 2 && inmsl <= 3)
	{
	  if (strlength (tname3, 10) > 0)
	    {			/* if tname3 not empty */
	      for (iq = 1; iq <= 3; ++iq)
		{
		  movevt640_(ipxf, ipy + iq);
		  drawvt640_(ipxt, ipy + iq);
		}
	    }
	}
      movevt640_(itx, ity);
      textvt640_(tname3, nchars);
L32:
      ;
    }
  movevt640_(10, iys + (ihspac << 4) + ihspac);
  textvt640_(chplt, 5);
  return 0;
} /* plotcont4_ */






/* plots single contig in CHPLT style */
int plotcont5_(long int *ixs, long int *iys, long int *ixmin, 
	       long int *ixmax, long int *ictr, long int *iscale,
	       long int *ncomp, long int *icnuma)
{
  /* System generated locals */
  float r__1, r__2;
  
  /* Local variables */
  char tname[9];
  static float genl;
  static long int ilft, iysc;
  static float csiz;
  static float xmax;
  static char gene1[9];
  static float scale;
  static long int itemp, iykey, iq, ix, iy;
  static float xl;
  static float xr;
  static char remark[41];
  static float gpt;
  static long int irt;
  
  /* Parameter adjustments */
  --icnuma;
  
  /* Function Body */
  csiz = 2.7f;
  scale = (float) (*iscale) / 10.f;
  scale1 = 1.f;
  xzero1 = (float) (*ixmin);
  xmax = (float) (*ixmax);
  levgap1 = 24;
  maxlev1 = 25;
  ichrgp = *iys;
  iysc = 30;
  iykey = 55;
  for (iq = 1; iq <= 6; ++iq)
    {
      genlev1[iq + 5993] = 0;
    }
  ilft = iposa[icnuma[3999] - 1];
  irt = iposa[icnuma[4000] - 1] + ilena[icnuma[4000] - 1];
  xl = dmax(xzero1,(float) (*ictr) + (float) (ilft + *ixs)*scale);
  xr = dmin(xmax,(float) (*ictr) + (float) (irt + *ixs) * scale);
  movevt640_((long int) xl, *iys);
  drawvt640_((long int) xr, *iys);
  strcpy(tname, "^");
  movevt640_(*ictr - 4, *iys - 15);
  textvt640_(tname, 1);
  for (ix = *ncomp; ix >= 1; --ix)
    {
      iy = icnuma[ix];
      if (irema[iy - 1] != 0)
	{
	  gpt = (float) (*ictr) + (float) (iposa[iy - 1] + 
					   ilena[iy - 1] / 2 + *ixs) *
					     scale;
	  if (gpt >= xzero1 && gpt <= xmax)
	    {
	      seek_11 (irema[iy - 1]) ;
	      fread (&itemp, sizeof(long int), 1, unit_11);
	      fread (gene1, sizeof(char), 8, unit_11);
	      fread (remark, sizeof(char), 40, unit_11);
	      if (!(*gene1 == ' ' || *gene1 == 0)) /* if not empty */
		{
		  gnl_(gene1, &genl, &csiz);
		  r__1 = gpt - xzero1;
		  r__2 = genl * 4.4f;
		  genplot1_(&r__1, &r__2, &sh__1, gene1, &c__6);
		}
	    }
	}
    }
  return 0;
} /* plotcont5_ */



int remplot_(char *prem, long int *ichar, long int *remlev, 
	     long int *iplev, long int *ixmin, long int *ixmax,
	     long int *itx, long int *ity)
{
  /* Local variables */
  static long int ilft;
  static long int itxr, iv, ncharr, levgap, iremsp, ilr, lev, irt;
  
  /* Parameter adjustments */
  --remlev;
  --prem;
  
  /* Function Body */
  iremsp = 10;
  levgap = 20;
  itxr = *itx;

/* find length of remark */
  ilr = strlength (prem, 40) ;

/* skip if no remark */
  if (ilr == 0)
    {
      return 0;
    }
/* find plot level for remark */
  ilft = itxr;
  irt = itxr + ilr * *ichar;
  for (lev = 1; lev <= *iplev; ++lev)
    {
      if (ilft >= remlev[lev])
	{
	  goto L73;
	}
    }
  lev = *iplev;
/* clip remark */
L73:
  remlev[lev] = irt + iremsp;
  ncharr = max(0,min(40,(*ixmax - *itx) / *ichar));
  if (itxr < *ixmin)
    {
      ncharr = max(0,40 - (*ixmin - *itx) / *ichar - 1);
      itxr += (40 - ncharr) * *ichar;
      for (iv = 1; iv <= ncharr; ++iv)
	{
	  prem[iv] = prem[iv + 40 - ncharr];
	}
    }
  movevt640_(itxr, max(1, *ity - levgap * lev));
  textvt640_(prem + 1, ncharr);
  return 0;
} /* remplot_ */
#endif /* ! NO_GRAPHICS */

int contrd_(long int *icont, long int *iconta, long int *nclone)
{
  /* Parameter adjustments */
  --iconta;
  
  /* Function Body */
L101:
  contstack_("r", icont);
  contchk_(icont, &iconta[1], nclone);
  if (*icont == 0)
    {
      goto L101;
    }
  return 0;
} /* contrd_ */

int contchk_(long int *icont, long int *iconta, long int *nclone)
{
  /* Local variables */
  static long int ix;
  
  /* Parameter adjustments */
  --iconta;
  
  /* Function Body */
  for (ix = 2; ix <= *nclone + 1; ++ix)
    {
      if (iconta[ix] == *icont)
	{
	  *icont = 0;
	  return 0;
	}
    }
  return 0;
} /* contchk_ */

int contstack_(char *rw, long int *icontt)
{
  /* Local variables */
  static long int nextn, ia, ix;
  
  /* read new contig number */
  if (*rw == 'r')
    {
      ix = 0;
L100:
      ++ix;
      seek_10 (ix) ;
      fread (&ia, sizeof(long int), 1, unit_10) ;
      if (ia != 0)
	{
	  goto L100;
	}
      seek_10 (ix - 1) ;
      fread (&(*icontt), sizeof(long int), 1, unit_10) ;
      seek_10 (ix - 1) ;
      fwrite (&c__0, sizeof(long int), 1, unit_10) ;
/* update if down to first entry */
      if (ix == 2)
	{
	  nextn = *icontt + 1;
	  seek_10 (1) ;
	  fwrite (&nextn, sizeof(long int), 1, unit_10) ;
	}
      fwrt_(unit_10);
      return 0;
    }
/* write disused contig number */
  if (*rw == 'w')
    {
      ix = 0;
L101:
      ++ix;
      seek_10 (ix) ;
      fread (&ia, sizeof(long int), 1, unit_10) ;

      if (ia == *icontt)
	{
	  *icontt = 0;
	  return 0;
	}
      if (ia != 0)
	{
	  goto L101;
	}
      seek_10 (ix) ;
      fwrite (&(*icontt), sizeof(long int), 1, unit_10) ;
      seek_10 (ix + 1) ;
      fwrite (&c__0, sizeof(long int), 1, unit_10) ;
      fwrt_(unit_10);

      *icontt = 0;
      return 0;
    }
  else
    {
      exit (-1) ;
    }
  return 0;
} /* contstack_ */

int contloss_(long int *icont2, long int *iconta, long int *nclone)
{
  /* Local variables */
  static long int igel, isum;
  static long int lband, nband;
  static char tname[9];
  static long int itemp;
  static long int ix, iclone;
  
  /* Parameter adjustments */
  --iconta;
  
  /* Function Body */
  isum = 0;
  for (ix = 2; ix <= *nclone + 1; ++ix)
    {
      if (iconta[ix] == *icont2)
	{
	  ++isum;
	  iclone = ix;
	}
    }
  if (isum > 1)
    {
      return 0;
    }
  if (isum == 1)
    {
      iconta[iclone] = 0;
      seek_1 (iclone) ;
      fread (tname, sizeof(char), 8, unit_1);
      fread (&nband, sizeof(long int), 1, unit_1);
      fread (&lband, sizeof(long int), 1, unit_1);
      fread (&igel, sizeof(long int), 1, unit_1);
      fread (&itemp, sizeof(long int), 1, unit_1);

      seek_1 (iclone) ;
      fwrite (tname, sizeof(char), 8, unit_1);
      fwrite (&nband, sizeof(long int), 1, unit_1);
      fwrite (&lband, sizeof(long int), 1, unit_1);
      fwrite (&igel, sizeof(long int), 1, unit_1);
      fwrite (&c__0, sizeof(long int), 1, unit_1);
      fwrt_(unit_1);
    }
  contstack_("w", icont2);
  return 0;
} /* contloss_ */

#ifndef NO_GRAPHICS
int cursBox = 0 ;		/* will be zero'ed again by graphclear */
extern float winWidth, winHeight ; /* determined by graphWhere in main() */

int cursor_(long int *ichar, long int *ipcx, long int *iscale, 
	    long int *isch)
{
  /* Local variables */
  static long int icdisx, icsetx, icx;
  int posBox=0 ;
  int key ;

  selectvt640_();
L1:
  if (*ipcx < -(490 / *iscale))
    {
      *ipcx = -(480 / *iscale);
    }
  if (*ipcx > 490 / *iscale)
    {
      *ipcx = 480 / *iscale;
    }
  icdisx = *ipcx - icsetx;

  selectvt100_();
  sprintf (stemp, "%ld      ", icdisx) ;
  if (posBox) graphBoxClear (posBox) ;
  posBox = graphBoxStart () ;
  graphText (stemp, MAPX(1*8), MAPY(WIN_HEIGHT-(2*13))) ;
  graphBoxEnd () ;
  graphBoxDraw (posBox, BLACK, WHITE) ;

  selectvt640_();
  icx = *ipcx * *iscale + 500 ;
  

  if (!cursBox)
    {
      cursBox = graphBoxStart () ;
      graphLine (MAPX(icx), MAPY(2), MAPX(icx), MAPY(WIN_HEIGHT-2)) ;
      graphBoxEnd () ;
      graphBoxDraw (cursBox, BLUE, BLUE) ;
    }
  graphBoxShift (cursBox, MAPX(icx), 0) ;

/* wait */
L2:
  key = waitForKey (0) ;
  if (key == 0)
    {
      goto L2;
    }
  else if (key == 44 || key == 60 || key == LEFT_KEY)
    {
      --(*ipcx);
    }
  else if (key == 46 || key == 62 || key == RIGHT_KEY)
    {
      ++(*ipcx);
    }
  else if (key == 115 || key == 83)
    {
      icsetx = *ipcx;
    }
  else if (key == 122 || key == 90)
    {
      *ipcx = icsetx;
    }
  else
    {
      *ichar = key;
      graphBoxClear (posBox) ;
      graphBoxClear (cursBox) ; cursBox = 0 ;
      selectvt100_();
      return 0;
    }
  goto L1;
} /* cursor_ */

#endif /* ! NO_GRAPHICS */

int scale_(long int *iscale)
{
  /* Local variables */
  static long int itemp;
 L73:
  gprintf (stdout, "   scale factor (def = %d) = ", *iscale) ;
  readinput ("", stemp, 5) ; if (nonint(stemp)) goto L73 ;
  itemp = a2int (stemp) ;
  if (itemp > 0)
    {
      *iscale = itemp;
    }
  return 0;
} /* scale_ */

int lsiz_(long int *nclone)
{
  /* Local variables */
  static long int ibnd, iend, gdir[2000], ichr, itnc, ilft, igpt, ctot[6];
  static char gene1[9];
  static float pchr1, rkbnd;
  static long int ichro, itskb, icnto, iszkb, ipost;
  static short int hflag1;
  static long int icnum1[4000], ncomp1;
  static short int hsitu1;
  static long int iq, iu;
  static long int iclone;
  static long int chrnum;
  static long int iposav, chstrt[9];
  static short int hchrom1;
  static char remark1[41];
  static float cpc[1200]	/* was [200][6] */;
  static long int cpn[1200]	/* was [200][6] */;
  static long int irt;
  static FILE *iunit ;
  
/* 200 in above lines was pncnt */
/* kb per band */
  rkbnd = 1.8f;
  iunit = stdout ;

  gprintf (stdout,
	   "   0-6   single chromosome\n"
	   "   7     all\n"
	   "   8     alphabetical\n"
	   "   9     by contig\n") ;
L1000:
  gprintf (stdout, "     enter choice : ") ;
  readinput ("", stemp, 2) ; if (strlength (stemp, 2) == 0) goto L1092 ;
  if (nonint(stemp)) goto L1000 ;
  ichr = a2int (stemp) ;
  if (ichr == 9)
    {
      itskb = 0;
      itnc = 0;
      gencont1_(nclone, gdir, &igpt, chstrt, ctot, cpn, cpc);
      iunit = getfile_();
      
      gprintf (iunit, "chrom  contig   clone       gene      "
	       "genetic%%  in situ%%    kb from\n"
	       "        size                          "
	       "                     leftmost gene\n") ;
      icnto = -1;
      for (ichro = 0; ichro <= 6; ++ichro)
	{
	  strtcpy (ntemp1, chrom + ichro * 5, 5) ;
	  gprintf (iunit, "%5s\n", ntemp1);
	  for (iu = chstrt[ichro]; iu <= chstrt[ichro + 1] - 1; ++iu)
	    {
	      seek_11 (gdir[iu - 1]) ;
	      fread (&iclone, sizeof(long int), 1, unit_11);
	      fread (gene1, sizeof(char), 8, unit_11);
	      fread (remark1, sizeof(char), 40, unit_11);
	      fread (&hchrom1, sizeof(short int), 1, unit_11);
	      fread (&hflag1, sizeof(short int), 1, unit_11);
	      fread (&pchr1, sizeof(float), 1, unit_11);
	      fread (&hsitu1, sizeof(short int), 1, unit_11);
	      if (iconta[iclone - 1] != icnto ||
		  iconta[iclone - 1] == 0)
		{
		  icnto = iconta[iclone - 1];
		  ipost = iposa[iclone - 1];
		  if (iconta[iclone - 1] == 0)
		    {
		      iszkb = (long int) (ilena[iclone - 1] * rkbnd);
		    }
		  else
		    {
		      sizcont_(nclone, &iconta[iclone - 1], &ibnd);
		      iszkb = ibnd * rkbnd;
		    }
		  itskb += iszkb;
		  ++itnc;
		  strtcpy (ntemp1, namea + ((iclone - 1) << 3), 8) ;
		  strtcpy (ntemp2, gene1, 8) ;
		  gprintf (iunit, "     %6dkb   %-8s    %-8s  %6.1f"
			   "   %3d -%3d%8d\n",
			   iszkb, ntemp1, ntemp2,
			   pchr1, hsitu1 / 128, hsitu1 % 128, 0);
		}
	      else
		{
		  strtcpy (ntemp1,  namea + ((iclone - 1) << 3), 8) ;
		  strtcpy (ntemp2, gene1, 8) ;
		  gprintf (iunit, "                %-8s    %-8s  %6.1f"
			   "   %3d -%3d%8.0f\n",
			   ntemp1, ntemp2,
			   pchr1, hsitu1 / 128, hsitu1 % 128,
			   (float)(iposa[iclone - 1] - ipost) * rkbnd) ;
		}
	    }
	}
      gprintf (iunit, "     total named kb = %6d\n"
	       "     named contigs  = %6d\n"
	       "     no of markers  = %6d\n"
	       " NB  All kb estimates depend on a "
	       "number of approximations\n",
	       itskb, itnc, igpt);
    }
  else
    {
      iend = ichr;
      chrnum = ichr >= 0 && ichr <= 7;
      if (ichr == 7)
	{
	  ichr = 0;
	  iend = 6;
	}
      iunit = getfile_();
L1092:
      gprintf (iunit, " clone       gene              "
	       "genetic%%    in situ%%\n") ;

      gensort_(gdir, &igpt);
      for (ichr = ichr; ichr <= iend; ++ichr)
	{
	  gprintf (iunit, "\n") ;
	  for (iu = 1; iu <= igpt; ++iu)
	    {
	      seek_11 (gdir[iu - 1]) ;
	      fread (&iclone, sizeof(long int), 1, unit_11);
	      fread (gene1, sizeof(char), 8, unit_11);
	      fread (remark1, sizeof(char), 40, unit_11);
	      fread (&hchrom1, sizeof(short int), 1, unit_11);
	      fread (&hflag1, sizeof(short int), 1, unit_11);
	      fread (&pchr1, sizeof(float), 1, unit_11);
	      fread (&hsitu1, sizeof(short int), 1, unit_11);
	      if (iclone > 0 && strlength (gene1, 8) > 0)
		{		/* if gene1 not empty */
		  if (chrnum && hchrom1 != ichr)
		    {
		      goto L69;
		    }
		  asscont_(nclone, &iconta[iclone - 1], 
			   iconta, iposa, &ncomp1, icnum1,
			   &iposav, icana);
		  if (iconta[iclone - 1] == 0)
		    {
		      iszkb = (long int) (ilena[iclone - 1] * rkbnd);
		    }
		  else
		    {
		      ilft = iposa[icnum1[0] - 1];
		      irt = iposa[icnum1[ncomp1 - 1] - 1] + 
			ilena[icnum1[ncomp1 - 1] - 1];
		      for (iq = ncomp1 - 1; iq >= max(1,ncomp1 - 10); --iq)
			{
			  irt = max(irt,iposa[icnum1[iq - 1]-1] +
				    ilena[icnum1[iq - 1] - 1]);
			}
		      iszkb = (long int) ((irt - ilft) * rkbnd);
		    }
		  strtcpy (ntemp1, namea + ((iclone - 1) << 3), 8) ;
		  strtcpy (ntemp2, gene1, 8) ;
		  strtcpy (ntemp3, chrom + hchrom1 * 5, 5) ;
		  gprintf (iunit, " %-8s    %-8s %5s   %6.1f     %3d -"
			   "%3d     %6dkb\n",
			   ntemp1, ntemp2, ntemp3,
			   pchr1, hsitu1 / 128, hsitu1 % 128, iszkb);
		}
L69:
	      ;
	    }
	}
    }
  if (iunit != stdout)
    fclose (iunit) ;
  return 0;
} /* lsiz_ */


int gensort_(long int *gdir, long int *igpt)
{
  /* Local variables */
  static char gene1[9], genea[8*2000];
  static long int itemp, ij, iq, iz, iclone;
  static BOOL test;
  
  /* Parameter adjustments */
  --gdir;
  
  /* Function Body */
  *igpt = 0;
  iz = 0;
L69:
  ++iz;
  seek_11 (iz) ;
  fread (&iclone, sizeof(long int), 1, unit_11);
  fread (gene1, sizeof(char), 8, unit_11);
  if (iclone > 0 && strlength (gene1, 8) > 0)
    {		      /* if gene1 not empty */
      ++(*igpt);
      gdir[*igpt] = iz;
      strncpy(genea + ((*igpt - 1)  << 3), gene1, 8);
    }
/* sort genes into name order */
  if (iclone != -9)
    {
      goto L69;
    }
L6:
  test = FALSE;
  for (ij = 2; ij <= *igpt; ++ij)
    {
      for (iq = 1; iq <= 8; ++iq)
	{
	  if (genea[((ij - 1) << 3) + (iq - 1)] >
	      genea[((ij - 2) << 3) + (iq - 1)])
	    {
	      goto L25;
	    }
	  if (genea[((ij - 1) << 3) + (iq - 1)] <
	      genea[((ij - 2) << 3) + (iq - 1)])
	    {
	      goto L8;
	    }
	}
      goto L25;
L8:
      test = TRUE;
      itemp = gdir[ij];
      strtcpy(gene1, genea + ((ij - 1) << 3), 8);
      gdir[ij] = gdir[ij - 1];
      strncpy(genea + ((ij - 1) << 3), genea + ((ij - 2) << 3), 8);
      gdir[ij - 1] = itemp;
      strncpy(genea + ((ij - 2) << 3), gene1, 8);
L25:
      ;
    }
  if (test)
    {
      goto L6;
    }
  return 0;
} /* gensort_ */

int getbnds_(long int *ic, long int *nbandm, long int *coords)
{
  /* Local variables */
  static long int iq;
  
  /* Parameter adjustments */
  --coords;
  
  /* Function Body */
  *nbandm = nbanda[*ic - 1];
  for (iq = 1; iq <= *nbandm; ++iq)
    {
      coords[iq] = coorda[lbanda[*ic - 1] - 1 + iq - 1];
    }
  return 0;
} /* getbnds_ */

/* apparently only used for mapsub7.c */
int getset_(long int *ncln, long int *clst, long int *nclone)
{
  /* Local variables */
  static char name[9];
  static long int i, ifrom, iq, ito;
  
  /* Parameter adjustments */
  --clst;
  
  gprintf (stdout, "   first clone in set = ") ;
  memset (name, 0, 8) ;
  readinput ("", name, 8) ;
  fprintf (stdout, "%s\n", name) ; /* output to screen (map.out) */

  if (strlength (name, 8) == 0)
    {
      i = 2;
      goto L17;
    }
  getline_(name, &i, nclone);
L17:
  ifrom = i;
  gprintf (stdout, "   last clone in set = ") ;
  memset (name, 0, 8) ;
  readinput ("", name, 8) ;
  fprintf (stdout, "%s\n", name) ; /* output to screen (map.out) */

  if (strlength (name, 8) == 0)
    {
      i = *nclone + 1;
      goto L19;
    }
  getline_(name, &i, nclone);
L19:
  ito = i;
  for (iq = ifrom; iq <= ito; ++iq)
    {
      ++(*ncln);
      clst[*ncln] = iq;
    }
  return 0;
} /* getset_ */

int readlst_(long int *ncln, long int *clst, long int *nclone)
{
  /* Local variables */
  static char name[9];
  static long int line;
  
  /* Parameter adjustments */
  --clst;
  
  gprintf (stdout, "   enter clones individually, ending with # ") ;
 L101:
  memset (name, 0, 8) ;
  readinput ("", name, 8) ;
  if (strlength (name, 8) == 0)
    {
      goto L101;
    }
  if (strcmp(name, "#") == 0)
    {
      return 0;
    }
  getline_(name, &line, nclone);
  ++(*ncln);
  clst[*ncln] = line;
  goto L101;
} /* readlst_ */

int getcont_(long int *ncln, long int *clst, long int *nclone, 
	     long int *iconta)
{
  /* Local variables */
  static char name[9];
  static long int i, icont, ifrom, iq, ito;
  
  /* Parameter adjustments */
  --iconta;
  --clst;
  
  /* Function Body */
L10:
  gprintf (stdout, "   contig number = ") ;
  readinput ("", stemp, 10) ; if (nonint (stemp)) goto L10 ;
  icont = a2int (stemp) ;

  gprintf (stdout, "   first clone to consider = ") ;
  memset (name, 0, 8) ;
  readinput ("", name, 8) ;
  if (strlength (name, 8) == 0)
    {
      i = 2;
      goto L17;
    }
  getline_(name, &i, nclone);
L17:
  gprintf (stdout, "   last clone to consider = ") ;
  readinput ("", name, 8) ;
  if (strlength (name, 8) == 0)
    {
      i = *nclone + 1;
      goto L19;
    }
  getline_(name, &i, nclone);
 L19:
  ito = i;
  for (iq = ifrom; iq <= ito; ++iq)
    {
      if (iconta[iq] == icont)
	{
	  ++(*ncln);
	  clst[*ncln] = iq;
	}
    }
  return 0;
} /* getcont_ */

int getline_(char *name, long int *line, long int *nclone)
{
  /* Local variables */
  static long int i;
  
  for (i = 2; i <= *nclone + 1; ++i)
    {
      if (strncmp(name, namea + ((i - 1) << 3), 8) == 0)
	{
	  goto L17;
	}
    }
  gprintf (stdout, "%s not found \n", name) ;
  exit (-1) ;
L17:
  *line = i;
  return 0;
} /* getline_ */

int getline1_(char *name, long int *line, long int *nclone)
{
  /* Local variables */
  static long int i;
  
  for (i = 2; i <= *nclone + 1; ++i)
    {
      if (strncmp(name, namea + ((i - 1) << 3), 8) == 0)
	{
	  goto L17;
	}
    }
  gprintf (stdout, "%s not found\n", name) ;
  *line = 0;
  return 0;
L17:
  *line = i;
  return 0;
} /* getline1_ */

/* writes individual bits into FLG file */
/* n=1  autopos */
/*   2  reject */
/*   3  possible */
/*   4  virtual */
/*   5  gridded cosmid */
/*   6  canonical of gridded cosmid */
/*   7  gridded YAC */
/*   8  hidden contig */
int wrflag_(long int *i, long int *n, long int *iflag)
{
  /* Local variables */
  static long int irec;
  static long int modrec, intrec;
  
  if (*iflag != 0 && *iflag != 1)
    {
      goto L101;
    }
  seek_13 (*i) ;
  fread (&irec, sizeof(long int), 1, unit_13);
  if (FILerror(unit_13)) goto L100 ;
  intrec = irec / pow_ii(2, *n);
  modrec = irec % pow_ii(2, *n - 1);
  irec = ((intrec << 1) + *iflag) * pow_ii(2, *n - 1) + modrec;
  seek_13 (*i) ;
  fwrite (&irec, sizeof(long int), 1, unit_13) ;
  fwrt_(unit_13);
  return 0;
L100:
  gprintf (stdout, " .flg file not open\n");
  return 0;
L101:
  gprintf (stdout, " IFLAG out of range\n");
  return 0;
} /* wrflag_ */

int iflag_(long int i, long int n)
{
  /* Local variables */
  static long int irec;
  
  seek_13 (i) ;
  fread (&irec, sizeof(long int), 1, unit_13);
  if (FILerror(unit_13) || feof(unit_13)) return 0 ;

  return ((irec / pow_ii(2, n - 1)) % 2) ;
} /* iflag_ */

/* returns char in cflag if flag(n) of clone i is set */
char cflag_(long int i, long int n, char *char_)
{
  if (iflag_(i, n) == 1)
    {
      return *char_;
    }
  else
    {
      return ' ';
    }
} /* cflag_ */

/* allows toggling of flags belonging to clone i */
int flag_(long int *i)
{
  /* Local variables */
  static long int inew, iflg1, iflg2;
  static char ipres;
  static char ans[1];
  
  ipres = iflag_(*i, 1);

  gprintf (stdout, " type Y to switch autopos flag (def = %d) ", ipres) ;
  readinput ("", stemp, 2) ; *ans = *stemp ;
  if (*ans == '^')
    {
    L10:
      gprintf (stdout, " enter : >flag number,flag value<  : ") ;
      readinput ("", stemp, 3) ;
      ntemp1[0] = stemp[0] ; ntemp1[1] = 0 ;
      ntemp2[0] = stemp[2] ; ntemp2[1] = 0 ;
      if (nonint(ntemp1) || nonint(ntemp2) || stemp[1] != ',') goto L10 ;
      iflg1 = a2int(ntemp1) ; iflg2 = a2int (ntemp2) ;
      wrflag_(i, &iflg1, &iflg2);
    }
  if (*ans != 'y' && *ans != 'Y')
    {
      return 0;
    }
  if (ipres == 0)
    {
      inew = 1;
    }
  else
    {
      inew = 0;
    }
  wrflag_(i, &c__1, &inew);
  return 0;
} /* flag_ */

/* returns 'limbo' in clim if clone(i) is in limbo */
void clim_(char *ret_val, long int *i)
{
  if (icana[*i - 1] > 10000000)
    {
      strtcpy(ret_val, "limbo", 5);
    }
  else
    {
      strtcpy(ret_val, "     ", 5);
    }
} /* clim_ */

/* keyword routines */
int keysub_(BOOL *key, long int *i, long int *nclone, long int *iret)
{
  /* Local variables */
  static long int klen, ikey;
  static char resp[10];
  static long int ilst;
  static long int iclnc;
  static char kname[25];
  static long int isrch, icpnt, ktlen;
  static char tname1[9];
  static long int iq;
  static char scname[41];
  static long int irsrch;
  static long int acc ;
  static char ans[1];
  static BOOL cnt, all, intl ;
  
  if (*key)
    {
      goto L81;
    }
L80:
  ikey = 0;
  all = FALSE;
  intl = FALSE;
  cnt = FALSE;
/* default is all */
  ilst = -1;
  *key = TRUE;
  icpnt = 0;
  keyw1_(kname, &ktlen, &klen);
  if (klen == 0 || ktlen == 0)
    {
      *key = FALSE;
      *iret = 8;
      return 0;
    }
/* get options */
  gprintf (stdout,
	   " Options :\n"
	   "  F = search first characters only\n"
	   "  C = output by contig\n"
	   "  P = pure contigs only\n"
	   "  S = selected contig\n"
	   "  R = reduced format (def = search all) : ") ;
  readinput ("", resp, 5) ;
  for (iq = 0; iq < strlen(resp); ++iq)
    {
      *ans = resp[iq];
      if (*ans == 'F' || *ans == 'f')
	{
	  intl = TRUE;
	}
      if (*ans == 'S' || *ans == 's')
	{
	  cnt = TRUE;
	}
      if (*ans == 'C' || *ans == 'c')
	{
	  ilst = 1;
	}
      if (*ans == 'G' || *ans == 'g')
	{
	  ilst = 2;
	}
      if (*ans == 'R' || *ans == 'r')
	{
	  ilst = 3;
	}
    }
/* default search is all lists */
  if (ilst == -1)
    {
      ilst = 1;
      all = TRUE;
/* begin clone search at second record */
    }
/* set extent of search fields */
  if (ilst == 1)
    {
      ikey = 1;
    }
  if (intl)
    {
      isrch = 1;
      irsrch = 1;
    }
  else
    {
      isrch = 9 - klen;
      irsrch = 41 - klen;
/* begin or continue key search */
    }
/* pick up next name according to chosen option */
L81:
  ++ikey;
  if (ilst == 1)
    {
/* action at end of clone list */
      if (ikey > *nclone + 1)
	{
	  if (all)
	    {
	      ilst = 4;
	      ikey = 0;
	      goto L81;
	    }
	  else
	    {
	      goto L80;
	    }
	}
      else
	{
	  if (keychk_(&ikey, &isrch, &ktlen, kname))
	    {
	      if (cnt)
		{
		  if (! cntchk_(&ikey, &icpnt))
		    {
		      goto L81;
		    }
		}
	      *i = ikey;
	      *iret = 2;
	      return 0;
	    }
	}
    }
  else
    {
      seek_11 (ikey) ;
      fread (&iclnc, sizeof(long int), 1, unit_11);
      fread (tname1, sizeof(char), 8, unit_11);
      fread (scname, sizeof(char), 40, unit_11);
      if (FILerror (unit_11)) goto L80 ;

      if (ilst == 2)
	{
	  acc = keychk1_(tname1, &isrch, &ktlen, kname);
	}
      if (ilst == 3)
	{
	  acc = keychk1_(scname, &irsrch, &ktlen, kname);
	}
      if (ilst == 4)
	{
	  acc = keychk1_(tname1, &isrch, &ktlen, kname) || 
	    keychk1_(scname, &irsrch, &ktlen, kname);
	}
      if (acc)
	{
	  if (cnt)
	    {
	      if (! cntchk_(&iclnc, &icpnt))
		{
		  goto L81;
		}
	    }
	  *i = iclnc;
	  *iret = 2;
	  return 0;
	}
    }
  goto L81;
} /* keysub_ */

BOOL keychk_(long int *ikey, long int *isrch, long int *ktlen, char *kname)
{
  /* System generated locals */
  BOOL ret_val;
  
  /* Local variables */
  static long int iasc, knum, ip, iq;
  static char kchar[2], nmchar[2];
  static BOOL code;
  
  ret_val = FALSE;
/* search with categories allowed */
  for (iq = 1; iq <= *isrch; ++iq)
    {
      code = FALSE;
      knum = -1;
      for (ip = 1; ip <= *ktlen; ++ip)
	{
	  *kchar = kname[ip - 1];
	  if (*kchar == '{')
	    {
	      code = TRUE;
	      goto L70;
	    }
	  if (*kchar == '}')
	    {
	      code = FALSE;
	      goto L70;
	    }
	  ++knum;
	  strtcpy(nmchar, namea + (((*ikey - 1) << 3) + iq + knum - 1), 1);
	  if (code)
	    {
	      iasc = *nmchar;
	      if (*kchar == 'i' || *kchar == 'I')
		{
		  if (iasc < 48 || iasc > 57)
		    {
		      goto L60;
		    }
		}
	      else if (*kchar == 'l' || *kchar == 'L')
		{
		  if (iasc < 65 || (iasc > 90 && iasc < 97) || iasc > 122)
		    {
		      goto L60;
		    }
		}
	      else
		{
		  goto L70;
		}
	    }
	  else
	    {
	      if (*nmchar != *kchar)
		{
		  goto L60;
		}
	    }
L70:
	  ;
	}
      ret_val = TRUE;
      return ret_val;
L60:
      ;
    }
  return ret_val;
} /* keychk_ */

BOOL keychk1_(char *scname, long int *isrch, long int *ktlen, char *kname)
{
  /* Local variables */
  static long int iasc, knum, ip, iq;
  static char kchar[1], nmchar[1];
  static BOOL code;
  BOOL ret_val ;

  ret_val = FALSE;

  /* search with categories allowed */
  for (iq = 1; iq <= *isrch; ++iq)
    {
      code = FALSE;
      knum = -1;
      for (ip = 1; ip <= *ktlen; ++ip)
	{
	  *kchar = kname[ip - 1];
	  if (*kchar == '{')
	    {
	      code = TRUE;
	      goto L70;
	    }
	  if (*kchar == '}')
	    {
	      code = FALSE;
	      goto L70;
	    }
	  ++knum;
	  strncpy(nmchar, scname + iq + knum - 1, 1);
	  if (code)
	    {
	      iasc = *nmchar;
	      if (*kchar == 'i' || *kchar == 'I')
		{
		  if (iasc < 48 || iasc > 57)
		    {
		      goto L60;
		    }
		}
	      else if (*kchar == 'l' || *kchar == 'L')
		{
		  if (iasc < 65 || (iasc > 90 && iasc < 97) || iasc > 122)
		    {
		      goto L60;
		    }
		}
	      else
		{
		  goto L70;
		}
	    }
	  else
	    {
	      if (*nmchar != *kchar)
		{
		  goto L60;
		}
	    }
L70:
	  ;
	}
      ret_val = TRUE;
      return ret_val;
L60:
      ;
    }
  return ret_val;
} /* keychk1_ */

BOOL cntchk_(long int *iclnc, long int *icpnt)
{
  /* System generated locals */
  BOOL ret_val;
  
  /* Local variables */
  static long int kcont, ic, cntlis[4000];
  
  ret_val = TRUE;
  kcont = iconta[*iclnc - 1];
  for (ic = 1; ic <= *icpnt; ++ic)
    {
      if (kcont == cntlis[ic - 1])
	{
	  ret_val = FALSE;
	  return ret_val;
	}
    }
  ++(*icpnt);
  cntlis[*icpnt - 1] = kcont;
  return ret_val;
} /* cntchk_ */

int keyw_(char *kname, long int *klen)
{
  memset (kname, 0, 8) ;
  gprintf (stdout, " enter keyword: ") ;
  readinput ("", kname, 8) ;

  *klen = strlength (kname, 8) ;
  return 0;
} /* keyw_ */

int keyw1_(char *kname, long int *ktlen, long int *klen)
{
  /* Local variables */
  static char kchar[1];
  static long int iq;
  
  memset (kname, 0, 24) ;
  gprintf (stdout, " enter keyword ({I}=integer;{L}=letter;{A}=either) : ") ;
  readinput ("", kname, 24) ;

  for (iq = 24; iq >= 1; --iq)
    {
      if (!(kname[iq - 1] == ' ' || kname[iq - 1] == 0))
	{
	  goto L11;
	}
    }
  *ktlen = 0;
  return 0;
L11:
  *ktlen = iq;
  *klen = 0;
  for (iq = 1; iq <= *ktlen; ++iq)
    {
      *kchar = kname[iq - 1];
      if (*kchar != '{' && *kchar != '}')
	{
	  ++(*klen);
	}
    }
  return 0;
} /* keyw1_ */

FILE *getfile_(void)
{
  /* Local variables */
  char filename[128];
  FILE *iunit ;

  strcpy (filename, "") ;
 fileinput:
  gprintf (stdout, "   file for output (def = screen) = ") ;
  readinput (filename, filename, 128) ;
  
  if (strlength (filename, 128) == 0)
    {
      iunit = stdout ;
    }
  else
    {
      if (!(iunit = FILopen (filename, "", "w"))) 
	goto fileinput ;
    }
  return iunit ;
} /* getfile_ */

int gencont1_(long int *nclone, long int *gout, long int *igpt, 
	      long int *chstrt, long int *ctot, long int *cpn, float *cpc)
{
  /* System generated locals */
  float r__1;
  
  /* Local variables */
  static long int gdir[2000];
  static short int rchr[2000];
  static float sval[2000];
  static long int rcnt[2000];
  static long int iprt;
  static char gene1[9];
  static float pchr1;
  static long int icchr, iccnt;
  static short int rline[2000];
  static long int rclon[2000], iplft, itemp, ipctr;
  static float tsval;
  static short int hflag1, hsitu1;
  static long int ig, ij, iq;
  static float tg;
  static long int ir, iz;
  static float ts;
  static long int iclone;
  static long int ijstrt;
  static short int hchrom1;
  static char remark1[41];
  static long int cne[6];
  static float cpg[1200]	/* was [200][6] */,
  ppb, cpp[1200]	/* was [200][6] */;
  static long int igp;
  static float rgp[2000];
  static long int isp;
  static float rst[2000];
  static BOOL test;
  
  /* Parameter adjustments */
  cpc -= 201;
  cpn -= 201;
  --ctot;
  --gout;
  
  /* Function Body */
  ppb = .01f;			/* % per band */
  *igpt = 0;
  iz = 0;
L69:
  ++iz;
  seek_11 (iz) ;
  fread (&iclone, sizeof(long int), 1, unit_11);
  fread (gene1, sizeof(char), 8, unit_11);
  fread (remark1, sizeof(char), 40, unit_11);
  fread (&hchrom1, sizeof(short int), 1, unit_11);
  fread (&hflag1, sizeof(short int), 1, unit_11);
  fread (&pchr1, sizeof(float), 1, unit_11);
  fread (&hsitu1, sizeof(short int), 1, unit_11);
  if (iclone > 0 && strlength (gene1, 8) > 0)
    {				/* if gene1 is not empty */
      ++(*igpt);
      gdir[*igpt - 1] = *igpt;
      rline[*igpt - 1] = iz;
      rclon[*igpt - 1] = iclone;
      rchr[*igpt - 1] = hchrom1;
/* physical */
      rst[*igpt - 1] = (float) (hsitu1 / 128 + hsitu1 % 128) / 2.f;
/* genetic */
      rgp[*igpt - 1] = pchr1;
      rcnt[*igpt - 1] = iconta[iclone - 1];
    }
  if (iclone != -9)
    {
      goto L69;
    }
/* sort genes into contig order */
  gcsort_(igpt, gdir, rcnt);
  for (iq = 1; iq <= 6; ++iq)
    {
      cne[iq - 1] = 1;
      cpp[iq * 200 - 200] = 0.f;
      cpg[iq * 200 - 200] = 0.f;
      ctot[iq] = 0;
    }
  for (ij = 1; ij <= *igpt; ++ij)
    {
      ig = gdir[ij - 1];

      if (rcnt[ig - 1] != 0)
	{
	  goto L201;		/* end of unattached */
	}
/* unattached */
      if (rgp[ig - 1] > 0.f)
	{
/* to be reworked in next pass */
	  sval[ig - 1] = -(double)rgp[ig - 1];
	}
      else if (rst[ig - 1] > 0.f)
	{
	  sval[ig - 1] = rst[ig - 1] + 1.f + rchr[ig - 1] * 1e6f;
	}
      else
	{
	  sval[ig - 1] = rchr[ig - 1] * 1e6f;
	}
/* save data for notional svals */
      if (rgp[ig - 1] > 0.f && rst[ig - 1] > 0.f)
	{
	  ++cne[icchr - 1];
	  cpp[cne[icchr - 1] + icchr * 200 - 201] =
	    dmax(.01f,dmin(99.99f,ts / (float) isp));
	  cpg[cne[icchr - 1] + icchr * 200 - 201] =
	    dmax(.01f,dmin(99.99f,tg / (float) igp));
	}
    }
/* attached */
  goto L202;
L201:
  iccnt = rcnt[ig - 1];
  icchr = 0;
  ijstrt = ij;
  ts = 0.f;
  tg = 0.f;
  isp = 0;
  igp = 0;

/* get end positions of contig, to normalise in sits to centre */
  sizcont1_(nclone, &iccnt, &iplft, &iprt);
  ipctr = (iplft + iprt) / 2;
  while(iccnt == rcnt[ig - 1])	/* pick up rest of contig */
    {
      if (rchr[ig - 1] > icchr)
	{
	  icchr = rchr[ig - 1];
	}
      if (rst[ig - 1] != 0.f)
	{
	  ts = ts + rst[ig - 1] +
	    (float) (ipctr - iposa[rclon[ig - 1] - 1]) * ppb;
	  ++isp;
	}
      if (rgp[ig - 1] != 0.f)
	{
	  tg += rgp[ig - 1];
	  ++igp;
	}
      ++ij;
      if (ij > *igpt)
	{
	  goto L203;
	}
      ig = gdir[ij - 1];
    }
L203:
  if (icchr == 0)
    {
      tsval = 0.f;
    }
  else if (igp > 0)
    {
/* to be reworked in next pass */
      tsval = -(double)(tg / (float) igp);
    } 
  else if (isp > 0)
    {
      tsval = ts / (float) isp + 1.f + icchr * 1e6f;
    }
  else
    {
      tsval = icchr * 1e6f;
/* save data for notional svals */
    }
  if (isp > 0 && igp > 0)
    {
      ++cne[icchr - 1];
      cpp[cne[icchr - 1] + icchr * 200 - 201] =
	dmax(.01f,dmin(99.99f,ts / (float) isp));
      cpg[cne[icchr - 1] + icchr * 200 - 201] =
	dmax(.01f,dmin(99.99f,tg / (float) igp));
    }
  for (ir = ijstrt; ir <= ij - 1; ++ir)
    {
      sval[gdir[ir - 1] - 1] = tsval;
      rchr[gdir[ir - 1] - 1] = icchr;
/* sort on iposa */
    }
L204:
  test = FALSE;
  for (ir = ijstrt + 1; ir <= ij - 1; ++ir)
    {
      if (iposa[rclon[gdir[ir - 1] - 1] - 1] <
	  iposa[rclon[gdir[ir - 2] - 1] - 1])
	{
	  test = TRUE;
	  itemp = gdir[ir - 1];
	  gdir[ir - 1] = gdir[ir - 2];
	  gdir[ir - 2] = itemp;
	}
    }
  if (test)
    {
      goto L204;
    }
  if (iccnt != rcnt[ig - 1])
    {
      goto L201;
    }

/* reorganise and sort cpp and cpg */
/* another contig to do */
  for (icchr = 1; icchr <= 6; ++icchr)
    {
      ++cne[icchr - 1];
      cpp[cne[icchr - 1] + icchr * 200 - 201] = 100.f;
      cpg[cne[icchr - 1] + icchr * 200 - 201] = 100.f;
      sortc_(&cpg[icchr * 200 - 200], &cne[icchr - 1]);
      sortc_(&cpp[icchr * 200 - 200], &cne[icchr - 1]);
    }

/* go through list again, putting in notional svals */
  for (ij = 1; ij <= *igpt; ++ij)
    {
      if (sval[gdir[ij - 1] - 1] < 0.f)
	{
	  icchr = rchr[gdir[ij - 1] - 1];
	  r__1 = -(double)sval[gdir[ij - 1] - 1];
	  sval[gdir[ij - 1] - 1] = fval_(&cne[icchr - 1], &r__1,
					 &cpp[icchr * 200 - 200],
					 &cpg[icchr * 200 - 200]) + 1.f +
					   icchr * 1e6f;
	}
/* sort on sval (NB shell sort scrambles IPOSA sort) */
    }
L202:
  test = FALSE;
  for (ir = 2; ir <= *igpt; ++ir)
    {
      if (sval[gdir[ir - 1] - 1] < sval[gdir[ir - 2] - 1])
	{
	  test = TRUE;
	  itemp = gdir[ir - 1];
	  gdir[ir - 1] = gdir[ir - 2];
	  gdir[ir - 2] = itemp;
	}
    }
  if (test)
    {
      goto L202;
    }

/* retrieve REM lines, set up chromosome starts,
     save contig centres and nos */
  icchr = 0;
  iccnt = -99;
  chstrt[0] = 1;
  for (iq = 1; iq <= 6; ++iq)
    {
      chstrt[iq] = 0;
    }
  chstrt[7] = *igpt + 1;
  for (ir = 1; ir <= *igpt; ++ir)
    {
      gout[ir] = rline[gdir[ir - 1] - 1];
      if (rchr[gdir[ir - 1] - 1] != icchr)
	{
	  icchr = rchr[gdir[ir - 1] - 1];
	  chstrt[icchr] = ir;	/* NB can have blanks */
	}
      if ((rcnt[gdir[ir - 1] - 1] != iccnt || rcnt[gdir[ir - 1] - 1] == 0) 
	  && sval[gdir[ir - 1] - 1] > icchr * 1e6f)
	{
/* new contig */
	  iccnt = rcnt[gdir[ir - 1] - 1];
	  ++ctot[icchr];
	  cpn[ctot[icchr] + icchr * 200] = iccnt;
	  if (iccnt == 0)
	    {
	      cpn[ctot[icchr] + icchr * 200] = -rclon[gdir[ir - 1] - 1];
	    }
	  cpc[ctot[icchr] + icchr * 200] = sval[gdir[ir - 1] - 1] - 1.f - 
	    icchr * 1e6f;
	}
    }
  return 0;
} /* gencont1_ */

double fval_(long int *icne, float *gval, float *cpps, float *cpgs)
{
  /* System generated locals */
  float ret_val;
  
  /* Local variables */
  static long int iq;

  /* Parameter adjustments */
  --cpgs;
  --cpps;
  
  /* Function Body */
  for (iq = 2; iq <= *icne - 1; ++iq)
    {
      if (*gval < cpgs[iq])
	{
	  goto L101;
	}
    }
  iq = *icne;
L101:
  ret_val = cpps[iq] - (cpgs[iq] - *gval) * (cpps[iq] - cpps[iq - 1]) /
    (cpgs[iq] - cpgs[iq - 1]);
  return ret_val;
} /* fval_ */

int sortc_(float *cp, long int *itot)
{
  /* Local variables */
  static float temp;
  static long int test;
  static long int ic;
  
  /* Parameter adjustments */
  --cp;
  
  /* Function Body */
L205:
  test = FALSE;
  for (ic = 2; ic <= *itot; ++ic)
    {
      if (cp[ic] < cp[ic - 1])
	{
	  test = TRUE;
	  temp = cp[ic];
	  cp[ic] = cp[ic - 1];
	  cp[ic - 1] = temp;
	}
    }
  if (test)
    {
      goto L205;
    }
  return 0;
} /* sortc_ */

/* sort clones */
int gcsort_(long int *igpt, long int *gdir, long int *crit)
{
  /* Local variables */
  static long int i, j, m, itemp;
  
  /* Parameter adjustments */
  --crit;
  --gdir;
  
  /* Function Body */
  m = *igpt;
L15:
  m = (m + 2) / 3;
  for (i = m + 1; i <= *igpt; ++i)
    {
      for (j = i; -m < 0 ? j >= m + 1 : j <= m + 1; j += -m)
	{
	  if (crit[gdir[j]] >= crit[gdir[j - m]])
	    {
	      goto L16;
	    }
	  itemp = gdir[j];
	  gdir[j] = gdir[j - m];
	  gdir[j - m] = itemp;
	}
L16:
      ;
    }
  if (m != 1)
    {
      goto L15;
    }

  return 0;
} /* gcsort_ */

void fwrt_(FILE *unit)
{
  fflush (unit) ;
} /* fwrt_ */

/* selects names */
/* inmsl=1     gridded */
/*      =2     YACs */
int namsel_(char *tname3, long int *inmsl, long int *nclone, 
	    long int *iy)
{
  /* Local variables */
  static long int icont, iq;
  static char iqq;
  
  /* only gridded clones */
  if (*inmsl == 1)
    {
      strcpy(tname3, "");
      iqq = iflag_(*iy, 5);
      if (iqq == 1)
	{
	  strtcpy(tname3, namea + ((*iy - 1) << 3), 8);
	  return 0;
	}
      else
	{
/* search among burieds */
	  if (icana[*iy - 1] == -1 && iflag_(*iy, 6) == 1)
	    {
	      icont = iconta[*iy - 1];
	      iq = 1;
	      while(iqq == 0 && iq < *nclone + 1)
		{
		  ++iq;
		  if (iconta[iq - 1] == icont)
		    {
		      if (abs(icana[iq - 1]) % 10000000 == *iy)
			{
			  iqq = iflag_(iq, 5);
			  if (iqq == 1)
			    {
			      strtcpy(tname3, namea + ((iq - 1) << 3), 8);
			    }
			}
		    }
		}
	    }
	}
    }
/* YACs only */
  else if (*inmsl == 2)
    {
      strcpy(tname3, "");
      if (keychk_(iy, &c__2, &c__4, "Y{I}"))
	{
	  strtcpy(tname3, namea + ((*iy - 1) << 3), 8);
	}
      return 0;
    }
/* gridded YACs only */
  else if (*inmsl == 3)
    {
      strcpy(tname3, "");
      if (iflag_(*iy, 7) != 0)
	{
	  strtcpy(tname3, namea + ((*iy - 1) << 3), 8);
	}
      return 0;
    }
  return 0;
} /* namsel_ */

int gencont_(long int *gout, long int *igpt, long int *chstrt)
{
  /* Local variables */
  static long int gdir[2000];
  static short int rchr[2000];
  static long int sval[2000], rcnt[2000];
  static long int test;
  static char gene1[9];
  static float pchr1, o;
  static long int icchr, iccnt;
  static short int rline[2000];
  static long int rclon[2000], isval, itemp;
  static short int hflag1, hsitu1;
  static long int ig, ij, iq, ir, iz, iclone;
  static long int ijstrt;
  static short int hchrom1;
  static char remark1[41];
  static long int igp, itg, rgp[2000], isp, its;
  static short int rst[2000];
  
  /* Parameter adjustments */
  --gout;
  
  /* Function Body */
  *igpt = 0;
  iz = 0;
L69:
  ++iz;
  seek_11 (iz) ;
  fread (&iclone, sizeof(long int), 1, unit_11);
  fread (gene1, sizeof(char), 8, unit_11);
  fread (remark1, sizeof(char), 40, unit_11);
  fread (&hchrom1, sizeof(short int), 1, unit_11);
  fread (&hflag1, sizeof(short int), 1, unit_11);
  fread (&pchr1, sizeof(float), 1, unit_11);
  fread (&hsitu1, sizeof(short int), 1, unit_11);
  if (iclone > 0 && strlength (gene1, 8) > 0)
    {/* if gene1 not empty */
      ++(*igpt);
      gdir[*igpt - 1] = *igpt;
      rline[*igpt - 1] = iz;
      rclon[*igpt - 1] = iclone;
      rchr[*igpt - 1] = hchrom1;
      rst[*igpt - 1] = (hsitu1 / 128 + hsitu1 % 128) / 2;
      rgp[*igpt - 1] = (long int) (pchr1 * 1e3f);
      rcnt[*igpt - 1] = iconta[iclone - 1];
    }
/* sort genes into contig order */
  if (iclone != -9)
    {
      goto L69;
    }
  gcsort_(igpt, gdir, rcnt);
  for (ij = 1; ij <= *igpt; ++ij)
    {
      ig = gdir[ij - 1];
/* unattached */
      if (rcnt[ig - 1] != 0)
	{
	  goto L201;
	}
      if (rst[ig - 1] > 0)
	{
	  sval[ig - 1] = rst[ig - 1] + 1 + rchr[ig - 1] * 1000000;
	}
      else if (rgp[ig - 1] > 0)
	{
	  sval[ig - 1] = rgp[ig - 1] + 200 + rchr[ig - 1] * 1000000;
	}
      else
	{
	  sval[ig - 1] = rchr[ig - 1] * 1000000;
	}
    }
/* attached */
  goto L202;
L201:
  iccnt = rcnt[ig - 1];
  icchr = 0;
  ijstrt = ij;
  its = 0;
  itg = 0;
  isp = o;
  igp = 0;
  while(iccnt == rcnt[ig - 1])
    {
      if (rchr[ig - 1] > icchr)
	{
	  icchr = rchr[ig - 1];
	}
      if (rst[ig - 1] != 0)
	{
	  its += rst[ig - 1];
	  ++isp;
	}
      if (rgp[ig - 1] != 0)
	{
	  itg += rgp[ig - 1];
	  ++igp;
	}
      ++ij;
      if (ij > *igpt)
	{
	  goto L203;
	}
      ig = gdir[ij - 1];
    }
L203:
  if (isp > 0)
    {
      isval = its / isp + 1 + icchr * 1000000;
    } 
  else if (igp > 0)
    {
      isval = itg / igp + 200 + icchr * 1000000;
    }
  else
    {
      isval = icchr * 1000000;
    }
  for (ir = ijstrt; ir <= ij - 1; ++ir)
    {
      sval[gdir[ir - 1] - 1] = isval;
      rchr[gdir[ir - 1] - 1] = icchr;
/* sort on iposa */
    }
L204:
  test = FALSE;
  for (ir = ijstrt + 1; ir <= ij - 1; ++ir)
    {
      if (iposa[rclon[gdir[ir - 1] - 1] - 1] <
	  iposa[rclon[gdir[ir - 2] - 1] - 1])
	{
	  test = TRUE;
	  itemp = gdir[ir - 1];
	  gdir[ir - 1] = gdir[ir - 2];
	  gdir[ir - 2] = itemp;
	}
    }
  if (test)
    {
      goto L204;
    }
/* sort on sval (NB shell sort scrambles IPOSA sort) */
  if (iccnt != rcnt[ig - 1])
    {
      goto L201;
    }
L202:
  test = FALSE;
  for (ir = 2; ir <= *igpt; ++ir)
    {
      if (sval[gdir[ir - 1] - 1] < sval[gdir[ir - 2] - 1])
	{
	  test = TRUE;
	  itemp = gdir[ir - 1];
	  gdir[ir - 1] = gdir[ir - 2];
	  gdir[ir - 2] = itemp;
	}
    }
/* retrieve REM lines and set up chromosome starts */
  if (test)
    {
      goto L202;
    }
  icchr = 0;
  chstrt[0] = 1;
  for (iq = 2; iq <= 6; ++iq)
    {
      chstrt[iq] = 0;
    }
  chstrt[0] = *igpt + 1;
  for (ir = 1; ir <= *igpt; ++ir)
    {
      gout[ir] = rline[gdir[ir - 1] - 1];
      if (rchr[gdir[ir - 1] - 1] != icchr)
	{
	  icchr = rchr[gdir[ir - 1] - 1];
/* NB can have blanks */
	  chstrt[icchr] = ir;
	}
    }
    return 0;
} /* gencont_ */

#ifndef NO_GRAPHICS
/* finds split point in clone list */
int getsplt_(long int *ipcx, long int *ncomp, long int *icnuma, 
	long int *ixs, long int *isplit)
{
  /* Local variables */
  static long int ix, iy;
  
  /* Parameter adjustments */
  --icnuma;
  
  /* Function Body */
  *isplit = 0;
  for (ix = 1; ix <= *ncomp; ++ix)
    {
      iy = icnuma[ix];
      if (iposa[iy - 1] + ilena[iy - 1] / 2 + *ixs > *ipcx)
	{
	  goto L83;
	}
    }
  ix = *ncomp + 1;
L83:
  *isplit = ix - 1;
  return 0;
} /* getsplt_ */

int squ_(long int *ipcx, long int *ncomp, long int *icnuma, 
	 long int *ixs, long int *iscale, long int *isch,
	 long int *imode, long int *nclone, char *projectbase)
{
  /* Local variables */
  static long int ican;
  static long int irem, ipsq;
  static long int ichar;
  static long int itemp;
  static long int ix, iy;
  static long int itempl, itempr, isplit;
  static char ans[1];
  static long int isq;
  
    /* Parameter adjustments */
  --icnuma;
  
  /* Function Body */
  gprintf (stdout, "  This routine will lengthen or shorten the current contig\n"
	  "  at a given point.  It will move normal clones bodily, but will\n"
	  "  alter the length of virtual clones that cross this point.\n") ;
 L10:
  gprintf (stdout, "   length change in bands (+ve lengthens, -ve shortens) = ") ;
  readinput ("", stemp, 5) ; if (nonint(stemp)) goto L10 ;
  isq = a2int (stemp) ;
  if (isq == 0)
    {
      return 0;
    }
  gprintf (stdout,
	   "  position cursor at squeeze point, defined by\n"
	   "  centres of clones; then hit ");
  colorvt640_(BLACK+100) ;
  gprintf (stdout, "SPACE");
  colorvt640_(BLACK) ;
  gprintf (stdout, " bar\n");

  cursor_(&ichar, ipcx, iscale, isch);
/* find split point in clone list */
  if (ichar != SPACE_KEY)
    {
      return 0;
    }
  getsplt_(ipcx, ncomp, &icnuma[1], ixs, &isplit);
  if (isplit < 1 || isplit >= *ncomp)
    {
      return 0;
    }
  ipsq = *ipcx - *ixs;
  itempl = ipsq - iposa[icnuma[3999] - 1];
/* find shorter arm of contig to adjust */
  itempr = iposa[icnuma[4000] - 1] + ilena[icnuma[4000] - 1] 
    - ipsq;
  if (itempr > itempl)
    {
      for (ix = 1; ix <= isplit; ++ix)
	{
	  iy = icnuma[ix];
	  if (iposa[iy - 1] + ilena[iy - 1] > ipsq)
	    {
	      if (iflag_(iy, 4) == 1)
		{
		  ilena[iy - 1] += isq;
		}
	    }
	  iposa[iy - 1] -= isq;
	}
      for (ix = isplit + 1; ix <= *ncomp; ++ix)
	{
	  iy = icnuma[ix];
	  if (iposa[iy - 1] < ipsq)
	    {
	      if (iflag_(iy, 4) == 1)
		{
		  iposa[iy - 1] -= isq;
		  ilena[iy - 1] += isq;
		}
	    }
	}
    }
  else
    {
      for (ix = 1; ix <= isplit; ++ix)
	{
	  iy = icnuma[ix];
	  if (iposa[iy - 1] + ilena[iy - 1] > ipsq)
	    {
	      if (iflag_(iy, 4) == 1)
		{
		  ilena[iy - 1] += isq;
		}
	    }
	}
      for (ix = isplit + 1; ix <= *ncomp; ++ix)
	{
	  iy = icnuma[ix];
	  if (iposa[iy - 1] < ipsq && iflag_(iy, 4) == 1)
	    {
	      ilena[iy - 1] += isq;
	    }
	  else
	    {
	      iposa[iy - 1] += isq;
	    }
	}
    }
  emphclone_(&c_n1, ixs, &c__380, ncomp, &icnuma[1], iscale, imode);
  clearvt640_();
  selectvt640_();
  plotcont4_(ixs, &c__380, iscale, ncomp, &icnuma[1],
	     projectbase, nclone, imode);
  selectvt100_();
  clearvt100_();
  movevt100_(0, 0) ;
  gprintf (stdout, "   OK to file data (Y/N) ? ") ;
  readinput ("", stemp, 2) ; *ans = *stemp ;
  if (*ans != 'y' && *ans != 'Y')
    {
      for (ix = 1; ix <= *ncomp; ++ix)
	{
	  iy = icnuma[ix];
	  seek_4 (iy) ;
	  fread (&iposa[iy - 1], sizeof(long int), 1, unit_4);
	  fread (&ilena[iy - 1], sizeof(long int), 1, unit_4);
	  fread (&itemp, sizeof(long int), 1, unit_4);
	  fread (&itemp, sizeof(long int), 1, unit_4);
	}
    }
  else
    {
      for (ix = 1; ix <= *ncomp; ++ix)
	{
	  iy = icnuma[ix];
	  seek_4 (iy) ;
	  fread (&itemp, sizeof(long int), 1, unit_4);
	  fread (&itemp, sizeof(long int), 1, unit_4);
	  fread (&ican, sizeof(long int), 1, unit_4);
	  fread (&irem, sizeof(long int), 1, unit_4);
	  seek_4 (iy) ;
	  fwrite (&iposa[iy - 1], sizeof(long int), 1, unit_4);
	  fwrite (&ilena[iy - 1], sizeof(long int), 1, unit_4);
	  fwrite (&ican, sizeof(long int), 1, unit_4);
	  fwrite (&irem, sizeof(long int), 1, unit_4);
	}
      fwrt_(unit_4);
    }
  return 0;
} /* squ_ */

int gnl_(char *gene1, float *genl, float *csiz)
{
  static long int igl;
  
  igl = strlength (gene1, 8) ;
/*
  for (igl = 8; igl >= 1; --igl)
    {
      if (!(gene1[igl - 1] == ' ' || gene1[igl - 1] == 0))
	{
	  goto L70;
	}
    }
L70:
*/
  *genl = *csiz * (float) igl;
  return 0;
} /* gnl_ */

int genplot1_(float *pchr1, float *genl, short int *hchrom1,
	      char *gene1, long int *isp)
{
  /* System generated locals */
  static char equiv_0[9];
#define tname (equiv_0)
#define tnameb (equiv_0)
  
  /* Local variables */
  static long int ilft, nent;
  static float x;
  static long int ix, iy;
  static long int lev, irt, num;
  
  x = xzero1 + *pchr1 * scale1;
/* get plot level; use 2 * plot units */
  strtcpy(tname, gene1, 8);
  ilft = (long int) (x / 2.f);
  irt = (long int) ((x + *genl) / 2.f);
  nent = genlev1[*hchrom1 + 5993];
  for (lev = 1; lev <= maxlev1; ++lev)
    {
      ilft += 1000;
      irt += 1000;
      for (num = 1; num <= nent; num += 2)
	{
	  if (ilft < genlev1[*hchrom1 + (num + 1) * 6 - 7])
	    {
	      if (irt > genlev1[*hchrom1 + num * 6 - 7])
		{
		  goto L74;
		}
	    }
	}
      goto L76;
    L74:
      ;
    }
 L76:
  genlev1[*hchrom1 + 5993] = nent + 2;
  genlev1[*hchrom1 + (nent + 1) * 6 - 7] = ilft - 5;
/* place gene */
  genlev1[*hchrom1 + (nent + 2) * 6 - 7] = irt + 5;
  iy = (7 - *hchrom1) / *isp * ichrgp;
  ix = (long int) x;
  movevt640_(ix, iy);
  drawvt640_(ix, iy + 8 + levgap1 * (lev - 1));
  movevt640_((long int) x, iy + 14 + levgap1 * (lev - 1));
  textvt640_(tnameb, 8);
  return 0;
} /* genplot1_ */

#undef tnameb
#undef tname

#endif /* ! NO_GRAPHICS */
