/*  File: co9.h
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: header file for Contig programs
 * SCCS: $Id: co9.h,v 1.1 2001/02/14 12:44:36 edgrif Exp $
 * HISTORY:
 * Last edited: Nov 24 10:07 1999 (fw)
 * Created: Wed Jun 15 12:33:05 1994 (fw)
 *-------------------------------------------------------------------
 */

#ifndef CONTIGC_INC

#define BANDMAX 750000

#include <stdio.h>
#ifdef SOLARIS
  #include <string.h>
#else
  #include <strings.h>
#endif
#include <ctype.h>		/* fir isdigit etc... */
#ifndef NO_GRAPHICS
 #include "regular.h"
 #include "graph.h"
 #include "key.h"
 #include "mydirent.h" /* define BSD-getwd() */
#else
 #include <fcntl.h>
 #include <stdlib.h> 
 #include <unistd.h>
 /* some things from regular.h */
 #ifdef FALSE
   typedef int BOOL ;
 #else
   typedef enum {FALSE=0,TRUE=1} BOOL ;
 #endif
 
 #define RETURN_KEY 13		/* for wairForMore() */
 #define SPACE_KEY 32
#endif

#define WIN_WIDTH 1024
#define WIN_HEIGHT 780

#define MAPX(x) ((float)(x) * (float)n_int(winWidth) / 1024.0)
#define MAPY(y) ((780.0 - (float)(y)) * (float)n_int(winHeight) / 780.0)

#define n_int(x) ((x) > 0 ? ((int)(x + .5)) : -((int)(.5 - x)) )
#define abs(x) ((x) >= 0 ? (x) : -(x))
#define dabs(x) (double)abs(x)
#define min(a,b) ((a) <= (b) ? (a) : (b))
#define max(a,b) ((a) >= (b) ? (a) : (b))
#define dmin(a,b) (double)min(a,b)
#define dmax(a,b) (double)max(a,b)

extern char stemp[100], ntemp1[100], ntemp2[100], ntemp3[100] ;

extern void gprintf (FILE *stream, const char *format, ...) ;
extern char *getdatetime(void) ;
extern void seek_1 (long int position) ;
extern void seek_4 (long int position) ;
extern void seek_10 (long int position) ;
extern void seek_11 (long int position) ;
extern void seek_13 (long int position) ;
extern void seek_14 (long int position) ;
extern void seek_16 (long int position) ;
extern void seek_21 (long int position) ;
extern void seek_22 (long int position) ;
extern void seek_23 (long int position) ;
extern void seek_29 (long int position) ;
extern BOOL FILerror (FILE *) ;
extern FILE *FILopen (char *, char *, char *) ;
extern void readinput (char *src, char *dest, int len) ;
extern void strtcpy (char *, char *, int) ;
extern int strlength (char *, int) ;
extern BOOL nonint (char *) ;
extern long int a2int (char *) ;
extern BOOL nonflt (char *) ;
extern float a2flt (char *) ;
extern void readstring (FILE *, char *, int) ;
extern void readline (FILE *, char *) ;
extern void forward (FILE *) ;

extern int graphFortranEntry (char*, int, float, float, void (*fn)(char*)) ;
extern void waitForMore (int key) ;
extern int waitForKey (int key) ;
extern void selectvt100_(void);
extern int clearvt100_(void);
extern void movevt100_(int, int) ;

extern void clearvt640_ (void) ;
extern int initvt640_(void);/*return 1 when graphics was found and cleared*/
extern void selectvt640_(void);
extern void drawvt640_(long int, long int) ;
extern void rectvt640_(long int, long int) ;
extern void movevt640_(long int, long int) ;
extern void colorvt640_(int) ;
extern void textvt640_(char *, long int);
extern void pointvt640_ (long int, long int) ;
extern void xhairvt640_ (long int *, long int *, long int *) ;

extern double log(double);
extern double pow_dd(double, double), sqrt(double), r_lg10(float *);
extern double pow_ri(float, long int), exp(double) ;
extern long int pow_ii (long int, long int) ;
extern double ran_(void);

extern void dumpAce (char *, char *, long int, long int) ;
extern int contloss_(long int *, long int *, long int *);
extern void fwrt_(FILE *);
extern int wrflag_(long int *, long int *, long int *);
extern int asscont_(long int *, long int *, long int *, 
		    long int *, long int *, long int *,
		    long int *, long int *);
extern int posclone_(long int *, long int *, long int *, 
		     long int *, long int *) ;
extern int bestmatch_(long int *, long int *, long int *, 
		      long int *, long int);
extern int getbnds_(long int *, long int *, long int *);
extern int repos2_(long int *, long int *, long int *, long int *);

extern int matchs_(long int, long int, long int *,
		   long int, long int *);

extern int score2_(long int *, long int *);
extern int emph_(long int, long int, long int, long int);
extern int probcoinc_(long int, long int, long int,
		      float *, long int, float *);
extern int keysub_(BOOL *, long int *, long int *, long int *);
extern int scale_(long int *);
extern int pmode_(long int *) ;
extern int modnam_(void);
extern int checkcont1_(char *, long int *);
extern int difmap_(long int *, long int *, long int *, long int *);
extern int gensort_(long int *, long int *);
extern int lsiz_(long int *);
extern int plotcont4_(long int *, long int *, long int *, long int *,
		      long int *, char *, long int *, long int *);
extern int marker_(long int *, long int *, long int *, float *,
		   float *, float *, float *, float *, long int *,
		   long int *, long int *, long int *,
		   long int *, long int *);
extern int asscont3_(long int *, long int *, long int *, 
		     long int *, long int *, long int *) ;
extern int asscont4_(long int *, long int *, 
		     long int *, long int *, long int *);
extern int text_(float *, float *, char *, long int *,
		 float *, float *, float *,
		 float *, long int *, long int *,
		 long int *, long int *, 
		 long int *, long int *) ;
extern int line1_(float *, float *, float *, float *, 
		  float *, float *, float *, float *,
		  long int *, long int *, long int *, 
		  long int *, long int *, long int *);
extern int analyse_(char *, long int *, long int *, long int *, 
		    long int *, long int *);
extern int map_(long int *, long int *, char *, long int *, 
		long int *, long int *, long int *);
extern char cflag_(long int, long int, char *);
extern /* Character */ void clim_(char *, long int *);
extern int contrd_(long int *, long int *, long int *);
extern int totplt_(long int *, long int *, long int *, long int *,
		   long int *, long int *, long int *, char *,
		   long int *, long int *, long int *);
extern int cursor_(long int *, long int *, long int *, long int *);
extern int abstr_(long int *, long int *);
extern int clist_(long int *, long int *);
extern int rejpss_(char *, long int *, long int *,
		   long int *, char *, long int *) ;
extern int emphclone_(long int *, long int *, long int *, 
		      long int *, long int *, long int *, long int *);
extern int emphclone1_(long int *, long int *, long int *, 
		      long int *, long int *, long int *);
extern int clc_(long int *, long int *, long int *,
		long int *, long int *, long int *);
extern int flag_(long int *);
extern int bury_(long int *, long int *, long int *, long int *);
extern int invert_(long int *, long int *, long int *,
		   long int *, long int *, long int *);
extern int remchng_(long int *, long int *);
extern int squ_(long int *, long int *, long int *,
		long int *, long int *, long int *,
		long int *, long int *, char *);
extern int getsplt_(long int *, long int *, long int *, 
		    long int *, long int *);
extern int auto_(BOOL *, long int *, long int *, 
		 long int *, long int *);
extern int contstack_(char *, long int *);
extern int plotcont5_(long int *, long int *, long int *,
		      long int *, long int *, long int *,
		      long int *, long int *);
extern int rem2_(short int *, long int *, long int *);
extern int delrem_(long int *);
extern int fitrem_(long int *, char *);
extern int iflag_(long int, long int);
extern int exchg_(long int *, long int *);
extern int marker1_(long int *, long int *);
extern int pltbnds_(long int *, long int *, long int *, long int *) ;
extern int plotcont3_(long int *, long int *, long int *, 
		      long int *, long int *);
extern int bandselect_(long int *, long int *, long int *, long int *);
extern int clondif_(long int *, long int *, long int *, 
		    long int *, long int *);
extern int patch5_(long int, long int, long int,
		   long int *, long int, float *, float *,
		   long int *, float, long int,
		   long int *, long int);
extern int bestsubtr_(long int *, long int *, long int *, 
		      long int *, long int *);
extern int overlap_(long int *, long int *, long int *, long int *,
		    long int *, long int *, long int *) ;
extern int pltclone_(long int *, long int *, long int *) ;
extern int getclone1_(char *, long int *, BOOL *);
extern int pltbnd_(long int *, long int *, long int *, BOOL);
extern int getmatch_(long int, long int *,
		     long int, long int *, long int, long int *);
extern FILE *getfile_(void);
extern int clonaccum_(long int *, long int *, long int *, long int *);
extern int redset_(long int *, long int *, BOOL *);
extern int keyw1_(char *, long int *, long int *);
extern int getinit1_(long int *, long int *, long int *, 
		     char *, long int *);
extern int getline1_(char *, long int *, long int *);
extern int getset1_(long int *, long int *, long int *, char *);
extern int tycont_(long int *, FILE *, long int *);
extern BOOL cntchk_(long int *, long int *);
extern BOOL keychk_(long int *, long int *, long int *, char *);
extern BOOL lstchk_(long int *, long int *, long int *);
extern int cydubl_(void) ;
extern int gencont1_(long int *, long int *,
		     long int *, long int *, 
		     long int *, long int *, float *);
extern int keyw_(char *, long int *);
extern int patch6_(long int, long int, long int, long int,
		   long int *, long int,  float, float) ;
extern int getclone_(long int *, char *, long int *, char *,
		     long int *, long int *, long int *) ;
extern int matchd_(long int *, long int *, long int *, long int *,
		   long int *, long int *, long int *, char *,
		   long int *, long int *, long int *, long int *,
		   long int, float *) ;
extern int srtctr_(long int *, long int *, long int *) ;
extern int namsel_(char *, long int *, long int *, long int *) ;
extern int remplot_(char *, long int *, long int *, 
		    long int *, long int *, long int *,
		    long int *, long int *) ;
extern int dislnk_(long int *, long int *, long int *, 
		   long int *, char *, long int *) ;
extern int gnl_(char *, float *, float *) ;
extern int contchk_(long int *, long int *, long int *) ;
extern int drawcurs_(long int, long int) ;
extern int genplot1_(float *, float *, short int *, char *, long int *) ;
extern int genplot_(float *, float *, short int *, float *, char *) ;
extern int serialin_(long int *, char *, long int *, 
		     long int *, long int *, long int *) ;
extern int flushser_(long int *) ;
extern int getline_(char *, long int *, long int *) ;
extern BOOL keychk1_(char *, long int *, long int *, char *) ;
extern double fval_(long int *, float *, float *, float *) ;
extern int sortc_(float *, long int *) ;
extern int gcsort_(long int *, long int *, long int *) ;
extern BOOL elim_(long int *) ;
extern int wrbuff_(char *) ;
extern int mvcurs_(long int *) ;
extern int delchar_(char *) ;
extern int nestcurs_(char *) ;
extern int wrchar_(char *, char *) ;
extern int rddat_(char *, long int *, long int *, long int *);
extern int rjps1_(char *, long int *, long int *, long int *,
		  long int *, char *, long int *, long int *,
		  long int *, char *, long int *);

extern int dltot_(long int *, long int *, long int *, 
		  long int *, long int *);
extern int rdtot_(long int *, long int *, long int *, 
		  long int *, long int *, long int *);
extern int qutot_(long int *, long int *, 
		  long int *, long int *, long int *);
extern int wrtot_(long int *, long int *, char *, long int *,
		  long int *, long int *, long int *) ;
extern int tytot_(long int *, long int *, long int *);
extern int sitot_(long int *, long int *, long int *, 
		  long int *, long int *, long int *) ;

/* needed for mapsub7.c */
extern int readlst_(long int *ncln, long int *clst, long int *nclone) ;
extern int getset_(long int *ncln, long int *clst, long int *nclone) ;
extern int getcont_(long int *ncln, long int *clst, long int *nclone, 
	     long int *iconta) ;
extern int patch3_(long int ifromp, long int itop, long int nband1, 
	    long int *coords, long int toler, float *logfact,
	    float *rank, long int *rnkdir, float *rankt,
	    long int *rktdir, float probm, float ps,
	    long int *irnkpt, long int nrnk,
            long int *mat, long int *matt) ;
extern int patch4_(long int ifromp, long int itop, long int nband1, 
	    long int *coords, long int toler, float *logfact,
	    float *rankt, long int *rktdir, float ps,
	    long int nrnk, long int *matt) ;

/* cemsub.c */
extern int prcont1_(long int, long int *, long int,
		    long int, long int, long int, long int) ;
extern int gnhl_(char *, float *, float *);
extern int botscale1_(long int *, float *,float *);
extern int gsort_(long int *, long int *, float *);
extern int setparam_(void);
extern int plotdots_(float *);
extern int nomch_(char *);
extern int gsort_(long int *, long int *, float *);
extern int corners_(long int, long int, long int, long int) ;
extern int setfont_(long int *);
extern int remplot1_(char *, char *, long int *,
		     long int *, long int *, long int,
		     long int, long int *, long int *);
extern int pltdate_(int, int) ;
extern int dot_(long int *, long int *, float *);
extern int tricont_(long int, long int, long int, 
		    long int, long int, long int *);
extern int chromplt_(long int *, long int *, long int *, 
		     float *, long int *, long int *,
		     char *, long int *, BOOL) ;
extern int sizcont_(long int *, long int *, long int *);
extern int gencont1_(long int *, long int *, long int *,
		     long int *, long int *, long int *, float *);
extern int cbump_(long int *, float *, float *);
extern int getpct_(long int *, long int *, long int *,
		   float *, float *, float *);
extern int sizcont_(long int *, long int *, long int *);
extern int enter_(long int *, long int *, long int *, BOOL *);
extern int repos_(long int *, long int *, long int *, long int *);
extern int assgrp_(long int *, long int *, long int *, long int *);
extern long int ibg_(long int *, long int *, long int *, long int *);
extern int getbnds_(long int *, long int *, long int *) ;
extern int findnew_(long int *, long int *, BOOL *, long int *, short int *);
extern int getzone_(long int *, long int *, long int *, long int *,
		    long int *, long int *, long int *) ;
extern int plotgrp_(long int, long int, long int, long int,
		    long int *, BOOL, BOOL) ;
extern int getline1_(char *, long int *, long int *);
extern int botscale_(float *, float *, float *, float *) ;
extern BOOL inlist_(char *) ;
void plotBigChromNum (long int, long int, long int) ;

/* contaslib.c */
extern int srtctr_(long int *, long int *, long int *);
extern int score1_(long int *, long int *);
extern int getcpb_(long int *, long int *, long int *, 
		   long int *, long int *);

/* pmap.c */
extern int cemaphelp_(void);
extern int cntss_(long int *, char *, BOOL *, long int *);
extern int cechr_(void) ;
extern int scale_(long int *);
extern int emphclone1_(long int *, long int *, long int *, 
		       long int *, long int *, long int *);
extern int chplt_(long int *, BOOL);



#define CONTIGC_INC
#endif /* ifdef CONTIGC_INC */
