/*  File: contasp.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description:  to order clones in project automatically
 * SCCS: $Id: contasp.c,v 1.1 2001/02/14 12:44:37 edgrif Exp $
 * HISTORY:
 * Last edited: Jan 23 15:47 1995 (fw)
 * Created: Wed May 25 11:06:36 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;			/* .nam */
extern FILE *unit_2 ;			/* .cor */
extern FILE *unit_4 ;			/* .cnt */
extern FILE *unit_10 ;			/* .stk */

/* Common Block Declarations */

/* csec CB */
short int coorda[500000];

/* cint CB */
long int nbanda[30000], lbanda[30000], igela[30000], iconta[30000],
         iposa[30000], ilena[30000], icana[30000], irema[30000];

/* chars CB */
char namea[240000], chrom[65];

/* matdat CB */
long int itolmn, itoldf;
float logfact[201];

/* chr1 CB */
float xzero1, scale1;
long int ichrgp, maxlev1, levgap1;
short int genlev1[6000]	/* was [6][1000] */;

/* cass CB */
long int gpos[400], glen[400], gclst[400], ngrp;
short int gmat[160000]	/* was [400][400] */;

/* BLNK CB */
long int isch, icurs, ibtot, iwend;

/* Table of constant values */

static long int c__0 = 0;
static long int c__1 = 1;
static long int c__2 = 2;
static long int c__3 = 3;

extern int firsterm_(float *, float *, long int *, long int *, float *,
		     long int *);
extern int enter_(long int *, long int *, long int *, BOOL *);
extern int repos_(long int *, long int *, long int *, long int *);
extern int assgrp_(long int *, long int *, long int *, long int *);
extern long int ibg_(long int *, long int *, long int *, long int *);
extern int findnew_(long int *, long int *, BOOL *, long int *, short int *);

void main (void)
{
  /* Local variables */
  static long int ican, igel, bmat[400];
  static float fbtl;
  static long int icyc, ilen, irem, nmat[400], irng, icrt, tlen[400], maxm, 
  cstk[400];
  static short int tmat[160000]	/* was [400][400] */;
  static BOOL unat[400];
  static long int i, j, lband, nband;
  static char tname[9];
  static float probc;
  static long int maxcl, ibest, iqold, icont, itemp;
  static long int ncomp, itgrp, inext, nxtcl, tclst[400];
  static long int nband1;
  static long int coord1[200], icnum1[4000], iq, ir, it, nbandm, iscale, 
                 ix, nbands, imatch;
  static long int iclone, nclone;
  static long int ifstcl, coords[200], iconto;
  static float fbtlmx;
  static long int iposav, itolmn;
  static long int ihi, ilo;
  static float logfact[201], psl;
  static long int ixs, iys;
  char projectbase[128] ;
  int size_of_cor=0 ;

  ifstcl = 2;			/* first clone to consider */
  iscale = 5;
  iys = 60;
/* calc log factorials */
  logfact[0] = 0.f;
  logfact[1] = 0.f;
  for (i = 2; i <= 200; ++i)
    {
      logfact[i] = logfact[i - 1] + log((float) i);
    }
  strcpy (chrom,
	  "     "" (I) "" (II)""(III)"" (IV)"
	  " (V) "" (X) "" SIT "" GEN "" BOTH") ;
  itolmn = 7 ; itoldf = 10 ;
  gprintf (stdout, "\n  -=* CONTASP *=-\n\n") ;
  gprintf (stdout,
	   "     ********************************************\n"
	   "     *   THIS PROGRAMME DESTROYS EXISTING MAP   *\n"
	   "     *   DATA;  BACKUP BEFORE PROCEEDING        *\n"
	   "     ********************************************\n") ;

  goto L0 ;
 L1:
  gprintf (stdout, "\nPlease enter valid project name !\n") ;
 L0:
  gprintf (stdout, "     project name = ") ;
  readinput ("", projectbase, 128) ;
  
  if (strlength(projectbase, 128) == 0)
    {
      exit (1) ;
    }

/* open files */
  if (!(unit_1 = FILopen (projectbase, "nam", "r+"))) goto L1 ;
  seek_1 (1) ;
  fread (&nclone, sizeof(long int), 1, unit_1) ;
  fread (&nbands, sizeof(long int), 1, unit_1) ;
  gprintf (stdout, "number of clones = %d, "
	   "number of bands = %d\n", nclone, nbands) ;

  if (!(unit_2 = FILopen (projectbase, "cor", "r"))) goto L1 ;
/* determine filesize of *.cor file */
  fseek (unit_2, 0, SEEK_END) ;
  size_of_cor = ftell (unit_2) ;
  size_of_cor /= sizeof(short int) ;
  rewind (unit_2) ;
/* read *.cor file into coorda array */
/* cor file are shorts to make it load faster */
  fread (&coorda, sizeof(short int), size_of_cor, unit_2) ;

  if (!(unit_4 = FILopen (projectbase, "cnt", "r+"))) goto L1 ;
  if (!(unit_10 = FILopen (projectbase, "stk", "r+"))) goto L1 ;

/* read in .NAM database */
  for (ix = ifstcl; ix <= nclone + 1; ++ix)
    {
      seek_1 (ix) ;
      fread (namea + ((ix - 1) << 3), sizeof(char), 8, unit_1) ;
      fread (&nbanda[ix - 1], sizeof(long int), 1, unit_1) ;
      fread (&lbanda[ix - 1], sizeof(long int), 1, unit_1) ;
      fread (&igela[ix - 1], sizeof(long int), 1, unit_1) ;
      fread (&itemp, sizeof(long int), 1, unit_1) ;
    }

/* get tolerance and probability limits */
 L2:
  gprintf (stdout, "  tolerance = ") ;
  readinput ("", stemp, 10) ; itolmn = a2int (stemp) ;
  if (itolmn == 0)
    {
      gprintf (stdout, " 0 tolerance not allowed\n") ;
      goto L2 ;
    }
  gprintf (stdout, "  probability cutoff = ") ;
  readinput ("", stemp, 20) ; probc = a2flt (stemp) ;
  fbtlmx = log(probc);
  psl = log(1 - (float) (itolmn << 1) / 3300.f);

/* clear contstack */
  seek_10 (1) ;
  fwrite (&c__1, sizeof(long int), 1, unit_10);
  seek_10 (2) ;
  fwrite (&c__0, sizeof(long int), 1, unit_10);

/* contig assembly */
  for (iclone = ifstcl; iclone <= nclone + 1; ++iclone)
    {
      iconta[iclone - 1] = 0;
      icont = 0;
      strtcpy (ntemp1, namea + ((iclone - 1) << 3), 8) ;
      gprintf (stdout, "%-8s\n", ntemp1) ;
      getbnds_(&iclone, &nbandm, coord1);
      if (nbandm == 0)
	{
	  goto L212;
	}
      for (it = ifstcl; it <= iclone - 1; ++it)
	{
	  if (nbanda[it - 1] == 0)
	    {
		goto L213;
	    }
/* already in contig */
	    if (icont != 0 && icont == iconta[it - 1])
	      {
		goto L213;
	      }
	  matchs_(it, nbandm, coord1, itolmn, &imatch);
	  firsterm_(&fbtl, &psl, &nbandm, &nbanda[it - 1],
		    logfact, &imatch);
	  if (fbtl < fbtlmx)
	    {
/* have significant match */
	      if (icont == 0)
		{
/* ICLONE is not yet attached */
		    if (iconta[it - 1] == 0)
		      {
/* matching clone is unattached also: get new contig number */
			contstack_("r", &icont);
			iconta[iclone - 1] = icont;
			iconta[it - 1] = icont;
		      }
		    else 
		      {
/* matching clone is attached: put ICLONE into its contig */
			icont = iconta[it - 1];
			iconta[iclone - 1] = icont;
		      }
		  }
	      else 
		{
/* ICLONE is already attached */
		  if (iconta[it - 1] == 0)
		    {
/* matching clone is unattached: put it in ICLONE contig */
		      iconta[it - 1] = icont;
		    } 
		  else 
		    {
/*  matching clone is attached: change its entire contig to ICLONE contig */
		      iconto = iconta[it - 1];
		      for (ir = 2; ir <= iclone - 1; ++ir)
			{
			  if (iconta[ir - 1] == iconto)
			    {
			      iconta[ir - 1] = icont;
			    }
			}
		      contstack_("w", &iconto);
		    }
		}
	    }
L213:
	  ;
	}
L212:
      ;
    }

/* start main loop */
  seek_10 (1) ;
  fread (&inext, sizeof(long int), 1, unit_10);

/* get group */
  for (icont = 1; icont <= inext; ++icont)
    {
      itgrp = 0;
      for (iq = 2; iq <= nclone + 1; ++iq)
	{
	  if (iconta[iq - 1] == icont)
	    {
	      ++itgrp;
	      tclst[itgrp - 1] = iq;
	    }
	}
      if (itgrp == 0)
	{
	  goto L214;
	}

/* get clone sizes, clear arrays */
      for (iq = 1; iq <= itgrp; ++iq)
	{
	  ix = tclst[iq - 1];
	  tlen[iq - 1] = nbanda[ix - 1];
	  bmat[iq - 1] = 0;
	  nmat[iq - 1] = 0;
	  unat[iq - 1] = TRUE;
	}

/* get match matrix for group */
	maxm = 0;
      for (iq = 1; iq <= itgrp - 1; ++iq)
	{
	  getbnds_(&tclst[iq - 1], &nband1, coords);
	  for (j = iq + 1; j <= itgrp; ++j)
	    {
	      matchs_(tclst[j - 1], nband1, coords, itolmn, &imatch);
	      imatch = max(0,imatch -
			   ibg_(&tlen[j - 1], &tlen[iq - 1],
				&imatch, &itolmn));
	      if (imatch > nmat[j - 1])
		{
		  nmat[j - 1] = imatch;
		  bmat[j - 1] = iq;
		}
	      if (imatch > nmat[iq - 1])
		{
		  nmat[iq - 1] = imatch;
		  bmat[iq - 1] = j;
		}
	      if (imatch > maxm)
		{
		  maxm = imatch;
		  maxcl = iq;
		}
		tmat[j + iq * 400 - 401] = imatch;
/* probably not needed */
		tmat[iq + j * 400 - 401] = imatch;
	    }
	}

/* get entry order */
      ihi = 1;
      cstk[0] = maxcl;
      unat[maxcl - 1] = FALSE;
      for (ilo = 1; ilo <= itgrp; ++ilo)
	{
	  nxtcl = cstk[ilo - 1];
	  enter_(&bmat[nxtcl - 1], &ihi, cstk, unat);
	  for (iq = 1; iq <= itgrp; ++iq)
	    {
	      if (bmat[iq - 1] == nxtcl)
		{
		  enter_(&iq, &ihi, cstk, unat);
		}
	    }
	  if (ihi == itgrp)
	    {
	      goto L211;
	    }
	  if (ilo == ihi)
	    {
	      findnew_(&ihi, cstk, unat, &itgrp, tmat);
	    }
	  if (ihi == itgrp)
	    {
	      goto L211;
	    }
	}

/* sort by entry order */
L211:
      for (iq = 1; iq <= itgrp; ++iq)
	{
	  iqold = cstk[iq - 1];
	  gclst[iq - 1] = tclst[iqold - 1];
	  glen[iq - 1] = tlen[iqold - 1];
	  for (j = 1; j <= itgrp; ++j)
	    {
	      gmat[j + iq * 400 - 401] = tmat[cstk[j - 1] +
					      iqold * 400 - 401];
	    }
	}
/* initiate contig */
      ngrp = 1;
      gpos[0] = 0;
/* add clones to contig one by one */
      assgrp_(&ncomp, icnum1, &iposav, &icrt);
      for (ngrp = 2; ngrp <= itgrp; ++ngrp)
	{
	  ibest = -1000000;
	  gpos[ngrp - 1] = iposav;
	  irng = iposav - gpos[icnum1[0] - 1] + 50;
	  ixs = -iposav;
	  repos_(&ngrp, &ibest, &irng, &c__2);
	  assgrp_(&ncomp, icnum1, &iposav, &icrt);
	  icyc = 0;
	  for (iq = 1; iq <= ngrp; ++iq)
	    {
	      repos_(&iq, &ibest, &c__3, &c__1);
	    }
	    assgrp_(&ncomp, icnum1, &iposav, &icrt);
	}
      ngrp = itgrp;
      for (irng = 30; irng >= 10; irng += -10)
	{
	  for (iq = 1; iq <= ngrp; ++iq)
	    {
	      repos_(&iq, &ibest, &irng, &c__1);
	    }
	}
/* write POS to main array */
      for (iq = 1; iq <= itgrp; ++iq)
	{
	  iclone = gclst[iq - 1];
	  iposa[iclone - 1] = gpos[iq - 1];
	}
L214:
	;
    }
/* write to files */
  for (iclone = ifstcl; iclone <= nclone + 1; ++iclone)
    {
      seek_4 (iclone) ;
      fread (&itemp, sizeof(long int), 1, unit_4);
      fread (&ilen, sizeof(long int), 1, unit_4);
      fread (&ican, sizeof(long int), 1, unit_4);
      fread (&irem, sizeof(long int), 1, unit_4);

      seek_4 (iclone) ;
      fwrite (&iposa[iclone - 1], sizeof(long int), 1, unit_4);
      fwrite (&ilen, sizeof(long int), 1, unit_4);
      fwrite (&ican, sizeof(long int), 1, unit_4);
      fwrite (&irem, sizeof(long int), 1, unit_4);

      seek_1 (iclone) ;
      fread (tname, sizeof(char), 8, unit_1);
      fread (&nband, sizeof(long int), 1, unit_1);
      fread (&lband, sizeof(long int), 1, unit_1);
      fread (&igel, sizeof(long int), 1, unit_1);
      fread (&itemp, sizeof(long int), 1, unit_1);

      seek_1 (iclone) ;
      fwrite (tname, sizeof(char), 8, unit_1);
      fwrite (&nband, sizeof(long int), 1, unit_1);
      fwrite (&lband, sizeof(long int), 1, unit_1);
      fwrite (&igel, sizeof(long int), 1, unit_1);
      fwrite (&iconta[iclone - 1], sizeof(long int), 1, unit_1);
    }
  fclose (unit_1) ;
  fclose (unit_4) ;
  return ;
} /* main */
