/*  File: contig.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description:
 * SCCS: $Id: contig.c,v 1.1 2001/02/14 12:44:37 edgrif Exp $
 * HISTORY:
 * Last edited: Nov 24 10:07 1999 (fw)
 * Created: Thu May 19 15:42:43 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;			/* .nam */
extern FILE *unit_2 ;			/* .cor */
extern FILE *unit_3 ;			/* standard.dat */
extern FILE *unit_4 ;			/* .cnt */
extern FILE *unit_10 ;			/* .stk */
extern FILE *unit_11 ;			/* .rem */
extern FILE *unit_12 ;			/* map.out */
extern FILE *unit_13 ;			/* .flg */
extern FILE *unit_16 ;			/* scratch.dat */

/* Common Block Declarations */

/* csec CB */
short int coorda[BANDMAX];

/* cint CB */
long int nbanda[30000], lbanda[30000], igela[30000], iconta[30000],
         iposa[30000], ilena[30000], icana[30000], irema[30000];

/* chars CB */
char namea[240000], chrom[65];

/* matdat CB */
long int itolmn, itoldf;
float logfact[201];

/* chr1 CB */
float xzero1, scale1;
long int ichrgp, maxlev1, levgap1;
short int genlev1[6000]	/* was [6][1000] */;

/* cass CB */
long int gpos[1000], glen[1000], gmat[1000], gclst[1000], ngrp;

/* BLNK CB */
long int isch, icurs, ibtot, iwend;


/* Table of constant values */

static long int c__1 = 1;
static long int c__0 = 0;
static long int c__380 = 380;
static long int c__8 = 8;
static long int c__4 = 4;
static long int c_n1 = -1;
static long int c__60 = 60;
static long int c__15 = 15;
static long int c__100 = 100;
static long int c__924 = 924;
static long int c__500 = 500;
static long int c_n9 = -9;

Graph graph = 0 ;
float wChar, hChar ;
float winWidth, winHeight ;

void dummy (void) {;}

static void about (void)
{
  messout
    ("Contig\n"
     "Interactive contig assembly for the ContigC package.\n "
     "Authors :\n"
     "John Sulston et al. (jes@sanger.ac.uk)\n"
     "     - original FORTRAN program\n"
     "Friedemann Wobus (fw@sanger.ac.uk)\n"
     "     - UNIX port and new graphics\n"
     "Co-authors :\n"
     "Richard Durbin (rd@sanger.ac.uk)\n"
     "     - graphics library");
}

/* CONTIG */
void main (int argc, char **argv)
{
  char projectbase[128] ;
  int dx, dy ;
  extern void contigc (char *) ;
  MENUOPT contigMenu[] = {
    {dummy,""},
    {graphPrint, "Print"},
    {about, "About"},
    {dummy,""},
    {0,0}};

  if (argc > 1 && *argv[1] != '-')
    strcpy (projectbase, argv[1]) ;
  else
    memset (projectbase, 0, 128) ;

  graphInit (&argc, argv) ;
  graph = graphCreate (TEXT_FIT, "ContigC",
		       0, 0, WIN_WIDTH/900.0, WIN_HEIGHT/900.0) ;
/*  graphRectangle (-1, -1, WIN_WIDTH+1, WIN_HEIGHT+1) ; */
  graphTextFormat (FIXED_WIDTH) ;

  graphRetitle ("ContigC (graphical vt640 emulation)") ;
  graphMenu (contigMenu) ;
  graphWhere (0, 0, &winWidth, &winHeight) ;
  graphTextInfo (&dx, &dy, &wChar, &hChar) ;

  gprintf (stdout,
	   "\n  -=* Welcome to ContigC *=-\n\n"
	   " This program provides read/write access to genome map projects,\n"
	   " and offers interactive editing and analysis facilities.\n"
	   "\n  project name = \n") ;

  graphRedraw () ;

  graphFortranEntry (projectbase, 128, 17*wChar, 6*hChar, contigc) ;
  graphStart (graph) ;
}

void contigc (char *projectbase)
{
  /* Initialized data */
  
  static float xmax = 3600.f;
  static float xmin = 0.f;
  static float ymax = 100.f;
  static float ymin = 0.f;
  static long int margxl = 10;
  static long int margxr = 10;
  static long int margyl = 10;
  static long int margyr = 10;
  static long int isxmax = 1023; /* x, y resolution */
  static long int isymax = 779;
  static long int nchars = 8;
  
  /* System generated locals */
  long int i__1;
  char ch__5[6];
  static char equiv_0[9], equiv_1[9];
#define tname (equiv_0)
#define tnameb ((char *)equiv_0)
#define tname1 (equiv_1)
#define tname1b ((char *)equiv_1)
#define name1 ((char *)equiv_2)
#define names (equiv_2)

    /* Builtin functions */
  double log(double);

    /* Local variables */
  static long int ican;
  static char gene[9];
  static long int iabs;
  static char name[9];
  static long int ilna[100], gdir[2000];
  static long int isch;
  static long int igel;
  static float pchr;
  static long int icont, ictrl, nclone ;
  static long int itcl, incs, ilen, ipcx, iret, igpt;
  static long int ilft, irem, ipos, idrt, idcx;
  static long int isrt;
  static long int icnr, iact, iend;
  static char gene1[9];
  static long int igel1;
  static float pchr1;
  static long int isrt1, i;
  static long int iband, lband;
  static short int hflag;
  static char cname[9];
  static long int nband, ichar;
  static float y, x;
  static long int idlen, iamat, imode, idlft;
  static long int ipcxf, ispan, itemp;
  static long int islft, ilast, ianum, izero;
  static short int hsitu;
  static long int lband1;
  static short int hflag1;
  static long int nband1;
  static long int icnum1[4000], icnum2[4000], icnum3[4000], ncomp1, ncomp3;
  static long int islft1, icont1, ncomp2, icont2;
  static short int hsitu1;
  static long int id;
  static long int iq, is, nbandm, iscale, ix;
  static long int iz;
  static long int nbands;
  static long int iclone;
  static float xf;
  static float xt;
  static short int hchrom;
  static char remark[41];
  static long int coordm[500];
  static float yf, yt;
  static long int iy;
  static long int iconto, ishift;
  static long int icontt, iposav;
  static long int isplit, istart;
  static short int hchrom1;
  static char remark1[41];
  static long int icp;
  static long int ird;
  static char ans[1];
  static long int irt;
  static long int ixs;
  static long int iln1, iln2, ixs2;
  static BOOL key, aut;
  long int size_of_cor = 0 ;	/* filesize of *.cor file */
  int stdk, itmp ;
  float stdr, ftmp ;
  int bandColorCount ;

  movevt100_(17, 6) ;
  gprintf (stdout, "%s\n", projectbase) ;

/* calc log factorials */
  logfact[0] = 0.f;
  logfact[1] = 0.f;
  for (iq = 2; iq <= 200; ++iq)
    {
      logfact[iq] = logfact[iq - 1] + log((float) iq);
    }
  /* init chrom[] array */
  strcpy (chrom,
	  "     "" (I) "" (II)""(III)"" (IV)"
	  " (V) "" (X) "" SIT "" GEN "" BOTH") ;
  itolmn = 7 ; itoldf = 10 ;
  goto L0 ;
 L1:
  gprintf (stdout, "Please enter valid project name !\n\n") ;
  gprintf (stdout, "  project name = ") ;
  readinput (projectbase, projectbase, 128) ;
 L0:
  if (strlength(projectbase, 128) == 0)
    {
      exit (1) ;
    }
  graphRetitle (messprintf ("ContigC (%s)", projectbase)) ;

  if (!(unit_1 = FILopen (projectbase, "nam", "r+"))) goto L1 ;
  seek_1 (1) ;
  fread (&nclone, sizeof(long int), 1, unit_1) ;
  fread (&nbands, sizeof(long int), 1, unit_1) ;
  gprintf (stdout, "number of clones = %d, "
	   "number of bands = %d\n", nclone, nbands) ;

  if (!(unit_4 = FILopen (projectbase, "cnt", "r+"))) goto L1 ;

  if (!(unit_10 = FILopen (projectbase, "stk", "r+"))) goto L1 ;
  seek_10 (1) ;
  fread (&itemp, sizeof(long int), 1, unit_10) ;

  if (!(unit_16 = FILopen ("scratch", "dat", "w+"))) goto L1 ;
  for (iq = 1; iq <= itemp; ++iq)
    {
      fwrite (&c__0, sizeof(long int), 1, unit_16) ; /* write zero's */
    }
  fwrt_(unit_16) ;

  if (!(unit_11 = FILopen (projectbase, "rem", "r+"))) goto L1 ;
  if (!(unit_13 = FILopen (projectbase, "flg", "r+"))) goto L1 ;
  unit_12 = FILopen ("map", "out", "R") ;

  /* read in data to arrays */
  {
    int unit_cor, unit_nam, unit_cnt ;
    char file[128] ;
    unsigned long_len = sizeof(long int), char_len = 8 * sizeof(char) ;
    sprintf (file, "%s.cor", projectbase) ;

    unit_2 = FILopen (file, "", "r") ;
    fseek (unit_2, 0, SEEK_END) ;
    /* determine filesize of *.cor file */
    size_of_cor = ftell (unit_2) ;
    fclose (unit_2) ;

    unit_cor = open (file, O_RDONLY) ;
    /* read *.cor file into coorda array */
    /* cor file are shorts to make it load faster */
    read (unit_cor, &coorda, size_of_cor) ;
    close (unit_cor) ;

    sprintf (file, "%s.nam", projectbase) ;
    unit_nam = open(file, O_RDONLY) ;
    sprintf (file, "%s.cnt", projectbase) ;
    unit_cnt = open(file, O_RDONLY) ;
    lseek (unit_nam, 8*sizeof(char)+4*sizeof(long int), SEEK_SET) ;
    lseek (unit_cnt, 4*sizeof(long int), SEEK_SET) ;
    for (ix = 1; ix < nclone + 1; ++ix)
      {
	read (unit_nam, namea + (ix << 3), char_len) ;
	read (unit_nam, &nbanda[ix], long_len) ;
	read (unit_nam, &lbanda[ix], long_len) ;
	read (unit_nam, &igela[ix],  long_len) ;
	read (unit_nam, &iconta[ix], long_len) ;

	read (unit_cnt, &iposa[ix],  long_len) ;
	read (unit_cnt, &ilena[ix],  long_len) ;
	read (unit_cnt, &icana[ix],  long_len) ;
	read (unit_cnt, &irema[ix],  long_len) ;
      }
    close (unit_nam) ; close (unit_cnt) ;
  }


  /* read in marker bands, if available */
  nbandm = 0;
  if (!(unit_3 = FILopen ("standard", "dat", "R"))) goto L5 ;
  /* skip header */
  fscanf (unit_3,"%d,%d,%f,%f,%f,%d,%d,%d,%f,%f,%d,%f\n",
	  &itmp, &itmp, &ftmp, &ftmp, &ftmp, &itmp,
	  &itmp, &itmp, &ftmp, &ftmp, &itmp, &ftmp) ;
  /* load standard-file */
  while (!feof(unit_3) && nbandm < 500)
    {
      fscanf (unit_3,"%d,%d,%d,%f,%d", &stdk, &itmp, &itmp, &stdr,&itmp);
      while (!feof(unit_3) && fgetc (unit_3) != '\n') ;
      if (stdk == 0) continue ;
      coordm[nbandm++] = (long int)(stdr*10.0) ;
    }
  --nbandm;
  fclose (unit_3) ;
L5:
  is = 1;
  i = 0;
  id = 0;
  icont = 0;
  iconto = 0;
  ixs = 0;
  iabs = 3;
  izero = 0;
  ipcx = 0;
  iscale = 5;
  aut = FALSE;
  key = FALSE;
  imode = 1 + 20;		/* display all remarks */

L8:
  selectvt100_() ;
/* get clone/control name */
  gprintf (stdout, "   clone name/control (H gets help) = ") ;
  readinput ("", name, 8) ;
/* first check for control name */
  if (strlength(name, 8) == 0)
    {
      is = 1;
      if (aut)
	{
	  goto L79;
	}
      if (key)
	{
	  keysub_(&key, &i, &nclone, &iret);
	  if (key)
	    {
	      goto L2;
	    }
	  goto L8;
	}
      clearvt640_() ;
      selectvt100_();
      clearvt100_();
      movevt100_(0, 0) ;
      goto L8;
    }
/*
	do while(name(1:1).eq.' ')
	 do iq=1,7
	  name(iq:iq)=name(iq+1:iq+1)
	  name(8:8)=' '
	 end do
	end do
*/
  if (strcmp(name, "*") == 0)
    {
      goto L13;
    }
  if (strcasecmp(name, "h") == 0 || strcasecmp(name, "help") == 0)
    {
      gprintf (stdout, "\n"
	       "    <RET>  stop band display\n"
	       " <CTRL>+L  clear graphics and reveal text\n"
	       "       D   display clone bands\n"
	       "      DL   display clone bands leftwards\n"
	       "      DR   display clone bands rightwards\n"
	       "      DI   reinitiate display mode\n"
	       "       C   display contig bands\n"
	       "       A   analyse\n"
	       "       M   match analysis\n"
	       "       L   list bands\n"
	       "     ABS   absolute data entry\n"
	       "     REL   relative data entry\n"
	       "    CURS   cursor data entry\n"
	       "   SCALE   set scale\n"
	       "    GENE   find gene\n"
	       "    CONT   get contig\n"
	       "   CHECK   check files\n"
	       "  MODNAM   modify NAM file\n"
	       "  DIFMAP   match difference bands\n"
	       "     AUT   AUTO mode\n"
	       "     MAN   MANUAL mode\n"
	       "     KEY   enter KEYword mode\n"
	       "   UNKEY   leave keyword mode\n"
	       "   PMODE   switch plot mode\n"
	       " DUMPACE   dump current database into ace file\n"
	       "       *   quit the program\n"
	       ) ;
      goto L8;
    }
  if (strcasecmp(name, "dumpace") == 0)
    {
      static char aceproject[128] ;
      FILE *filtest;

      gprintf (stdout, "\n  .ace file to dump into = ") ;
      strcpy (aceproject, projectbase) ;
      strcat (aceproject, ".ace") ;

      readinput (aceproject, aceproject, 128) ;
      if (strlength(aceproject, 128) == 0)
	goto L8;

      if ((filtest = fopen(aceproject, "r")) != NULL)
	{
	  fclose (filtest) ;
	  gprintf (stdout, "  Do you want to overwrite \"%s\" (y/n) ? ", aceproject) ;
	  readinput ("", stemp, 1) ;
	  if (!(*stemp == 'y' || *stemp == 'Y'))
	    goto L8;
	}

      dumpAce (projectbase, aceproject, nclone, nbands) ;

      gprintf (stdout, "\n") ;
      graphProcessEvents () ;
      clearvt640_() ;

      goto L8 ;
    }
  if (strcasecmp(name, "abs") == 0)
    {
      iabs = 2;
      gprintf (stdout, "absolute data entry\n") ;
      goto L8;
    }
  if (strcasecmp(name, "rel") == 0)
    {
      iabs = 1;
      gprintf (stdout, "relative data entry\n") ;
      goto L8;
    }
  if (strcasecmp(name, "curs") == 0) 
    {
      iabs = 3;
      gprintf (stdout, "cursor data entry\n") ;
      goto L8;
    }
  if (strcasecmp(name, "scale") == 0)
    {
      scale_(&iscale);
      goto L8;
    }
  if (strcasecmp(name, "pmode") == 0)
    {
      pmode_(&imode);
      goto L8;
    }
  if (strcasecmp(name, "key") == 0)
    {
      keysub_(&key, &i, &nclone, &iret);
      if (key)
	{
	  goto L2;
	}
      else
	{
	  goto L8;
	}
    }
  if (strcasecmp(name, "unkey") == 0)
    {
      key = FALSE;
      gprintf (stdout, " key mode cancelled\n");
      goto L8;
    }
  if (strcasecmp(name, "aut") == 0)
    {
      goto L79;
    }
  if (strcasecmp(name, "man") == 0)
    {
      if (ictrl == 3)
	{
	  gprintf (stdout, "can't leave auto mode during trial\n") ;
	}
      else
	{
	  aut = FALSE;
	  gprintf (stdout, "set to manual mode\n") ;
	}
      goto L8;
    }
  if (strcasecmp(name, "check") == 0)
    {
      checkcont1_(projectbase, &nclone);
      goto L8;
    }
  if (strcasecmp(name, "modnam") == 0)
    {
      modnam_();
      goto L8;
    }
  if (strcasecmp(name, "difmap") == 0)
    {
      difmap_(&isch, &icont, &ixs, &iscale);
      goto L8;
    }
  if (strcasecmp(name, "cont") == 0) 
    {
L75:
      gprintf (stdout, "  contig number = ") ;
      readinput ("", stemp, 10) ; if (nonint(stemp)) goto L75 ;
      ird = a2int (stemp) ;

      if (ird == 0) goto L8;

      for (i = 2; i <= nclone + 1; ++i)
	{
	  if (iconta[i - 1] == ird)
	    {
	      goto L77;
	    }
	}
      gprintf (stdout, " contig not found !\n") ;
      goto L75;
L77:
      seek_1 (i) ;
      fread (tname, sizeof(char), 8, unit_1) ;
      fread (&nband, sizeof(long int), 1, unit_1) ;
      fread (&lband, sizeof(long int), 1, unit_1) ;
      fread (&igel, sizeof(long int), 1, unit_1) ;
      fread (&icontt, sizeof(long int), 1, unit_1) ;
      is = 1;
      goto L37;
    }
  if (strcasecmp(name, "gene") == 0) 
    {
L67:
      gprintf (stdout, "   gene name (L for list, LS for chrom/size, E for edit) = ") ;
      readinput ("", gene, 8) ;
      if (strlength(gene, 8) == 0) goto L8;

      if (strcasecmp(gene, "l") == 0)
	{
	  gensort_(gdir, &igpt);
	  gprintf (stdout, " clone       gene              "
		   "genetic%%    in situ%%\n");
	  for (iq = 1; iq <= igpt; ++iq)
	    {
	      seek_11 (gdir[iq - 1]) ;
	      fread (&iclone, sizeof(long int), 1, unit_11) ;
	      fread (gene1, sizeof(char), 8, unit_11) ;
	      fread (remark1, sizeof(char), 40, unit_11) ;
	      fread (&hchrom1, sizeof(short int), 1, unit_11) ;
	      fread (&hflag1, sizeof(short int), 1, unit_11) ;
	      fread (&pchr1, sizeof(float), 1, unit_11) ;
	      fread (&hsitu1, sizeof(short int), 1, unit_11) ;
	      strtcpy (ntemp1, namea + ((iclone - 1) << 3), 8) ;
	      strtcpy (ntemp2, chrom + hchrom1 * 5, 5) ;
	      gprintf (stdout, " %-8s    %-8s%5s   %6.1f     %3d -%3d\n",
		      ntemp1, gene1, ntemp2,
		      pchr1, hsitu1 / 128, hsitu1 % 128) ;
	    }
	  goto L67;
	}
      else if (strcasecmp(gene, "ls") == 0)
	{
	  lsiz_(&nclone);
	  goto L67;
	}
      if (strcasecmp(gene, "e") == 0)
	{
L1085:
	  gprintf (stdout, "   gene to edit = ") ;
	  readinput("", gene1, 8) ;
	  if (strlength(gene1, 8) == 0)
	    {
	      goto L67;
	    }
	  iz = 0;
L1086:
	  ++iz;
	  seek_11 (iz) ;
	  fread (&iclone, sizeof(long int), 1, unit_11) ;
	  fread (gene, sizeof(char), 8, unit_11) ;
	  fread (remark, sizeof(char), 40, unit_11) ;
	  fread (&hchrom, sizeof(short int), 1, unit_11) ;
	  fread (&hflag, sizeof(short int), 1, unit_11) ;
	  fread (&pchr, sizeof(float), 1, unit_11) ;
	  fread (&hsitu, sizeof(short int), 1, unit_11) ;
	  if (strncmp(gene1, gene, 8) == 0)
	    {
	      goto L1087;
	    }
	  if (iclone != -9)	/* check for end of file */
	    {
	      goto L1086;
	    }
	  goto L1085;		/* because gene not found */
L1087:
	  if (iclone > 0 && strlength(gene, 8) > 0)
	    {	                  /* if gene not empty */
	      strtcpy (ntemp1, namea + ((iclone - 1) << 3), 8) ;
	      strtcpy (ntemp2, gene, 8) ;
	      strtcpy (ntemp3, chrom + hchrom * 5, 5) ;
	      gprintf (stdout, "  %-8s     %-8s%5s   %6.1f    %3d - %3d\n",
		      ntemp1, ntemp2, ntemp3,
		      pchr, hsitu / 128, hsitu % 128) ;
	    }
	  gprintf (stdout, "  chromosome (1-6) (-ve deletes) = ") ;
	  readinput ("", stemp, 5) ; if (nonint (stemp)) goto L1087 ;
	  hchrom1 = (short int) a2int (stemp) ;
	  if (hchrom1 > 6)  goto L1087;
	  if (hchrom1 == 0) hchrom1 = hchrom;
	  if (hchrom1 < 0)  hchrom1 = 0;
	  hflag1 = hflag;
L1089:
	  gprintf (stdout, "  genetic map percentage (real) (-ve deletes) = ") ;
	  readinput ("", stemp, 15) ; if (nonflt (stemp)) goto L1089 ;
	  pchr1 = a2flt (stemp) ;
	  if (pchr1 > 100.f) goto L1089;
	  if (pchr1 == 0.f)  pchr1 = pchr;
	  if (pchr1 < 0.f)   pchr1 = 0.f;
	  islft = hsitu / 128;
	  isrt = hsitu % 128;
L1093:
	  gprintf (stdout, "  in situ left end %% (-ve deletes) = ") ;
	  readinput ("", stemp, 5) ; if (nonint (stemp)) goto L1093 ;
	  islft1 = a2int (stemp) ;
	  if (islft1 > 100) goto L1093;
	  if (islft1 == 0)  islft1 = islft;
	  if (islft1 < 0)   islft1 = 0;
L1094:
	  gprintf (stdout, "  in situ right end %% (-ve deletes) = ") ;
	  readinput ("", stemp, 5) ; if (nonint (stemp)) goto L1094 ;
	  isrt1 = a2int (stemp) ;
	  if (isrt1 > 110) goto L1094;
	  if (isrt1 == 0)  isrt1 = isrt;
	  if (isrt1 < 0)   isrt1 = 0;
	  if (isrt1 < islft1) goto L1094;
	  hsitu1 = (islft1 << 7) + isrt1;
	  
	  seek_11 (iz) ;
	  fwrite (&iclone, sizeof(long int), 1, unit_11) ;
	  fwrite (gene, sizeof(char), 8, unit_11) ;
	  fwrite (remark, sizeof(char), 40, unit_11) ;
	  fwrite (&hchrom1, sizeof(short int), 1, unit_11) ;
	  fwrite (&hflag1, sizeof(short int), 1, unit_11) ;
	  fwrite (&pchr1, sizeof(float), 1, unit_11) ;
	  fwrite (&hsitu1, sizeof(short int), 1, unit_11) ;
	  fwrt_(unit_11);
	  seek_11 (iz) ;
	  fread (&iclone, sizeof(long int), 1, unit_11) ;
	  fread (gene1, sizeof(char), 8, unit_11) ;
	  fread (remark1, sizeof(char), 40, unit_11) ;
	  fread (&hchrom1, sizeof(short int), 1, unit_11) ;
	  fread (&hflag1, sizeof(short int), 1, unit_11) ;
	  fread (&pchr1, sizeof(float), 1, unit_11) ;
	  fread (&hsitu1, sizeof(short int), 1, unit_11) ;

	  if (iclone > 0 && strlength(gene1, 8) > 0)
	    {			/* if gene1 not empty */
	      strtcpy (ntemp1, namea + ((iclone - 1) << 3), 8) ;
	      strtcpy (ntemp2, gene1, 8) ;
	      strtcpy (ntemp3, chrom + hchrom1 * 5, 5) ;
	      gprintf (stdout, "  %-8s     %-8s%5s   %6.1f    %3d - %3d\n",
		      ntemp1, ntemp2, ntemp3,
		      pchr1, hsitu1 / 128, hsitu1 % 128) ;
	    }
	  goto L1085;
	}
/* not control name, so find gene */
      iz = 0;
L68:
      ++iz;
      seek_11 (iz) ;
      fread (&iclone, sizeof(long int), 1, unit_11) ;
      fread (gene1, sizeof(char), 8, unit_11) ;
      if (strncmp(gene1, gene, 8) == 0)
	{
	  goto L66;
	}
      if (iclone != -9)		/* check for end of file */
	{
	  goto L68;
	}
      goto L67;			/* because gene not found */
L66:
      i = iclone;
      seek_1 (i) ;
      fread (tname, sizeof(char), 8, unit_1) ;
      fread (&nband, sizeof(long int), 1, unit_1) ;
      fread (&lband, sizeof(long int), 1, unit_1) ;
      fread (&igel, sizeof(long int), 1, unit_1) ;
      fread (&icontt, sizeof(long int), 1, unit_1) ;
      is = 1;
      goto L37;
    }
  if (strcasecmp(name, "d") == 0)
    {
/* plot current contig above */
      clearvt640_();
      i__1 = 1 + 10;
      plotcont4_(&ixs, &c__380, &iscale, &ncomp1, icnum1, projectbase,
		 &nclone, &i__1);
      marker_(coordm, &nbandm, &id, &y, &xmax, &xmin, &ymax, &ymin,
	      &margxl, &margxr, &margyl, &margyr, &isxmax, &isymax);
      is = 2;
      if (i == 0)
	{
	  goto L8;
	}
      goto L12;
    }
  if (strcasecmp(name, "di") == 0)
    {
/* plot current contig above */
      clearvt640_();
      i__1 = 1 + 10;
      plotcont4_(&ixs, &c__380, &iscale, &ncomp1, icnum1, projectbase,
		 &nclone, &i__1);
      marker_(coordm, &nbandm, &id, &y, &xmax, &xmin, &ymax, &ymin, &margxl,
	      &margxr, &margyl, &margyr, &isxmax, &isymax);
      is = 2;
      goto L8;
    }
  if (strcasecmp(name, "dl") == 0 || strcasecmp(name, "dr") == 0)
    {
      gprintf (stdout, "  start clone (prefix with * to get burieds) = ") ;
      readinput("", remark, 9) ;
      *ans = *remark;
      if (*ans == '*')
	{
	  strtcpy(tname, remark + 1, 8);
	}
      else
	{
	  strtcpy(tname, remark, 8);
	}
      for (itcl = 2; itcl <= nclone + 1; ++itcl)
	{
	  if (strncmp(namea + ((itcl - 1) << 3), tname, 8) == 0)
	    {
	      goto L1081;
	    }
	}
      gprintf (stdout, "clone not found\n") ;
      goto L15;
L1081:
      gprintf (stdout, "  span = ") ;
      readinput ("", stemp, 5) ; if (nonint(stemp)) goto L1081 ;
      ispan = a2int (stemp) ;
      if (*ans == '*')
	{
	  asscont3_(&nclone, &iconta[itcl - 1], &ncomp3, icnum3,
		    &itemp, &itemp);
	}
      else
	{
	  asscont4_(&nclone, &iconta[itcl - 1], &ncomp3, icnum3,
		    &itemp);
	}
      for (icp = 1; icp <= ncomp3; ++icp)
	{
	  if (strncmp(namea + ((icnum3[icp - 1] - 1) << 3),
		    tname, 8) == 0)
	    {
	      goto L1076;
	    }
	}
      goto L15;
L1076:
      if (name[1] == 'L' || name[1] == 'l')
	{
	  ispan = max(1,icp - ispan);
	  incs = -1;
	}
      else
	{
	  ispan = min(ncomp3, icp + ispan);
	  incs = 1;
	}
      selectvt640_();
      for (iq = icp; incs < 0 ? iq >= ispan : iq <= ispan; iq += incs)
	{
	  iclone = icnum3[iq - 1];
	  seek_1 (iclone) ;
	  fread (tname1, sizeof(char), 8, unit_1) ;
	  fread (&nband1, sizeof(long int), 1, unit_1) ;
	  fread (&lband1, sizeof(long int), 1, unit_1) ;
	  fread (&igel1, sizeof(long int), 1, unit_1) ;
	  fread (&icont1, sizeof(long int), 1, unit_1) ;

	  y += -1.5;
	  if (y <= margyr) goto L15 ;

	  bandColorCount++ ;
	  if (bandColorCount % 3 == 1) colorvt640_(BLACK) ;
	  if (bandColorCount % 3 == 2) colorvt640_(RED) ;
	  if (bandColorCount % 3 == 0) colorvt640_(BLUE) ;
  
	  text_(&x, &y, tname1b, &nchars, &xmax,
		&xmin, &ymax, &ymin, &margxl, &margxr,
		&margyl, &margyr, &isxmax, &isymax);
	  
	  for (iband = lband1; iband <= lband1 + nband1 - 1; ++iband)
	    {
	      xf = (float) coorda[iband - 1];
	      xt = xf;
	      yf = y;
	      if (yf <= margyr) goto L15 ; /* stop at the bottom */
	      yt = y + 1.f;
	      line1_(&xf, &xt, &yf, &yt, &xmax, &xmin, &ymax, &ymin,
		     &margxl, &margxr, &margyl, &margyr, &isxmax, &isymax);
	    }
	}
      goto L15;
    }
  if (strcasecmp(name, "c") == 0)
    {
      if (is == 1)
	{
	  initvt640_();
/* plot current contig above */
	  i__1 = 1 + 10;
	  plotcont4_(&ixs, &c__380, &iscale, &ncomp1, icnum1, projectbase,
		     &nclone, &i__1);
	  marker_(coordm, &nbandm, &id, &y, &xmax, &xmin, &ymax, &ymin,
		  &margxl, &margxr, &margyl, &margyr, &isxmax, &isymax);
	}
      is = 2;
      if (i == 0)
	{
	  goto L8;
	}
      goto L18;
    }
  if (strcasecmp(name, "a") == 0)
    {
      if (is == 2)
	{
	  initvt640_();
	}
      is = 1;
      analyse_(namea, nbanda, lbanda, igela, iconta, &nclone);
      goto L8;
    }
  if (strcasecmp(name, "m") == 0)
    {
      if (is == 2)
	{
	  initvt640_();
	}
      is = 1;
      map_(&nclone, &nbands, namea, nbanda, lbanda, igela, iconta);
      goto L8;
    }
  if (strcasecmp(name, "l") == 0)
    {
      if (is == 2)
	{
	  initvt640_();
	}
      is = 1;
      if (i == 0) goto L8;
      
      clim_(ch__5, &i);
      gprintf (stdout, "%4d-%-8s(%3db,%4d) %c%c%c\n",
	      igel, tname, nband, icont,
	       cflag_(i, 1, "p"), cflag_(i, 8, "h"), *ch__5) ;

      for (ix = lbanda[i - 1], itemp = 1;
	   ix <= lbanda[i - 1] + nbanda[i - 1] - 1; ++ix, ++itemp)
	{
	  gprintf (stdout, " %6d", coorda[ix - 1]);
	  if (!(itemp%10)) gprintf (stdout, "\n") ;
	}
      if ((itemp-1)%10) gprintf (stdout, "\n") ;
      goto L8;
    }
/* not control name, so find name */
  for (i = 2; i <= nclone + 1; ++i)
    {
      if (strncmp(namea + ((i - 1) << 3), name, 8) == 0)
	{
	  goto L2;
	}
    }
  gprintf (stdout, "clone not found\n") ;
  goto L8;
L2:
  seek_1 (i) ;
  fread (tname, sizeof(char), 8, unit_1) ;
  fread (&nband, sizeof(long int), 1, unit_1) ;
  fread (&lband, sizeof(long int), 1, unit_1) ;
  fread (&igel, sizeof(long int), 1, unit_1) ;
  fread (&icontt, sizeof(long int), 1, unit_1) ;
  switch (is)
    {
    case 1:  goto L37;
    case 2:  goto L12;
    }
L12:
  icont = icontt;
  selectvt640_();
  x = 0.f;

/* display named clone */
  y += -1.5;
  if (y <= margyr) goto L15 ;

  bandColorCount++ ;
  if (bandColorCount % 3 == 1) colorvt640_(BLACK) ;
  if (bandColorCount % 3 == 2) colorvt640_(RED) ;
  if (bandColorCount % 3 == 0) colorvt640_(BLUE) ;
  
  id = i;
  text_(&x, &y, tnameb, &nchars, &xmax, &xmin, &ymax, &ymin,
	&margxl, &margxr, &margyl, &margyr, &isxmax, &isymax);
  for (iband = lband; iband <= lband + nband - 1; ++iband)
    {
      xf = (float) coorda[iband - 1];
      xt = xf;
      yf = y;
      if (yf <= margyr) goto L15 ; /* stop at the bottom */
      yt = y + 1.f;
      line1_(&xf, &xt, &yf, &yt, &xmax, &xmin, &ymax, &ymin, &margxl,
	     &margxr, &margyl, &margyr, &isxmax, &isymax);
    }
  goto L15;
/* display current contig */
L18:
  if (icont == 0)
    {
      goto L15;
    }
  selectvt640_() ;
  asscont3_(&nclone, &icont, &ncomp3, icnum3, &itemp, &itemp);
  for (iq = 1; iq <= ncomp3; ++iq)
    {
      iclone = icnum3[iq - 1];
      seek_1 (iclone) ;
      fread (tname1, sizeof(char), 8, unit_1) ;
      fread (&nband1, sizeof(long int), 1, unit_1) ;
      fread (&lband1, sizeof(long int), 1, unit_1) ;
      fread (&igel1, sizeof(long int), 1, unit_1) ;
      fread (&icont1, sizeof(long int), 1, unit_1) ;

      if (nband1 == 0) continue ; /* no point in displaying virtual clones */

      ++bandColorCount ;
      if (bandColorCount % 3 == 1) colorvt640_(BLACK) ;
      if (bandColorCount % 3 == 2) colorvt640_(RED) ;
      if (bandColorCount % 3 == 0) colorvt640_(BLUE) ;

      y -= 1.5;

      if (y <= 1.0) 
	{
	  movevt640_(margxl, 10) ; colorvt640_(BLACK) ;
	  textvt640_(messprintf (". . . (%d clones from contig in band display, %d [incl. virtuals] not displayed)", iq, ncomp3-iq),
		     strlen(messprintf (". . . (%d clones from contig in band display, %d [incl. virtuals] not displayed)", iq, ncomp3-iq)) ) ;
	  goto L15; ; /* stop , when display reaches bottom of window */
	}

      text_(&x, &y, tname1b, &nchars, &xmax, &xmin, &ymax, &ymin,
	    &margxl, &margxr, &margyl, &margyr, &isxmax, &isymax);
      for (iband = lband1; iband <= lband1 + nband1 - 1; ++iband)
	{
	  xf = (float) coorda[iband - 1];
	  xt = xf;
	  yf = y;
	  yt = y + 1.f;
	  line1_(&xf, &xt, &yf, &yt, &xmax, &xmin, &ymax, &ymin, &margxl,
		 &margxr, &margyl, &margyr, &isxmax, &isymax);
	}
    }
  movevt640_(margxl, 10) ; colorvt640_(BLACK) ;
  textvt640_(messprintf ("(%2d clones in band display)  ", ncomp3), 12) ;
L15:
  selectvt100_() ;
  clearvt100_();
  movevt100_(0, 0) ;
  colorvt640_(BLACK) ;

  clim_(ch__5, &i);
  gprintf (stdout, "\n%4d-%-8s(%3db,%4d) %c%c%c"
	  "                                           %s\n",
	  igel, tname, nband, icont,
	   cflag_(i, 1, "p"), cflag_(i, 8, "h"), *ch__5, projectbase) ;
  goto L8;
L37:
  /* is old contig worth keeping? */
  contloss_(&icont, iconta, &nclone);
/* file old contig */
  if (icont != 0)
    {
      for (ix = 1; ix <= ncomp1; ++ix)
	{
	  icnum2[ix - 1] = icnum1[ix - 1];
	}
      ixs2 = ixs;
      seek_16 (icont) ;
      fwrite (&ixs, sizeof(long int), 1, unit_16) ;
      fwrt_(unit_16);
      ncomp2 = ncomp1;
      icnum2[3999] = icnum1[3999];
      icnum2[3998] = icnum1[3998];
      icnum2[3997] = icnum1[3997];
    }
  if (iconta[i - 1] == 0)
    {  /* choose new contig number */
      contrd_(&icontt, iconta, &nclone);
      iconta[i - 1] = icontt;
      seek_1 (i) ;
      fread (tname, sizeof(char), 8, unit_1) ;
      fread (&nband, sizeof(long int), 1, unit_1) ;
      fread (&lband, sizeof(long int), 1, unit_1) ;
      fread (&igel, sizeof(long int), 1, unit_1) ;
      fread (&itemp, sizeof(long int), 1, unit_1) ;

      seek_1 (i) ;
      fwrite (tname, sizeof(char), 8, unit_1) ;
      fwrite (&nband, sizeof(long int), 1, unit_1) ;
      fwrite (&lband, sizeof(long int), 1, unit_1) ;
      fwrite (&igel, sizeof(long int), 1, unit_1) ;
      fwrite (&icontt, sizeof(long int), 1, unit_1) ;
      fwrt_(unit_1);

      ixs = 0;
    }
  else
    {
      seek_16 (iconta[i - 1]) ;
      fread (&ixs, sizeof(long int), 1, unit_16) ;
    }
/* initialise new contig */
  iconto = icont;
  icont = icontt;
/* centre in single mode */
  if (imode / 1000 % 10 == 0)
    {
      ican = abs(icana[i - 1]) % 10000000;
      if (ican > 1)
	{
	  ipos = iposa[ican - 1];
	  ilen = ilena[ican - 1];
	}
      else
	{
	  ipos = iposa[i - 1];
	  ilen = ilena[i - 1];
	}
      ixs = -ipos - ilen / 2;
    }
L72:
  asscont4_(&nclone, &icont, &ncomp1, icnum1, &iposav);
L7:
   totplt_(&ixs, &iscale, &ncomp1, icnum1, &i, &ncomp2, icnum2, projectbase,
	  &nclone, &imode, &ixs2);
L70:
  selectvt100_() ;
  clearvt100_();
  movevt100_(0, 0) ;

  clim_(ch__5, &i);
  gprintf (stdout, "\n%4d-%-8s(%3db,%4d) %c%c%c"
	  "                                           %s\n",
	   igel, tname, nband, icont,
	   cflag_(i, 1, "p"), cflag_(i, 8, "h"), *ch__5, projectbase) ;
/* get new parameters for current clone */
  switch (iabs)
    {
    case 1:  goto L33;
    case 2:  goto L63;
    case 3:  goto L71;
    }
L63:
/* absolute data entry:- */
  gprintf (stdout, " left end = ") ;
  readinput ("", stemp, 5) ; if (nonint (stemp)) goto L63 ;
  ilft = a2int (stemp) ;
L64:
  gprintf (stdout, " right end = ") ;
  readinput ("", stemp, 5) ; if (nonint (stemp)) goto L64 ;
  irt = a2int (stemp) ;
  if (ilft == 0 && irt == 0) goto L35;
  if (irt - ilft < 1) goto L63;
  ilena[i - 1] = irt - ilft;
  iposa[i - 1] = ilft;
  
  seek_4 (i) ;
  fread (&itemp, sizeof(long int), 1, unit_4) ;
  fread (&itemp, sizeof(long int), 1, unit_4) ;
  fread (&ican, sizeof(long int), 1, unit_4) ;
  fread (&irem, sizeof(long int), 1, unit_4) ;

  seek_4 (i) ;
  fwrite (&iposa[i - 1], sizeof(long int), 1, unit_4) ;
  fwrite (&ilena[i - 1], sizeof(long int), 1, unit_4) ;
  fwrite (&ican, sizeof(long int), 1, unit_4) ;
  fwrite (&irem, sizeof(long int), 1, unit_4) ;
  fwrt_(unit_4);
  goto L65;
L33:
/* relative data entry */
  gprintf (stdout, " change in left end = ") ;
  readinput ("", stemp, 5) ; if (nonint(stemp)) goto L33 ;
  idlft = a2int (stemp) ;
L34:
  gprintf (stdout, " change in right end = ") ;
  readinput ("", stemp, 5) ; if (nonint(stemp)) goto L34 ;
  idrt = a2int (stemp) ;
  
  if (idlft == 0 && idrt == 0)
    {
      goto L35;
    }
  idlen = idrt - idlft;
  if (idlen + ilena[i - 1] < 1)
    {
      goto L33;
    }
  ilena[i - 1] += idlen;
  iposa[i - 1] += idlft;
  seek_4 (i) ;
  fread (&itemp, sizeof(long int), 1, unit_4) ;
  fread (&itemp, sizeof(long int), 1, unit_4) ;
  fread (&ican, sizeof(long int), 1, unit_4) ;
  fread (&irem, sizeof(long int), 1, unit_4) ;

  seek_4 (i) ;
  fwrite (&iposa[i - 1], sizeof(long int), 1, unit_4) ;
  fwrite (&ilena[i - 1], sizeof(long int), 1, unit_4) ;
  fwrite (&ican, sizeof(long int), 1, unit_4) ;
  fwrite (&irem, sizeof(long int), 1, unit_4) ;
  fwrt_(unit_4);
  asscont4_(&nclone, &icont, &ncomp1, icnum1, &iposav);
  goto L7;
/* cursor data entry */
L71:
  cursor_(&ichar, &ipcx, &iscale, &isch);
  ican = abs(icana[i - 1]) % 10000000;
  if (ican > 1)
    {
      ipos = iposa[ican - 1];
      ilen = ilena[ican - 1];
    }
  else
    {
      ipos = iposa[i - 1];
      ilen = ilena[i - 1];
    }
/* #1,left move */
  if (ichar == '1')
    {
      iposa[i - 1] = ipcx - ixs;
      seek_4 (i) ;
      fread (&itemp, sizeof(long int), 1, unit_4) ;
      fread (&ilen, sizeof(long int), 1, unit_4) ;
      fread (&ican, sizeof(long int), 1, unit_4) ;
      fread (&irem, sizeof(long int), 1, unit_4) ;
      
      seek_4 (i) ;
      fwrite (&iposa[i - 1], sizeof(long int), 1, unit_4) ;
      fwrite (&ilen, sizeof(long int), 1, unit_4) ;
      fwrite (&ican, sizeof(long int), 1, unit_4) ;
      fwrite (&irem, sizeof(long int), 1, unit_4) ;
      fwrt_(unit_4);
      clearvt640_() ;
      goto L72;
    }
/* #2,left adjust */
  else if (ichar == '2')
    {
      ipos = ipcx - ixs;
      ilen = ilena[i - 1] + iposa[i - 1] - ipos;
      if (ilen < 1)
	{
	  goto L71;
	}
      iposa[i - 1] = ipos;
      ilena[i - 1] = ilen;

      seek_4 (i) ;
      fread (&itemp, sizeof(long int), 1, unit_4) ;
      fread (&itemp, sizeof(long int), 1, unit_4) ;
      fread (&ican, sizeof(long int), 1, unit_4) ;
      fread (&irem, sizeof(long int), 1, unit_4) ;
      
      seek_4 (i) ;
      fwrite (&iposa[i - 1], sizeof(long int), 1, unit_4) ;
      fwrite (&ilena[i - 1], sizeof(long int), 1, unit_4) ;
      fwrite (&ican, sizeof(long int), 1, unit_4) ;
      fwrite (&irem, sizeof(long int), 1, unit_4) ;
      fwrt_(unit_4);
      clearvt640_() ;
      goto L72;
    }
/* #3,right adjust */
  else if (ichar == '3')
    {
      ilen = ipcx - ixs - iposa[i - 1];
      if (ilen < 1)
	{
	  goto L71;
	}
      ilena[i - 1] = ilen;
      seek_4 (i) ;
      fread (&ipos, sizeof(long int), 1, unit_4) ;
      fread (&itemp, sizeof(long int), 1, unit_4) ;
      fread (&ican, sizeof(long int), 1, unit_4) ;
      fread (&irem, sizeof(long int), 1, unit_4) ;
      
      seek_4 (i) ;
      fwrite (&ipos, sizeof(long int), 1, unit_4) ;
      fwrite (&ilena[i - 1], sizeof(long int), 1, unit_4) ;
      fwrite (&ican, sizeof(long int), 1, unit_4) ;
      fwrite (&irem, sizeof(long int), 1, unit_4) ;
      fwrt_(unit_4);
      clearvt640_() ;
      goto L72;
    }
/* #4,right move */
  else if (ichar == '4')
    {
      iposa[i - 1] = ipcx - ixs - ilena[i - 1];
      seek_4 (i) ;
      fread (&itemp, sizeof(long int), 1, unit_4) ;
      fread (&ilen, sizeof(long int), 1, unit_4) ;
      fread (&ican, sizeof(long int), 1, unit_4) ;
      fread (&irem, sizeof(long int), 1, unit_4) ;
      
      seek_4 (i) ;
      fwrite (&iposa[i - 1], sizeof(long int), 1, unit_4) ;
      fwrite (&ilen, sizeof(long int), 1, unit_4) ;
      fwrite (&ican, sizeof(long int), 1, unit_4) ;
      fwrite (&irem, sizeof(long int), 1, unit_4) ;
      fwrt_(unit_4);
      clearvt640_() ;
      goto L72;
    }
/* #6,curs left end */
  else if (ichar == '6')
    {
      ipcx = ipos + ixs;
      graphRedraw () ;
      goto L71;
    }
/* #7,curs right end */
  else if (ichar == '7')
    {
      ipcx = ipos + ilen + ixs;
      graphRedraw () ;
      goto L71;
    }
/* #9,move cont,left curr clone */
  else if (ichar == '9')
    {
      ixs = ipcx - ipos;
      clearvt640_() ;
      goto L7;
    }
/* #0,move cont,right curr clone */
  else if (ichar == '0')
    {
      ixs = ipcx - ipos - ilen;
      clearvt640_() ;
      goto L7;
    }
/* scroll left */
  else if (ichar == '[')
    {
      ixs += 750 / iscale;
      clearvt640_() ;
      goto L7;
    }
/* scroll right */
  else if (ichar == ']')
    {
      ixs -= 750 / iscale;
      clearvt640_() ;
      goto L7;
    }
/* scroll small step left */
  else if (ichar == '{')
    {
      ixs += 350 / iscale;
      clearvt640_() ;
      goto L7;
    }
/* scroll small step right */
  else if (ichar == '}')
    {
      ixs -= 350 / iscale;
      clearvt640_() ;
      goto L7;
    }
  else if (ichar =='h' || ichar == 'H')
    {
      selectvt100_();
      movevt100_(0, 5) ;
      gprintf (stdout,
	       "  <   cursor left\n"
	       "  >   cursor right\n"
	       "  1   move clone to align left end with cursor\n"
	       "  2   adjust left end of clone to cursor\n"
	       "  3   adjust right end of clone to cursor\n"
	       "  4   move clone to align right end with cursor\n"
	       "  6   align cursor with left end of clone\n"
	       "  7   align cursor with right end of clone\n"
	       "  9   move cont to align left end curr clone with curs\n"
	       "  0   move cont to align right end curr clone with curs\n"
	       "  L   move contig to align Left end with cursor\n"
	       "  R   move contig to align Right end with cursor\n"
	       " [,{  scroll contig-view left  (big=[ small={ steps)\n"
	       " ],}  scroll contig-view right (big=] small=} steps)\n"
	       "  S   Set cursor counter\n"
	       "  Z   cursor to Zero\n"
	       "  F   measure cursor movement From\n"
	       "  T   measure cursor movement To\n"
	       "  M   Move cursor required distance\n"
	       "  H   help\n"
	       );
      selectvt640_();
      goto L71;
    }
/* left end of contig */
  else if (ichar == 'l' || ichar == 'L')
    {
      ixs = ipcx - iposa[icnum1[3998] - 1];
      clearvt640_() ;
      goto L7;
    }
/* right end of contig */
  else if (ichar == 'r' || ichar == 'R')
    {
      irt = iposa[icnum1[ncomp1 - 1] - 1] +
	ilena[icnum1[ncomp1 - 1] - 1];
      for (iq = ncomp1 - 1; iq >= max(1,ncomp1 - 10); --iq)
	{
	  irt = max(irt,iposa[icnum1[iq - 1] - 1] + 
		    ilena[icnum1[iq - 1] - 1]);
	}
      ixs = ipcx - irt;
      clearvt640_() ;
      goto L7;
    }
/* cursor from */
  else if (ichar == 'f' || ichar == 'F')
    {
      ipcxf = ipcx;
      goto L71;
    }
/* cursor to */
  else if (ichar == 't' || ichar == 'T')
    {
      selectvt100_();
      idcx = ipcx - ipcxf;
      gprintf (stdout, "cursor movement was %d\n", idcx) ;
      goto L71;
    }
/* move cursor */
  else if (ichar == 'm' || ichar == 'M')
    {
      selectvt100_();
L74:
      gprintf (stdout, "   required cursor movement (+/-) = ") ;
      readinput ("", stemp, 5) ; if (nonint(stemp)) goto L74 ;
      idcx = a2int (stemp) ;
      ipcx += idcx;
      graphRedraw () ;
      goto L71;
    }

  graphRedraw () ;		/* to get rid of cleared cursBox, before
				 returning to contig control*/
  selectvt100_();
L35:
/* get another clone for current contig, or control name */
  gprintf (stdout, "  next clone for current contig/contig control = ") ;
  readinput ("", name, 8) ;
  if (strlength(name, 8) == 0)
    {
      goto L8;
    }
/* first check for control name */
  if (strcmp(name, "*") == 0)
    {
      goto L13;
    }
  if (strcasecmp(name, "h") == 0 || strcasecmp(name, "help") == 0)
    {
      gprintf (stdout, "\n"
	       "    <RET>  return to  global control\n"
	       " <CTRL>+L  clear graphics and reveal text\n"
	       "      ID   set clone identity\n"
	       "      SD   show clone identity\n"
	       "     REV   reveal hidden clone\n"
	       "     INV   invert contig\n"
	       "      SC   scroll contig-view left or right\n"
	       "     DEL   delete clone from contig\n"
	       "    JOIN   join contigs\n"
	       "   SPLIT   split contigs\n"
	       "      SQ   squeeze or stretch contigs\n"
	       "     REM   create or change remark\n"
	       "     POS   auto-position current clone\n"
	       "    BURY   auto-burial\n"
	       "   LIMBO   auto-limbo\n"
	       "    FLAG   alter flags\n"
	       "     CLC   plot clone occurrence\n"
	       "       L   list bands\n"
	       "  CANCEL   cancel clone\n"
	       "  DIFMAP   match difference bands\n"
	       "     REJ   reject match\n"
	       "     PSS   possible match\n"
	       "     CLE   get clean plot\n"
	       "       F   file current clone in \"clone.lis\"\n"
	       "     VIR   create virtual clone\n"
	       "    HIDE   hide contig\n"
	       "  UNHIDE   unhide contig\n"
	       "   PMODE   switch plot mode\n"
	       "   ABSTR   abstract current contig\n"
	       "   SCALE   set scale\n"
	       "      *   quit the program\n"
	       ) ;
      goto L35;
    }
  if (strcasecmp(name, "scale") == 0)
    {
      scale_(&iscale);
      goto L7;
    }
  if (strcasecmp(name, "abstr") == 0)
    {
      abstr_(&icont, &nclone);
      goto L35;
    }
  if (strcasecmp(name, "pmode") == 0)
    {
      pmode_(&imode);
      goto L7;
    }
  if (strcasecmp(name, "a") == 0)
    {
      analyse_(namea, nbanda, lbanda, igela, iconta, &nclone);
      goto L65;
    }
  if (strcasecmp(name, "m") == 0)
    {
      map_(&nclone, &nbands, namea, nbanda, lbanda, 
	   igela, iconta);
      goto L65;
    }
  if (strcasecmp(name, "hide") == 0) 
    {
      if (icont == 0)
	{
	  gprintf (stdout, "   contig not selected\n") ;
	  goto L35;
	}
      for (iq = 2; iq <= nclone + 1; ++iq)
	{
	  if (iconta[iq - 1] == icont)
	    {
	      wrflag_(&iq, &c__8, &c__1);
	    }
	}
      gprintf (stdout, "   contig hidden\n") ;
      goto L35;
    }
  if (strcasecmp(name, "unhide") == 0)
    {
      if (icont == 0)
	{
	  gprintf (stdout, "   contig not selected\n") ;
	  goto L35;
	}
      for (iq = 2; iq <= nclone + 1; ++iq)
	{
	  if (iconta[iq - 1] == icont)
	    {
	      wrflag_(&iq, &c__8, &c__0);
	    }
	}
      gprintf (stdout, "   contig unhidden\n") ;
      goto L35;
    }
  if (strcasecmp(name, "vir") == 0)
    {
      gprintf (stdout, "   name of virtual clone = ") ;
      readinput ("", tname, 8) ;
      if (strlength(tname, 8) == 0) goto L35;

      i = nclone + 2;
      nband = 0;
      lband = 1;
      igel = 0;
      ipos = ipcx - ixs;
      ilen = 20;
      strncpy(namea + ((i - 1) << 3), tname, 8);
      lbanda[i - 1] = 1;
      iconta[i - 1] = icont;
      iposa[i - 1] = ipos;
/* enlarge files */
      ilena[i - 1] = ilen;
      seek_1 (i) ;
      fwrite (tname, sizeof(char), 8, unit_1) ;
      fwrite (&nband, sizeof(long int), 1, unit_1) ;
      fwrite (&lband, sizeof(long int), 1, unit_1) ;
      fwrite (&igel, sizeof(long int), 1, unit_1) ;
      fwrite (&icont, sizeof(long int), 1, unit_1) ;

      seek_4 (i) ;
      fwrite (&ipos, sizeof(long int), 1, unit_4) ;
      fwrite (&ilen, sizeof(long int), 1, unit_4) ;
      fwrite (&c__0, sizeof(long int), 1, unit_4) ;
      fwrite (&c__0, sizeof(long int), 1, unit_4) ;

      seek_13 (i) ;
      fwrite (&c__0, sizeof(long int), 1, unit_13) ;
      ++nclone;

      seek_1 (1) ;
      fwrite (&nclone, sizeof(long int), 1, unit_1) ;
      fwrite (&nbands, sizeof(long int), 1, unit_1) ;
      wrflag_(&i, &c__4, &c__1);
      fwrt_(unit_13);
      fwrt_(unit_4);
      fwrt_(unit_1);
      goto L72;
    }
  if (strcasecmp(name, "f") == 0)
    {
      clist_(&nclone, &i);
    }
  if (strcasecmp(name, "cle") == 0)
    {
      clearvt640_();
      selectvt640_();
      plotcont4_(&ixs, &c__380, &iscale, &ncomp1, icnum1, projectbase,
		 &nclone, &imode);
      selectvt100_();
      waitForMore (RETURN_KEY) ;
      clearvt640_() ;
      goto L7;
    }
  if (strcasecmp(name, "rej") == 0)
    {
      iln1 = i;
      rejpss_("wR", &iln1, &iln2, ilna, projectbase, &nclone);
      goto L65;
    }
  if (strcasecmp(name, "pss") == 0)
    {
      iln1 = i;
      rejpss_("wP", &iln1, &iln2, ilna, projectbase, &nclone);
      goto L65;
    }
  if (strcasecmp(name, "difmap") == 0)
    {
      difmap_(&isch, &icont, &ixs, &iscale);
      goto L65;
    }
/* prohibit cancellation of hidden or canonical clone */
  if (strcasecmp(name, "cancel") == 0)
    {
      if (icana[i - 1] != 0)
	{
	  gprintf (stdout, "can't cancel buried or canonical clone\n") ;
	  goto L35;
	}
/* ensure cancel required */
      gprintf (stdout, "   cancel current clone (Y/N) ? ") ;
      readinput ("", stemp, 2) ; *ans = *stemp ;
/* cancel current clone */
      if (*ans != 'y' && *ans != 'Y')
	{
	  goto L35;
	}
      iconta[i - 1] = 0;
      nbanda[i - 1] = 0;
      nband = 0;
      strncpy(tname, namea + ((i - 1) << 3), 8);
      for (iq = 7; iq >= 1; --iq)
	{
	  tname[iq] = tname[iq - 1] ;
	}
      tname[0] = '!';
      strncpy(namea + ((i - 1) << 3), tname, 8);
      seek_1 (i) ;
      fread (tname, sizeof(char), 8, unit_1) ;
      fread (&itemp, sizeof(long int), 1, unit_1) ;
      fread (&lband, sizeof(long int), 1, unit_1) ;
      fread (&igel, sizeof(long int), 1, unit_1) ;
      fread (&itemp, sizeof(long int), 1, unit_1) ;

      seek_1 (i) ;
      fwrite (namea + ((i - 1) << 3), sizeof(char), 8, unit_1) ;
      fwrite (&c__0, sizeof(long int), 1, unit_1) ;
      fwrite (&lband, sizeof(long int), 1, unit_1) ;
      fwrite (&igel, sizeof(long int), 1, unit_1) ;
      fwrite (&c__0, sizeof(long int), 1, unit_1) ;
      fwrt_(unit_1);
/* reset pos flag */
      wrflag_(&i, &c__1, &c__0);
      emphclone_(&c_n1, &ixs, &c__380, &ncomp1, icnum1, &iscale, &imode);
      contloss_(&icont, iconta, &nclone);
      asscont4_(&nclone, &icont, &ncomp1, icnum1, &iposav);
      if (icont == 0)
	{
	  clearvt640_();
	  selectvt640_();
/* plot old contig below */
	  if (imode / 1000 % 10 == 1)
	    {
	      plotcont4_(&ixs2, &c__60, &iscale, &ncomp2, icnum2,
			 projectbase, &nclone, &imode);
	    }
	  selectvt100_();
	  clearvt100_();
	  movevt100_(0, 0) ;
	  goto L8;
	}
      clearvt640_() ;
      goto L40;
    }
  if (strcasecmp(name, "l") == 0)
    {
      if (i == 0)
	{
	  goto L35;
	}
   
      for (ix = lbanda[i - 1], itemp = 1;
	   ix <= lbanda[i - 1] + nbanda[i - 1] - 1; ++ix, ++itemp)
	{
	  gprintf (stdout, " %6d", coorda[ix - 1]);
	  if (!(itemp%10)) gprintf (stdout, "\n") ;
	}
      if ((itemp-1)%10) gprintf (stdout, "\n") ;
      goto L35;
    }
  if (strcasecmp(name, "clc") == 0)
    {
      clc_(&nclone, &icont, &ixs, &iscale, &ncomp1, icnum1);
      goto L7;
    }
  if (strcasecmp(name, "flag") == 0) 
    {
      flag_(&i);
      goto L70;
    }
  if (strcasecmp(name, "bury") == 0) 
    {
      iret = 1;
      bury_(&i, &iret, &ncomp1, icnum1);
      if (iret == 35)
	{
	  goto L35;
	}
      else
	{
	  goto L65;
	}
    }
  if (strcasecmp(name, "limbo") == 0)
    {
      iret = 2;
      bury_(&i, &iret, &ncomp1, icnum1);
      if (iret == 35)
	{
	  goto L35;
	}
      else
	{
	  goto L65;
	}
    }
  if (strcasecmp(name, "pos") == 0)
    {
      if (ncomp1 == 0 || nbanda[i - 1] <= 0)
	{
	  goto L35;
	}
      for (iq = 1; iq <= ncomp1; ++iq)
	{
	  if (icnum1[iq - 1] == i)
	    {
	      posclone_(&iq, &ncomp1, icnum1, &iposa[i - 1], &c__0);

	      seek_4 (i) ;
	      fread (&itemp, sizeof(long int), 1, unit_4) ;
	      fread (&ilen, sizeof(long int), 1, unit_4) ;
	      fread (&ican, sizeof(long int), 1, unit_4) ;
	      fread (&irem, sizeof(long int), 1, unit_4) ;
	      
	      seek_4 (i) ;
	      fwrite (&iposa[i - 1], sizeof(long int), 1, unit_4) ;
	      fwrite (&ilen, sizeof(long int), 1, unit_4) ;
	      fwrite (&ican, sizeof(long int), 1, unit_4) ;
	      fwrite (&irem, sizeof(long int), 1, unit_4) ;
	      fwrt_(unit_4);

	      asscont4_(&nclone, &icont, &ncomp1, icnum1, &iposav);
	      totplt_(&ixs, &iscale, &ncomp1, icnum1, &i, &ncomp2, icnum2, 
		      projectbase, &nclone, &imode, &ixs2);
	      gprintf (stdout, " type N if autopos flag Not required: ") ;
	      readinput ("", stemp, 2) ; *ans = *stemp ;
	      if (*ans != 'n' && *ans != 'N')
		{
		  wrflag_(&i, &c__1, &c__1);
		}
	      else
		{
		  wrflag_(&i, &c__1, &c__0);
		}
	      goto L70;
	    }
	}
    }
  if (strcasecmp(name, "id") == 0)
    {
      if (icana[i - 1] == -1)
	{
	  gprintf (stdout, "can't bury canonical\n") ;
	  goto L35;
	}
      if (icana[i - 1] != 0)
	{
	  gprintf (stdout, "reveal clone before reassigning canonical\n") ;
	  goto L35;
	}
      if (irema[i - 1] != 0)
	{
	  gprintf (stdout, "can't bury clone with remark\n") ;
	  goto L35;
	}
      gprintf (stdout, "   canonical clone = ") ;
      readinput ("", cname, 8) ;
      for (ix = 1; ix <= ncomp1; ++ix)
	{
	  iy = icnum1[ix - 1];
	  if (strncmp(namea + ((iy - 1) << 3), cname, 8) == 0)
	    {
	      if (iy == i)
		{
		  gprintf (stdout, "can't be canonical to self\n") ;
		  goto L35;
		}
	      gprintf (stdout, "   exact identity (E), partial identity (P)\n"
		      "     or limbo (L) ? ") ;
	      readinput ("", stemp, 2) ; *ans = *stemp ;
	      if (*ans == 'e' || *ans == 'E')
		{
		  ican = iy;
		}
	      else if (*ans == 'p' || *ans == 'P')
		{
		  ican = -iy;
		}
	      else if (*ans == 'l' || *ans == 'L')
		{
		  ican = iy + 10000000;
		}
	      else
		{
		  goto L35;
		}
	      icana[iy - 1] = -1;

	      seek_4 (iy) ;
	      fread (&ipos, sizeof(long int), 1, unit_4) ;
	      fread (&ilen, sizeof(long int), 1, unit_4) ;
	      fread (&itemp, sizeof(long int), 1, unit_4) ;
	      fread (&irem, sizeof(long int), 1, unit_4) ;
	      
	      seek_4 (iy) ;
	      fwrite (&ipos, sizeof(long int), 1, unit_4) ;
	      fwrite (&ilen, sizeof(long int), 1, unit_4) ;
	      fwrite (&icana[iy - 1], sizeof(long int), 1, unit_4) ;
	      fwrite (&irem, sizeof(long int), 1, unit_4) ;
	      fwrt_(unit_4);

	      iposa[i - 1] = ipos;
	      icana[i - 1] = ican;
	      if (ican == iy)
		{
		  ilena[i - 1] = ilen;
		  seek_4 (i) ;
		  fread (&itemp, sizeof(long int), 1, unit_4) ;
		  fread (&itemp, sizeof(long int), 1, unit_4) ;
		  fread (&itemp, sizeof(long int), 1, unit_4) ;
		  fread (&irem, sizeof(long int), 1, unit_4) ;
		  
		  seek_4 (i) ;
		  fwrite (&ipos, sizeof(long int), 1, unit_4) ;
		  fwrite (&ilen, sizeof(long int), 1, unit_4) ;
		  fwrite (&ican, sizeof(long int), 1, unit_4) ;
		  fwrite (&irem, sizeof(long int), 1, unit_4) ;
		  fwrt_(unit_4);
		}
	      else
		{
		  seek_4 (i) ;
		  fread (&itemp, sizeof(long int), 1, unit_4) ;
		  fread (&ilen, sizeof(long int), 1, unit_4) ;
		  fread (&itemp, sizeof(long int), 1, unit_4) ;
		  fread (&irem, sizeof(long int), 1, unit_4) ;
		  
		  seek_4 (i) ;
		  fwrite (&ipos, sizeof(long int), 1, unit_4) ;
		  fwrite (&ilen, sizeof(long int), 1, unit_4) ;
		  fwrite (&ican, sizeof(long int), 1, unit_4) ;
		  fwrite (&irem, sizeof(long int), 1, unit_4) ;
		  fwrt_(unit_4);
		}
	      i = iy;
	      seek_1 (i) ;
	      fread (tname, sizeof(char), 8, unit_1) ;
	      fread (&nband, sizeof(long int), 1, unit_1) ;
	      fread (&lband, sizeof(long int), 1, unit_1) ;
	      fread (&igel, sizeof(long int), 1, unit_1) ;
	      fread (&icont2, sizeof(long int), 1, unit_1) ;
	      goto L65;
	    }
	}
      gprintf (stdout, "  canonical not found\n") ;
      goto L35;
    }
  if (strcasecmp(name, "sd") == 0)
    {
      iy = abs(icana[i - 1]) % 10000000;
      if (iy == 0)
	{
	  goto L35;
	}
      if (iy > 1)
	{
	  ican = iy;
	  
	  strtcpy (ntemp1, namea + ((iy - 1) << 3), 8) ;
	  gprintf (stdout, "%4d-%-8s(%3db,%4d)           (canonical)\n",
		  igela[iy - 1],
		  ntemp1,
		  nbanda[iy - 1],
		  iconta[iy - 1]) ;
	}
      if (iy == 1)
	{
	  ican = i;
	}
      icnr = ican + 10000000;
      for (ix = 2; ix <= nclone + 1; ++ix)
	{
	  if (icana[ix - 1] == ican)
	    {
	      strtcpy (ntemp1, namea + ((ix - 1) << 3), 8) ;
	      gprintf (stdout, "%4d-%-8s(%3db,%4d)\n",
		      igela[ix - 1],
		      ntemp1,
		      nbanda[ix - 1],
		      iconta[ix - 1]) ;
	    }
	  else if (icana[ix - 1] == -ican)
	    {
	      strtcpy (ntemp1, namea + ((ix - 1) << 3), 8) ;
	      gprintf (stdout, "%4d-%-8s(%3db,%4d)           (partial identity)\n",
		      igela[ix - 1],
		      ntemp1,
		      nbanda[ix - 1],
		      iconta[ix - 1]) ;
	    }
	  else if (icana[ix - 1] == icnr)
	    {
	      strtcpy (ntemp1, namea + ((ix - 1) << 3), 8) ;
	      gprintf (stdout, "%4d-%-8s(%3db,%4d)           (limbo)\n",
		      igela[ix - 1],
		      ntemp1,
		      nbanda[ix - 1],
		      iconta[ix - 1]);
	    }
	}
      goto L35;
    }
  if (strcasecmp(name, "rev") == 0)
    {
      if (icana[i - 1] == 0 || icana[i - 1] == -1)
	{
	  goto L35;
	}
      if (icana[i - 1] < -1 || icana[i - 1] > 10000000)
	{
	  gprintf (stdout, " partial identity or limbo: ready to position clone (Y/N) ? ");
	  readinput ("", stemp, 2) ; *ans = *stemp ;
	  if (*ans != 'y' && *ans != 'Y')
	    {
	      goto L35;
	    }
	}
      iy = abs(icana[i - 1]) % 10000000;
      iposa[i - 1] = iposa[iy - 1];
      icana[i - 1] = 0;
      seek_4 (i) ;
      fread (&itemp, sizeof(long int), 1, unit_4) ;
      fread (&ilen, sizeof(long int), 1, unit_4) ;
      fread (&itemp, sizeof(long int), 1, unit_4) ;
      fread (&irem, sizeof(long int), 1, unit_4) ;
      
      seek_4 (i) ;
      fwrite (&iposa[i - 1], sizeof(long int), 1, unit_4) ;
      fwrite (&ilen, sizeof(long int), 1, unit_4) ;
      fwrite (&izero, sizeof(long int), 1, unit_4) ;
      fwrite (&irem, sizeof(long int), 1, unit_4) ;
      fwrt_(unit_4);

/* check whether any clones left in group */
      for (ix = 2; ix <= nclone + 1; ++ix)
	{
	  if (abs(icana[ix - 1]) % 10000000 == iy)
	    {
	      goto L72;
	    }
	}
      icana[iy - 1] = 0;
      seek_4 (iy) ;
      fread (&ipos, sizeof(long int), 1, unit_4) ;
      fread (&ilen, sizeof(long int), 1, unit_4) ;
      fread (&itemp, sizeof(long int), 1, unit_4) ;
      fread (&irem, sizeof(long int), 1, unit_4) ;
      
      seek_4 (iy) ;
      fwrite (&ipos, sizeof(long int), 1, unit_4) ;
      fwrite (&ilen, sizeof(long int), 1, unit_4) ;
      fwrite (&izero, sizeof(long int), 1, unit_4) ;
      fwrite (&irem, sizeof(long int), 1, unit_4) ;
      fwrt_(unit_4);
      goto L72;
    }
  if (strcasecmp(name, "inv") == 0)
    {
      invert_(&ncomp1, icnum1, &ipcx, &iscale, &isch, &ixs);
      goto L72;
    }
  if (strcasecmp(name, "sc") == 0)
    {
L43:
      gprintf (stdout, "   scroll (in band units) = ") ;
      readinput ("", stemp, 5) ; if (nonint(stemp)) goto L43 ;
      ishift = a2int (stemp) ;
      ixs -= ishift;
      goto L7;
    }
  if (strcasecmp(name, "rem") == 0)
    {
/* look at current clone ; check for burial */
      if (abs(icana[i - 1]) > 1)
	{
	  gprintf (stdout, "  clone buried\n");
	  goto L35;
	}
      remchng_(&i, &isch);
      clearvt640_() ;
      goto L40;
    }
/* prohibit deletion of hidden clone */
  if (strcasecmp(name, "del") == 0)
    {
      iy = abs(icana[i - 1]) % 10000000;
      if (iy > 1)
	{
	  gprintf (stdout, "  can't delete buried clone\n") ;
	}
/* ensure delete required */
      gprintf (stdout, "  delete current clone + any equivalents (Y/N) ? ") ;
      readinput ("", stemp, 2) ; *ans = *stemp ;
      if (*ans != 'y' && *ans != 'Y')
	{
	  goto L35;
	}
/* delete current clone if not member of group */
      if (iy == 0)
	{
	  iconta[i - 1] = 0;
	  seek_1 (i) ;
	  fread (tname, sizeof(char), 8, unit_1) ;
	  fread (&nband, sizeof(long int), 1, unit_1) ;
	  fread (&lband, sizeof(long int), 1, unit_1) ;
	  fread (&igel, sizeof(long int), 1, unit_1) ;
	  fread (&itemp, sizeof(long int), 1, unit_1) ;

	  seek_1 (i) ;
	  fwrite (tname, sizeof(char), 8, unit_1) ;
	  fwrite (&nband, sizeof(long int), 1, unit_1) ;
	  fwrite (&lband, sizeof(long int), 1, unit_1) ;
	  fwrite (&igel, sizeof(long int), 1, unit_1) ;
	  fwrite (&c__0, sizeof(long int), 1, unit_1) ;
	  fwrt_(unit_1);
/* reset pos flag */
	  wrflag_(&i, &c__1, &c__0);
	}
/* assign new contig and look for equivalents */
	if (iy == 1)
	  {
	    ican = i;
	    contrd_(&icontt, iconta, &nclone);
	    iconta[ican - 1] = icontt;
	    
	    seek_1 (ican) ;
	    fread (tname, sizeof(char), 8, unit_1) ;
	    fread (&nband, sizeof(long int), 1, unit_1) ;
	    fread (&lband, sizeof(long int), 1, unit_1) ;
	    fread (&igel, sizeof(long int), 1, unit_1) ;
	    fread (&itemp, sizeof(long int), 1, unit_1) ;

	    seek_1 (ican) ;
	    fwrite (tname, sizeof(char), 8, unit_1) ;
	    fwrite (&nband, sizeof(long int), 1, unit_1) ;
	    fwrite (&lband, sizeof(long int), 1, unit_1) ;
	    fwrite (&igel, sizeof(long int), 1, unit_1) ;
	    fwrite (&icontt, sizeof(long int), 1, unit_1) ;
	    fwrt_(unit_1);

	    for (ix = 2; ix <= nclone + 1; ++ix)
	      {
		if (abs(icana[ix - 1]) % 10000000 == ican)
		  {
		    iconta[ix - 1] = icontt;

		    seek_1 (ix) ;
		    fread (tname, sizeof(char), 8, unit_1) ;
		    fread (&nband, sizeof(long int), 1, unit_1) ;
		    fread (&lband, sizeof(long int), 1, unit_1) ;
		    fread (&igel, sizeof(long int), 1, unit_1) ;
		    fread (&itemp, sizeof(long int), 1, unit_1) ;

		    seek_1 (ix) ;
		    fwrite (tname, sizeof(char), 8, unit_1) ;
		    fwrite (&nband, sizeof(long int), 1, unit_1) ;
		    fwrite (&lband, sizeof(long int), 1, unit_1) ;
		    fwrite (&igel, sizeof(long int), 1, unit_1) ;
		    fwrite (&icontt, sizeof(long int), 1, unit_1) ;
		    fwrt_(unit_1);
		  }
	      }
	  }
      emphclone_(&c_n1, &ixs, &c__380, &ncomp1, icnum1, &iscale, &imode);
      contloss_(&icont, iconta, &nclone);
      asscont4_(&nclone, &icont, &ncomp1, icnum1, &iposav);
      if (icont == 0)
	{
	  clearvt640_();
	  selectvt640_();
/* plot old contig below */
	  if (imode / 1000 % 10 == 1)
	    {
	      plotcont4_(&ixs2, &c__60, &iscale, &ncomp2, icnum2,
			 projectbase, &nclone, &imode);
	    }
	  selectvt100_();
	  clearvt100_();
	  movevt100_(0, 0) ;
	  goto L8;
	}
      clearvt640_() ;
      goto L40;
    }
  if (strcasecmp(name, "join") == 0) 
    {
      if (iconto == 0)
	{
	  gprintf (stdout, "  can't join to empty contig\n");
	  goto L8;
	}
      else if (iconto == icont)
	{
	  gprintf (stdout, "  can't join to self\n");
	  goto L8;
	}
      else if (imode / 1000 % 10 != 1)
	{
	  gprintf (stdout, "  can't join in this plot mode: use PMODE to alter\n");
	  goto L8;
	}
      else if (ncomp1 + ncomp2 >= 3998)
	{
	  gprintf (stdout, "  joining will exceed ICNUMA array size\n");
	  goto L8;
	}
      else if (icnum1[3997] != 0 && icnum2[3997] != 0 && icnum1[3997] != 
		icnum2[3997]) 
	{
	  gprintf (stdout, "  contigs are on different chromosomes\n");
	  goto L8;
	}
      else if (ncomp2 - ncomp1 > 10)
	{
	  gprintf (stdout, " do you need to join larger to smaller contig (Y/N) ? ");
	  readinput ("", stemp, 2) ; *ans = *stemp ;
	  if (*ans != 'y' && *ans != 'Y')
	    {
	      goto L8;
	    }
	}
      for (ix = 1; ix <= ncomp2; ++ix)
	{
	  iy = icnum2[ix - 1];
	  if (iconta[iy - 1] == iconto)
	    {
	      iconta[iy - 1] = icont;
	      iposa[iy - 1] = iposa[iy - 1] + ixs2 - ixs;
	    }
	  else
	    {
	      gprintf (stdout, " fault in icnum2:-\n"
		      " ix= %d iconto= %d iconta(iy)= %d\n",
		      ix, iconto, iconta[iy - 1]) ;
	    }
	}
      emphclone_(&c_n1, &ixs, &c__380, &ncomp1, icnum1, &iscale, &imode);
      asscont4_(&nclone, &icont, &ncomp1, icnum1, &iposav);
      clearvt640_();
      selectvt640_();
/* plot merged contig above */
      plotcont4_(&ixs, &c__380, &iscale, &ncomp1, icnum1,
		 projectbase, &nclone, &imode);
      selectvt100_();
      clearvt100_();
      movevt100_(0, 0) ;

      gprintf (stdout, "   OK to file join (Y/N) ? ") ;
      readinput ("", stemp, 2) ; *ans = *stemp ;
      if (*ans != 'y' && *ans != 'Y')
	{
	  for (ix = 1; ix <= ncomp2; ++ix)
	    {
	      iy = icnum2[ix - 1];
	      if (iconta[iy - 1] == icont)
		{
		  iconta[iy - 1] = iconto;
		  iposa[iy - 1] = iposa[iy - 1] + ixs - ixs2;
		}
	    }
	}
      else
	{
	  for (ix = 1; ix <= ncomp2; ++ix)
	    {
	      iy = icnum2[ix - 1];
	      if (iconta[iy - 1] != icont)
		{
		  goto L47;
		}
	      seek_1 (iy) ;
	      fread (tname, sizeof(char), 8, unit_1);
	      fread (&nband, sizeof(long int), 1, unit_1) ;
	      fread (&lband, sizeof(long int), 1, unit_1) ;
	      fread (&igel, sizeof(long int), 1, unit_1) ;
	      fread (&itemp, sizeof(long int), 1, unit_1) ;

	      seek_1 (iy) ;
	      fwrite (tname, sizeof(char), 8, unit_1);
	      fwrite (&nband, sizeof(long int), 1, unit_1) ;
	      fwrite (&lband, sizeof(long int), 1, unit_1) ;
	      fwrite (&igel, sizeof(long int), 1, unit_1) ;
	      fwrite (&icont, sizeof(long int), 1, unit_1) ;
	      fwrt_(unit_1);

	      seek_4 (iy) ;
	      fread (&itemp, sizeof(long int), 1, unit_4) ;
	      fread (&ilen, sizeof(long int), 1, unit_4) ;
	      fread (&ican, sizeof(long int), 1, unit_4) ;
	      fread (&irem, sizeof(long int), 1, unit_4) ;
	      
	      seek_4 (iy) ;
	      fwrite (&iposa[iy - 1], sizeof(long int), 1, unit_4) ;
	      fwrite (&ilen, sizeof(long int), 1, unit_4) ;
	      fwrite (&ican, sizeof(long int), 1, unit_4) ;
	      fwrite (&irem, sizeof(long int), 1, unit_4) ;
	      fwrt_(unit_4);
L47:
	      ;
	    }
/* look for equivalents as well */
	  for (ix = 2; ix <= nclone + 1; ++ix)
	    {
	      if (iconta[ix - 1] == iconto)
		{
		  iconta[ix - 1] = icont;
		  seek_1 (ix) ;
		  fread (tname, sizeof(char), 8, unit_1);
		  fread (&nband, sizeof(long int), 1, unit_1) ;
		  fread (&lband, sizeof(long int), 1, unit_1) ;
		  fread (&igel, sizeof(long int), 1, unit_1) ;
		  fread (&itemp, sizeof(long int), 1, unit_1) ;
		  
		  seek_1 (ix) ;
		  fwrite (tname, sizeof(char), 8, unit_1);
		  fwrite (&nband, sizeof(long int), 1, unit_1) ;
		  fwrite (&lband, sizeof(long int), 1, unit_1) ;
		  fwrite (&igel, sizeof(long int), 1, unit_1) ;
		  fwrite (&icont, sizeof(long int), 1, unit_1) ;
		  fwrt_(unit_1);
		}
	    }
	  contstack_("w", &iconto);
	  ncomp2 = 0;
	}
      goto L65;
    }
  if (strcasecmp(name, "sq") == 0)
    {
      squ_(&ipcx, &ncomp1, icnum1, &ixs, &iscale, &isch, &imode,
	   &nclone, projectbase);
      clearvt640_() ;
      goto L65;
    }
  if (strcasecmp(name, "split") == 0)
    {
      gprintf (stdout, "  position cursor at split point, defined by\n"
	      "  centres of clones; then hit space bar\n");
      cursor_(&ichar, &ipcx, &iscale, &isch);
/* find split point in clone list */
      if (ichar != 32)
	{
	  goto L35;
	}
      getsplt_(&ipcx, &ncomp1, icnum1, &ixs, &isplit);
/* find active clone in list */
      if (isplit < 1 || isplit >= ncomp1)
	{
	  goto L35;
	}
      ican = abs(icana[i - 1]) % 10000000;
      if (ican <= 1)
	{
	  iact = i;
	}
      else
	{
	  iact = ican;
	}
      for (ix = 1; ix <= ncomp1; ++ix)
	{
	  if (icnum1[ix - 1] == iact)
	    {
	      goto L84;
	    }
	}
L84:
/* arrange to split in the most economical fashion */
      if (isplit < ncomp1 / 2 && ix <= isplit)
	{
	  ix = isplit + 1;
	  iact = icnum1[ix - 1];
	  i = iact;
	}
      else if (isplit > ncomp1 / 2 && ix > isplit)
	{
	  ix = isplit;
	  iact = icnum1[ix - 1];
	  i = iact;
	}
      if (ix <= isplit)
	{
	  istart = isplit + 1;
	  iend = ncomp1;
	}
      else
	{
	  istart = 1;
	  iend = isplit;
	}
      contrd_(&iconto, iconta, &nclone);
      for (ix = istart; ix <= iend; ++ix)
	{
	  iy = icnum1[ix - 1];
	  iconta[iy - 1] = iconto;

	  seek_1 (iy) ;
	  fread (tname, sizeof(char), 8, unit_1);
	  fread (&nband, sizeof(long int), 1, unit_1) ;
	  fread (&lband, sizeof(long int), 1, unit_1) ;
	  fread (&igel, sizeof(long int), 1, unit_1) ;
	  fread (&itemp, sizeof(long int), 1, unit_1) ;
	  
	  seek_1 (iy) ;
	  fwrite (tname, sizeof(char), 8, unit_1);
	  fwrite (&nband, sizeof(long int), 1, unit_1) ;
	  fwrite (&lband, sizeof(long int), 1, unit_1) ;
	  fwrite (&igel, sizeof(long int), 1, unit_1) ;
	  fwrite (&iconto, sizeof(long int), 1, unit_1) ;
/* look for equivalents as well */
	  if (icana[iy - 1] == -1)
	    {
	      for (iq = 2; iq <= nclone + 1; ++iq)
		{
		  ican = abs(icana[iq - 1]) % 10000000;
		  if (ican == iy)
		    {
		      iconta[iq - 1] = iconto;
		      seek_1 (iq) ;
		      fread (tname, sizeof(char), 8, unit_1);
		      fread (&nband, sizeof(long int), 1, unit_1) ;
		      fread (&lband, sizeof(long int), 1, unit_1) ;
		      fread (&igel, sizeof(long int), 1, unit_1) ;
		      fread (&itemp, sizeof(long int), 1, unit_1) ;
		      
		      seek_1 (iq) ;
		      fwrite (tname, sizeof(char), 8, unit_1);
		      fwrite (&nband, sizeof(long int), 1, unit_1) ;
		      fwrite (&lband, sizeof(long int), 1, unit_1) ;
		      fwrite (&igel, sizeof(long int), 1, unit_1) ;
		      fwrite (&iconto, sizeof(long int), 1, unit_1) ;
		    }
		}
	    }
	}
      fwrt_(unit_1);
      ixs2 = ixs;
      contloss_(&iconto, iconta, &nclone);
      asscont4_(&nclone, &iconto, &ncomp2, icnum2, &itemp);
      clearvt640_() ;
      goto L65;
    }
/* not control, so find name */
  if (ncomp1 >= 3997)
    {
      gprintf (stdout, "  addition will exceed ICNUMA array size\n");
      goto L8;
    }
  ilast = i;
  for (i = 2; i <= nclone + 1; ++i)
    {
      if (strncmp(namea + ((i - 1) << 3), name, 8) == 0)
	{
	  seek_1 (i) ;
	  fread (tname, sizeof(char), 8, unit_1);
	  fread (&nband, sizeof(long int), 1, unit_1) ;
	  fread (&lband, sizeof(long int), 1, unit_1) ;
	  fread (&igel, sizeof(long int), 1, unit_1) ;
	  fread (&icont2, sizeof(long int), 1, unit_1) ;

	  if (icont2 == icont)
	    {
	      clearvt640_() ;
	      totplt_(&ixs, &iscale, &ncomp1, icnum1, &i, &ncomp2,
		      icnum2, projectbase, &nclone, &imode, &ixs2);
 
	      i__1 = -i;
	      emphclone_(&i__1, &ixs, &c__380, &ncomp1, icnum1, &iscale,
			 &imode);
	      selectvt100_();
	      goto L70;
	    }
/* warn if clone is in another contig or unattached */
	  gprintf (stdout, "  change contig number from %d to %d (Y/N) ? ",
		  icont2, icont) ;
	  readinput ("", stemp, 2) ; *ans = *stemp ;
	  if (*ans != 'Y' && *ans != 'y')
	    {
	      goto L36;
	    }
	  iy = abs(icana[i - 1]) % 10000000;
/* single clone entry */
	  if (iy == 0)
	    {
	      iconta[i - 1] = icont;
	      seek_1 (i) ;
	      fwrite (tname, sizeof(char), 8, unit_1);
	      fwrite (&nband, sizeof(long int), 1, unit_1) ;
	      fwrite (&lband, sizeof(long int), 1, unit_1) ;
	      fwrite (&igel, sizeof(long int), 1, unit_1) ;
	      fwrite (&icont, sizeof(long int), 1, unit_1) ;
	    }
	  else
	    {
/* group entry */
	      if (iy > 1)
		{
		  ican = iy;
		}
/* first deal with canonical */
	      if (iy == 1)
		{
		  ican = i;
		}
	      iconta[ican - 1] = icont;

	      seek_1 (ican) ;
	      fread (tname, sizeof(char), 8, unit_1);
	      fread (&nband, sizeof(long int), 1, unit_1) ;
	      fread (&lband, sizeof(long int), 1, unit_1) ;
	      fread (&igel, sizeof(long int), 1, unit_1) ;
	      fread (&itemp, sizeof(long int), 1, unit_1) ;
	      
	      seek_1 (ican) ;
	      fwrite (tname, sizeof(char), 8, unit_1);
	      fwrite (&nband, sizeof(long int), 1, unit_1) ;
	      fwrite (&lband, sizeof(long int), 1, unit_1) ;
	      fwrite (&igel, sizeof(long int), 1, unit_1) ;
	      fwrite (&icont, sizeof(long int), 1, unit_1) ;
/* now deal with buried clones */
	      for (ix = 2; ix <= nclone + 1; ++ix)
		{
		  if (abs(icana[ix - 1]) % 10000000 == ican)
		    {
		      iconta[ix - 1] = icont;
		      seek_1 (ix) ;
		      fread (tname, sizeof(char), 8, unit_1);
		      fread (&nband, sizeof(long int), 1, unit_1) ;
		      fread (&lband, sizeof(long int), 1, unit_1) ;
		      fread (&igel, sizeof(long int), 1, unit_1) ;
		      fread (&itemp, sizeof(long int), 1, unit_1) ;
		      
		      seek_1 (ix) ;
		      fwrite (tname, sizeof(char), 8, unit_1);
		      fwrite (&nband, sizeof(long int), 1, unit_1) ;
		      fwrite (&lband, sizeof(long int), 1, unit_1) ;
		      fwrite (&igel, sizeof(long int), 1, unit_1) ;
		      fwrite (&icont, sizeof(long int), 1, unit_1) ;
		    }
		}
	    }
	  fwrt_(unit_4);
	  fwrt_(unit_1);
/* has a contig disappeared? */
	  contloss_(&icont2, iconta, &nclone);
	  asscont4_(&nclone, &iconto, &ncomp2, icnum2, &itemp);
	  asscont4_(&nclone, &icont, &ncomp1, icnum1, &iposav);
	  goto L7;
	}
L36:
      ;
    }
  i = ilast;
  goto L35;
L65:
  asscont4_(&nclone, &icont, &ncomp1, icnum1, &iposav);
L40:
  totplt_(&ixs, &iscale, &ncomp1, icnum1, &i, &ncomp2, icnum2,
	  projectbase, &nclone, &imode, &ixs2);
  selectvt100_() ;
  clearvt100_();
  movevt100_(0, 0) ;

  clim_(ch__5, &i);
  strtcpy (ntemp1, namea + ((i - 1) << 3), 8) ;
  gprintf (stdout, "\n%4d-%-8s(%3db,%4d) %c%c%c"
	  "                                           %s\n",
	   igela[i - 1], ntemp1, nbanda[i - 1], iconta[i - 1],
	   cflag_(i, 1, "p"), cflag_(i, 8, "h"), *ch__5, projectbase) ;
  gprintf (stdout, "\n") ;
  goto L35;
/* communication with auto mode */
L79:
  auto_(&aut, &ianum, &iamat, &ictrl, &nclone);
  if (! aut)
    {
      goto L8;
    }
  if (ictrl == 3 || iconta[ianum - 1] != 0)
    {
      i = ianum;
      if (iconta[ianum - 1] == icont)
	{
	  asscont4_(&nclone, &icont, &ncomp1, icnum1, &iposav);
	}
    }
  else
    {
      if (iamat == 0) 
	{
	  goto L8;
	}
      i = iamat;
    }
  seek_1 (i) ;
  fread (tname, sizeof(char), 8, unit_1) ;
  fread (&nband, sizeof(long int), 1, unit_1) ;
  fread (&lband, sizeof(long int), 1, unit_1) ;
  fread (&igel, sizeof(long int), 1, unit_1) ;
  fread (&icontt, sizeof(long int), 1, unit_1) ;
  switch (is)
    {
    case 1:  goto L37;
    case 2:  goto L12;
    }
L13:
  if (ictrl == 3)
    {
      gprintf (stdout, "can't exit during trial\n") ;
      goto L8;
    }

  contloss_(&icont, iconta, &nclone);
  fclose (unit_1) ;
  fclose (unit_3) ;
  fclose (unit_4) ;
  fclose (unit_10) ;
  fclose (unit_11) ;
  fclose (unit_12) ;
  fclose (unit_13) ;
  fclose (unit_16) ;  unlink ("scratch.dat") ;
  graphFinish () ;

  return ;
} /* main */

#undef tname1b
#undef tnameb
#undef tname1
#undef tname
#undef names
#undef name1


int totplt_(long int *ixs, long int *iscale, long int *ncomp1, 
	    long int *icnum1, long int *i, long int *ncomp2,
	    long int *icnum2, char *projectbase, long int *nclone,
	    long int *imode, long int *ixs2)
{
  static long int itemp;

  initvt640_();
/* plot current contig above */
  selectvt640_();
  plotcont4_(ixs, &c__380, iscale, ncomp1, icnum1,
	     projectbase, nclone, imode);
/* plot old contig below */
  emphclone_(i, ixs, &c__380, ncomp1, icnum1, iscale, imode);
  if (*imode / 1000 % 10 == 1)
    {
      plotcont4_(ixs2, &c__60, iscale, ncomp2, icnum2,
		 projectbase, nclone, imode);
    }
  else
    {
      itemp = *imode / 100 % 10;
      if (itemp < 2 || itemp > 3)
	{
	  plotcont5_(ixs, &c__15, &c__100, &c__924, &c__500,
		     iscale, ncomp1, icnum1);
	}
    }
  selectvt100_();
  return 0;
} /* totplt_ */

int remchng_(long int *iclone, long int *isch)
{
  /* Local variables */
  static long int ican;
  static char gene[9], gene1[9], remark[41], remark1[41];
  static long int ilen;
  static float pchr;
  static long int ipos;
  static short int hflag;
  static long int itemp;
  static short int hsitu;
  static long int iz;
  static short int hchrom;
  static short int hchrom1;

  if (irema[*iclone - 1] == 0)
    {
      gprintf (stdout, " clone has no remark\n") ;
      goto L100;
    }
  else
    {
      gprintf (stdout, " clone already has remark\n") ;
      goto L101;
    }
/* new remark */
L100:
  gprintf (stdout, "   gene name = ") ;
  readinput ("", remark1, 8) ;
  strtcpy(gene1, remark1, 8);
  hchrom1 = 0;
  if (strlength(gene1, 8) > 0)
    {				/* if gene1 not empty */
L1075:
      gprintf (stdout, "   chromosome (1-6) (-ve deletes) = ") ;
      readinput ("", stemp, 5) ; if (nonint(stemp)) goto L1075 ;
      hchrom1 = a2int (stemp) ;
      if (hchrom1 > 6)
	{
	  goto L1075;
	}
      if (hchrom1 < 0)
	{
	  hchrom1 = 0;
	}
    }
  gprintf (stdout, "      remark = ") ;
  readinput ("", remark1, 40) ;
/* regardless of remark1 status, call secondary routine */
  hflag = 0;
  rem2_(&hflag, iclone, isch);
  if (strlength(gene1, 8) == 0 && strlength(remark1, 40) == 0 && hflag == 0)
    {				/* if everything empty */
      return 0;
    }
  fitrem_(&iz, "i");
  irema[*iclone - 1] = iz;
  seek_4 (*iclone) ;
  fread (&ipos, sizeof(long int), 1, unit_4) ;
  fread (&ilen, sizeof(long int), 1, unit_4) ;
  fread (&ican, sizeof(long int), 1, unit_4) ;
  fread (&itemp, sizeof(long int), 1, unit_4) ;

  seek_4 (*iclone) ;
  fwrite (&ipos, sizeof(long int), 1, unit_4) ;
  fwrite (&ilen, sizeof(long int), 1, unit_4) ;
  fwrite (&ican, sizeof(long int), 1, unit_4) ;
  fwrite (&iz, sizeof(long int), 1, unit_4) ;
  fwrt_(unit_4);

  seek_11 (iz) ;
  fwrite (&(*iclone), sizeof(long int), 1, unit_11) ;
  fwrite (gene1, sizeof(char), 8, unit_11) ;
  fwrite (remark1, sizeof(char), 40, unit_11) ;
  fwrite (&hchrom1, sizeof(short int), 1, unit_11) ;
  fwrite (&hflag, sizeof(short int), 1, unit_11) ;
  fwrt_(unit_11);
  return 0;
/* existing remark */
L101:
  seek_11 (irema[*iclone - 1]) ;
  fread (&itemp, sizeof(long int), 1, unit_11) ;
  fread (gene, sizeof(char), 8, unit_11) ;
  fread (remark, sizeof(char), 40, unit_11) ;
  fread (&hchrom, sizeof(short int), 1, unit_11) ;
  fread (&hflag, sizeof(short int), 1, unit_11) ;
  fread (&pchr, sizeof(float), 1, unit_11) ;
  fread (&hsitu, sizeof(short int), 1, unit_11) ;
  
  strtcpy (ntemp1, namea + ((*iclone - 1) << 3), 8) ;
  strtcpy (ntemp2, chrom + hchrom * 5, 5) ;
  gprintf (stdout, " %-8s %-8s%5s %-40s\n", ntemp1, gene, ntemp2, remark);
  gprintf (stdout, "   gene name = ") ;
  readinput (gene, remark1, 8) ;
  strtcpy(gene1, remark1, 8);
  hchrom1 = hchrom;
  if (strncmp(gene1, gene, 8) != 0 && strlength(gene1, 8) > 0)
    {	/* if gene1 != gene and gene1 not empty */
L1073:
      gprintf (stdout, "   chromosome (1-6) (-ve deletes) = ") ;
      readinput ("", stemp, 5) ; if (nonint(stemp)) goto L1073 ;
      hchrom1 = a2int (stemp) ;
      if (hchrom1 > 6)
	{
	  goto L1073;
	}
      if (hchrom1 == 0)
	{
	  hchrom1 = hchrom;
	}
      if (hchrom1 < 0)
	{
	  hchrom1 = 0;
	}
    }
  gprintf (stdout, "      remark = ") ;
  readinput (remark, remark1, 40) ;
  rem2_(&hflag, iclone, isch);
/* both gene and all remarks were deleted, so eliminate entry */
  if (strlength(gene1, 8) == 0 && strlength(remark1, 40) == 0 && hflag == 0)
    { /* if everything empty */
      delrem_(&irema[*iclone - 1]);
      irema[*iclone - 1] = 0;

      seek_4 (*iclone) ;
      fread (&ipos, sizeof(long int), 1, unit_4) ;
      fread (&ilen, sizeof(long int), 1, unit_4) ;
      fread (&ican, sizeof(long int), 1, unit_4) ;
      fread (&itemp, sizeof(long int), 1, unit_4) ;

      seek_4 (*iclone) ;
      fwrite (&ipos, sizeof(long int), 1, unit_4) ;
      fwrite (&ilen, sizeof(long int), 1, unit_4) ;
      fwrite (&ican, sizeof(long int), 1, unit_4) ;
      fwrite (&c__0, sizeof(long int), 1, unit_4) ;
      fwrt_(unit_4);
    }
  else
    {
/* update entry */
      seek_11 (irema[*iclone - 1]) ;
      fwrite (&(*iclone), sizeof(long int), 1, unit_11) ;
      fwrite (gene1, sizeof(char), 8, unit_11) ;
      fwrite (remark1, sizeof(char), 40, unit_11) ;
      fwrite (&hchrom1, sizeof(short int), 1, unit_11) ;
      fwrite (&hflag, sizeof(short int), 1, unit_11) ;
      fwrite (&pchr, sizeof(float), 1, unit_11) ;
      fwrite (&hsitu, sizeof(short int), 1, unit_11) ;
      fwrt_(unit_11);
    }
    return 0;
} /* remchng_ */

/* old system */
int delrem_(long int *iz)
{
  char blnk = 0 ;
  
  seek_11 (*iz) ;
  fwrite (&c__0, sizeof(long int), 1, unit_11) ;
  fwrite (&blnk, sizeof (char), 8, unit_11) ;
  fwrite (&blnk, sizeof(char), 40, unit_11) ;
  fwrt_(unit_11);
  return 0;
} /* delrem_ */

int fitrem_(long int *iz, char *ih)
{
  /* Initialized data */
  char blnk = 0 ;
  
  /* Local variables */
  static long int iclone, itemp ;
  
  /* allow for long int *2/+4 spec */
  *iz = 0;
 L60:
  ++(*iz);
  seek_11 (*iz) ;
  fread (&iclone, sizeof(long int), 1, unit_11);
  if (iclone == 0) goto L61;
  goto L60;
L61:
  seek_11 (*iz + 1) ;
  fread (&itemp, sizeof(long int), 1, unit_11);
/* end of file, so enlarge */
  if (itemp == -9)
    {
      seek_11 (*iz + 1) ;
      fwrite (&c__0, sizeof(long int), 1, unit_11);
      fwrite (&blnk, sizeof(char), 8, unit_11) ;
      fwrite (&blnk, sizeof(char), 40, unit_11) ;

      seek_11 (*iz + 2) ;
      fwrite (&itemp, sizeof(long int), 1, unit_11);
      fwrite (&blnk, sizeof(char), 8, unit_11) ;
      fwrite (&blnk, sizeof(char), 40, unit_11) ;
      fwrt_(unit_11);
    }
    return 0;
} /* fitrem_ */

/* overwrites remark with last in file, adjusts .nam or .rem */
/* accordingly, resets end of file records and pointer */
int delrem1_(long int *iz)
{
  char blnk = 0;
  
  /* System generated locals */
  long int i__1;
  
  /* Local variables */
  static long int ican;
  static char gene[9];
  static long int ilen;
  static float pchr;
  static long int irem, ipos;
  static char gene1[9];
  static float pchr1;
  static short int hflag;
  static long int itemp, i1;
  static short int hsitu, hflag1, hflag2, hsitu1;
  static long int ip, iclone;
  static short int hchrom;
  static char remark[41];
  static long int iclone1;
  static short int hchrom1;
  static char remark1[41];
  
  seek_11 (1) ;
  fread (&ip, sizeof(long int), 1, unit_11);
  if (ip < *iz)
    {
      gprintf (stdout, "  delrem error 1\n");
      return 0;
    }
  seek_11 (ip) ;
  fread (&iclone, sizeof(long int), 1, unit_11);
  fread (gene, sizeof(char), 8, unit_11);
  fread (remark, sizeof(char), 40, unit_11);
  fread (&hchrom, sizeof(short int), 1, unit_11);
  fread (&hflag, sizeof(short int), 1, unit_11);
  fread (&pchr, sizeof(float), 1, unit_11);
  fread (&hsitu, sizeof(short int), 1, unit_11);

  irem = irema[iclone - 1];
/* last remark is 1ary; alter .nam */
  if (irem == ip)
    {
      irema[iclone - 1] = *iz;
      seek_4 (iclone) ;
      fread (&ipos, sizeof(long int), 1, unit_4) ;
      fread (&ilen, sizeof(long int), 1, unit_4) ;
      fread (&ican, sizeof(long int), 1, unit_4) ;
      fread (&itemp, sizeof(long int), 1, unit_4) ;
      
      seek_4 (iclone) ;
      fwrite (&ipos, sizeof(long int), 1, unit_4) ;
      fwrite (&ilen, sizeof(long int), 1, unit_4) ;
      fwrite (&ican, sizeof(long int), 1, unit_4) ;
      fwrite (&(*iz), sizeof(long int), 1, unit_4) ;
      fwrt_(unit_4);
    }
  else
    {
      i1 = irem;
      seek_11 (i1) ;
      fread (&iclone1, sizeof(long int), 1, unit_11);
      fread (gene1, sizeof(char), 8, unit_11);
      fread (remark1, sizeof(char), 40, unit_11);
      fread (&hchrom1, sizeof(short int), 1, unit_11);
      fread (&hflag2, sizeof(short int), 1, unit_11);
      fread (&pchr1, sizeof(float), 1, unit_11);
      fread (&hsitu1, sizeof(short int), 1, unit_11);
      while(hflag2 != ip)
	{
	  if (hflag2 == 0)
	    {
	      gprintf (stdout, "  delrem error 2\n");
	      return 0;
	    }
	  i1 = hflag2;
	  seek_11 (i1) ;
	  fread (&iclone1, sizeof(long int), 1, unit_11);
	  fread (gene1, sizeof(char), 8, unit_11);
	  fread (remark1, sizeof(char), 40, unit_11);
	  fread (&hchrom1, sizeof(short int), 1, unit_11);
	  fread (&hflag2, sizeof(short int), 1, unit_11);
	  fread (&pchr1, sizeof(float), 1, unit_11);
	  fread (&hsitu1, sizeof(short int), 1, unit_11);
	}
      hflag1 = *iz;
      seek_11 (i1) ;
      fwrite (&iclone1, sizeof(long int), 1, unit_11);
      fwrite (gene1, sizeof(char), 8, unit_11);
      fwrite (remark1, sizeof(char), 40, unit_11);
      fwrite (&hchrom1, sizeof(short int), 1, unit_11);
      fwrite (&hflag1, sizeof(short int), 1, unit_11);
      fwrite (&pchr1, sizeof(float), 1, unit_11);
      fwrite (&hsitu1, sizeof(short int), 1, unit_11);
    }
  seek_11 (*iz) ;
  fwrite (&iclone, sizeof(long int), 1, unit_11);
  fwrite (gene, sizeof(char), 8, unit_11);
  fwrite (remark, sizeof(char), 40, unit_11);
  fwrite (&hchrom, sizeof(short int), 1, unit_11);
  fwrite (&hflag, sizeof(short int), 1, unit_11);
  fwrite (&pchr, sizeof(float), 1, unit_11);
  fwrite (&hsitu, sizeof(short int), 1, unit_11);

  seek_11 (ip) ;
  fwrite (&c__0, sizeof(long int), 1, unit_11);
  fwrite (&blnk, sizeof(char), 8, unit_11) ;
  fwrite (&blnk, sizeof(char), 40, unit_11) ;

  seek_11 (ip + 1) ;
  fwrite (&c_n9, sizeof(long int), 1, unit_11);
  fwrite (&blnk, sizeof(char), 8, unit_11) ;
  fwrite (&blnk, sizeof(char), 40, unit_11) ;

  i__1 = ip - 1 ;
  seek_11 (1) ;
  fwrite (&i__1, sizeof(long int), 1, unit_11);
  fwrite (&blnk, sizeof(char), 8, unit_11) ;
  fwrite (&blnk, sizeof(char), 40, unit_11) ;

  fwrt_(unit_11);
  return 0;
} /* delrem1_ */

int fitrem1_(long int *iz, char *ih)
{
  /* Initialized data */
  char blnk = 0 ;
  
  /* Local variables */
  static long int ip;
  
  /* allow for long int *2/+4 spec */
  seek_11 (1) ;
  fread (&ip, sizeof(long int), 1, unit_11);
  *iz = ip + 1;


  seek_11 (*iz + 1) ;
  fwrite (&c__0, sizeof(long int), 1, unit_11);
  fwrite (&blnk, sizeof(char), 8, unit_11) ;
  fwrite (&blnk, sizeof(char), 40, unit_11) ;
  
  seek_11 (*iz + 2) ;
  fwrite (&c_n9, sizeof(long int), 1, unit_11);
  fwrite (&blnk, sizeof(char), 8, unit_11) ;
  fwrite (&blnk, sizeof(char), 40, unit_11) ;

  seek_11 (1) ;
  fwrite (&(*iz), sizeof(long int), 1, unit_11);
  fwrite (&blnk, sizeof(char), 8, unit_11) ;
  fwrite (&blnk, sizeof(char), 40, unit_11) ;

  return 0;
} /* fitrem1_ */

/* deals with secondary remark stack */
/* i2=hflag2 is record number of remark being edited */
/* i1=hflag1 is record number of next higher remark */
/* i3=hflag3 is record number of next deeper remark */
int rem2_(short int *hflage, long int *iclonee, long int *isch)
{
  /* Initialized data */
  short int hblnk = 0 ;
  char blnk = 0 ;
  
  /* Local variables */
  static char gene[9];
  static float pchr, temp;
  static char trem[41];
  static short int htmp1, htmp2, htmp3;
  static char tname[9];
  static long int itemp, i1, i2;
  static short int hsitu, hflag1, hflag2, hflag3;
  static long int iclone, idepth;
  static short int hchrom;
  static char remark[41];
  
  iclone = *iclonee;
  idepth = 2;
/* editing existing secondary remarks */
  if (*hflage != 0)
    {
      hflag2 = *hflage;
      i2 = hflag2;
L101:
      seek_11 (i2) ;
      fread (&iclone, sizeof(long int), 1, unit_11);
      fread (gene, sizeof(char), 8, unit_11);
      fread (remark, sizeof(char), 40, unit_11);
      fread (&hchrom, sizeof(short int), 1, unit_11);
      fread (&hflag3, sizeof(short int), 1, unit_11);
      fread (&pchr, sizeof(float), 1, unit_11);
      fread (&hsitu, sizeof(short int), 1, unit_11);
      
      gprintf (stdout, "    remark%d = ", idepth) ;
L102:
      readinput (remark, remark, 40) ;
/* cut out this record from stack */
      if (strlength(remark, 40) == 0)
	{
	  delrem_(&i2);
	  if (idepth == 2)
	    {
	      *hflage = hflag3;
	    }
	  else
	    {
	      seek_11 (i1) ;
	      fread (&itemp, sizeof(long int), 1, unit_11);
	      fread (tname, sizeof(char), 8, unit_11);
	      fread (trem, sizeof(char), 40, unit_11);
	      fread (&htmp1, sizeof(short int), 1, unit_11);
	      fread (&htmp2, sizeof(short int), 1, unit_11);
	      fread (&temp, sizeof(float), 1, unit_11);
	      fread (&htmp3, sizeof(short int), 1, unit_11);
	      
	      seek_11 (i1) ;
	      fwrite (&itemp, sizeof(long int), 1, unit_11);
	      fwrite (tname, sizeof(char), 8, unit_11);
	      fwrite (trem, sizeof(char), 40, unit_11);
	      fwrite (&htmp1, sizeof(short int), 1, unit_11);
	      fwrite (&hflag3, sizeof(short int), 1, unit_11);
	      fwrite (&temp, sizeof(float), 1, unit_11);
	      fwrite (&htmp3, sizeof(short int), 1, unit_11);
	      fwrt_(unit_11);
	    }
	  if (hflag3 == 0)
	    {
	      return 0;
	    }
	  hflag2 = hflag3;
	  i2 = hflag2;
	  seek_11 (i2) ;
	  fread (&iclone, sizeof(long int), 1, unit_11);
	  fread (gene, sizeof(char), 8, unit_11);
	  fread (remark, sizeof(char), 40, unit_11);
	  fread (&hchrom, sizeof(short int), 1, unit_11);
	  fread (&hflag3, sizeof(short int), 1, unit_11);
	  fread (&pchr, sizeof(float), 1, unit_11);
	  fread (&hsitu, sizeof(short int), 1, unit_11);
	  goto L102;
	} 
      else
	{
	  seek_11 (i2) ;
	  fwrite (&iclone, sizeof(long int), 1, unit_11);
	  fwrite (gene, sizeof(char), 8, unit_11);
	  fwrite (remark, sizeof(char), 40, unit_11);
	  fwrite (&hchrom, sizeof(short int), 1, unit_11);
	  fwrite (&hflag3, sizeof(short int), 1, unit_11);
	  fwrite (&pchr, sizeof(float), 1, unit_11);
	  fwrite (&hsitu, sizeof(short int), 1, unit_11);
      	  fwrt_(unit_11);
	
	  ++idepth;
	  hflag1 = hflag2;
	  i1 = hflag1;
	  if (hflag3 != 0)
	    {
	      hflag2 = hflag3;
	      i2 = hflag2;
	      goto L101;
	    }
	  else
	    {
	      goto L103;
	    }
	}
    }
  else
    {
      goto L103;
    }
/* generating new secondary remarks */
    return 0;
L103:
  strcpy(remark, "");
  gprintf (stdout, "    remark%d = ", idepth) ;
  readinput (remark, remark, 40) ;
  if (strlength (remark, 40) > 0)
    {				/* if remark not empty */
      fitrem_(&i2, "h");
      hflag2 = i2;
      seek_11 (i2) ;
      fwrite (&(*iclonee), sizeof(long int), 1, unit_11);
      fwrite (&blnk, sizeof(char), 8, unit_11);
      fwrite (remark, sizeof(char), 40, unit_11);
      fwrite (&hblnk, sizeof(short int), 1, unit_11);
      fwrite (&hblnk, sizeof(short int), 1, unit_11);
      
      if (idepth > 2)
	{
	  seek_11 (i1) ;
	  fread (&itemp, sizeof(long int), 1, unit_11);
	  fread (tname, sizeof(char), 8, unit_11);
	  fread (trem, sizeof(char), 40, unit_11);
	  fread (&htmp1, sizeof(short int), 1, unit_11);
	  fread (&htmp2, sizeof(short int), 1, unit_11);
	  fread (&temp, sizeof(float), 1, unit_11);
	  fread (&htmp3, sizeof(short int), 1, unit_11);
	  
	  seek_11 (i1) ;
	  fwrite (&itemp, sizeof(long int), 1, unit_11);
	  fwrite (tname, sizeof(char), 8, unit_11);
	  fwrite (trem, sizeof(char), 40, unit_11);
	  fwrite (&htmp1, sizeof(short int), 1, unit_11);
	  fwrite (&hflag2, sizeof(short int), 1, unit_11);
	  fwrite (&temp, sizeof(float), 1, unit_11);
	  fwrite (&htmp3, sizeof(short int), 1, unit_11);
	}
      else
	{
	  *hflage = hflag2;
	}
      fwrt_(unit_11);
      ++idepth;
      hflag1 = hflag2;
      i1 = hflag1;
      goto L103;
    }
    return 0;
} /* rem2_ */

int pmode_(long int *imode)
{
  static long int itemp1, itemp2, itemp3;
  
L3:
  gprintf (stdout, "   contigs:-\n"
	  "        0    single\n"
	  "        1    double\n") ;
L31:
  gprintf (stdout, "   enter choice : ") ;
  readinput ("", stemp, 2); if (nonint(stemp)) goto L31 ;
  itemp3 = a2int (stemp) ;
  if (itemp3 < 0 || itemp3 > 1)
    {
      goto L3;
    }
L1:
  gprintf (stdout, "   clone names:-\n"
	  "        0    all canonicals\n"
	  "        1    only gridded cosmids\n"
	  "        2    only YACs\n"
	  "        3    only gridded YACs\n") ;
L11:
  gprintf (stdout, "   enter choice : ") ;
  readinput ("", stemp, 2) ; if (nonint(stemp)) goto L11 ;
  itemp1 = a2int (stemp) ;
  if (itemp1 < 0 || itemp1 > 3)
    {
      goto L1;
    }
L2:
  gprintf (stdout, "   remarks:-\n"
	  "        0    only first level\n"
	  "        1    none\n"
	  "        2    all; single contig display\n") ;
L21:
  gprintf (stdout, "   enter choice : ") ;
  readinput ("", stemp, 2); if (nonint(stemp)) goto L21 ;
  itemp2 = a2int (stemp) ;
  if (itemp2 < 0 || itemp2 > 2)
    {
      goto L2;
    }
  *imode = itemp3 * 1000 + itemp1 * 100 + itemp2 * 10 + 1;
  return 0;
} /* pmode_ */

/* inverts whole or part of contig */
int invert_(long int *ncomp, long int *icnuma, long int *ipcx, 
	long int *iscale, long int *isch, long int *ixs)
{
  /* Local variables */
  static long int ican, ilen, irem, ilft, ipos;
  static long int ichar;
  static long int itemp, iposl, iposn, iposr, ix, iy, iposrn;
  static long int isplit1, isplit2;
  static char ans[1];
  static long int ipl, ipr, irt;
  
  /* Parameter adjustments */
  --icnuma;
  
  gprintf (stdout, "   invert whole contig (W) or just a section (S)? ") ;
  readinput ("", stemp, 2) ; *ans = *stemp ;
  if (*ans == 'w' || *ans == 'W')
    {
      for (ix = 1; ix <= *ncomp; ++ix)
	{
	  iy = icnuma[ix];
	  iposa[iy - 1] = -iposa[iy - 1] - ilena[iy - 1] - *ixs;
	  seek_4 (iy) ;
	  fread (&itemp, sizeof(long int), 1, unit_4);
	  fread (&ilen, sizeof(long int), 1, unit_4);
	  fread (&ican, sizeof(long int), 1, unit_4);
	  fread (&irem, sizeof(long int), 1, unit_4);

	  seek_4 (iy) ;
	  fwrite (&iposa[iy - 1], sizeof(long int), 1, unit_4);
	  fwrite (&ilen, sizeof(long int), 1, unit_4);
	  fwrite (&ican, sizeof(long int), 1, unit_4);
	  fwrite (&irem, sizeof(long int), 1, unit_4);
	}
      fwrt_(unit_4);
      *ixs = 0;
    }
/*  define section */
  else if (*ans == 's' || *ans == 'S')
    {
      gprintf (stdout, "  position cursor at first split point, defined by\n"
	      "  centres of clones; then hit SPACE\n") ;
      cursor_(&ichar, ipcx, iscale, isch);
      if (ichar != SPACE_KEY)
	{
	  return 0;
	}
      getsplt_(ipcx, ncomp, &icnuma[1], ixs, &isplit1);
      clearvt100_();
      gprintf (stdout, "  position cursor at second split point, defined by\n"
	      "  centres of clones; then hit SPACE\n") ;
      cursor_(&ichar, ipcx, iscale, isch);
      if (ichar != SPACE_KEY)
	{
	  return 0;
	}
      getsplt_(ipcx, ncomp, &icnuma[1], ixs, &isplit2);
      if (isplit1 > isplit2)
	{
	  exchg_(&isplit1, &isplit2);
	}
      if (isplit2 < 1 || isplit1 > *ncomp - 1)
	{
	  return 0;
	}
      ilft = 1000000;
      irt = -1000000;
/* find true left and right ends of normal clone group */
      for (ix = isplit1 + 1; ix <= isplit2; ++ix)
	{
	  iy = icnuma[ix];
	  if (nbanda[iy - 1] > 0)
	    {
	      ilft = min(iposa[iy - 1],ilft);
	      irt = max(iposa[iy - 1] + ilena[iy - 1],irt);
	    }
	}
/* invert normal clones */
      for (ix = isplit1 + 1; ix <= isplit2; ++ix)
	{
	  iy = icnuma[ix];
	  if (nbanda[iy - 1] > 0)
	    {
	      ipr = iposa[iy - 1] + ilena[iy - 1] - ilft;
	      iposa[iy - 1] = irt - ipr;
	      
	      seek_4 (iy) ;
	      fread (&itemp, sizeof(long int), 1, unit_4);
	      fread (&ilen, sizeof(long int), 1, unit_4);
	      fread (&ican, sizeof(long int), 1, unit_4);
	      fread (&irem, sizeof(long int), 1, unit_4);
	      
	      seek_4 (iy) ;
	      fwrite (&iposa[iy - 1], sizeof(long int), 1, unit_4);
	      fwrite (&ilen, sizeof(long int), 1, unit_4);
	      fwrite (&ican, sizeof(long int), 1, unit_4);
	      fwrite (&irem, sizeof(long int), 1, unit_4);
	    }
	}
/* adjust virtuals */
      for (ix = 1; ix <= *ncomp; ++ix)
	{
	  iy = icnuma[ix];
	  if (nbanda[iy - 1] == 0)
	    {
/* consider possible virtual configurations */
	      if (iflag_(iy, 4) == 1)
		{
		  iposl = iposa[iy - 1];
		  iposr = iposl + ilena[iy - 1];
/* wholly within group */
		  if (iposl >= ilft && iposl <= irt && iposr >= ilft && 
		      iposr <= irt)
		    {
		      ipr = iposr - ilft;
		      iposa[iy - 1] = irt - ipr;
		      seek_4 (iy) ;
		      fread (&itemp, sizeof(long int), 1, unit_4);
		      fread (&ilen, sizeof(long int), 1, unit_4);
		      fread (&ican, sizeof(long int), 1, unit_4);
		      fread (&irem, sizeof(long int), 1, unit_4);
		      
		      seek_4 (iy) ;
		      fwrite (&iposa[iy - 1], sizeof(long int), 1, unit_4);
		      fwrite (&ilen, sizeof(long int), 1, unit_4);
		      fwrite (&ican, sizeof(long int), 1, unit_4);
		      fwrite (&irem, sizeof(long int), 1, unit_4);
		    }
/* left end in group */
		  else if (iposl >= ilft && iposl <= irt)
		    {
		      ipl = iposl - ilft;
		      iposn = irt - ipl;
		      iposa[iy - 1] = iposn;
		      ilena[iy - 1] = ilena[iy - 1] + iposl - 
			iposn;

		      seek_4 (iy) ;
		      fread (&itemp, sizeof(long int), 1, unit_4);
		      fread (&itemp, sizeof(long int), 1, unit_4);
		      fread (&ican, sizeof(long int), 1, unit_4);
		      fread (&irem, sizeof(long int), 1, unit_4);
		      
		      seek_4 (iy) ;
		      fwrite (&iposa[iy - 1], sizeof(long int), 1, unit_4);
		      fwrite (&ilena[iy - 1], sizeof(long int), 1, unit_4);
		      fwrite (&ican, sizeof(long int), 1, unit_4);
		      fwrite (&irem, sizeof(long int), 1, unit_4);
		    }
/* right end in group */
		  else if (iposr >= ilft && iposr <= irt)
		    {
		      ipr = iposr - ilft;
		      iposrn = irt - ipr;
		      ilena[iy - 1] = ilena[iy - 1] + iposrn 
			- iposr;
		      seek_4 (iy) ;
		      fread (&ipos, sizeof(long int), 1, unit_4);
		      fread (&itemp, sizeof(long int), 1, unit_4);
		      fread (&ican, sizeof(long int), 1, unit_4);
		      fread (&irem, sizeof(long int), 1, unit_4);
		      
		      seek_4 (iy) ;
		      fwrite (&ipos, sizeof(long int), 1, unit_4);
		      fwrite (&ilena[iy - 1], sizeof(long int), 1, unit_4);
		      fwrite (&ican, sizeof(long int), 1, unit_4);
		      fwrite (&irem, sizeof(long int), 1, unit_4);
		    }
		}
	    }
	}
      fwrt_(unit_4);
    }
  return 0;
} /* invert_ */
