/*  File: extend.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: merged from listend.c, huntend2.c,
 finds overlaps of contig-ends
 * SCCS: $Id: extend.c,v 1.1 2001/02/14 12:44:37 edgrif Exp $
 * HISTORY:
 * Last edited: Mar 17 14:27 1995 (fw)
 * Created: Thu May 19 15:44:59 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;			/* .nam */
extern FILE *unit_2 ;			/* .cor */
extern FILE *unit_4 ;			/* .cnt */
extern FILE *unit_10 ;			/* .stk */
extern FILE *unit_16 ;			/* here: tmp-file in /tmp */

/* Common Block Declarations */
/* some of them aren't needed in here */

/* csec CB */
short int coorda[BANDMAX];

/* cint CB */
long int nbanda[30000], lbanda[30000], igela[30000], iconta[30000],
         iposa[30000], ilena[30000], icana[30000], irema[30000];

/* chars CB */
char namea[240000], chrom[65];

/* matdat CB */
long int itolmn, itoldf;
float logfact[201];

/* chr1 CB */
float xzero1, scale1;
long int ichrgp, maxlev1, levgap1;
short int genlev1[6000]	/* was [6][1000] */;

/* cass CB */
long int gpos[1000], glen[1000], gmat[1000], gclst[1000], ngrp;

/* BLNK CB */
long int isch, icurs, ibtot, iwend;

BOOL listend (char*, long int) ;
BOOL huntend2 (char*, char*, long int, FILE *, char *) ;
int asscont1_(long int *, long int *, long int *, 
	      long int *, long int *, long int *);

/* EXTEND, combines listend and huntend2 in one program */

void main (int argc, char **argv)
{
  char projectbase[128], end_temp[128], out_file[128] ;
  FILE *filout ;
  int size_of_cor, ix ;
  long int nclone, nbands ;

/* check, if output file specified */
  if (argc == 1)
    {
      filout = stdout ;
      *out_file = '\0' ;
    }
  else if (argc == 2)
    {
      strcpy (out_file, argv[1]) ;
      if (!(filout = FILopen (out_file, "", "w")))
	goto ERROR ;
    }

/* get temp filename for END.DAT file (unit_16) */
  strcpy (end_temp, tempnam("/tmp", "EXTEND")) ;

/* open files */
/* get name of project */
  fprintf (stdout, "   project name = ") ;
  readinput ("", projectbase, 0) ;	/* get project name */
  if (strlength(projectbase, 128) == 0) { printf ("--STOP-- (no project)\n") ; exit (1) ;}

  gprintf (stdout, "%s\n", projectbase) ; /* output to screen (extend.out) */

  if (!(unit_1 = FILopen (projectbase, "nam", "r"))) goto ERROR ;
  seek_1 (1) ;
  fread (&nclone, sizeof(long int), 1, unit_1) ;
  fread (&nbands, sizeof(long int), 1, unit_1) ;
  printf ("number of clones = %ld,"
	  " number of bands = %ld\n", nclone, nbands) ;

  if (!(unit_2 = FILopen (projectbase, "cor", "r"))) goto ERROR ;
/* determine filesize of *.cor file */
  fseek (unit_2, 0, SEEK_END) ;
  size_of_cor = ftell (unit_2) ;
  size_of_cor /= sizeof(short int) ;
  rewind (unit_2) ;
/* read *.cor file into coorda array */
  fread (&coorda, sizeof(short int), size_of_cor, unit_2) ;

  if (!(unit_4 = FILopen (projectbase, "cnt", "r"))) goto ERROR ;
  if (!(unit_10 = FILopen (projectbase, "stk", "r"))) goto ERROR ;

  /*   read in data to arrays */
  for (ix = 2; ix <= nclone + 1; ++ix)
    {
      seek_1 (ix) ;
      fread (namea + ((ix - 1) << 3), sizeof(char), 8, unit_1) ;
      fread (&nbanda[ix - 1], sizeof(long int), 1, unit_1) ;
      fread (&lbanda[ix - 1], sizeof(long int), 1, unit_1) ;
      fread (&igela[ix - 1], sizeof(long int), 1, unit_1) ;
      fread (&iconta[ix - 1], sizeof(long int), 1, unit_1) ;

      seek_4 (ix) ;
      fread (&iposa[ix - 1], sizeof(long int), 1, unit_4) ;
      fread (&ilena[ix - 1], sizeof(long int), 1, unit_4) ;
      fread (&icana[ix - 1], sizeof(long int), 1, unit_4) ;
      fread (&irema[ix - 1], sizeof(long int), 1, unit_4) ;
    }

  if (!listend (end_temp, nclone)) goto ERROR ;
  if (!huntend2 (projectbase, end_temp, nclone, filout, out_file))goto ERROR;
  unlink (end_temp) ;
  return ;
 ERROR:
  unlink (end_temp) ;
  gprintf (stderr, "--STOP-- (error occured)\n") ;
  exit (1) ;
}
/*   LISTEND */
/*   program to list ends of contigs and unattached clones */
BOOL listend (char *end_temp, long int nclone)
{
  /* Local variables */
  static long int ican, ilft, icrt;
  static char cname[9];
  static long int ifctc, iclft;
  static long int nicnt, itlen, icont, ncomp, ncont;
  static float sztkb;
  static long int ix, iy;
  static long int isztbd, icount, ntcomp;
  static long int irt;
  static char tname[9] ;

  printf ("*** listend ***\n") ;
  if (!(unit_16 = FILopen (end_temp, "", "w")))
    goto ERROR ;
  
  printf ("   first contig to consider = ") ;
  readinput ("", stemp, 0) ;
  ifctc = a2int (stemp) ;
  fprintf (stdout, "%ld\n", ifctc) ; /* output to screen (extend.out) */

L301:
  seek_10 (1) ;
  fread (&nicnt, sizeof(long int), 1, unit_10);
  printf ("   last clone analysed = ") ;
  readinput ("", cname, 0) ;


  if (strlength(cname, 8) == 0)
    {
      ix = nclone + 1;
      strtcpy (cname, namea + ((ix - 1) << 3), 8) ; /* use last by default */
      fprintf (stdout, "%s\n", cname) ; /* output to screen (extend.out) */
      goto L307;
    }
  else
    fprintf (stdout, "%s\n", cname) ; /* output to screen (extend.out) */

  for (ix = 2; ix <= nclone + 1; ++ix)
    {
      if (strncmp(namea + ((ix - 1) << 3), cname, 8) == 0)
	{
	  goto L307;
	}
    }
  printf ("clone not found\n") ;
    goto L301;
L307:
/*   enter last clone analysed as header */
  gprintf (unit_16, "%6ld %-8s last clone analysed\n", ix, cname) ;
/*   enter unattached clones, omitting cancellations and repeats */
  icount = 0;
  for (iy = 2; iy <= ix; ++iy)
    {
      if (iconta[iy - 1] == 0)
	{
	  strtcpy(tname, namea + ((iy - 1) << 3), 8);
	  if (tname[0] == '!' || tname[1] == '!' || tname[2] == '!' || 
	      tname[3] == '!' || tname[4] == '!' || tname[5] == '!' 
	      || tname[6] == '!' || tname[7] == '!')
	    {
	      goto L308;
	    }
	  ++icount;
	  gprintf (unit_16, "%6ld%6d  %-8s\n", iy, 0, tname) ;
	}
L308:
      ;
    }
  ncont = 0;
  ntcomp = 0;
  itlen = 0;
  isztbd = 0;
  sztkb = 0.f;
/*   loop for each contig */
  for (icont = ifctc; icont <= nicnt; ++icont)
    {
      ilft = 100000;
      irt = -100000;
      ncomp = 0;
/*   loop for each clone */
      for (iy = 2; iy <= ix; ++iy)
	{
	  if (iconta[iy - 1] == icont)
	    {
/*   omit cancellations and repeats */
	      strtcpy(tname, namea + ((iy - 1) << 3), 8L);
	      if ((tname [0] == '!' || tname[1] == '!' ||
		   tname[2] == '!' || tname[3] == '!' ||
		   tname[4] == '!' || tname[5] == '!' ||
		   tname[6] == '!' || tname[7] == '!') && 
		  icana[iy - 1] != -1)
		{
		  goto L303;
		}
	      ++ncomp;
/*   don't use hidden clones for contig dimensions (data may be wrong) */
	      if (abs(icana[iy - 1]) < 2)
		{
		  if (iposa[iy - 1] < ilft)
		    {
		      iclft = iy;
		      ilft = iposa[iy - 1];
		    }
		  if (iposa[iy - 1] + ilena[iy - 1] > irt)
		    {
		      icrt = iy;
		      irt = iposa[iy - 1] + ilena[iy - 1];
		    }
		}
	    }
L303:
	  ;
	}
      strtcpy (ntemp1, namea + ((iclft - 1) << 3), 8) ;
      gprintf (unit_16, "%6ld%6ld  %-8s\n", iclft, icont, ntemp1);
      strtcpy (ntemp1, namea + ((icrt - 1) << 3), 8) ;
      gprintf (unit_16, "%6ld%6ld  %-8s\n", icrt, icont, ntemp1);
/*   get clones in limbo at ends */
      for (iy = 2; iy <= ix; ++iy)
	{
	  if (iconta[iy - 1] == icont)
	    {
	      if (icana[iy - 1] > 10000000)
		{
		  ican = icana[iy - 1] % 10000000;
		  if (ican == iclft || ican == icrt)
		    {
		      strtcpy (ntemp1, namea + ((iy - 1) << 3), 8) ;
		      gprintf (unit_16, "%6ld%6ld  %-8s\n",
			       iy, icont, ntemp1);
		    }
		}
	    }
	}
    }
  fclose (unit_16) ;
  return TRUE ;
 ERROR:
  gprintf (stderr, "File access failed in listend routine !!!\n") ;
  return FALSE ;
}

/**********************************************************************
 **********************************************************************
 **********************************************************************
 **********************************************************************/

/*   HUNTEND */
/*   program to compare ends of contigs and unattached clones */
/*   RANKF forward prob, RANKB back prob, RANKP total prob (inc bonus) */
BOOL huntend2 (char *projectbase, char *end_temp, long int nclone,
	       FILE *filout, char *out_file)
{
  /* Local variables */
  static char tname[9];
  static long int enda[30000];
  static long int ican, bndm[30000];
  static char name[9];
  static long int ilna[100], matf[30000];
  static float rank[100];
  static long int ihnd;
  static BOOL disp;
  static long int icrt;
  static float prob;
  static long int irnk, matt[10], nrnk, itot, nout;
  static long int i, j, k, m, ifctc;
  static float rankb[30000];
  static long int ilctc, clmat[30000];
  static float rankf[30000];
  static long int iclft;
  static long int itend;
  static float bprob;
  static long int clsub[30000];
  static float tbprb, rankp[30000];
  static long int nicnt, icont;
  static float rankt[10];
  static long int ncomp, icrtm;
  static float probp;
  static long int isubm;
  static float tprob;
  static long int itemp;
  static long int lband1, nband1;
  static long int coord1[200], icnum1[4000], icnum2[4000];
  static long int ic, im, nbandd, kk, ip, iq, ix, iy;
  static float ps;
  static long int minbnd;
  static long int imatch, rkfdir[30000];
  static long int rnkdir[100];
  static long int itolsb, ncompm, rktdir[10];
  static long int iposav;
  static long int srtdir[30000], itolmt, isubtr, irnkpt;
  static long int ifm, mat[100];
  static long int imm;
  static float logfact[201];
  static long int nbd1, iposavm, iln1, iln2;

  printf ("*** huntend ***\n") ;
/* dont subtract for smaller clones */
    minbnd = 6;
/* number in probability rank */
    nrnk = 5;
/*   calc log factorials */
  logfact[0] = 0.f;
  logfact[1] = 0.f;
  for (i = 2; i <= 200; ++i)
    {
      logfact[i] = logfact[i - 1] + log((float) i);
    }

  if (!(unit_16 = FILopen (end_temp, "", "r")))
    goto ERROR ;

  printf ("   first contig to consider = ") ;
  readinput ("", stemp, 0) ; ifctc = a2int (stemp) ;
  fprintf (stdout, "%ld\n", ifctc) ; /* output to screen (extend.out) */

  seek_10 (1) ;
  fread (&nicnt, sizeof(long int), 1, unit_10);

  printf ("   last contig to consider = ") ;
  readinput ("", stemp, 0) ; ilctc = a2int (stemp) ;

  if (ilctc < 1 || ilctc > nicnt)
    {
      ilctc = nicnt;
    }
  fprintf (stdout, "%ld\n", nicnt) ; /* output to screen (extend.out) */


  printf ("   subtract tolerance (def = 7) = ") ;
  readinput ("", stemp, 0) ; itolsb = a2int (stemp) ;
  if (itolsb < 1)
    {
      itolsb = 7 ;
    }
  fprintf (stdout, "%ld\n", itolsb) ; /* output to screen (extend.out) */

  printf ("   match tolerance (def = 7) = ") ;
  readinput ("", stemp, 0) ; itolmt= a2int (stemp) ;
  if (itolmt < 1)
    {
      itolmt = 7 ;
    }
  fprintf (stdout, "%ld\n", itolmt) ; /* output to screen (extend.out) */

  ps = 1 - (float) (itolmt << 1) / 3300.f;
  printf ("   no of clones for output (def = 10) = ") ;
  readinput ("", stemp, 0) ; nout = a2int (stemp) ;
  if (nout < 1) nout = 10 ;
  fprintf (stdout, "%ld\n", nout) ; /* output to screen (extend.out) */

  disp = TRUE ;
  strcpy(namea, "");
  itend = 0;
  rewind (unit_16) ;
  fscanf (unit_16, "%ld", &ix);
  forward (unit_16) ;
  if (feof(unit_16)) goto L17 ;
L18:
  ++itend;
  if (feof(unit_16)) goto L17 ;
  fscanf (unit_16, "%ld", &enda[itend - 1]);
  forward (unit_16) ;
  if (feof(unit_16)) goto L17 ;
  goto L18;
L17:
  --itend;
/* loop for each contig */
  itot = 0;
/* assemble contig */
  for (icont = ifctc; icont <= ilctc; ++icont)
    {
      asscont1_(&nclone, &icont, &ncomp, icnum1, &iposav, &icrt);
      if (ncomp == 0)		/* EMPTY CONTIG */
	{
	  goto L302;
	}
      iclft = icnum1[0];

/* loop for each end of contig */
      for (ip = 1; ip <= 2; ++ip)
	{
	  if (ip == 1)
	    {
	      ic = iclft;
	    }
	  if (ip == 2)
	    {
	      ic = icrt;
	    }
	  if (nbanda[ic - 1] < minbnd)
	    {
	      isubtr = 1;
	      goto L21;
	    }
/* find best clone to subtract */
	  bestsubtr_(&ic, &ip, icnum1, &ncomp, &isubtr);
L21:
/*   subtract */
	  clondif_(&ic, &isubtr, &itolsb, &nbd1, coord1);
/*clear prob ranks */
	  for (iq = 1; iq <= 10; ++iq)
	    {
	      rankt[iq - 1] = 1.f;
	    }
	  for (iq = 1; iq <= 100; ++iq)
	    {
	      rank[iq - 1] = 1.f;
	    }
	  irnkpt = 0;
/*   match to each clone in enda */
	  for (iq = 1; iq <= itend; ++iq)
	    {
	      iy = enda[iq - 1];

	      if (iconta[iy - 1] == icont)
		{
		  goto L16;	/* EXCLUDE OWN CONTIG */
		}
	      iln1 = ic;
	      iln2 = iy;
	      rejpss_("qR", &iln1, &iln2, ilna, projectbase, &nclone);
	      if (iln1 == -99)
		{
		  goto L16;
		}
	      patch3_(iy, iy, nbd1, coord1, itolmt, logfact, rank, 
		      rnkdir, rankt, rktdir, 0, ps, &irnkpt, nrnk, 
		      mat, matt);
L16:
	      ;
	    }
	  ++itot;
	  clmat[itot - 1] = ic;
	  clsub[itot - 1] = isubtr;
	  bndm[itot - 1] = nbd1;
	  srtdir[itot - 1] = itot;

/* choose clone with lowest product of forward and reverse probabilities */
	  tprob = 1.f;
	  for (irnk = 1; irnk <= nrnk; ++irnk)
	    {
	      im = rktdir[irnk - 1];
	      if (iconta[im - 1] == 0)
		{
		  probp = rankt[irnk - 1] * rankt[irnk - 1];
		  bprob = 90.f;
		  goto L26;
		}
/*allow for presence of limbos */
		if (abs(icana[im - 1]) <= 1)
		  {
		    imm = im;
		  }
		else
		  {
		    imm = abs(icana[im - 1]) % 10000000;
		  }
	      asscont1_(&nclone, &iconta[im - 1], &ncompm, icnum2,
			&iposavm, &icrtm);
/* is clone at left or right end of contig? */
	      if (imm == icnum2[0])
		{
		  ihnd = 1;
		}
	      else if (imm == icrtm)
		{
		  ihnd = 2;
		}
	      else
		{
		    goto L27;
		}
/* find best clone to subtract */
	      bestsubtr_(&im, &ihnd, icnum2, &ncompm, &isubm);
/* subtract */
	      clondif_(&im, &isubm, &itolsb, &nbandd, coord1);
/* do reverse match */
	      matchs_(ic, nbandd, coord1, itolmt, &imatch);
	      probcoinc_(imatch,
			 min(nbandd,nbanda[ic - 1]),
			 max(nbandd,nbanda[ic - 1]),
			 &bprob, itolmt, logfact);

	      probp = rankt[irnk - 1] * bprob;
	      if (disp)
		{
		  probp *= sqrt(dmin(rankt[irnk - 1],bprob) /
				dmax(rankt[irnk - 1],bprob));
		}
L26:
/* choose lowest probability */
	      if (probp < tprob)
		{
		  tprob = probp;
		  tbprb = bprob;
		  ifm = irnk;
		}
L27:
	      ;
	    }
	  rankp[itot - 1] = tprob;
	  rankf[itot - 1] = rankt[ifm - 1];
	  rankb[itot - 1] = tbprb;
	  rkfdir[itot - 1] = rktdir[ifm - 1];
	  matf[itot - 1] = matt[ifm - 1];

	  if (ncomp == 1)	/* no point in doing second end */
	    {
	      goto L302;
	    }
	}
L302:
	;
    }
/*   sort results into order of increasing probability */
  m = itot;
L25:
  m = (m + 2) / 3;
  for (i = m + 1; i <= itot; ++i)
    {
      for (j = i; -m < 0 ? j >= m + 1 : j <= m + 1; j += -m)
	{
	  if (rankp[srtdir[j - 1] - 1] >= rankp[srtdir[j - m - 1] - 1])
	    {
	      goto L6;
	    }
	  itemp = srtdir[j - 1];
	  srtdir[j - 1] = srtdir[j - m - 1];
	  srtdir[j - m - 1] = itemp;
	}
L6:
      ;
    }
  if (m != 1)
    {
      goto L25;
    }
/*   output */
  if (!*out_file)
    printf ("sending output to screen\n\n") ;
  else
    printf ("sending output to screen and to file %s\n\n", out_file) ;
  for (iq = 1; iq <= min(nout,itot); ++iq)
    {
      irnk = srtdir[iq - 1];
      i = clmat[irnk - 1];
      j = rkfdir[irnk - 1];
      k = clsub[irnk - 1];
      if (i == 0 || j == 0)
	{
	  goto L22;
	}
/*   find canonical */
	ican = abs(icana[j - 1]) % 10000000;
	if (ican > 1)
	  {
	    strtcpy(name, namea + ((ican - 1) << 3), 8);
	  }
	else
	  {
	    if (icana[j - 1] == -1)
	      {
		strcpy(name, "canon");
	      }
	    else
	      {
		strcpy(name, "");
	      }
	}
/*   read in data for clone I */
      lband1 = lbanda[i - 1];
      nband1 = nbanda[i - 1];
	for (kk = 1; kk <= nband1; ++kk)
	  {
	    coord1[kk - 1] = coorda[lband1 + kk - 2];
	  }
      iln1 = i;
      iln2 = j;
      rejpss_("qP", &iln1, &iln2, ilna, projectbase, &nclone);
      if (iln1 == -99)
	{
	  strcpy(tname, "poss");
	}
      else
	{
	  strcpy(tname, "");
	}
      matchs_(j, nband1, coord1, itolmt, &imatch);
      probcoinc_(imatch,
		 min(nband1,nbanda[j - 1]),
		 max(nband1,nbanda[j - 1]), &prob, itolmt, logfact);

      strtcpy (ntemp1, namea + ((i - 1) << 3), 8) ;
      strtcpy (ntemp2, namea + ((k - 1) << 3), 8) ;
      strtcpy (ntemp3, namea + ((j - 1) << 3), 8) ;
      gprintf (filout,
	       "%4ld-%-8s(%3ldb,%4ld) minus %-8s  %3ld/%-5ld"
	       "%4ld-%-8s(%3ldb,%4ld)%3ldm E%-2ld  %-8s%4ld%4ld%4ld  %s\n",
	       igela[i - 1], ntemp1, nbanda[i - 1], iconta[i - 1],
	       ntemp2, matf[irnk - 1],
	       bndm[irnk - 1], igela[j - 1], ntemp3, 
	       nbanda[j - 1], iconta[j - 1], imatch, 
	       -((long int) r_lg10(&prob)),
	       name,
	       -((long int) r_lg10(&rankf[irnk - 1])),
	       -((long int) r_lg10(&rankb[irnk - 1])),
	       -((long int) r_lg10(&rankp[irnk - 1])),
	       tname);
      if (*out_file)		/* when outfile, then also screen */
	gprintf (stdout,
		"%4ld-%-8s(%3ldb,%4ld) minus %-8s  %3ld/%-5ld"
		"%4ld-%-8s(%3ldb,%4ld)%3ldm E%-2ld  %-8s%4ld%4ld%4ld  %s\n",
		 igela[i - 1], ntemp1, nbanda[i - 1], iconta[i - 1],
		 ntemp2, matf[irnk - 1],
		 bndm[irnk - 1], igela[j - 1], ntemp3, 
		 nbanda[j - 1], iconta[j - 1], imatch, 
		 -((long int) r_lg10(&prob)),
		 name,
		 -((long int) r_lg10(&rankf[irnk - 1])),
		 -((long int) r_lg10(&rankb[irnk - 1])),
	       -((long int) r_lg10(&rankp[irnk - 1])),
		 tname);
      
L22:
      ;
    }
  if (filout != stdout) fclose (filout) ;
  fclose (unit_16) ;
  return TRUE ;
ERROR:
  gprintf (stderr, "File access failed in huntend2 routine !!!\n") ;
  return FALSE ;
} /* huntend2 */

#undef tname1b
#undef tname
#undef tname1
#undef tname
#undef names
#undef name1


/*   assembles contig, omits ! clones, finds true right end clone */
int asscont1_(long int *nclone, long int *icont, long int *ncomp,
	      long int *icnuma, long int *iposav, long int *icrt)
{
  /* Local variables */
  static long int ipos;
    static char tname[9];
  static long int ix, iy, irt;
  
  /* Parameter adjustments */
  --icnuma;
  
/* protect against contig 0 */
  if (*icont == 0)
    {
      return 0;
    }
/* initialise right end hunt */
  irt = -100000;
    *ncomp = 0;
/* loop for each clone */
  for (iy = 2; iy <= *nclone + 1; ++iy)
    {
      if (iconta[iy - 1] == *icont)
	{
/* ignore buried clones */
	  if (abs(icana[iy - 1]) > 1)
	    {
	      goto L30;
	    }
/* omit cancellations and repeats */
	  strtcpy(tname, namea + ((iy - 1) << 3), 8);
	  if ((*tname == '!' || tname[1] == '!' || tname[2] == '!' ||
	       tname[3] == '!' || tname[4] == '!' || tname[5] == '!' ||
	       tname[6] == '!' || tname[7] == '!') && icana[iy - 1] !=  -1)
	    {
	      goto L30;
	    }
	  ipos = iposa[iy - 1];
/* compute right end */
	  if (ipos + ilena[iy - 1] > irt)
	    {
	      *icrt = iy;
	      irt = ipos + ilena[iy - 1];
	    }
/* place clone in rank of left ends */
	  ++(*ncomp);
	  for (ix = *ncomp; ix >= 2; --ix)
	    {
	      if (ipos > iposa[icnuma[ix - 1] - 1])
		{
		  icnuma[ix] = iy;
		  goto L30;
		}
	      icnuma[ix] = icnuma[ix - 1];
	    }
	  icnuma[1] = iy;
	}
L30:
      ;
    }
  if (*ncomp == 0)
    {
      return 0;
    }
  *iposav = (iposa[icnuma[1] - 1] + iposa[icnuma[*ncomp] - 1]) / 2;
  return 0;
} /* asscont1_ */

