/*  File: f2c.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: contains functions, that behave similar to the
 functions used in FORTRAN
 * SCCS: $Id: f2c.c,v 1.1 2001/02/14 12:44:37 edgrif Exp $
 * HISTORY:
 * Last edited: Jul  3 13:58 1995 (fw)
 * * May 19 15:46 1994 (fw): defined all FILE *unit_? here
 and get them via extern into each source
 * Created: Thu May 19 15:45:58 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"
#include <stdarg.h>

/*****************************************************************/

char ntemp1[100], ntemp2[100], ntemp3[100] ;

#ifdef NO_GRAPHICS  /* graphical applications find readinput in vt640.c */
/****************************************/
/*                                      */
/* FUNCTION TO READ A STRING FROM stdin */
/*                                      */
/****************************************/
char stemp[100] ;

void readinput (char *src, char *line, int len)
{
  int c, i;
  char inp[100] ;

  for ( i = 0; ((c=getchar()) != EOF && c != '\n'); ++i)
    inp[i] = c;
  inp[i] = '\0';
  strcpy (line, inp) ;
/*
  strtcpy (line, inp, len) ;
*/
  if (i == 0)
    strcpy (line, " ") ;
}

static char messbuf[4096] ;
void gprintf (FILE *fil, const char *format, ...)
{ va_list args ;

  va_start (args,format) ;
    vsprintf (messbuf,format,args) ;
  va_end (args) ;

  if (fil == stderr)
    {
      fprintf (stderr, " ERROR : ") ;
    }
  fprintf (fil, messbuf) ;
  fflush (fil) ;
}

void waitForMore (int key)
{
  if (key == RETURN_KEY)
    {
      gprintf (stdout, " hit RETURN to coninue \n") ;
      readinput ("", stemp, 0) ;
    }
  if (key == SPACE_KEY)
    {
      gprintf (stdout, " more - press RETURN \n") ;
      readinput ("", stemp, 0) ;
    }
}
#endif

/********************************************/
/*                                          */
/* AUXILIARY FUNCTIONS FOR THE MAP.OUT FILE */
/*                                          */
/********************************************/

/* read [len] chars from file [unit] into string [line]
   and empty string will be " ", like FORTRAN does */
void readstring (FILE *unit, char *line, int len)
{
  char s[100] ;
  int i ;

  for (i = 0; i < len; ++i) s[i] = fgetc(unit) ;
  s[i] = '\0';
  if (line == 0) return ;      /* just move fileptr */

  strtcpy (line, s, len) ;
}
void readline (FILE *unit, char *line)
{
  char s[200] ;
  int i ;

  for (i = 0; !feof(unit) && i < 200; ++i)
    {
      s[i] = fgetc(unit) ;
      if (s[i] == '\n')
	{
	  s[i] = 0 ;
	  break ;
	}
    }
  if (line)	/* may be used just to skip a line, if 0 is passed */
    strcpy (line, s) ;
}

/* move filepointer to the beginning of the next line */
void forward (FILE *unit)
{
  while (!feof(unit) && fgetc (unit) != '\n') ;
}

/*********************************************************/
/*                                                       */
/* SPECIAL STRING FUNCTIONS TO COPE WITH FORTRAN STRINGS */
/*                                                       */
/*********************************************************/

/* copies len characters of FORTRAN-string src to temporrary C-string dest */
void strtcpy (char *dest, char *src, int len)
{
  strncpy (dest, src, len) ;
  dest[len] = '\0' ;	    /* end-marker for C-strings */
}
/* gets length from a FORTRAN string, [len] is the maximum length,
   which will be returned, if the string is completely full.
   (FORTRAN) doesn't use an extra 0-byte at the end of each string */
int strlength (char *s, int len)
{
  int i = len ;
  if (!(s[len-1] == ' ' || s[len-1] == '\0' || !isascii(s[len-1])))
    return len ;
  while (i>0 && (s[i-1] == ' ' || s[i-1] == '\0' || !isascii(s[i-1])))
    --i ;
  return i ;
}

/**********************************************************/
/*                                                        */
/* FUNCTIONS TO HANDLE INPUTS OF INTEGER AND FLOAT VALUES */
/*                                                        */
/**********************************************************/

/* returns TRUE, when input string is not 
   appropriate for integer input */
BOOL nonint (char *in)
{
  int i ;
  
  if (strcmp(in, " ") == 0) return FALSE ; /* empty input */
  for (i = 0; i < strlen (in); ++i)
    if (!(isdigit(in[i]) || in[i] == '-')) return TRUE ;/* allow negative */
  return FALSE ;
}
/* returns TRUE, when input string is not
   apprpriate for float input */
BOOL nonflt (char *in)
{
  float test ;
  int i ;

  if (strcmp(in, " ") == 0) return FALSE ; /* empty input */
  sscanf (in, "%f", &test) ;
  /* result 0 only allowed, when
     string consists only of 0 and ., e.g.:  0.00 */
  if (test == 0)
    {	      for (i = 0; i < strlen (in); ++i)
	if (!(in[i] == '0' || in[i] == '.'))
	  return TRUE ;
    }
  return FALSE ;	/* assume, any other value is float */
}

/* returns the integer value of a passed string [in] */
long int a2int (char *in)
{
  long int ret_val ;

  if (strcmp(in, " ") == 0) return 0 ; /* empty input */
  if (nonint(in)) return 0 ;
  sscanf (in, "%ld", &ret_val) ;
  return ret_val ;
}

/* returns the float value of a passed string [in] */
float a2flt (char *in)
{
  float ret_val ;

  if (strcmp(in, " ") == 0) return 0 ; /* empty input */
  if (nonflt(in)) return 0 ;
  sscanf (in, "%e", &ret_val) ;
  return ret_val ;
}

/*****************************************************************/
/*                                                               */
/* FUNCTIONS TO POSITION THE FILEPOINTER IN A DIRECT-ACCESS FILE */
/*                                                               */
/*****************************************************************/

/* all the file-pointer are defind here, so using extern one 
   can get the needed ones into the source-file */
FILE *unit_1 ;			/* .nam */
FILE *unit_2 ;			/* .cor */
FILE *unit_3 ;			/* standard.dat */
FILE *unit_4 ;			/* .cnt */
FILE *unit_10 ;			/* .stk */
FILE *unit_11 ;			/* .rem */
FILE *unit_12 ;			/* map.out */
FILE *unit_13 ;			/* .flg */
FILE *unit_14 ;			/* scratch[1,2].dat, .rej, .pss */
FILE *unit_16 ;			/* scratch.dat */

FILE *unit_20=0;		/* CLONE.LIS */
FILE *unit_21 ;			/* new*.nam */
FILE *unit_22 ;			/* new*.cor */
FILE *unit_23 ;			/* new*.cnt */
FILE *unit_29 ;			/* new*.flg */

#ifdef USE_INTS
  /* extensions made by people at FORTH in Greece */
  /* this file contains the intensity of each band - the volume number in .fea */
  FILE *unit_53 ;			/* int */
#endif

/* .nam */
void seek_1 (long int pos)			
{
  int rec = 8*sizeof(char) + 4*sizeof(long int) ;
  fseek (unit_1, (pos - 1) * rec, SEEK_SET) ;
}
/* new*.nam */
void seek_21 (long int pos)			
{
  int rec = 8*sizeof(char) + 4*sizeof(long int) ;
  fseek (unit_21, (pos - 1) * rec, SEEK_SET) ;
}

/* new*.cor */
void seek_22 (long int pos)
{
  int rec = sizeof(short int) ;
  fseek (unit_22, (pos - 1) * rec, SEEK_SET) ;
}
/* .cnt */
void seek_4 (long int pos)
{
  int rec = 4*sizeof(long int) ;
  fseek (unit_4, (pos - 1) * rec, SEEK_SET) ;
}
/* new*.cnt */
void seek_23 (long int pos)
{
  int rec = 4*sizeof(long int) ;
  fseek (unit_23, (pos - 1) * rec, SEEK_SET) ;
}

/* .stk */
void seek_10 (long int pos)
{
  int rec = sizeof(long int) ;
  fseek (unit_10, (pos - 1) * rec, SEEK_SET) ;
}

/* .rem */
void seek_11 (long int pos)
{
  int rec = 1*sizeof(long int) + 48*sizeof(char) + 3*sizeof(short int) +
    1*sizeof(float) ;
  fseek (unit_11, (pos - 1) * rec, SEEK_SET) ;
}

/* .flg */
void seek_13 (long int pos)
{
  int rec = sizeof(long int) ;
  fseek (unit_13, (pos - 1) * rec, SEEK_SET) ;
}
/* new*.flg */
void seek_29 (long int pos)
{
  int rec = sizeof(long int) ;
  fseek (unit_29, (pos - 1) * rec, SEEK_SET) ;
}

/* scratch.dat */
void seek_16 (long int pos)
{
  int rec = sizeof(long int) ;
  fseek (unit_16, pos  * rec, SEEK_SET) ;
}


/* .pss, .rej */
void seek_14 (long int pos)
{
  int rec = 2*sizeof(long int) ;
  fseek (unit_14, (pos - 1) * rec, SEEK_SET) ;
}

/***********************************************/
/*                                             */
/* FUNCTIONS FOR MORE CONVIENIENT FILEHANDLING */
/*                                             */
/***********************************************/

FILE *FILopen (char *base, char *end, char *rw)
{
  char file[128] ;
  FILE *fil ;
  char mesg[100] ;
  char rwflag[2] ;

  if (*end)
    sprintf (file, "%s.%s", base, end) ;
  else
    strcpy (file, base) ;
  
  strcpy(rwflag, rw) ;

  if (isupper(*rwflag))
    { *rwflag = tolower (*rwflag) ; *mesg = 0 ; }
  else
    { *rwflag = tolower (*rwflag) ;
      if (strcmp(rwflag, "r") == 0) strcpy (mesg, "reading") ;
      if (strcmp(rwflag, "r+") == 0) strcpy (mesg, "reading and appending") ;
      if (strcmp(rwflag, "w") == 0) strcpy (mesg, "creating and writing") ;
      if (strcmp(rwflag, "w+") == 0) strcpy (mesg, "creating and reading/writing") ;
    }
  if (!(fil = fopen(file, rwflag)))
    {
      if (*mesg)
	gprintf (stderr, "Failed to open for %s : %s\n", mesg, file) ;

      return 0 ;
    }

  return fil ;
}

BOOL FILerror (FILE *unit)
{
  if (ferror(unit) || feof(unit)) return TRUE ;
  return FALSE ;
}

/*******************************************************************/
/*                                                                 */
/* MATHEMATICAL FUNCTIONS TAKEN FROM THE SRC OF THE LIBF2C-LIBRARY */
/*                                                                 */
/*******************************************************************/

#undef abs
#include "math.h"

double pow_ri(float ap, long int bp)
{
  double pow, x;
  long int n;
  
  pow = 1;
  x = ap;
  n = bp;
  
  if(n != 0)
    {
      if(n < 0)
	{
	  n = -n;
	  x = 1/x;
	}
      for( ; ; )
	{
	  if(n & 01)
	    pow *= x;
	  if((n >>= 1))
	    x *= x;
	  else
	    break;
	}
    }
  return(pow);
}

long int pow_ii(long int ap, long int bp)
{
  long int pow, x, n;
  
  x = ap;
  n = bp;
  
  if (n <= 0)
    {
      if (n == 0 || x == 1)
	return 1;
      if (x != -1)
	return x == 0 ? 1/x : 0;
      n = -n;
    }
  for(pow = 1; ; )
    {
      if(n & 01)
	pow *= x;
      if((n >>= 1))
	x *= x;
      else
	break;
    }
  return(pow);
}
double pow_dd(double ap, double bp)
{
  return(pow(ap, bp) );
}

#define log10e 0.43429448190325182765

double r_lg10(float *x)
{
  return( log10e * log(*x) );
}

#if ! defined (SUN)
#include <stdlib.h>		/* for RAND_MAX */
#endif

double ran_(void)
{
#if defined (SUN)
  return ((double)rand()/(double)((1 << 15) - 1)) ;
#else
  return ((double)rand()/(double)RAND_MAX) ;
#endif
}


/*******************************************/
/*                                         */
/* FUNCTION TO PRINT DATE/TIME INTO STRING */
/*                                         */
/*******************************************/
#include <sys/time.h>
#if (defined SUN || defined SOLARIS)    /* isn't defined in SUN includes */
extern int gettimeofday(struct timeval *tp,struct timezone *tzp);
#endif

static char *datetime = 0 ;

char *getdatetime (void)	/* ddmmyy hh:mm:ss  = 19 chars long*/
{
  struct tm  *ct ;
  struct timeval tv ;
  
  if (!datetime) datetime = (char*)malloc (20) ;

  gettimeofday(&tv, (struct timezone *) NULL);
  ct = localtime(&tv.tv_sec);
  sprintf(datetime, "%02d/%02d/19%02d %02d:%02d:%02d",
	  ct->tm_mday, ct->tm_mon + 1, ct->tm_year,
	  ct->tm_hour, ct->tm_min, ct->tm_sec) ;
  return datetime ;
}
