/*  File: fpabi.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: appends fingerprint data ABI datafile
                to a ContigC-database,
 * SCCS: $Id: fpabi.c,v 1.1 2001/02/14 12:44:38 edgrif Exp $
 * HISTORY: 
 * Last edited: Jan 20 16:15 1995 (fw)
 * Created: Thu May 19 15:47:45 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;		/* .nam */
extern FILE *unit_2 ;		/* .cor */
extern FILE *unit_4 ;		/* .cnt */
extern FILE *unit_13 ;		/* .flg */
extern FILE *unit_16 ;		/* here: abi-file */

static long int c__0 = 0;

void main (int argc, char **argv)
{
  /* Local variables */
  static short int band[10000];
  static long int lband, nband, iq, nbands1, nclone1, gelnr;
  static char name[9], tname[9], innam[128], filnam[128], line[200];
  float fsize ;
  gelnr = 0 ;			/* not read from file */

  printf ("\n  FPABI\n\n"
	  "   converts ABI fingerprint data to ContigC format\n"
	  "   appends to output project\n") ;

  
  /** get the input file name **/
  printf ("   ABI data file = ") ;
  if (argc >= 2)  /* specified one or both files */
    { strcpy (innam, argv[1]) ;
      printf ("%s\n", innam) ;
    }
  else
    readinput ("", innam, 128) ;
  if (strlength(innam, 128) == 0) { printf ("BREAK\n") ; exit (1) ; }


  /** get the project name of the database **/
  printf ("   output project name = ") ;
  if (argc == 3) /* specified also project name */
    { strcpy (filnam, argv[2]) ;
      printf ("%s\n", filnam) ;
    }
  else
    readinput ("", filnam, 128) ;
  if (strlength(filnam, 128) == 0) { printf ("BREAK\n") ; exit (1) ;}



  if (!(unit_1 = FILopen (filnam, "nam", "r+"))) exit (-1) ;
  if (!(unit_2 = FILopen (filnam, "cor", "r+"))) exit (-1) ;
  fseek (unit_2, 0, SEEK_END) ;	/* append at the end */

  if (!(unit_4 = FILopen (filnam, "cnt", "r+"))) exit (-1) ;

  if (!(unit_13 = FILopen (filnam, "flg", "r+"))) exit (-1) ;
  seek_1 (1) ;
  fread (&nclone1, sizeof(long int), 1, unit_1);
  fread (&nbands1, sizeof(long int), 1, unit_1);

  printf ("%8ld clones    %8ld bands\n", nclone1, nbands1) ;

  if (!(unit_16 = FILopen(innam, "", "r"))) exit (-1) ;

  while(!feof(unit_16))
    {
      readline (unit_16, line) ; /* read a whole text line of the file */

      /* go on reading until line beginning with "Cosmid" is found */
      if (strncmp (line, "Cosmid", 6) != 0)
	continue ;

      forward (unit_16) ; /* go to next line */

      /* second element in line is the clone name */
      sscanf (line, "%s %s", stemp, tname) ; 

/*      printf ("Found clone : %s\n", tname) ;
*/

      nband = 0;
      while(!feof(unit_16))
	{
	  memset (line, 0, 200) ;
	  readline (unit_16, line) ;
	  if (strlength (line, 199) == 0) break ;
	  strtcpy (stemp, line + 16, 7) ;
	  fsize = a2flt (stemp) ;
	  
	  ++nband;
	  band[nband - 1] = n_int (fsize) ;
	  /* 
	     printf ("%f\n", fsize) ;
	   */
	}
      
      if (nband == 0)
	{
	  printf (" !!!  discard empty clone %s\n", tname) ;
	  continue ;
	}
      
      lband = nbands1 + 1;
      nbands1 += nband;
      strtcpy (name, tname, 8) ;
      
      seek_1 (nclone1+2) ;
      fwrite (name, sizeof(char), 8, unit_1) ;
      fwrite (&nband, sizeof(long int), 1, unit_1);
      iq = lband + 1 ;
      fwrite (&iq, sizeof(long int), 1, unit_1);
      fwrite (&gelnr, sizeof(long int), 1, unit_1);
      fwrite (&c__0, sizeof(long int), 1, unit_1);
      
      seek_4 (nclone1+2) ;
      fwrite (&c__0, sizeof(long int), 1, unit_4);
      fwrite (&nband, sizeof(long int), 1, unit_4);
      fwrite (&c__0, sizeof(long int), 1, unit_4);
      fwrite (&c__0, sizeof(long int), 1, unit_4);
      
      seek_13 (nclone1+2) ;
      fwrite (&c__0, sizeof(long int), 1, unit_13);
      
      fseek (unit_2, lband * sizeof(short int), SEEK_SET) ;
      fwrite (&band, sizeof(short int), nband, unit_2);

      ++nclone1;
    }
  seek_1 (1) ;
  fwrite (&nclone1, sizeof(long int), 1, unit_1);
  fwrite (&nbands1, sizeof(long int), 1, unit_1);
  printf ("  transfer done ...\n") ;
  printf ("%8ld clones    %8ld bands\n", nclone1, nbands1) ;

  fclose (unit_1) ;
  fclose (unit_2) ;
  fclose (unit_4) ;
  fclose (unit_13) ;
}
