/*  File: fphinx.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: appends fingerprint data from IMAGE
 to a ContigC-database,
 loads the *.bands file (saved by TAR-mode in IMAGE (Version > 1.1))
 * SCCS: $Id: fphinx.c,v 1.1 2001/02/14 12:44:38 edgrif Exp $
 * HISTORY: 
 * Last edited: Jul  5 10:32 1995 (fw)
 * Created: Thu May 19 15:47:45 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;		/* .nam */
extern FILE *unit_2 ;		/* .cor */
extern FILE *unit_4 ;		/* .cnt */
extern FILE *unit_13 ;		/* .flg */
extern FILE *unit_16 ;		/* here: <project>.bands */

static long int c__0 = 0;

void main (int argc, char** argv)
{
  /* Local variables */
  static short int band[10000];
  static long int gelnr;
  static BOOL test;
  static char name[9], tname[9], innam[128], filnam[128];
  static long int lband, nband, itemp, iq, icount, nbands1, nclone1;

  printf ("\n  FPHINX\n\n"
	  "   converts IMAGE fingerprint data to ContigC format\n"
	  "   appends to output project\n") ;

  /** get the input file name **/
  printf ("   TAR output file from IMAGE = ") ;
  if (argc >= 2)  /* specified one or both files */
    { strcpy (innam, argv[1]) ;
      printf ("%s\n", innam) ;
    }
  else
    readinput ("", innam, 128) ;
  if (strlength(innam, 128) == 0) { printf ("BREAK\n") ; exit (1) ; }


  /** get the project name of the database **/
  printf ("   output project name = ") ;
  if (argc == 3) /* specified also project name */
    { strcpy (filnam, argv[2]) ;
      printf ("%s\n", filnam) ;
    }
  else
    readinput ("", filnam, 128) ;
  if (strlength(filnam, 128) == 0) { printf ("BREAK\n") ; exit (1) ;}



  if (!(unit_1 = FILopen (filnam, "nam", "r+"))) exit (-1) ;
  if (!(unit_2 = FILopen (filnam, "cor", "r+"))) exit (-1) ;
  fseek (unit_2, 0, SEEK_END) ;	/* append at the end */

  if (!(unit_4 = FILopen (filnam, "cnt", "r+"))) exit (-1) ;

  if (!(unit_13 = FILopen (filnam, "flg", "r+"))) exit (-1) ;
  seek_1 (1) ;
  fread (&nclone1, sizeof(long int), 1, unit_1);
  fread (&nbands1, sizeof(long int), 1, unit_1);

  printf ("%8ld clones    %8ld bands\n", nclone1, nbands1) ;

  if (!(unit_16 = FILopen(innam, "", "r"))) exit (-1) ;

  while(TRUE)
    {
      fscanf (unit_16, "%s %ld %ld", tname, &nband, &gelnr) ;
      if (feof(unit_16)) break;
/*      printf ("clone:%-8s   band No:%4d    Gel No:%4d\n",
	      tname, nband, gelnr) ;
*/
      icount = 0;
      fscanf (unit_16, "%ld", &iq) ;
      while(iq != -1)
	{
	  ++icount;
	  band[icount - 1] = iq;
/*	  printf ("%d\n", iq) ;
*/
	  fscanf (unit_16, "%ld", &iq) ;
	}
      if (icount == 0 && nband == 0)
	{
	  printf (" !!!  discard empty clone %s\n", tname) ;
	  continue ;
	}
      if (icount != nband)
	{
	  printf (" !!!  band count discrepancy for %s\n", tname);
	  continue ;
	}
L17:
	test = FALSE;
	for (iq = 1; iq <= nband - 1; ++iq)
	  {
	    if (band[iq - 1] > band[iq]) 
	      {
		itemp = band[iq - 1];
		band[iq - 1] = band[iq];
		band[iq] = itemp;
		test = TRUE;
	      }
	  }
      if (test)
	{
	  goto L17;
	}
      lband = nbands1 + 1;
      nbands1 += nband;
      strtcpy (name, tname, 8) ;

      seek_1 (nclone1+2) ;
      fwrite (name, sizeof(char), 8, unit_1) ;
      fwrite (&nband, sizeof(long int), 1, unit_1);
      iq = lband + 1 ;
      fwrite (&iq, sizeof(long int), 1, unit_1);
      fwrite (&gelnr, sizeof(long int), 1, unit_1);
      fwrite (&c__0, sizeof(long int), 1, unit_1);

      seek_4 (nclone1+2) ;
      fwrite (&c__0, sizeof(long int), 1, unit_4);
      fwrite (&nband, sizeof(long int), 1, unit_4);
      fwrite (&c__0, sizeof(long int), 1, unit_4);
      fwrite (&c__0, sizeof(long int), 1, unit_4);

      seek_13 (nclone1+2) ;
      fwrite (&c__0, sizeof(long int), 1, unit_13);

/* BUG: was "lband * sizeof()" before, but in c indexing it is one lower than that */
      fseek (unit_2, (lband - 1) * sizeof(short int), SEEK_SET) ;
      fwrite (&band, sizeof(short int), nband, unit_2);
      ++nclone1;
    }
  seek_1 (1) ;
  fwrite (&nclone1, sizeof(long int), 1, unit_1);
  fwrite (&nbands1, sizeof(long int), 1, unit_1);
  printf ("  transfer done ...\n") ;
  printf ("%8ld clones    %8ld bands\n", nclone1, nbands1) ;

  fclose (unit_1) ;
  fclose (unit_2) ;
  fclose (unit_4) ;
  fclose (unit_13) ;
}
