/*  File: getascdb.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: converts ascified VAX-database to unix-format
 * SCCS: $Id: getascdb.c,v 1.1 2001/02/14 12:44:38 edgrif Exp $
 * HISTORY:
 * Last edited: Mar 17 12:07 1995 (fw)
 * Created: Tue May 24 16:00:34 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/************************************************************/
static void asc2cnt (void) ;
static void asc2cor (void) ;
static void asc2flg (void) ;
static void asc2nam (void) ;
static void asc2pss (void) ;
static void asc2rej (void) ;
static void asc2rem (void) ;
static void asc2stk (void) ;
static void usage (void) ;

extern FILE *unit_1 ;			/* .nam */
extern FILE *unit_2 ;			/* .cor */
extern FILE *unit_4 ;			/* .cnt */
extern FILE *unit_10 ;			/* .stk */
extern FILE *unit_11 ;			/* .rem */
extern FILE *unit_13 ;			/* .flg */
extern FILE *unit_14 ;                  /* .rej, .pss */

/************************************************************/

static char src[128], dest[128] ;
static char name[9] ;
static char gene[9], remark[41] ;
static FILE *fil ;
static long int nclone, nbands, nband, lband, igel, icont,
         ipos, ilen, ican, irem, stkv, iclone, iflag, tot1, tot2, ntot ;
static short int coord, hchrom, hflag, hsitu ;
static float pchr ;
static int i, ix ;
BOOL ISUPPER ;
static char cnt[4], cor[4], flg[4], nam[4], pss[4], rej[4], rem[4], stk[4] ;

/************************************************************/

void main (int argc, char **argv)
{
  int i, m ;

/* first check for HELP options */
  for (i = 1; i < argc; ++i)
    if (strcasecmp(argv[i], "-h") == 0) usage () ;

  printf ("\nGETASCDB - program to convert ascified database\n"
	  "           to binary format used by the UNIX contig package.\n");
  printf ("\n  ascii project name (def = \"ascdb\") = ") ;
  readinput ("ascdb", src, 128) ;
  if (strlength(src, 128) == 0) strcpy (src, "ascdb") ;
  m = 0 ;
  for (i = 0; i < strlen (src); ++i)
    if (isupper (src[i])) ++m ;
  if (m == strlen(src))		/* uppercase file endings */
    {
      strcpy (cnt, "CNT") ; strcpy (cor, "COR") ;
      strcpy (flg, "FLG") ; strcpy (nam, "NAM") ;
      strcpy (pss, "PSS") ; strcpy (rej, "REJ") ;
      strcpy (rem, "REM") ; strcpy (stk, "STK") ;
    }
  else				/* lowercase file endings */
    {
      strcpy (cnt, "cnt") ; strcpy (cor, "cor") ;
      strcpy (flg, "flg") ; strcpy (nam, "nam") ;
      strcpy (pss, "pss") ; strcpy (rej, "rej") ;
      strcpy (rem, "rem") ; strcpy (stk, "stk") ;
    }

  printf ("  target project name = ") ;
  readinput ("", dest, 128) ;
  if (strlength(dest, 128) == 0) exit (1) ;
  
  if (!(fil = FILopen (src, nam, "r"))) exit (-1) ;
  fscanf (fil, "%ld%ld", &nclone, &nbands) ;

  if (argc == 1)		/* convert whole deal */
    {
      asc2cnt () ;
      asc2cor () ;
      asc2flg () ;
      asc2nam () ;
      asc2pss () ;
      asc2rej () ;
      asc2rem () ;
      asc2stk () ;
      exit (1) ;
    }
/* start selective conversion */

  m = 0 ;
  for (i = 1; i < argc; ++i)
    {
      if (strcasecmp(argv[i], "cnt") == 0) { asc2cnt () ; ++m ; }
      if (strcasecmp(argv[i], "cor") == 0) { asc2cor () ; ++m ; }
      if (strcasecmp(argv[i], "flg") == 0) { asc2flg () ; ++m ; }
      if (strcasecmp(argv[i], "nam") == 0) { asc2nam () ; ++m ; }
      if (strcasecmp(argv[i], "pss") == 0) { asc2pss () ; ++m ; }
      if (strcasecmp(argv[i], "rej") == 0) { asc2rej () ; ++m ; }
      if (strcasecmp(argv[i], "rem") == 0) { asc2rem () ; ++m ; }
      if (strcasecmp(argv[i], "stk") == 0) { asc2stk () ; ++m ; }
      if (strcasecmp(argv[i], "-h") == 0) usage () ;
    }

  if (argc - 1 != m)
    {
      printf ("Warning: found %d unparsable argument%s.\n",
	      argc - 1 - m, (argc - 1 - m == 1) ? "" : "s") ;
    }
  exit (1) ;
}

/************************************************************/

static void asc2nam (void)
{
  printf ("Processing (asc)%s.%s -> (bin)%s.nam\n", src, nam, dest) ;

  if (!(fil = FILopen (src, nam, "r"))) exit (-1) ;
  if (!(unit_1 = FILopen (dest, "nam", "w"))) exit (-1) ;

  fscanf (fil, "%ld%ld", &nclone, &nbands) ;
  seek_1 (1) ;
  fwrite (&nclone, sizeof(long int), 1, unit_1) ;
  fwrite (&nbands, sizeof(long int), 1, unit_1) ;

  for (ix = 2; ix <= nclone + 1; ++ix)
    {
      char ntemp[9] ;

      fscanf (fil, "%s%ld%ld%ld%ld", ntemp, &nband, &lband, &igel, &icont) ;
      for (i = 0; i < 8; ++i)
	if (ntemp[i] == '_') ntemp[i] = ' ' ;

      for (i = 0; i < 8; ++i) name[i] = 0 ;
      strtcpy (name, ntemp, strlength(ntemp, 8)) ;

      seek_1 (ix) ;
      fwrite (name, sizeof(char), 8, unit_1) ;
      fwrite (&nband, sizeof(long int), 1, unit_1) ;
      fwrite (&lband, sizeof(long int), 1, unit_1) ;
      fwrite (&igel, sizeof(long int), 1, unit_1) ;
      fwrite (&icont, sizeof(long int), 1, unit_1) ;
    }
  fclose (fil) ; fclose (unit_1) ;
}

/************************************************************/

static void asc2cnt (void)
{
  printf ("Processing (asc)%s.%s -> (bin)%s.cnt\n", src, cnt, dest) ;

  if (!(fil = FILopen (src, cnt, "r"))) exit (-1) ;
  if (!(unit_4 = FILopen (dest, "cnt", "w"))) exit (-1) ;
  
  for (ix = 2; ix <= nclone + 1; ++ix)
    {
      fscanf (fil, "%ld%ld%ld%ld", &ipos, &ilen, &ican, &irem) ;

      seek_4 (ix) ;
      fwrite (&ipos, sizeof(long int), 1, unit_4) ;
      fwrite (&ilen, sizeof(long int), 1, unit_4) ;
      fwrite (&ican, sizeof(long int), 1, unit_4) ;
      fwrite (&irem, sizeof(long int), 1, unit_4) ;
    }
  fclose (fil) ; fclose (unit_4) ;
}

/************************************************************/

static void asc2flg (void)
{
  printf ("Processing (asc)%s.%s -> (bin)%s.flg\n", src, flg, dest) ;

  if (!(fil = FILopen (src, flg, "r"))) exit (-1) ;
  if (!(unit_13 = FILopen (dest, "flg", "w"))) exit (-1) ;


  fscanf (fil, "%ld", &iflag) ;	/* read version data */
  seek_13 (1) ;
  fwrite (&iflag, sizeof(long int), 1, unit_13) ;
  
  for (ix = 2; ix <= nclone + 1; ++ix)
    {
      fscanf (fil, "%ld", &iflag) ;
      
      seek_13 (ix) ;
      fwrite (&iflag, sizeof(long int), 1, unit_13) ;
    }
  fclose (fil) ; fclose (unit_13) ;
}

/************************************************************/

static void asc2cor (void)
{
  printf ("Processing (asc)%s.%s -> (bin)%s.cor\n", src, cor, dest) ;

  if (!(fil = FILopen (src, cor, "r"))) exit (-1) ;
  if (!(unit_2 = FILopen (dest, "cor", "w"))) exit (-1) ;

  while (TRUE)
    {
      fscanf (fil, "%hd", &coord) ;
      fwrite (&coord, sizeof(short int), 1, unit_2) ;
      if (feof(fil)) break ;
    }
  fclose (fil) ; fclose (unit_2) ;
}

/************************************************************/

static void asc2stk (void)
{
  printf ("Processing (asc)%s.%s -> (bin)%s.stk\n", src, stk, dest) ;

  if (!(fil = FILopen (src, stk, "r"))) exit (-1) ;
  if (!(unit_10 = FILopen (dest, "stk", "w"))) exit (-1) ;
  
  ix = 0 ;
  while (TRUE)
    {
      fscanf (fil, "%ld", &stkv) ;
      ++ix ;
      seek_10 (ix) ;
      fwrite (&stkv, sizeof(long int), 1, unit_10) ;
      if (feof(fil)) break ;
    }
  fclose (fil) ; fclose (unit_10) ;
}

/************************************************************/

static void asc2rem (void)
{
  printf ("Processing (asc)%s.%s -> (bin)%s.rem\n", src, rem, dest) ;

  if (!(fil = FILopen (src, rem, "r"))) exit (-1) ;
  if (!(unit_11 = FILopen (dest, "rem", "w"))) exit (-1) ;

  ix = 0 ;
  while (TRUE)
    {
      char gtemp[9], rtemp[41] ;
      
      fscanf (fil, "%ld%s%s%hd%hd%f%hd",
	      &iclone, gtemp, rtemp, &hchrom, &hflag, &pchr, &hsitu) ;

      for (i = 0; i < 8; ++i)
	if (gtemp[i] == '_') gtemp[i] = ' ' ;
      for (i = 0; i < 8; ++i) gene[i] = 0 ;
      strtcpy (gene, gtemp, strlength(gtemp, 8)) ;

      for (i = 0; i < 40; ++i)
	{
	  if (rtemp[i] == '_') rtemp[i] = ' ' ;
/*	  if (rtemp[i] == '\"') rtemp[i] = '\'' ; *//* done in dumpAce() */
	}
      for (i = 0; i < 40; ++i) remark[i] = 0 ;
      strtcpy (remark, rtemp, strlength(rtemp, 40)) ;

      ++ix ;
      seek_11 (ix) ;
      fwrite (&iclone, sizeof(long int), 1, unit_11) ;
      fwrite (gene, sizeof(char), 8, unit_11) ;
      fwrite (remark, sizeof(char), 40, unit_11) ;
      fwrite (&hchrom, sizeof(short int), 1, unit_11) ;
      fwrite (&hflag, sizeof(short int), 1, unit_11) ;
      fwrite (&pchr, sizeof(float), 1, unit_11) ;
      fwrite (&hsitu, sizeof(short int), 1, unit_11) ;
      if (feof(fil)) break ;
    }
  fclose (fil) ; fclose (unit_11) ;
}

/************************************************************/

static void asc2pss (void)
{
  printf ("Processing (asc)%s.%s -> (bin)%s.pss\n", src, pss, dest) ;

  if (!(fil = FILopen (src, pss, "r"))) exit (-1) ;
  if (!(unit_14 = FILopen (dest, "pss", "w"))) exit (-1) ;
  
  fscanf (fil, "%ld", &ntot) ;
  seek_14 (1) ;
  fwrite (&ntot, sizeof(long int), 1, unit_14) ;

  for (ix = 2; ix <= ntot; ++ix)
    {
      fscanf (fil, "%ld%ld", &tot1, &tot2) ;
      
      seek_14 (ix) ;
      fwrite (&tot1, sizeof(long int), 1, unit_14) ;
      fwrite (&tot2, sizeof(long int), 1, unit_14) ;
    }
  fclose (fil) ; fclose (unit_14) ;
}

/************************************************************/

static void asc2rej (void)
{
  printf ("Processing (asc)%s.%s -> (bin)%s.rej\n", src, rej, dest) ;

  if (!(fil = FILopen (src, rej, "r"))) exit (-1) ;
  if (!(unit_14 = FILopen (dest, "rej", "w"))) exit (-1) ;
  
  fscanf (fil, "%ld", &ntot) ;
  seek_14 (1) ;
  fwrite (&ntot, sizeof(long int), 1, unit_14) ;

  for (ix = 2; ix <= ntot; ++ix)
    {
      fscanf (fil, "%ld%ld", &tot1, &tot2) ;
      
      seek_14 (ix) ;
      fwrite (&tot1, sizeof(long int), 1, unit_14) ;
      fwrite (&tot2, sizeof(long int), 1, unit_14) ;
    }
  fclose (fil) ; fclose (unit_14) ;
}

static void usage (void)
{
  printf ("getascdb : programme to convert ascified VAX-database\n"
	  "           of the ContigC software into UNIX binary format.\n\n"
	  "Usage:\n"
	  "  getascdb [ options ]\n"
	  "Options:\n"
	  "  -h  ... gets this help\n"
	  "  cnt ... convert only the .cnt file\n"
	  "  cor ... convert only the .cor file\n"
	  "  flg ... convert only the .flg file\n"
	  "  nam ... convert only the .nam file\n"
	  "  pss ... convert only the .pss file\n"
	  "  rej ... convert only the .rej file\n"
	  "  rem ... convert only the .rem file\n"
	  "  stk ... convert only the .stk file\n"
	  "When no options are given, the entire "
	  "database will be converted.\n") ;  
  exit (1) ;
}
