/*  File: graphc9.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: renegade version of graphTextEntry for non-callback
 programs
 * SCCS: $Id: graphc9.c,v 1.1 2001/02/14 12:44:38 edgrif Exp $
 * HISTORY:
 * Last edited: Nov 24 09:38 1999 (fw)
 * Created: Mon May 23 12:56:25 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "regular.h"
#include "../w2/graph_.h"     /* defines externals within graph package */
#include "key.h"
#if defined(MACINTOSH)
  #define isascii(x) ((x) < 128)
#endif
#include <ctype.h>

static char oldtext[128] ;

/******************** text entry box *********************/

typedef struct EntryBoxStruct
  { char*	text ;
    char*	cp ;
    int		len ;
    int		box ;
    int		cursorBox ;
    int         insertMode ;
    void	(*action)(char*) ;
  } *EntryBox ;

static EntryBox ebox ;		/* one global at a time */

/******************************/

static void graphSetCursor (void)
{ 
  float x1,x2,y1,y2 ;
      
  graphBoxDim (ebox->box, &x1, &y1, &x2, &y2) ;
  graphBoxShift (ebox->cursorBox, x1 + UtextX*(ebox->cp - ebox->text), y1) ;
  graphBoxDraw (ebox->cursorBox, RED, TRANSPARENT) ;
}

/******************************/

static void entryBoxEntry (int key)
{
  int oldbox ;
  char *cp, *cq, *cr ;
  static Stack yankStack = 0 ;
  void (*endfn)() ;
  char *endstr ;
  extern int initvt640_(void) ;

  if (!yankStack)
    yankStack = stackCreate(50) ;

  cp = ebox->cp ;
  if (strlen(ebox->text) > ebox->len)
    messcrash ("Over flow in input field") ;
  if (cp < ebox->text || cp >= ebox->text + ebox->len)
    cp = ebox->text + strlen(ebox->text) ;

  switch (key)
    {
    case 12:			/* C-l clear away graphics */
      if (initvt640_())
	graphRedraw () ;	/* only when graphics found */
      break ;
    case RETURN_KEY:
      strcpy (oldtext, ebox->text) ; /* keep old text */

      ebox->cp = cp ;
      graphSetCursor() ;
      graphBoxDraw (oldbox = ebox->box, BLACK, WHITE) ;
      graphUnBlock (TRUE) ;
      
      endfn = ebox->action ;
      endstr = ebox->text ;

      graphBoxClear (ebox->box) ;
      graphBoxClear (ebox->cursorBox) ;
      messfree (ebox) ;

      if (endfn)
	(*(endfn))(endstr) ;

      return ;
    case INSERT_KEY:
      ebox->insertMode = 1 - ebox->insertMode ;
      break ;
    case DELETE_KEY:
    case BACKSPACE_KEY:		/* delete one char to my left */
      if (cp == ebox->text)
	break ;
      cq = --cp ;
      while (*cq)
	{ *cq = *(cq + 1) ;
	  cq++ ;
	}
      break ;
    case 4:			/* C-d delete at cp */
      cq = cp ;
      while(*cq)
	{ *cq = *(cq + 1) ;
	  cq ++ ;
	}
      break ;
    case 23:			/* C-w delete a word leftwards at cp */
      if (cp == ebox->text)
	break ;
      cr = cq = cp ;
      while (*cq == ' ' && cq > ebox->text) cq-- ;
      while (*cq != ' ' && cq > ebox->text) cq-- ;
      cp = cq ;
      while ((*cq++ = *cr++)) ;	/* copy */
      while (*cq) *cq++ = 0 ;	/* clean up */
      break ;
    case LEFT_KEY:
    case 2:			/* C-b */
      if (cp > ebox->text)
	cp-- ;
      break ;
    case RIGHT_KEY:
    case 6:			/* C-f */
      if (*cp)
	cp++ ;
      if (cp > ebox->text + ebox->len && ebox->len > 0 )
	cp-- ;
      break ;
    case HOME_KEY:
    case 1:			/* C-a */
      cp = ebox->text ; 
      break ;
    case 11:			/* C-k kill from cp to the right */
      cq = ebox->text + ebox->len ;
      stackClear(yankStack) ;
      pushText(yankStack, cp) ;
      while (cq >= cp) 
	*cq-- = 0 ;
      break ;
    case 21:			/* C-u kill whole entry */
      cq = ebox->text + ebox->len ;
      cp = ebox->text ;
      stackClear(yankStack) ;
      pushText(yankStack, cp) ;
      while (cq >= cp) 
	*cq-- = 0 ;
      break ;
    case 25:			/* C-y yank buffer */
      cq = stackText(yankStack,0) ;
      while(*cq && cp <  ebox->text + ebox->len) *cp++ = *cq++ ;
      break ;
    case END_KEY:
    case 5:			/* C-e go to last char */
      cp = ebox->text + strlen(ebox->text) ;
      break ;
    case UP_KEY:		/* restore last input */
      strcpy (ebox->text, oldtext) ;
      break;
    default:
      if (!isascii(key) || !isprint(key))
	{ ebox->cp = cp ; 
	  return ; 
	}
      if (ebox->insertMode)
	{ if (strlen(ebox->text) < ebox->len)
	    { cq = ebox->text + strlen(ebox->text) ;
	      while (--cq >= cp) 
		*(cq + 1) = *cq ;
	      *cp++ = key ;
	    }
	}
      else
	if (cp < ebox->text + ebox->len)
	  *cp++ = key ;
    }

  ebox->cp = cp ;
  graphSetCursor () ;
  graphBoxDraw (ebox->box,-1,-1) ;
}  

/* blocking text input, no scrolling, fills text */

int graphFortranEntry (char* text, int len, float x, float y, void (*fn)())
{
  char *cp ;
  int box ;

  if (len < 0)
    messcrash ("Negative length in graphFortranEntry") ;
  if (!text)
    messcrash ("Null text in graphFortranEntry") ;
  cp = text - 1 ;  /* Clean the buffer */
  while (*++cp) ;
  while (cp < text + len)
    *cp++ = 0 ;

  ebox = (EntryBox) messalloc (sizeof (struct EntryBoxStruct)) ;
  ebox->text = text ;
  ebox->cp = text + strlen(text) ;
  ebox->action = fn ;
  ebox->len = len ;
  ebox->text = text ;
  ebox->insertMode = 1 ;
      
  ebox->box = graphBoxStart () ;
  box = ebox->box ;
  graphTextPtr (ebox->text,x,y,len) ;
  ebox->cursorBox = graphBoxStart () ;
  /*  graphLine (x, y, x, y+UtextY) ; */
  graphRectangle (x, y, x+UtextX, y+UtextY) ;
  graphBoxEnd () ;		/* cursor */
  graphBoxEnd () ;		/* ebox */
  
  graphSetCursor () ;
  graphBoxDraw (ebox->box, BLACK, WHITE) ;
  
  graphRegister (KEYBOARD, entryBoxEntry) ;
  graphBlock () ;
  if (!fn)
    graphRegister (KEYBOARD, 0) ;
  
  return box ;
}
