/*  File: lastclones.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description:  takes a project name and a number as input and list
 as many of the last entered clone names as specified from the *.nam file
 Example:     lastclones  mydatabase 40
    will list the last 40 clone names of the database 'mydatabase'
    the default filename is "cen2hs"
    the default number is 20
 * SCCS: $Id: lastclones.c,v 1.1 2001/02/14 12:44:38 edgrif Exp $
 * HISTORY:
 * Last edited: Mar 28 14:39 1995 (fw)
 * Created: Tue Oct 18 18:43:33 1994 (fw)
 *-------------------------------------------------------------------
 */
	
#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;		/* .nam */

void usage(void)
{
  printf ("Usage:  lastclones <database project> [ count ]\n") ;
  exit (-1) ;
}

void main (int argc, char **argv)
{
  /* Local variables */
  static char filnam[128], name[8], cname[9];
  static int number ;
  static long int icount, nbands1, nclone1;

  if (argc == 2) 
    {
      if (strcmp(argv[1], "-h") == 0) usage () ;
    }

  if (argc < 1 || argc > 3)
    {
      usage () ;
    }
  if (argc == 3)
    {
      strcpy (filnam, argv[1]) ;
      number = a2int (argv[2]) ;
    }
  else if (argc == 2)
    {
      strcpy (filnam, argv[1]) ;
      number = 20 ;
    }
  else
    {
      strcpy (filnam, "cen2hs") ; /* local default */
      number = 20 ;
    }

  if (!(unit_1 = FILopen (filnam, "nam", "r"))) exit (-1) ;

  seek_1 (1) ;
  fread (&nclone1, sizeof(long int), 1, unit_1);
  fread (&nbands1, sizeof(long int), 1, unit_1);
 
  if (number <= 0)
    number = nclone1 ;
  else
    number = min (number, nclone1) ;

  printf ("Database \"%s\"     listing %s %d clones :\n", 
	  filnam, nclone1 == number ? "all" : "last", number) ;

  for (icount = nclone1 - number; icount < nclone1; icount++)
    {
      seek_1 (icount + 2) ;
      fread (name, sizeof(char), 8, unit_1) ;
      strtcpy (cname, name, 8) ;
      printf ("%s\n", cname) ;
    }
  fclose (unit_1) ;
}


