/*  File: log.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: record of database progress
 * SCCS: $Id: logdb.c,v 1.1 2001/02/14 12:44:38 edgrif Exp $
 * HISTORY:
 * Last edited: Feb 24 16:19 1995 (fw)
 * Created: Tue Jun 21 11:45:15 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;			/* .nam */
extern FILE *unit_4 ;			/* .cnt */
extern FILE *unit_10 ;			/* .stk */
extern FILE *unit_11 ;			/* .rem */
extern FILE *unit_13 ;			/* .flg */

/* Common Block Declarations */

/* ir1 CB */
long int np;

/* cint CB */
long int nbanda[30000], lbanda[30000], igela[30000], iconta[30000],
iposa[30000], ilena[30000], icana[30000], irema[30000];

/* chars CB */
char namea[240000], chrom[65];

/* clprf CB */

long int itpr, prlen[20];
char pref[160];
float prins[20];

extern double clins_(char *);

/*   LOG */
/*   program to monitor progress in map */
void main (void)
{
  /* Initialized data */
  
  static BOOL abskb = FALSE;
  
  /* System generated locals */
  static char equiv_0[9];
#define tname (equiv_0)
#define tnameb (equiv_0)

  /* Local variables */
  static char gene[9];
  static long int ilft, lent;
  static float pcun, szkb;
  static char cname[9];
  static long int ifctc, iclen;
  static float rkbnd;
  static long int minkb;
  static float pccnt;
  static long int nicnt, itlen, icont, ncomp, ncont;
  static float tcins;
  static long int itemp, iszbd;
  static float sztkb;
  static long int iq, ix, iy, nbands;
  static long int nclone;
  static short int hchrom;
  static char remark[41], remark1[41] ;
  static float gensiz;
  static long int ntcomp, isztbd, iunatt, mcntsz;
  static float tunins;
  static float occ;
  static long int irt;
  static char projectbase[128] ;

  strcpy (chrom, "      (I)  (II)(III) (IV) (V)  (X)  SIT  GEN  BOTH") ;

/*   get name of project and project parameters */
L1:
  printf ("  project name = ") ;
  readinput ("", projectbase, 128) ;

  if (!(unit_1 = FILopen (projectbase, "nam", "r"))) goto L1 ;
  seek_1 (1) ;
  fread (&nclone, sizeof(long int), 1, unit_1) ;
  fread (&nbands, sizeof(long int), 1, unit_1) ;
  gprintf (stdout, "number of clones = %d, "
	   "number of bands = %d\n", nclone, nbands) ;

  if (!(unit_4 = FILopen (projectbase, "cnt", "r"))) goto L1 ;
  if (!(unit_10 = FILopen (projectbase, "stk", "r"))) goto L1 ;
  if (!(unit_11 = FILopen (projectbase, "rem", "r"))) goto L1 ;
  if (!(unit_13 = FILopen (projectbase, "flg", "r"))) goto L1 ;

/* genome size in kb */
  printf ("genome size in kb = ") ;
  readinput ("", stemp, 10) ; gensiz = a2flt (stemp) ;
/* default insert size in kb; absolute kb/bnd if -ve */
  printf ("absolute kb/band (if known, type neg valye, def=size in kb) = ") ;
  readinput ("", stemp, 10) ; prins[0] = a2flt (stemp) ;
  itpr = 1;
  if (prins[0] < 0.f)
    {
      abskb = TRUE;
      rkbnd = -(double)prins[0];
    }
  for (iq = 2; iq <= 20; ++iq)
    {
/* optional keywords... */
      strcpy (stemp, "        ") ;
      printf ("optional keyword (def=none) = ") ;
      readinput ("", stemp, 8) ;
      strncpy (pref + ((iq - 1) << 3), stemp, 8) ;
/* end list with blank */
      if (strlength(stemp, 8) == 0)
	goto L304;
      
      ++itpr;

      lent = strlength (pref + ((iq - 1) << 3), 8) ;
      prlen[iq - 1] = lent;
/* ...and corresponding insert sizes */
      printf ("corresponding insert size = ") ;
      readinput ("", stemp, 10) ;
      prins[iq - 1] = a2flt (stemp) ;
    }
/* first contig to consider */
L304:
  printf ("first contig to consider (def=1) = ") ;
  readinput ("", stemp, 6) ;
  if (strlength (stemp, 6) == 0)
    ifctc = 1 ;
  else
    ifctc = a2int (stemp) ;
L301:
  seek_10 (1) ;
  fread (&nicnt, sizeof(long int), 1, unit_10) ;
  printf ("   minimum contig size to print (kb) = ") ;
  readinput ("", stemp, 6) ; minkb = a2int (stemp) ;

  printf ("   last clone analysed (def=last clone of database) = ") ;
  readinput ("", cname, 8) ;
/*   read in data to arrays */
  {
    int unit_nam, unit_cnt ;
    unsigned long_len = sizeof(long int), char_len = 8 * sizeof(char) ;
    char file[128] ;
    sprintf (file, "%s.nam", projectbase) ;
    unit_nam = open(file, O_RDONLY) ;
    sprintf (file, "%s.cnt", projectbase) ;
    unit_cnt = open(file, O_RDONLY) ;

    lseek (unit_nam, 8*sizeof(char)+4*sizeof(long int), SEEK_SET) ;
    lseek (unit_cnt, 4*sizeof(long int), SEEK_SET) ;
    for (ix = 1; ix < nclone + 1; ++ix)
      {
	read (unit_nam, namea + (ix << 3), char_len) ;
	read (unit_nam, &nbanda[ix], long_len) ;
	read (unit_nam, &lbanda[ix], long_len) ;
	read (unit_nam, &igela[ix],  long_len) ;
	read (unit_nam, &iconta[ix], long_len) ;

	read (unit_cnt, &iposa[ix],  long_len) ;
	read (unit_cnt, &ilena[ix],  long_len) ;
	read (unit_cnt, &icana[ix],  long_len) ;
	read (unit_cnt, &irema[ix],  long_len) ;
      }
    close (unit_nam) ; close (unit_cnt) ;
  }

  if (strlength (cname, 8) == 0)
    {
      ix = nclone + 1;
      goto L307;
    }
  
  for (ix = 2; ix <= nclone + 1; ++ix)
    {
      if (strncmp(namea + ((ix - 1) << 3), cname, 8L) == 0)
	{
	  goto L307;
	}
    }
  printf ("clone not found");
  goto L301;

/* count unattached clones, omitting cancellations and repeats */
L307:
  iunatt = 0;
  tunins = 0.f;
  for (iy = 2; iy <= ix; ++iy)
    {
      if (iconta[iy - 1] == 0)
	{
	  strncpy(tname, namea + ((iy - 1) << 3), 8);
	  if (tnameb[0] == '!' || tnameb[1] == '!' || tnameb[2] == '!' || 
	      tnameb[3] == '!' || tnameb[4] == '!' || tnameb[5] == '!' ||
	      tnameb[6] == '!' || tnameb[7] == '!')
	    {
	      goto L308;
	    }
	  if (abskb)
	    {
	      tunins += (float) ilena[iy - 1] * rkbnd;
	    }
	  else
	    {
	      tunins += clins_(tname);
	    }
	  ++iunatt;
	}
L308:
      ;
    }
  ncont = 0;
  ntcomp = 0;
  itlen = 0;
  isztbd = 0;
  sztkb = 0.f;

/*   loop for each contig */
  for (icont = ifctc; icont <= nicnt; ++icont) 
    {
      ilft = 100000;
      irt = -100000;
      iclen = 0;
      ncomp = 0;
      strcpy (remark, "") ;
      tcins = 0.f;
/*   loop for each clone */
      for (iy = 2; iy <= ix; ++iy)
	{
/*   omit cancellations and repeats */
	  if (iconta[iy - 1] == icont)
	    {
	      strtcpy(tname, namea + ((iy - 1) << 3), 8L);
	      if ((tnameb[0] == '!' || tnameb[1] == '!' ||
		   tnameb[2] == '!' || tnameb[3] == '!' ||
		   tnameb[4] == '!' || tnameb[5] == '!' ||
		   tnameb[6] == '!' || tnameb[7] == '!') && 
		  icana[iy - 1] != -1)
		{
		  goto L303;
		}
	      ++ncomp;
/* don't use hidden clones for contig dimensions (data may be wrong) */
	      if (abs(icana[iy - 1]) < 2)
		{
		  ilft = min(ilft,iposa[iy - 1]);
		  irt = max(irt,iposa[iy - 1] + ilena[iy - 1]);
		}
	      iclen += ilena[iy - 1];
	      tcins += clins_(tname);
	      if (irema[iy - 1] > 0)
		{
		  seek_11 (irema[iy - 1]) ;
		  fread (&itemp, sizeof(long int), 1, unit_11);
		  fread (gene, sizeof(char), 8, unit_11);
		  fread (remark1, sizeof(char), 40, unit_11);
		  fread (&hchrom, sizeof(short int), 1, unit_11);
		  if (strlength(gene, 8) > 0)
		    {
		      strtcpy (stemp, chrom + hchrom * 5, 5L) ;
		      sprintf (remark, "%-8s%5s", gene, stemp) ;
		    }
		}
	    }
L303:
	  ;
	}
      if (ncomp == 0)
	{
	  goto L302;
	}
      if (irt < ilft)
	{
	  goto L302;
	}
      ++ncont;
      ntcomp += ncomp;
      itlen += iclen;
      iszbd = irt - ilft;
      isztbd += iszbd;
      if (abskb)
	{
	  szkb = (float) iszbd * rkbnd;
	}
      else
	{
	  szkb = (float) iszbd * tcins / (float) iclen;
	}
      sztkb += szkb;
      if (szkb < (float) minkb)
	{
	  goto L302;
	}
      printf (" %6ld     %6ld bands     %6ld kb     %-40s\n",
	      icont, iszbd, (long int) szkb, remark);
L302:
      ;
    }
  pccnt = sztkb * 100.f / gensiz;
  occ = (float) ntcomp / (float) ncont;
  mcntsz = (long int) sztkb / ncont;
  pcun = tunins * 100.f / gensiz;

  printf ("  clone analysed =%6ld\n"
	  "  clones in contigs =%6ld\n"
	  "  contigs =%6ld\n"
	  "  total bands in contigs =%6ld\n"
	  "  %% genome in contigs =%5.1f\n"
	  "  mean contig size =%6ld\n"
	  "  contig occupancy =%5.1f\n"
	  "  unattached clones =%6ld\n"
	  "  max %% genome in unattached clones =%5.1f\n",
	  ntcomp + iunatt, ntcomp, ncont, isztbd, pccnt, mcntsz,
	  occ, iunatt, pcun);
  return ;
} /* main */

#undef tname1b
#undef tnameb
#undef tname1
#undef tname
#undef names
#undef name1


double clins_(char *name)
{
  /* System generated locals */
  float ret_val;

  /* Local variables */
  static long int ip;

/* default value */
  ret_val = prins[0];

  for (ip = 2; ip <= itpr; ++ip)
    {
      if (strncmp(name, pref + ((ip - 1) << 3), prlen[ip - 1]) == 0) 
	{
	  ret_val = prins[ip - 1];
	  return ret_val;
	}
    }
  return ret_val;
} /* clins_ */

