/*  File: mapsub7.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: creates map.out file, used by the auto mode
 in ContigC
 * SCCS: $Id: mapsub7.c,v 1.1 2001/02/14 12:44:39 edgrif Exp $
 * HISTORY:
 * Last edited: Mar 17 13:41 1995 (fw)
 * Created: Thu May 19 16:43:27 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;			/* .nam */
extern FILE *unit_2 ;			/* .cor */
extern FILE *unit_4 ;			/* .cnt */

/* Common Block Declarations */
/* some of them aren't needed in here */

/* csec CB */
short int coorda[BANDMAX];

/* cint CB */
long int nbanda[30000], lbanda[30000], igela[30000], iconta[30000],
         iposa[30000], ilena[30000], icana[30000], irema[30000];

/* chars CB */
char namea[240000], chrom[65];

/* matdat CB */
float logfact[201];


extern int attrib_(long int *, char *, char *, long int *, 
		   long int *, long int *);

/*   mapsub7 */
/* options to get two sets: singles, two groups, contigs */
int main (void)
{
  /* Local variables */
  static char name[9], limb[9], ptmp[150] ;
  static float rank[100], psdf;
  static long int ilft;
  static float prob;
  static long int irnk, matt[10], iint, nrnk;
  static float psmn;
  static BOOL dtst, upto;
  static long int icln2, ncln1, ncln2, clst1[30000], clst2[30000], ipnt1, 
  ipnt2, i, j, k, idmat, icmax, ibest;
  static BOOL found;
  static long int icont;
  static float rankt[10], probm;
  static long int lband1, nband1, nband2;
  static long int coord1[200], icnum1[4000], ncomp1, iq, imatch;
  static BOOL rnkcan[100];
  static long int nclone;
  static long int itoldf;
  static long int coords[200], rnkdir[100];
  static char projectbase[128] ;
  static long int rktdir[10], icount;
  static long int itolmn, iposav, irnkpt, mat[100];
  static float logfact[201];
  static long int irt;
  static long int nbands, ix, size_of_cor, itmp ;
  static float probc ;

  nrnk = 5;			/* number in probability rank */
  icmax = 5;			/* max number of clones for one contig */
  upto = FALSE;		/* only up to self when true */

  logfact[0] = 0.f;
  logfact[1] = 0.f;
/* calc log factorials */
  for (i = 2; i <= 200; ++i)
    {
      logfact[i] = logfact[i - 1] + log((float) i);
    }
/* open files */
/* get name of project */
  readinput ("", projectbase, 0) ; 
  if (strlength(projectbase, 128) == 0) { printf ("--STOP-- (no project)\n") ; exit (1) ;}

  gprintf (stdout, "   project name = %s\n", projectbase) ;

  if (!(unit_1 = FILopen (projectbase, "nam", "r"))) goto ERROR ;
  seek_1 (1) ;
  fread (&nclone, sizeof(long int), 1, unit_1) ;
  fread (&nbands, sizeof(long int), 1, unit_1) ;

  gprintf (stdout, "   number of clones = %d, "
	   "number of bands = %d\n", nclone, nbands) ;

  if (!(unit_4 = FILopen (projectbase, "cnt", "r"))) goto ERROR ;

  /*   read in data to arrays */
  {
    int unit_cor, unit_nam, unit_cnt ;
    char file[128] ;
    unsigned long_len = sizeof(long int), char_len = 8 * sizeof(char) ;
    sprintf (file, "%s.cor", projectbase) ;

    unit_2 = FILopen (file, "", "r") ;
    fseek (unit_2, 0, SEEK_END) ;
    /* determine filesize of *.cor file */
    size_of_cor = ftell (unit_2) ;
    fclose (unit_2) ;

    unit_cor = open (file, O_RDONLY) ;
    /* read *.cor file into coorda array */
    /* cor file are shorts to make it load faster */
    read (unit_cor, &coorda, size_of_cor) ;
    close (unit_cor) ;

    sprintf (file, "%s.nam", projectbase) ;
    unit_nam = open(file, O_RDONLY) ;
    sprintf (file, "%s.cnt", projectbase) ;
    unit_cnt = open(file, O_RDONLY) ;
    lseek (unit_nam, 8*sizeof(char)+4*sizeof(long int), SEEK_SET) ;
    lseek (unit_cnt, 4*sizeof(long int), SEEK_SET) ;
    for (ix = 1; ix < nclone + 1; ++ix)
      {
	read (unit_nam, namea + (ix << 3), char_len) ;
	read (unit_nam, &nbanda[ix], long_len) ;
	read (unit_nam, &lbanda[ix], long_len) ;
	read (unit_nam, &igela[ix],  long_len) ;
	read (unit_nam, &iconta[ix], long_len) ;

	read (unit_cnt, &iposa[ix],  long_len) ;
	read (unit_cnt, &ilena[ix],  long_len) ;
	read (unit_cnt, &icana[ix],  long_len) ;
	read (unit_cnt, &irema[ix],  long_len) ;
      }
    close (unit_nam) ; close (unit_cnt) ;
  }
  fclose (unit_1) ;
  fclose (unit_4) ;
  ncln1 = 0;
  ncln2 = 0;
  readinput ("", name, 0) ;
  if (strcasecmp (name, "set1") != 0)
    {
      fprintf (stderr, " SET1 not found !\n") ;
      exit (-1) ;
    }
L208:
  readinput ("", name, 0) ;
  if (strcasecmp(name, "set2") == 0)
    {
      goto L209;
    }
  if (strcmp(name, "#") == 0)
    {
      readlst_(&ncln1, clst1, &nclone);
    }
  if (strcmp(name, "&") == 0)
    {
      getset_(&ncln1, clst1, &nclone);
    }
  if (strcasecmp(name, "cont") == 0)
    {
      getcont_(&ncln1, clst1, &nclone, iconta);
    }
  /* 100 spaces as empty line */
  gprintf (stdout, "\n") ;
  goto L208;

/* get second group */
L209:
  readinput ("", name, 0) ;
  if (strcasecmp(name, "end") == 0)
    {
      goto L2;
    }
  if (strcasecmp(name, "upto") == 0)
    {
      upto = TRUE;
      goto L2;
    }
  if (strcmp(name, "#") == 0)
    {
      readlst_(&ncln2, clst2, &nclone);
    }
  if (strcmp(name, "&") == 0)
    {
      getset_(&ncln2, clst2, &nclone);
    }
  if (strcasecmp(name, "cont") == 0)
    {
      getcont_(&ncln2, clst2, &nclone, iconta);
    }
  /* 100 spaces */
  gprintf (stdout, "\n") ;
  goto L209;

/* get tolerance and probability limits */
L2:
  readinput ("", stemp, 0) ;
  itolmn = a2int (stemp) ;
  if (itolmn == 0)
    {
      fprintf (stderr, " 0 tolerance not allowed !\n") ;
      exit (-1) ;
    }
  readinput ("", stemp, 0) ;
  itoldf = a2int (stemp) ;
  if (itoldf == 0)
    {
      itoldf = itolmn;
    }
  gprintf (stdout, "  tolerance in tenths of mm = %6ld (main)   "
	  "%6ld (difmap)\n", itolmn, itoldf) ;

  readinput ("", stemp, 0) ;
  probm = a2flt (stemp) ;
  gprintf (stdout, "  match probability cutoff = %8.1e\n", probm) ;

  readinput ("", stemp, 0) ;
  probc = a2flt (stemp) ;
  gprintf (stdout, "  analytical probability cutoff = %8.1e\n", probc) ;
  /* set start marker */
  gprintf (stdout, "\n-----start-of-mapsub-output-----@\n") ;

  psmn = 1 - (float) (itolmn << 1) / 3300.f;
  psdf = 1 - (float) (itoldf << 1) / 3300.f;
/* loop for each subclone */
  for (ipnt1 = 1; ipnt1 <= ncln1; ++ipnt1)
    {
      i = clst1[ipnt1 - 1];
/* read in data for subclone i */
      lband1 = lbanda[i - 1];
      nband1 = nbanda[i - 1];
      if (nband1 == 0)
	{
	  goto L200;
	}
      for (k = 1; k <= nband1; ++k)
	{
	  coords[k - 1] = coorda[lband1 + k - 2];
	}
      
/* clear prob ranks */
      for (iq = 1; iq <= 10; ++iq)
	{
	  rankt[iq - 1] = 1.f;
	  rktdir[iq - 1] = 0;
	}
      for (iq = 1; iq <= 100; ++iq)
	{
	  rank[iq - 1] = 1.f;
	  rnkcan[iq - 1] = FALSE;
	  rnkdir[iq - 1] = 0;
	}
      irnkpt = 0;

/* loop for each clone in group 2, excluding i */
      for (ipnt2 = 1; ipnt2 <= ncln2; ++ipnt2)
	{
	  icln2 = clst2[ipnt2 - 1];
	  if (icln2 == i)
	    {
	      goto L207;
	    }
	  if (upto && icln2 > i)
	    {
	      goto L207;
	    }
	  patch3_(icln2, icln2, nband1, coords, itolmn, logfact,
		  rank, rnkdir, rankt, rktdir, probm, psmn,
		  &irnkpt, nrnk, mat, matt);
L207:
	    ;
	}
/* transfer from temp rank as necessary, but avoid rubbish */
      itmp = nrnk - irnkpt;
      for (irnk = 1; irnk <= itmp; ++irnk)
	{
	  if (rankt[irnk - 1] < 1.f)
	    {
	      ++irnkpt;
	      rank[irnkpt - 1] = rankt[irnk - 1];
	      rnkdir[irnkpt - 1] = rktdir[irnk - 1];
	      mat[irnkpt - 1] = matt[irnk - 1];
	    }
	}
/* get clondif for top clone */
      ibest = rnkdir[0];
      clondif_(&i, &ibest, &itolmn, &nband2, coord1);

      strtcpy (ntemp1, namea + ((i - 1) << 3), 8) ;
      sprintf (ptmp, "%4ld-%-8s(%3ldb,%4ld)",
	      igela[i - 1], ntemp1, nband1, iconta[i - 1]);
      gprintf (stdout, "%-101s\n", ptmp) ;

L204:
/* sort into contigs, choosing no more than 5 clones from each */
      found = FALSE;
      for (irnk = 1; irnk <= irnkpt; ++irnk)
	{
	  if (rnkcan[irnk - 1])
	    {
	      goto L203;
	    }
	  j = rnkdir[irnk - 1];
	  if (j == 0)
	    {
	      goto L200;
	    }
	  if (! found)
	    {
/* starting new contig */
	      icount = 0;
	      icont = iconta[j - 1];
	      if (icont == 0)
		{
		  goto L205;
		}
	      found = TRUE;
	      asscont_(&nclone, &icont, iconta, iposa, &
		       ncomp1, icnum1, &iposav, icana);
	      ilft = iposa[icnum1[0] - 1];
	      irt = iposa[icnum1[ncomp1 - 1] - 1] +\
		ilena[icnum1[ncomp1 - 1] - 1];
	      for (iq = ncomp1 - 1; iq >= max(1,ncomp1 - 10); --iq)
		{
		  irt = max(irt,iposa[icnum1[iq - 1] - 1] + 
			    ilena[icnum1[iq - 1] - 1]);
		}
	    }
		else if (iconta[j - 1] != icont)
		{
		goto L203;
	    }
		else if (icount >= icmax)
		{
		rnkcan[irnk - 1] = TRUE;
		goto L203;
	    }
L205:
	  attrib_(&j, name, limb, &iint, &ilft, &irt);
	  if (irnk == 1)
	    {
	      dtst = nband2 > iint;
	      strtcpy (ntemp1, namea + ((j - 1) << 3), 8) ;
	      sprintf (ptmp, "                         %6ld matches  "
		      "%4ld-%-8s(%3ldb,%4ld)  %8.1e%6ld%6ldd   "
		      "%-8s  %s",
		      mat[irnk - 1], igela[j - 1],
		      ntemp1, nbanda[j - 1], iconta[j - 1],
		      rank[irnk - 1], iint, nband2, name, limb) ;
	      gprintf (stdout, "%-101s\n", ptmp) ;
	    }
	  else
	    {
	      matchs_(j, nband2, coord1, itoldf, &idmat);
	      strtcpy (ntemp1, namea + ((j - 1) << 3), 8) ;
	      sprintf (ptmp, "                         %6ld matches  "
		      "%4ld-%-8s(%3ldb,%4ld)  %8.1e%6ld%6ld    "
		      "%-8s  %s",
		      mat[irnk - 1], igela[j - 1],
		      ntemp1, nbanda[j - 1], iconta[j - 1],
		      rank[irnk - 1], iint, idmat, name, limb) ;
	      gprintf (stdout, "%-101s\n", ptmp) ;
	    }
	  rnkcan[irnk - 1] = TRUE;
	  ++icount;
L203:
	  ;
	}
      if (found)
	{
	  goto L204;
	}
      if (! dtst)
	{
	  goto L200;
	}
      for (iq = 1; iq <= 10; ++iq)
	{
	  rankt[iq - 1] = 1.f;
	  rktdir[iq - 1] = 0;
	}
      for (ipnt2 = 1; ipnt2 <= ncln2; ++ipnt2)
	{
	  icln2 = clst2[ipnt2 - 1];
	  if (icln2 == i)
	    {
	      goto L211;
	    }
	  if (upto && icln2 > i)
	    {
	      goto L211;
	    }
	  patch4_(icln2, icln2, nband2, coord1, itoldf,
		  logfact, rankt, rktdir, psdf, nrnk, matt);
L211:
	  ;
	}
      for (irnk = 1; irnk <= 5; ++irnk)
	{
	  j = rktdir[irnk - 1];
	  if (j == 0)
	    {
	      goto L1022;
	    }
	  for (iq = 1; iq <= irnkpt; ++iq)
	    {
	      if (rnkdir[iq - 1] == j)
		{
		  goto L1022;
		}
	    }
	  icont = iconta[j - 1];
	  if (icont == 0)
	    {
	      goto L206;
	    }
	  else
	    {
	      asscont_(&nclone, &icont, iconta, iposa,
		       &ncomp1, icnum1, &iposav, icana);
	      ilft = iposa[icnum1[0] - 1];
	      irt = iposa[icnum1[ncomp1 - 1] - 1] +
		ilena[icnum1[ncomp1 - 1] - 1];
	      for (iq = ncomp1 - 1; iq >= max(1,ncomp1 - 10); --iq)
		{
		  irt = max(irt,iposa[icnum1[iq - 1] - 1] + 
			    ilena[icnum1[iq - 1] - 1]);
		}
	    }
L206:
	  attrib_(&j, name, limb, &iint, &ilft, &irt);
	  matchs_(j, nband1, coords, itolmn, &imatch);
	  probcoinc_(imatch,
		     min(nband1,nbanda[j - 1]), 
		     max(nband1,nbanda[j - 1]),
		     &prob, itolmn, logfact);

	  strtcpy (ntemp1, namea + ((j - 1) << 3), 8) ;
	  sprintf (ptmp, "                difmap:  %6ld matches  "
		  "%4ld-%-8s(%3ldb,%4ld)  %8.1e%6ld%6ld    %-8s  %s",
		  imatch, igela[j - 1], ntemp1, nbanda[j - 1], 
		  iconta[j - 1], prob, iint, matt[irnk - 1], name, limb) ;
	  gprintf (stdout, "%-101s\n", ptmp) ;
L1022:
	  ;
	}
L200:
      ;
    }
 ERROR:
  return 0;
}

/* get clone attributes */
int attrib_(long int *j, char *name, char *limb, long int *iint,
	    long int *ilft, long int *irt)
{
  /* Local variables */
  static long int ican, ilen, ipos;

  if (iconta[*j - 1] == 0)
    {
      *iint = 0;
      strcpy (name, "") ;
      strcpy (limb, "") ;
      return 0;
    }
  ican = abs(icana[*j - 1]) % 10000000;
  if (ican > 1)
    {
      ipos = iposa[ican - 1];
      ilen = ilena[ican - 1];
      strtcpy(name, namea + ((ican - 1) << 3), 8);
      if (icana[*j - 1] > 10000000)
	{
	  strcpy (limb, "limbo");
	}
      else
	{
	  strcpy (limb, "");
	}
    }
  else
    {
      ipos = iposa[*j - 1];
      ilen = ilena[*j - 1];
      if (icana[*j - 1] == -1)
	{
	  strcpy (name, "canon");
	  strcpy (limb, "");
	}
      else
	{
	  strcpy (name, "");
	  strcpy (limb, "");
	}
    }
  *iint = min(ipos - *ilft,*irt - ipos - ilen);
  return 0;
} /* attrib_ */

