/*  File: model.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description:
 * SCCS: $Id: modelcc.c,v 1.1 2001/02/14 12:44:39 edgrif Exp $
 * HISTORY:
 * Last edited: Feb 21 14:40 1995 (fw)
 * Created: Thu Jun 23 08:55:55 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;			/* .nam */
extern FILE *unit_2 ;			/* .cor */
extern FILE *unit_4 ;			/* .cnt */
extern FILE *unit_10 ;			/* .stk */

/* Common Block Declarations */

long int nbanda[30000], lbanda[30000], igela[30000], iconta[30000],
        iposa[30000], ilena[30000], icana[30000], irema[30000];

char namea[240000], chrom[65];

/* Table of constant values */

static long int c__0 = 0;
static short int c__1000 = 1000;


extern int asscont1_(long int *, long int *, long int *, 
		     long int *, long int *, long int *) ;
extern int asscont2_(long int *, long int *, long int *,
		     long int *, long int *, long int *);
extern int ransort3_(long int *, long int *, long int *, 
		     long int *, long int *, long int *,
		     long int *, long int *, long int *);

/*   MODEL */
/*   program to monitor progress in map */
void main (void)
{
  /* System generated locals */
  long int i__1;
  static char equiv_0[9];
#define tname (equiv_0)
#define tnameb (equiv_0)

  /* Local variables */
  static long int ichc;
  static long int cdir[50000], icln;
  static float hcln;
  static long int mixa[50000], ilft, icrt;
  static BOOL cont, noun;
  static long int ipnt;
  static float ovlp;
  static long int ipos, iprt, iwrt;
  static char cname[9];
  static long int ifctc, mlena[50000];
  static float avgap;
  static long int nicnt, itlen, icont, mposa[50000], ncomp, ipass, nrset;
  static long int icnum1[4000], ia, ic, iq, ix, iy, nbands;
  static long int lengen, nclone, mconta[50000], lenran, ictlen;
  static char projectbase[128] ;
  static long int iposav, nrclst, imixpt;

  strcpy (chrom, "      (I)  (II)(III) (IV) (V)  (X)  SIT  GEN  BOTH") ;

/* get name of project and project parameters */
L1:
  printf ("  project name = ") ;
  readinput ("", projectbase, 128) ;

  if (!(unit_1 = FILopen (projectbase, "nam", "r"))) goto L1 ;
  seek_1 (1) ;
  fread (&nclone, sizeof(long int), 1, unit_1) ;
  fread (&nbands, sizeof(long int), 1, unit_1) ;
  gprintf (stdout, "number of clones = %d, "
	   "number of bands = %d\n", nclone, nbands) ;

  if (!(unit_4 = FILopen (projectbase, "cnt", "r"))) goto L1 ;
  if (!(unit_10 = FILopen (projectbase, "stk", "r"))) goto L1 ;

/* NUMBER OF SETS OF RANDOM CLONES (0-9) */
  printf (" number of sets of random clones (0-9) = ") ;
  readinput ("", stemp, 6) ; nrset = a2int (stemp) ;
  nrset = min(9,nrset);
/* NUMBER OF RANDOM CLONES PER SET(0-9999) */
  printf (" number of random clones per set (0-9999) = ") ;
  readinput ("", stemp, 6) ; nrclst = a2int (stemp) ;
  nrclst = min(9999,nrclst);
/* LENGTH OF RANDOM CLONE IN BANDS */
  printf (" length of random clone in bands = ") ;
  readinput ("", stemp, 6) ; lenran = a2int (stemp) ;
/* GENOME LENGTH IN BANDS */
  printf (" genome length in bands = ") ;
  readinput ("", stemp, 6) ; lengen = a2int (stemp) ;
/* FRACTIONAL OVERLAP DETECTED */
  printf (" fractional overlap detected = ") ;
  readinput ("", stemp, 10) ; ovlp = a2flt (stemp) ;
/* FIRST CONTIG TO CONSIDER */
  printf (" first contig to consider (def=1) = ") ;
  readinput ("", stemp, 6) ; 
  if (strlength(stemp, 6) == 0)
    ifctc = 1 ;
  else
    ifctc = a2int (stemp) ;
L301:
  seek_10 (1) ;
  fread (&nicnt, sizeof(long int), 1, unit_10);
/* LAST CLONE ANALYSED */
  printf (" last clone analysed = ") ;
  readinput ("", stemp, 8) ; strtcpy (cname, stemp, 8) ;
/* OPTIONS */
  printf (" type \"noun\" to exclude unattached clones (def=whole database) = ") ;
  readinput ("", tname, 8L);
  noun = FALSE;
  if (strcasecmp (tname, "noun") == 0)
    {
      noun = TRUE;
    }
  /* read in data to arrays */
  for (ix = 2; ix <= nclone + 1; ++ix)
    {
      seek_1 (ix) ;
      fread (namea + ((ix - 1) << 3), sizeof(char), 8, unit_1) ;
      fread (&nbanda[ix - 1], sizeof(long int), 1, unit_1) ;
      fread (&lbanda[ix - 1], sizeof(long int), 1, unit_1) ;
      fread (&igela[ix - 1], sizeof(long int), 1, unit_1) ;
      fread (&iconta[ix - 1], sizeof(long int), 1, unit_1) ;

      seek_4 (ix) ;
      fread (&iposa[ix - 1], sizeof(long int), 1, unit_4) ;
      fread (&ilena[ix - 1], sizeof(long int), 1, unit_4) ;
      fread (&icana[ix - 1], sizeof(long int), 1, unit_4) ;
      fread (&irema[ix - 1], sizeof(long int), 1, unit_4) ;
    }

  fclose (unit_1) ;
  fclose (unit_4) ;
  fclose (unit_10) ;

  if (strlength (cname, 8) == 0)
    {
      ix = nclone + 1;
      goto L307;
    }
  for (ix = 2; ix <= nclone + 1; ++ix)
    {
      if (strncmp(namea + ((ix - 1) << 3), cname, 8) == 0)
	{
	  goto L307;
	}
    }
  printf ("clone not found\n");
  goto L301;

/* find total length of unattached clones,
   omitting cancellations and repeats */
L307:
  imixpt = 0;
  itlen = 0;
  if (noun)
    {
      goto L309;
    }
  for (iy = 2; iy <= ix; ++iy)
    {
      if (iconta[iy - 1] == 0)
	{
	  strncpy(tname, namea + ((iy - 1) << 3), 8);
	  if (tnameb[0] == '!' || tnameb[1] == '!' || tnameb[2] == '!' || 
	      tnameb[3] == '!' || tnameb[4] == '!' || tnameb[5] == '!' ||
	      tnameb[6] == '!' || tnameb[7] == '!')
	    {
	      goto L308;
	    }
	  ++imixpt;
	  mixa[imixpt - 1] = -iy;
	  itlen += ilena[iy - 1];
	}
L308:
      ;
    }
/*   find total length of contigs, omitting cancellations and repeats */
/*   loop for each contig */
L309:
  printf (" no of unatt = %ld  total unatt len = %ld\n", imixpt, itlen) ;
  for (icont = ifctc; icont <= nicnt; ++icont)
    {
      asscont1_(&nclone, &icont, &ncomp, icnum1, &iposav, &icrt);
      if (ncomp == 0)
	{
	  goto L302;
	}
      ++imixpt;
      mixa[imixpt - 1] = icont;
      itlen = itlen + iposa[icrt - 1] + ilena[icrt - 1] -
	iposa[icnum1[0] - 1];
L302:
      ;
    }
/*   find average gap */
  avgap = (float) (lengen - itlen) / (float) imixpt;
  if (noun)
    {
      avgap = 0.f;
      lengen = itlen;
    }

  printf (" tot len = %ld  avgap = %.3f  total units = %ld\n"
	  " ovlap = %.3f\n", itlen, avgap, imixpt, ovlp) ;
  if (avgap < 0.f)
    {
      printf (" !!! WARNING , average gap < 0\n") ;
      printf (" !!! FATAL ERROR, because the total size will not accomodate this total length of units.\n") ;
      exit (1) ;
    }
/*   create fictitious genome */
  icln = 0;
  ipnt = 1;
  for (ipass = imixpt; ipass >= 1; --ipass)
    {
      ichc = (long int) (ipass * ran_()) + 1;
      if (mixa[ichc - 1] < 0)
	{
/* position unattached clone */
	  ++icln;
	  mposa[icln - 1] = ipnt;
	  mlena[icln - 1] = ilena[-mixa[ichc - 1] - 1];
	  cdir[icln - 1] = -mixa[ichc - 1];
	  hcln = (float) mlena[icln - 1] * ovlp;
	  ipnt += mlena[icln - 1];
	}
      else
	{
/*   position contig */
	  icont = mixa[ichc - 1];
	  asscont2_(&nclone, &icont, &ncomp, icnum1, &iposav, &icrt);
	  ilft = iposa[icnum1[0] - 1];
	  ictlen = 0;
	  for (iq = 1; iq <= ncomp; ++iq)
	    {
	      ++icln;
	      mposa[icln - 1] = ipnt + iposa[icnum1[iq - 1] - 1] - ilft;
	      mlena[icln - 1] = ilena[icnum1[iq - 1] - 1];
	      cdir[icln - 1] = icnum1[iq - 1];
	      ictlen += mlena[icln - 1];
	    }
	  hcln = (float) ictlen * ovlp / (float) ncomp;
	  ipnt = ipnt + iposa[icrt - 1] + ilena[icrt - 1] - ilft;
	}

/*   set pointer for next entry */
      ipnt += n_int((avgap + hcln - 1.f) * ran_() * 2.f - hcln + 1.f);

/*   close gap in MIXA */
      for (iq = ichc; iq <= ipass - 1; ++iq)
	{
	  mixa[iq - 1] = mixa[iq];
	}
    }
  printf (" ipnt before ran = %ld\n", ipnt) ;
/* add random clones, naming them in order of addition */
  ransort3_(&nclone, &icln, &nrset, &nrclst, mposa, mlena, cdir, 
	    &lengen, &lenran);
  strtcpy (ntemp1, namea + ((cdir[0] - 1) << 3), 8) ;
  strtcpy (ntemp2, namea + ((cdir[icln - 1] - 1) << 3), 8) ;
  printf (" max  pos after ran = %ld\n"
	  " first = %-8s   last = %-8s\n",
	  mposa[icln - 1], ntemp1, ntemp2);
/*   define contigs */
  cont = FALSE;
  icont = 0;
  ipos = mposa[0];
  iprt = ipos + mlena[0];
  for (ia = 2; ia <= icln; ++ia)
    {

      if ((float) (iprt - mposa[ia - 1]) >= mlena[ia - 1] * ovlp)
	{			/* overlap */
	  if (cont)
	    {			/* continue old contig */
	      iprt = max(iprt,mposa[ia - 1] + mlena[ia - 1]);
	      mposa[ia - 2] -= ipos;
	      mconta[ia - 2] = icont;
	    }
	  else			/* new contig */
	    {
	      iprt = max(mposa[ia - 2] + mlena[ia - 2],
			 mposa[ia - 1] + mlena[ia - 1]);
	      cont = TRUE;
	      ++icont;
	      ipos = mposa[ia - 2];
	      mposa[ia - 2] = 0;
	      mconta[ia - 2] = icont;
	    }
	}
      else			/* no overlap */
	{
	  iprt = mposa[ia - 1] + mlena[ia - 1];
	  if (cont)		/* end contig */
	    {
	      cont = FALSE;
	      mposa[ia - 2] -= ipos;
	      mconta[ia - 2] = icont;
	  }
	  else		/* continue unattached */
	    {
	      mposa[ia - 2] = 0;
	    }
	}
    }
/*   deal with last clone */
  ia = icln;
  if (cont)
    {
      mposa[ia - 1] -= ipos;
      mconta[ia - 1] = icont;
  }
  else
    {
      mposa[ia - 1] = 0;
    }
/*   reorganise data for output */
  for (ic = 2; ic <= nclone + 1; ++ic)
    {
      ilena[ic - 1] = -1;
    }
  for (ia = 1; ia <= icln; ++ia)
    {
      ic = cdir[ia - 1];
      iconta[ic - 1] = mconta[ia - 1];
      iposa[ic - 1] = mposa[ia - 1];
      ilena[ic - 1] = mlena[ia - 1];
    }
/*   write database to disc */
  unit_1 = FILopen ("model", "nam", "r+") ;
  unit_2 = FILopen ("model", "cor", "r+") ;
  unit_4 = FILopen ("model", "cnt", "r+") ;
  unit_10= FILopen ("model", "stk", "r+") ;

  iwrt = 1;
  for (ic = 2; ic <= nclone + 1; ++ic)
    {
      if (ilena[ic - 1] < 1)
	{
	  continue ;
	}
      ++iwrt;
      seek_1 (iwrt) ;
      fwrite (namea + ((ic - 1) << 3), sizeof(char), 8, unit_1);
      fwrite (&c__0, sizeof(long int), 1, unit_1);
      fwrite (&c__0, sizeof(long int), 1, unit_1);
      fwrite (&c__0, sizeof(long int), 1, unit_1);
      fwrite (&iconta[ic - 1], sizeof(long int), 1, unit_1) ;

      seek_4 (iwrt) ;
      fwrite (&iposa[ic - 1], sizeof(long int), 1, unit_4);
      fwrite (&ilena[ic - 1], sizeof(long int), 1, unit_4);
      fwrite (&c__0, sizeof(long int), 1, unit_4);
      fwrite (&c__0, sizeof(long int), 1, unit_4);
    }
  printf ("  iwrt = %ld   icln = %ld\n", iwrt, icln) ;
  seek_1 (1) ;
  i__1 = iwrt - 1 ;
  fwrite (&i__1, sizeof(long int), 1, unit_1);
  fwrite (&c__0, sizeof(long int), 1, unit_1);

  rewind (unit_2) ;
  fwrite (&c__1000, sizeof(short int), 1, unit_2);

  seek_10 (1) ;
  i__1 = icont + 1;
  fwrite (&i__1, sizeof(long int), 1, unit_10);

  fclose (unit_1) ;
  fclose (unit_2) ;
  fclose (unit_4) ;
  fclose (unit_10) ;

  return ;
} /* MAIN__ */

#undef tname1b
#undef tnameb
#undef tname1
#undef tname
#undef names
#undef name1



int ransort3_(long int *nclone, long int *icln, long int *nrset,
	      long int *nrclst, long int *jclone, long int *mlena,
	      long int *cdir, long int *ig, long int *isp)
{
  /* Local variables */
  static char name[9];
  static long int i, j, k, m, itemp, iq;
  static long int inr, irs;

  /* Parameter adjustments */
  --cdir;
  --mlena;
  --jclone;
  
  /* Function Body */
  strcpy (name, "ran") ;
  j = *icln;
  k = *nclone + 1;
  for (irs = 1; irs <= *nrset; ++irs)
    {
      name[3] = (char) (irs + 48);
      for (inr = 1; inr <= *nrclst; ++inr)
	{
	  ++j;
	  ++k;
L12:
	  jclone[j] = (long int) (*ig * ran_()) + 1;
	  if (jclone[j] > *ig - *isp)
	    {
	      goto L12;
	    }
	  cdir[j] = k;
	  mlena[j] = *isp;
	  for (iq = 5; iq <= 8; ++iq)
	    {
	      name[iq - 1] = (char) (inr / pow_ii(10, 8 - iq) % 10 + 48);
	    }
	  strncpy (namea + ((k - 1) << 3), name, 8);
	}
    }
  *icln += *nrset * *nrclst;
/*   sort clones */
  *nclone += *nrset * *nrclst;
  m = *icln;
L15:
  m = (m + 2) / 3;
  for (i = m + 1; i <= *icln; ++i)
    {
      for (j = i; -m < 0 ? j >= m + 1 : j <= m + 1; j += -m)
	{
	  if (jclone[j] >= jclone[j - m])
	    {
	      goto L16;
	    }
	  itemp = jclone[j];
	  jclone[j] = jclone[j - m];
	  jclone[j - m] = itemp;
	  itemp = mlena[j];
	  mlena[j] = mlena[j - m];
	  mlena[j - m] = itemp;
	  itemp = cdir[j];
	  cdir[j] = cdir[j - m];
	  cdir[j - m] = itemp;
	}
L16:
      ;
    }
  if (m != 1)
    {
      goto L15;
    }
  return 0;
} /* ransort3_ */

/*   assembles contig, omits ! clones, finds true right end clone */
int asscont1_(long int *nclone, long int *icont, long int *ncomp,
	      long int *icnuma, long int *iposav, long int *icrt)
{
  /* Local variables */
  static long int ipos;
  static char tname[9];
  static long int ix, iy, irt;
  
  /* Parameter adjustments */
  --icnuma;
  
  /* Function Body */
/* protect against contig 0 */
  if (*icont == 0)
    {
      return 0;
    }
/*   initialise right end hunt */
  irt = -100000;
  *ncomp = 0;
/*   loop for each clone */
  for (iy = 2; iy <= *nclone + 1; ++iy)
    {
      if (iconta[iy - 1] == *icont)
	{
/*   ignore buried clones */
	  if (abs(icana[iy - 1]) > 1)
	    {
		goto L30;
	    }
/*   omit cancellations and repeats */
	  strtcpy(tname, namea + ((iy - 1) << 3), 8);
	  if ((*tname == '!' || tname[1] == '!' || tname[2] == '!' ||
	       tname[3] == '!' || tname[4] == '!' || tname[5] == '!' ||
	       tname[6] == '!' || tname[7] == '!') &&
	      icana[iy - 1] != -1)
	    {
	      goto L30;
	    }
	  ipos = iposa[iy - 1];
/*   compute right end */
	  if (ipos + ilena[iy - 1] > irt)
	    {
	      *icrt = iy;
	      irt = ipos + ilena[iy - 1];
	    }
/*   place clone in rank of left ends */
	  ++(*ncomp);
	  for (ix = *ncomp; ix >= 2; --ix)
	    {
	      if (ipos > iposa[icnuma[ix - 1] - 1])
		{
		  icnuma[ix] = iy;
		  goto L30;
		}
	      icnuma[ix] = icnuma[ix - 1];
	    }
	  icnuma[1] = iy;
	}
L30:
      ;
    }
  if (*ncomp == 0)
    {
      return 0;
    }
  *iposav = (iposa[icnuma[1] - 1] + iposa[icnuma[*ncomp] - 1]) / 2;
  return 0;
} /* asscont1_ */

/*   assembles contig, omits ! clones, includes burieds,
     finds true right end clone */
int asscont2_(long int *nclone, long int *icont, long int *ncomp,
	      long int *icnuma, long int *iposav, long int *icrt)
{
  /* Local variables */
  static long int ican, ipos;
  static char tname[9];
  static long int ix, iy, irt;
  
  /* Parameter adjustments */
  --icnuma;
  
  /* Function Body */
/* protect against contig 0 */
  if (*icont == 0)
    {
      return 0;
    }
/*   initialise right end hunt */
  irt = -100000;
    *ncomp = 0;
/*   loop for each clone */
  for (iy = 2; iy <= *nclone + 1; ++iy)
    {
	if (iconta[iy - 1] == *icont)
	{
/*   omit cancellations and repeats */
	  strtcpy(tname, namea + ((iy - 1) << 3), 8);
/*   correct dimensions for buried clones */
	  if ((*tname == '!' || tname[1] == '!' || tname[2] == '!' ||
	       tname[3] == '!' || tname[4] == '!' || tname[5] == '!' ||
	       tname[6] == '!' || tname[7] == '!') && 
	      icana[iy - 1] != -1)
	    {
	      goto L30;
	    }
	  ican = abs(icana[iy - 1]) % 10000000;
	  if (ican > 1)
	    {
	      iposa[iy - 1] = iposa[ican - 1];
	      ilena[iy - 1] = ilena[ican - 1];
	    }
	  ipos = iposa[iy - 1];
/*   compute right end */
	  if (ipos + ilena[iy - 1] > irt)
	    {
	      *icrt = iy;
	      irt = ipos + ilena[iy - 1];
	    }
/*   place clone in rank of left ends */
	  ++(*ncomp);
	  for (ix = *ncomp; ix >= 2; --ix)
	    {
	      if (ipos > iposa[icnuma[ix - 1] - 1])
		{
		  icnuma[ix] = iy;
		  goto L30;
		}
	      icnuma[ix] = icnuma[ix - 1];
	    }
	  icnuma[1] = iy;
	}
L30:
	;
      }
  if (*ncomp == 0)
    {
      return 0;
    }
  *iposav = (iposa[icnuma[1] - 1] + iposa[icnuma[*ncomp] - 1]) / 2 ;
  return 0 ;
} /* asscont2_ */

