/*  File: plot1.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: contains misc graph functions
 * SCCS: $Id: plot1.c,v 1.1 2001/02/14 12:44:39 edgrif Exp $
 * HISTORY:
 * Last edited: Nov 15 17:23 1994 (fw)
 * Created: Thu May 19 17:08:38 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

/*   routines in this library */
/*   line1 */
/*   point1 */
/*   newframe */
/*   search2 */
/*   search1 */
/*   find4 */
/*   blip */
/*   xhairs */
/*   erase */
/*   text */
/*   scales */
/*    line1 */
/*   draws line from xf,yf to xt,yt  on screen of size */
/*   isxmax,isymax with margins */
/*   margxl,margxr and margyl,margyr. the min and max values */
/*   to fit on screen are xmin,xmax and ymin,ymax. the screen */
/*   size is isxmax,isymax. margin sizes are in screen units. */
int line1_(float *xf, float *xt, float *yf, float *yt, float *
	   xmax, float *xmin, float *ymax, float *ymin, long int *margxl,
	   long int *margxr, long int *margyl, long int *margyr,
	   long int *isxmax, long int *isymax)
{
  static float fracx, fracy;
  static long int leftx, lefty;
  static float scalex, scaley, xff, yff;
  static long int ixf, iyf, ixt, iyt;
  static float xtt, ytt;
  
  xff = *xf;
  xtt = *xt;
  yff = *yf;
/*   clip */
  ytt = *yt;
  if (*xf < *xmin)
    {
      xff = *xmin;
    }
  if (*xf > *xmax)
    {
      xff = *xmax;
    }
  if (*xt > *xmax)
    {
      xtt = *xmax;
    }
  if (*xt < *xmin)
    {
      xtt = *xmin;
    }
  if (*yf < *ymin)
    {
      yff = *ymin;
    }
  if (*yf > *ymax)
    {
      yff = *ymax;
    }
  if (*yt > *ymax)
    {
      ytt = *ymax;
    }
/*   remove margin from usable space */
  if (*yt < *ymin)
    {
      ytt = *ymin;
    }
  leftx = *isxmax - *margxl - *margxr;
/*   calc middle of margins for y */
/* 	mid=lefty/2 */
/*   max lengths to display */
  lefty = *isymax - *margyl - *margyr;
  scalex = *xmax - *xmin;
/*   have scale points to fit into left positions */
  scaley = *ymax - *ymin;
  fracx = (float) leftx / scalex;
/*   calc from position */
  fracy = (float) lefty / scaley;
  ixf = *margxl + fracx * (xff - *xmin);
/*   to position */
  iyf = *margyl + fracy * (yff - *ymin);
  ixt = *margxl + fracx * (xtt - *xmin);
/*   draw it */
  iyt = *margyl + fracy * (ytt - *ymin);
  movevt640_(ixf, iyf);
  drawvt640_(ixt, iyt);
  return 0;
/*    point */
} /* line1_ */

/*   draws point x,y on screen of size isxmax,isymax with margins */
/*   margxl,margxr and margyl,margyr. the min and max values */
/*   to fit on screen are xmin,xmax and ymin,ymax. the screen */
/*   size is isxmax,isymax. margin sizes are in screen units. */
/*   remove margin from usable space */
int point1_(float *x, float *y, float *xmax, float *xmin, float *ymax,
	    float *ymin, long int *margxl, long int *margxr,
	    long int *margyl, long int *margyr,
	    long int *isxmax, long int *isymax)
{
  static float fracx, fracy;
  static long int leftx, lefty;
  static float xf, yf;
  static long int ix, iy;
  static float scalex, scaley;

  leftx = *isxmax - *margxl - *margxr;
/*   clip */
  lefty = *isymax - *margyl - *margyr;
  xf = *x;
  yf = *y;
  if (xf < *xmin)
    {
      xf = *xmin;
    }
  if (xf > *xmax)
    {
      xf = *xmax;
    }
  if (yf < *ymin)
    {
      yf = *ymin;
    }
/*   max lengths to display */
  if (yf > *ymax)
    {
      yf = *ymax;
    }
  scalex = *xmax - *xmin;
/*   have scale points to fit into left positions */
  scaley = *ymax - *ymin;
  fracx = (float) leftx / scalex;
/*   calc position */
  fracy = (float) lefty / scaley;
  ix = *margxl + fracx * (xf - *xmin);
/*   plot it */
  iy = *margyl + fracy * (yf - *ymin);
  pointvt640_(ix, iy);
  return 0;
} /* point1_ */

int newframe_(long int *margxl, long int *margxr, long int * margyb,
	      long int *margyt, long int *isxmax, long int *isymax)
{
  static long int ix;
  static long int iy;

  selectvt640_();
  movevt640_(*margxl, *margyb);
  iy = *isymax - *margyt;
  drawvt640_(*margxl, iy);
  ix = *isxmax - *margxr;
  drawvt640_(ix, iy);
  drawvt640_(ix, *margyb);
  drawvt640_(*margxl, *margyb);
  return 0;
} /* newframe_ */

int blip_(float *xmax, float *xmin, float *ymax, float *ymin, 
	  long int *margl, long int *margr, long int *margb,
	  long int *margt, long int *isxmax, long int *isymax)
{
  /* Local variables */
  static long int left;
  static float sinc;
  static long int i;
  static float scale;
  static long int nb;
  static float uu;
  static long int ibh, inc, iat, iyp;
  
  
L1:
  gprintf (stdout, " blip separation(def=1000,<0=stop) = ") ;
  readinput ("", stemp, 10) ; if (nonint(stemp)) goto L1 ;
  inc = a2int (stemp) ;
  if (inc < 0)
    {
      goto L100;
    }
  if (inc == 0)
    {
      inc = 1000;
    }
L2:
  gprintf (stdout, " blip height in screen units(def=10) = ") ;
  readinput ("", stemp, 10) ; if (nonint(stemp)) goto L2 ;
  ibh = a2int (stemp) ;
  if (ibh < 1)
    {
      ibh = 10;
    }
  iyp = *margb + ibh;
  left = *isxmax - *margl - *margr;
  scale = *xmax - *xmin + 1.f;
  uu = (float) left / scale;
  sinc = inc * uu;
  nb = scale / (float) inc;
  clearvt100_();
  selectvt640_();
  for (i = 1; i <= nb; ++i)
    {
      iat = *margl + i * sinc;
      movevt640_(iat, *margb);
      drawvt640_(iat, iyp);
    }
  goto L1;
 L100:
  clearvt100_();
  return 0;
/*   xhairs */
} /* blip_ */

int xhairs_(float *xmax, float *xmin, float *ymax, float *ymin, 
	    long int *margl, long int *margr, long int *margb,
	    long int *margt, long int *isxmax,
	    long int *isymax, float *x, float *y)
{
  /* Local variables */
  static float scalx, scaly;
  static long int leftx, lefty;
  static long int ih1, ih2, ihc, ihx, ihy;
  static float sux, suy;
  
  scalx = *xmax - *xmin + 1.f;
  scaly = *ymax - *ymin + 1.f;
  ih1 = *xmin;
  ih2 = *ymin;
  leftx = *isxmax - *margl - *margr;
  lefty = *isymax - *margb - *margt;
/* one screen unit in seq nos */
  sux = scalx / leftx;
  suy = scaly / lefty;
/* go into vt640 mode */
  selectvt640_();
  xhairvt640_(&ihx, &ihy, &ihc);
/* back to vt100 mode */
  selectvt100_();
/* calc position in seq units */
  ihx = ih1 + (float) (ihx - *margl) * sux;
  ihy = ih2 + (float) (ihy - *margb) * suy;

  gprintf (stdout, " xhair at %d %d\n", ihx, ihy);
  return 0;
/*   erase */
} /* xhairs_ */

int erase_(void)
{
  /* Local variables */
  static long int i;
  static long int ic, mb;
  static long int ml, mr, ix;
  static long int iy, mt;
  
  selectvt100_();
  gprintf (stdout, " define box to erase using xhairs\n"
	  " position left margin\n") ;
  selectvt640_();
  xhairvt640_(&ml, &iy, &ic);
  selectvt100_();
  gprintf (stdout, " position right margin\n") ;
  selectvt640_();
  xhairvt640_(&mr, &iy, &ic);
  selectvt100_();
  gprintf (stdout, " position bottom margin\n") ;
  selectvt640_();
  xhairvt640_(&ix, &mb, &ic);
  selectvt100_();
  gprintf (stdout, " position top margin\n") ;
  selectvt640_();
  xhairvt640_(&ix, &mt, &ic);
  selectvt640_();
  colorvt640_(WHITE) ;
  for (i = mb; i <= mt; ++i)
    {
      movevt640_(ml, i);
      drawvt640_(mr, i);
    }
  colorvt640_(BLACK) ;
  selectvt100_();
  return 0;
} /* erase_ */

/*    text */
int text_(float *x, float *y, char *chars, long int *nchars,
	  float *xmax, float *xmin, float *ymax,
	  float *ymin, long int *margxl, long int *margxr,
	  long int *margyl, long int *margyr, 
	  long int *isxmax, long int *isymax)
{
  static float fracx, fracy;
  static long int leftx, lefty;
  static float scalex, scaley, xff, yff;
  static long int ixf, iyf;

/*   write nchars chars on screen of size */
/*   isxmax,isymax with margins */
/*   margxl,margxr and margyl,margyr. the min and max values */
/*   to fit on screen are xmin,xmax and ymin,ymax. the screen */
/*   size is isxmax,isymax. margin sizes are in screen units. */
  
  /* Function Body */
  xff = *x;
  yff = *y;
  if (xff > *xmax)
    {
      xff = *xmax;
    }
  if (xff < *xmin)
    {
      xff = *xmin;
    }
  if (yff > *ymax)
    {
      yff = *ymax;
    }
/*   remove margin from usable space */
  if (yff < *ymin)
    {
      yff = *ymin;
    }
  leftx = *isxmax - *margxl - *margxr;
/*   max lengths to display */
  lefty = *isymax - *margyl - *margyr;
  scalex = *xmax - *xmin;
/*   have scale points to fit into left positions */
  scaley = *ymax - *ymin;
  fracx = (float) leftx / scalex;
/*   calc position */
  fracy = (float) lefty / scaley;
  ixf = *margxl + fracx * (xff - *xmin);
/*   write it */
  iyf = *margyl + fracy * (yff - *ymin);
  movevt640_(ixf, iyf+8);	/* +8 is fw-hack */
  textvt640_(chars, *nchars);
  return 0;
} /* text_ */

/*   routine to put a scale along an axis either x or y */
/*   depending on iscale (1=x,2=y axes) */
/*   blips of height ibliph are placed along the axis every */
/*   bit of length starting from first. first can be used */
/*   to make sure that a blip occurs at a particular position */
/*   for example at a zero point, otherwise first is set to */
/*   xmin or ymin */
/*   x or y axis? */
int scales_(float *xmax, float *xmin, float *ymax, float *ymin, 
	    long int *margl, long int *margr, long int *margb,
	    long int *margt, long int *isxmax, long int *isymax,
	    float *bit, long int *ibliph, float *
	    first, long int *iscale)
{
  /* System generated locals */
  long int i__1;
  
  /* Local variables */
  static long int iend;
  static long int left;
  static float sinc;
  static long int i;
  static float scale;
  static long int np;
  static float uu;
  static long int istart;
  static float ufirst;

/*  x axis */
/*   how much screen left? */
  if (*iscale == 2)
    {
      goto L20;
    }
/*   scale in user units? */
  left = *isxmax - *margl - *margr;
/*   screen scale in user units */
  scale = *xmax - *xmin;
/*   increment size in screen units */
  uu = (float) left / scale;
/*   number of points to plot? */
  sinc = *bit * uu;
/*   blip endpoint */
  np = (*xmax - *first) / *bit + 1;
/*   first in screen units */
  iend = *margb + *ibliph;
/*  plot */
  ufirst = (*first - *xmin) * uu;
  i__1 = np;
  for (i = 1; i <= i__1; ++i)
    {
      istart = *margl + (i - 1) * sinc + ufirst;
      movevt640_(*margb, istart);
      drawvt640_(iend, istart);
    }
  return 0;
/*   yscale */
/*   how much screen left? */
L20:
/*   scale in user units? */
  left = *isymax - *margb - *margt;
/*   screen scale in user units */
  scale = *ymax - *ymin;
/*   increment size in screen units */
  uu = (float) left / scale;
/*   number of points to plot? */
  sinc = *bit * uu;
/*   blip endpoint */
  np = (*ymax - *first) / *bit + 1;
/*   first in screen units */
  iend = *margl + *ibliph;
/*  plot */
  ufirst = (*first - *ymin) * uu;
  i__1 = np;
  for (i = 1; i <= i__1; ++i)
    {
      istart = *margb + (i - 1) * sinc + ufirst;
      movevt640_(*margl, istart);
      drawvt640_(iend, istart);
    }
  return 0;
} /* scales_ */



