/*  File: pmap.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: contains main function for pmap program
 * SCCS: $Id: pmap.c,v 1.1 2001/02/14 12:44:39 edgrif Exp $
 * HISTORY:
 * Last edited: Nov 27 22:05 1995 (rd)
 * Created: Fri May 27 11:24:28 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;			/* .nam */
extern FILE *unit_4 ;			/* .cnt */
extern FILE *unit_10 ;			/* .stk */
extern FILE *unit_11 ;			/* .rem */
extern FILE *unit_12 ;		        /* cemap.gen */
extern FILE *unit_13 ;			/* .flg */

/* Common Block Declarations */

/* csec CB */
short int coorda[BANDMAX];

/* cint CB */
long int nbanda[30000], lbanda[30000], igela[30000], iconta[30000],
         iposa[30000], ilena[30000], icana[30000], irema[30000];

/* chars CB */
char namea[240000], chrom[65];

/* matdat CB */
long int itolmn, itoldf;
float logfact[201];

/* chr1 CB */
float xzero1, scale1;
long int ichrgp, maxlev1, levgap1;
short int genlev1[6000]	/* was [6][1000] */;

/* chr CB */
float xzero, scale1 ;
long int chrgap, maxlev, levgap;
short int genlev[6000]	/* was [6][1000] */;
float csiz, sitlev[1200], scale;

/* cass CB */
long int gpos[400], glen[400], gclst[400], ngrp;
short int gmat[160000]	/* was [400][400] */;

/* BLNK CB */
long int isch, icurs, ibtot, iwend;


/* Table of constant values */

static long int c__380 = 380;
static long int c__15 = 15;
static long int c__100 = 100;
static long int c__924 = 924;
static long int c__500 = 500;

Graph graph ;
float wChar, hChar ;
float winWidth, winHeight ;


void dummy (void) {;}

static void about (void)
{
  messout
    ("Pmap\n"
     "Analysis tool for the ContigC package.\n "
     "Authors :\n"
     "John Sulston et al. (jes@sanger.ac.uk)\n"
     "     - original FORTRAN program\n"
     "Friedemann Wobus (fw@sanger.ac.uk)\n"
     "     - UNIX port and new graphics\n"
     "Co-authors :\n"
     "Richard Durbin (rd@sanger.ac.uk)\n"
     "     - graphics library");
}

/* PMAP */

void main (int argc, char **argv)
{
  char projectbase[128] ;
  int dx, dy ;
  extern void pmap (char *) ;
  MENUOPT pmapMenu[] = {
    {dummy,""},
    {graphPrint, "Print"},
    {about, "About"},
    {dummy,""},
    {0,0}};

  if (argc > 1 && *argv[1] != '-')
    strcpy (projectbase, argv[1]) ;
  else
    memset (projectbase, 0, 128) ;

  graphInit (&argc, argv) ;
  graph = graphCreate (TEXT_FIT, "PMAP",
		       0, 0, WIN_WIDTH/900.0, WIN_HEIGHT/900.0) ;

  graphRetitle ("PMAP (graphical vt640 emulation)") ;
  graphMenu (pmapMenu) ;
  graphTextFormat (FIXED_WIDTH) ;
  graphWhere (0, 0, &winWidth, &winHeight) ;
  graphTextInfo (&dx, &dy, &wChar, &hChar) ;

  gprintf (stdout,
	   "\n  -=* Welcome to PMAP *=-\n\n"
	   " This program provides readonly access to genome map projects,\n"
	   " and offers analysis and printing facilities.\n"
	   "\n  project name = \n") ;
  graphRedraw () ;

  graphFortranEntry (projectbase, 128, 17*wChar, 6*hChar, pmap) ;
  graphStart(graph) ;		/* start main X-event loop */
}

void pmap (char *projectbase)
{
  /* System generated locals */
  static char equiv_0[9];
#define tname1 (equiv_0)
#define tname1b ((char *)equiv_0)

    /* Local variables */
  static char tname[9];
  static long int ican;
  static char gene[9];
  static long int iabs;
  static char name[9];
  static long int gdir[2000];
  static long int igel;
  static long int icnr, ipcx, iret, igpt;
  static long int ixlm, ipos, ixrm;
  static long int ityp, ixrt, ixst;
  static char gene1[9];
  static float pchr1;
  static long int i, lband, idatc;
  static long int nband, imode;
  static long int icont;
  static long int itemp, izero;
  static BOOL print, tripr;
  static short int hflag1;
  static long int icnum1[4000], ncomp1;
  static short int hsitu1;
  static long int id;
  static long int iq, is, iscale, ix;
  static long int iz, iy, nbands;
  static long int iclone, nclone;
  static short int hchrom;
  static char remark[41];
  static long int ichrom, ishift, iconto;
  static long int icontt, iposav;
  static short int hchrom1;
  static char remark1[41];
  static BOOL key;
  static long int ixs, ird;
  
  
  movevt100_(17, 6) ;
  gprintf (stdout, "%s\n", projectbase) ;

  /* init chrom[] array */
  strcpy (chrom,
	  "     "" (I) "" (II)""(III)"" (IV)"
	  " (V) "" (X) "" SIT "" GEN "" BOTH") ;
  itolmn = 7 ; itoldf = 10 ;
/* display all remarks */
  imode = 1 + 20;
  goto L1 ;
L89:
  gprintf (stdout, "Please enter valid project name !\n\n") ;
  gprintf (stdout, "  project name = ") ;
  readinput (projectbase, projectbase, 50) ;
L1:  
  if (strlength(projectbase, 128) == 0)
    {
      exit (1) ;
    }
  graphRetitle (messprintf ("PMAP (%s)", projectbase)) ;

/* start project pickup */

  if (!(unit_1 = FILopen (projectbase, "nam", "r"))) goto L89 ;
  if (!(unit_4 = FILopen (projectbase, "cnt", "r"))) goto L89 ;
  if (!(unit_10 = FILopen (projectbase, "stk", "r"))) goto L89 ;
  if (!(unit_11 = FILopen (projectbase, "rem", "r"))) goto L89 ;
  if (!(unit_13 = FILopen (projectbase, "flg", "r"))) goto L89 ;

  if (strcmp (projectbase, "cemap") == 0)
    {
      unit_12 = FILopen ("cemap", "gen", "R") ;
    }

  seek_13 (1) ;
  fread (&idatc, sizeof(long int), 1, unit_13);
  if (FILerror(unit_13)) goto L90 ;
  gprintf (stdout, "\n") ;
L90:
  gprintf (stdout, "  data being loaded, database version %d\n\n", idatc) ;
  
  seek_1 (1) ;
  fread (&nclone, sizeof(long int), 1, unit_1) ;
  fread (&nbands, sizeof(long int), 1, unit_1) ;

  /*   read in data to arrays */
/* old method using fread
  for (ix = 2; ix <= nclone + 1; ++ix)
    {
      seek_1 (ix) ;
      fread (namea + ((ix - 1) << 3), sizeof(char), 8, unit_1) ;
      fread (&nbanda[ix - 1], sizeof(long int), 1, unit_1) ;
      fread (&lbanda[ix - 1], sizeof(long int), 1, unit_1) ;
      fread (&igela[ix - 1], sizeof(long int), 1, unit_1) ;
      fread (&iconta[ix - 1], sizeof(long int), 1, unit_1) ;

      seek_4 (ix) ;
      fread (&iposa[ix - 1], sizeof(long int), 1, unit_4) ;
      fread (&ilena[ix - 1], sizeof(long int), 1, unit_4) ;
      fread (&icana[ix - 1], sizeof(long int), 1, unit_4) ;
      fread (&irema[ix - 1], sizeof(long int), 1, unit_4) ;
    }
*/
  {
    int unit_nam, unit_cnt ;
    unsigned long_len = sizeof(long int), char_len = 8 * sizeof(char) ;
    unit_nam = open(messprintf("%s.nam", projectbase), O_RDONLY) ;
    unit_cnt = open(messprintf("%s.cnt", projectbase), O_RDONLY) ;
    lseek (unit_nam, 8*sizeof(char)+4*sizeof(long int), SEEK_SET) ;
    lseek (unit_cnt, 4*sizeof(long int), SEEK_SET) ;
    for (ix = 1; ix < nclone + 1; ++ix)
      {
	read (unit_nam, namea + (ix << 3), char_len) ;
	read (unit_nam, &nbanda[ix], long_len) ;
	read (unit_nam, &lbanda[ix], long_len) ;
	read (unit_nam, &igela[ix],  long_len) ;
	read (unit_nam, &iconta[ix], long_len) ;

	read (unit_cnt, &iposa[ix],  long_len) ;
	read (unit_cnt, &ilena[ix],  long_len) ;
	read (unit_cnt, &icana[ix],  long_len) ;
	read (unit_cnt, &irema[ix],  long_len) ;
      }
    close (unit_nam) ; close (unit_cnt) ;
  }
  is = 1;
  i = 0;
  id = 0;
  icont = 0;
  iconto = 0;
  ixs = 0;
  iabs = 3;
  izero = 0;
  ipcx = 0;
  iscale = 5;
  print = FALSE;
  tripr = FALSE;
  key = FALSE;
  ityp = 0;
L8:
/*   get clone/gene/control name */
  selectvt100_();
  gprintf (stdout, "   clone name/control (H gets help) = ") ;
  readinput ("", name, 8) ;
/* first check for control name */
  if (strlength(name, 8) == 0)
    {
      if (key)
	{
	  keysub_(&key, &i, &nclone, &iret);
	  if (key)
	    {
	      goto L2;
	    }
	  goto L8;
	}
      clearvt640_();
      clearvt100_();
      goto L8;
    }
  if (strcmp(name, "*") == 0)
    {
      goto L13;
    }
  if (strcasecmp(name, "h") == 0 || strcasecmp(name, "help") == 0)
    {
      gprintf (stdout, "\n"
	       "    <RET>  clear screen or continue keysearch\n"
	       " <CTRL>+L  clear graphics and reveal text\n"
	       "     DOC   introductory documentation\n"
	       "   SCALE   set scale\n"
	       "    GENE   get gene\n"
	       "      SD   show related clones\n"
	       "      SC   scroll contig-view left or right\n"
	       "       [   scroll contig-view a page to the left\n"
	       "       ]   scroll contig-view a page to the right\n"
	       "     CTR   centre contig on current clone\n"
	       "     SLM   set left marker (initial value = left end of contig)\n" 
	       "     SRM   set right marker (initial value  = right end of contig)\n" 
	       "     MLM   move to left marker\n" 
	       "     MRM   move to right marker\n" 
	       "      TY   type contig\n"
	       "    CONT   get contig\n"
	       "     GIS   genetic and in situ plot\n"
	       "      PC   print whole or part of current contig\n"
	       "     KEY   enter KEYword mode\n"
	       "   UNKEY   leave keyword mode\n"
	       "   CHPLT   plot chromosomes\n"
	       " CONTASS   position and plot buried clones\n"
	       "       *   quit the program\n"
	       ) ;
	goto L8;
    }
  if (strcasecmp(name, "doc") == 0)
    {
      cemaphelp_();
      goto L8;
    }
  if (strcasecmp(name, "ctr") == 0)
    {
      if (i == 0)
	{
	  goto L8;
	}
      else if (abs(icana[i - 1]) > 1)
	{
	  ican = abs(icana[i - 1]) % 10000000;
	  ipos = iposa[ican - 1];
	}
      else
	{
	  ipos = iposa[i - 1];
	}
      ixs = -ipos - ilena[i - 1] / 2;
      goto L7;
    }
/* scroll to the left by changing ixs :
 IXS is a variable that converts clone posisitons (ipos)
 to sensible screen dimensions. To get a clone with ipos=5000
 on the screen, we need to set ixs to arounbd -5000 */
  if (strcmp(name, "[") == 0 || strcmp(name, "{") == 0)
    {
      ixs = min(ixst,ixs + 750 / iscale);
      goto L7;
    }
/* scroll to the right */
  if (strcmp(name, "]") == 0 || strcmp(name, "}") == 0)
    {
      ixs = max(ixrt,ixs - 750 / iscale);
      goto L7;
    }
  if (strcasecmp(name, "slm") == 0)
    {
      ixlm = ixs;
      gprintf (stdout, "  left marker set\n") ;
      goto L8;
    }
  if (strcasecmp(name, "srm") == 0)
    {
      ixrm = ixs;
      gprintf (stdout, "  right marker set\n") ;
      goto L8;
    }
  if (strcasecmp(name, "mlm") == 0)
    {
      ixs = ixlm;
      goto L7;
    }
  if (strcasecmp(name, "mrm") == 0)
    {
      ixs = ixrm;
      goto L7;
    }
  if (strcasecmp(name, "key") == 0)
    {
      keysub_(&key, &i, &nclone, &iret);
      if (key)
	{
	  goto L2;
	}
      else
	{
	  goto L8;
	}
    }
  if (strcasecmp(name, "unkey") == 0)
    {
      key = FALSE;
      gprintf (stdout, " key mode cancelled\n") ;
      goto L8;
    }
  if (strcasecmp(name, "gis") == 0)
    {
      cechr_();
      goto L8;
    }
  if (strcasecmp(name, "contass") == 0) 
    {
      cntss_(&nclone, projectbase, &tripr, &i);
      goto L8;
    }
  if (strcasecmp(name, "chplt") == 0)
    {
      chplt_(&nclone, FALSE);
      goto L8;
    }
  if (strcasecmp(name, "pc") == 0)
    {
      int dim ;
      dim = prcont1_(ncomp1, icnum1, iscale, ixst, ixrt, ixlm, ixrm) ;
      if (dim > 0) graphBoundsPrint((float)MAPX(dim), winHeight, 0) ;
      goto L7 ;
    }
  if (strcasecmp(name, "cont") == 0)
    {
L75:
      gprintf (stdout, " contig number = ") ;
      readinput ("", stemp, 6) ; if (nonint(stemp)) goto L75 ;
      ird = a2int (stemp) ;
      if (ird == 0) goto L8 ;
      for (i = 2; i <= nclone + 2; ++i)
	if (iconta[i - 1] == ird) goto L77 ;
      gprintf (stdout, "  contig not found\n") ;
      goto L75 ;
L77:
      seek_1 (i) ;
      fread (tname, sizeof(char), 8, unit_1) ;
      fread (&nband, sizeof(long int), 1, unit_1) ;
      fread (&lband, sizeof(long int), 1, unit_1) ;
      fread (&igel, sizeof(long int), 1, unit_1) ;
      fread (&icontt, sizeof(long int), 1, unit_1) ;
      is = 1 ;
      goto L37 ;
    }

  if (strcasecmp (name, "ty") == 0)
    {
L1063:
      gprintf (stdout, "   contig number (-1 for contig 0) = ") ;
      readinput ("", stemp, 6) ; icontt = a2int (stemp) ;
      if (icontt == 0)
	{
	  goto L8 ;
	}
      gprintf (stdout, "\n") ;
      tycont_(&icontt, stdout, &nclone) ;
      goto L1063 ;
    }
  if (strcasecmp(name, "scale") == 0)
    {
      scale_(&iscale);
      if (i == 0)
	{
	  goto L8;
	}
      goto L7;
    }
  if (strcasecmp(name, "gene") == 0) 
    {
L67:
      gprintf (stdout, "   gene name (L for list, "
	      "LS for list + contig sizes) = ") ;
      readinput ("", gene, 8) ;
      if (strlength(gene, 8) == 0)
	{
	  goto L8;
	}
      if (strcasecmp(gene, "l") == 0)
	{
	  gensort_(gdir, &igpt);
	  gprintf (stdout, " clone       gene              "
		   "genetic%%    in situ%%\n");
	  for (iq = 1; iq <= igpt; ++iq)
	    {
	      seek_11 (gdir[iq - 1]) ;
	      fread (&iclone, sizeof(long int), 1, unit_11) ;
	      fread (gene1, sizeof(char), 8, unit_11) ;
	      fread (remark1, sizeof(char), 40, unit_11) ;
	      fread (&hchrom1, sizeof(short int), 1, unit_11) ;
	      fread (&hflag1, sizeof(short int), 1, unit_11) ;
	      fread (&pchr1, sizeof(float), 1, unit_11) ;
	      fread (&hsitu1, sizeof(short int), 1, unit_11) ;
	      strtcpy (ntemp1, namea + ((iclone - 1) << 3), 8) ;
	      strtcpy (ntemp2, chrom + hchrom1 * 5, 5) ;
	      gprintf (stdout, " %-8s    %-8s%5s   %6.1f     %3d -%3d\n",
		      ntemp1, gene1, ntemp2,
		      pchr1, hsitu1 / 128, hsitu1 % 128) ;
	    }
	  goto L67;
	}
      else if (strcasecmp(gene, "ls") == 0)
	{
	  lsiz_(&nclone);
	  goto L67;
	}
      iz = 0;
L68:
      ++iz;
      seek_11 (iz) ;
      fread (&iclone, sizeof(long int), 1, unit_11);
      fread (gene1, sizeof (char), 8, unit_11);
      if (strncmp(gene1, gene, 8) == 0)
	{
	  goto L66;
	}
/* check for end of file */
      if (iclone != -9)
	{
	  goto L68;
	}
/* because gene not found */
      goto L67;
L66:
      i = iclone;
      seek_1 (i) ;
      fread (tname, sizeof (char), 8, unit_1);
      fread (&nband, sizeof(long int), 1, unit_1);
      fread (&lband, sizeof(long int), 1, unit_1);
      fread (&igel, sizeof(long int), 1, unit_1);
      fread (&icontt, sizeof(long int), 1, unit_1);
      is = 1;
      goto L37;
    }
  if (strcasecmp(name, "sh id") == 0 || strcasecmp(name, "sd") == 0) 
    {
      iy = abs(icana[i - 1]) % 10000000;
      if (iy == 0)
	{
	  goto L8;
	}
      if (iy > 1)
	{
	  ican = iy;
	  
	  strtcpy (ntemp1, namea + ((iy - 1) << 3), 8) ;
	  gprintf (stdout, "%4d-%-8s(%3db,%4d)           (canonical)\n",
		  igela[iy - 1],
		  ntemp1,
		  nbanda[iy - 1],
		  iconta[iy - 1]) ;
	}
      if (iy == 1)
	{
	  ican = i;
	}
      icnr = ican + 10000000;

      for (ix = 2; ix <= nclone + 1; ++ix)
	{
	  if (icana[ix - 1] == ican)
	    {
	      strtcpy (ntemp1, namea + ((ix - 1) << 3), 8) ;
	      gprintf (stdout, "%4d-%-8s(%3db,%4d)\n",
		      igela[ix - 1],
		      ntemp1,
		      nbanda[ix - 1],
		      iconta[ix - 1]) ;
	    }
	  else if (icana[ix - 1] == -ican)
	    {
	      strtcpy (ntemp1, namea + ((ix - 1) << 3), 8) ;
	      gprintf (stdout, "%4d-%-8s(%3db,%4d)           (partial identity)\n",
		      igela[ix - 1],
		      ntemp1,
		      nbanda[ix - 1],
		      iconta[ix - 1]) ;
	    }
	  else if (icana[ix - 1] == icnr)
	    {
	      strtcpy (ntemp1, namea + ((ix - 1) << 3), 8) ;
	      gprintf (stdout, "%4d-%-8s(%3db,%4d)           (limbo)\n",
		      igela[ix - 1],
		      ntemp1,
		      nbanda[ix - 1],
		      iconta[ix - 1]);
	    }
	}
      goto L8;
    }
  if (strcasecmp(name, "sc") == 0)
    {
L43:
      gprintf (stdout, "   scroll (in band units) = ") ;
      readinput ("", stemp, 6) ; if (nonint(stemp)) goto L43 ;
      ishift = -a2int (stemp) ;
      ixs = max(ixrt,min(ixst,ixs + ishift));
      goto L7;
    }
/* not control name, so find name */
  for (i = 2; i <= nclone + 1; ++i)
    {
      if (strncmp(namea + ((i - 1) << 3), name, 8) == 0)
	{
	  goto L2;
	}
    }
  gprintf (stdout, " clone not found\n") ;
  goto L8;
L2:
  seek_1 (i) ;
  fread (tname, sizeof(char), 8, unit_1);
  fread (&nband, sizeof(long int), 1, unit_1);
  fread (&lband, sizeof(long int), 1, unit_1);
  fread (&igel, sizeof(long int), 1, unit_1);
  fread (&icontt, sizeof(long int), 1, unit_1);
L37:
  /* initialise new contig */
  iconto = icont;
  icont = icontt;
  if (icont != iconto || icont == 0)
    {
      asscont4_(&nclone, &icont, &ncomp1, icnum1, &iposav);
      if (icont == 0)
	{
	  ncomp1 = 1;
	  icnum1[0] = i;
	  icnum1[3999] = i;
	  icnum1[3998] = i;
	  ichrom = 0;
	  if (irema[i - 1] > 0)
	    {
	      seek_11 (irema[i - 1]) ;
	      fread (&itemp, sizeof(long int), 1, unit_11);
	      fread (tname1, sizeof(char), 8, unit_11);
	      fread (remark, sizeof(char), 40, unit_11);
	      fread (&hchrom, sizeof(short int), 1, unit_11);
	      ichrom = hchrom;
	    }
	  icnum1[3997] = ichrom;
	}
      ixst = -iposa[icnum1[3998] - 1];
      ixrt = -iposa[icnum1[3999] - 1] - ilena[icnum1[3999] - 1];
      ixlm = ixst;
      ixrm = ixrt;
    }
  if (abs(icana[i - 1]) > 1)
    {
      ican = abs(icana[i - 1]) % 10000000;
      ipos = iposa[ican - 1];
    }
  else
    {
      ipos = iposa[i - 1];
    }
  ixs = -ipos - ilena[i - 1] / 2;
L7:
  clearvt640_();
  selectvt640_();
/* plot current contig above */
  plotcont4_(&ixs, &c__380, &iscale, &ncomp1, icnum1,
	     projectbase, &nclone, &imode);
  emphclone1_(&i, &ixs, &c__380, &ncomp1, icnum1, &iscale);
/* plot small-scale contig below */
  plotcont5_(&ixs, &c__15, &c__100, &c__924, &c__500,
	     &iscale, &ncomp1, icnum1);
  movevt640_(0, 780);
  selectvt100_();
  clearvt100_();
  movevt100_(0, 0);

  strtcpy (ntemp1, tname, 8) ;
  gprintf (stdout, "\n%4d-%-8s(%3db,%4d)\n",
	   igel, ntemp1, nband, icont);
  goto L8;
L13:
  fclose (unit_1) ;
  fclose (unit_4) ;
  fclose (unit_11) ;
  graphFinish () ;
} /* main */

#undef tname1b
#undef tname1
#undef tname


/*   emphasises clone IEM */
int emphclone1_(long int *iem, long int *ixs, long int *iys, 
		long int *ncomp, long int *icnuma, long int *iscale)
{
  /* Local variables */
  static long int ican;
  static long int ipye, ipye1, ipye2, ipxfe, ixmin,
  ixmax, ipxte, ix, iy, imy;
  
  /* Parameter adjustments */
  --icnuma;
  
  /* Function Body */
  ixmin = 10;
  ixmax = 1022;
  ican = abs(icana[*iem - 1]) % 10000000;
  if (ican > 1)
    {
      iy = ican;
    }
  else
    {
      iy = *iem;
    }
  for (ix = 1; ix <= *ncomp; ++ix)
    {
      if (icnuma[ix] == iy)
	{
	  ipxfe = (*ixs + iposa[iy - 1]) * *iscale + 500;
	  ipxte = ipxfe + ilena[iy - 1] * *iscale;
	  if (ipxfe > ixmax || ipxte < ixmin)
	    {
	      return 0;
	    }
	  ipxfe = max(ipxfe,ixmin);
	  ipxte = min(ipxte,ixmax);
	  ipye = *iys + (ix - 1) % 8 * 30 + (ix - 1) / 8 % 2 * 15;
	  imy = ipye % 13;
	  ipye1 = ipye - 2;
	  ipye2 = ipye + 1;

	  colorvt640_(RED) ;
	  movevt640_(ipxfe, ipye1);
	  rectvt640_(ipxte, 3) ;
	  colorvt640_(BLACK) ;
/*
	  drawvt640_(ipxte, ipye1);
	  movevt640_(ipxfe, ipye2);
	  drawvt640_(ipxte, ipye2);
*/
	  return 0;
	}
    }
  return 0;
} /* emphclone1_ */

int cemaphelp_(void)
{
  gprintf (stdout, "\n"
	   " PMAP begins by prompting you for a clone name or a\n"
	   " control code.  The control codes are listed in the help\n"
	   " menu, and some of them are explained more fully below. If\n"
	   " you give a clone name, the appropriate contig will be\n"
	   " displayed with the chosen clone (or, if buried, the\n"
	   " canonical clone) centred on the screen and highlighted.\n"
	   " Clones are represented by horizontal lines, with names\n"
	   " written above.  The displayed clones are known as canonical,\n"
	   " and have mostly been positioned not only by computation but\n"
	   " also by manual checking of the fingerprints or other data. \n"
	   " Asterisks indicate the presence of so-called buried clones,\n"
	   " which can be listed by the command SD or displayed by\n"
	   " CONTASS.  Bracketed names designate clones that have no\n"
	   " fingerprints but have been positioned by hybridisation data.\n"
	   " Immediately below the field of clones is a line reserved for\n"
	   " gene names, and below that is a series of lines for remarks.\n"
	   );
  gprintf (stdout,
	   " SCALE  allows you to alter the horizontal scale of the\n"
	   " contig.\n"
	   "\n"
	   " SH,[,],CTR  allow you to move through the contig in either\n"
	   " direction or to re-centre it on the current clone.\n"
	   "\n"
	   " GIS  plots genetic and in situ data for a chosen chromosome.\n"
	   "\n"
	   " CHPLT  uses genetic, in situ, and contig data to plot the\n"
	   " current best estimate for the overall map of a given\n"
	   " chromosome.  Each contig is represented by a continuous\n"
	   " horizontal line.\n"
	   "\n"
	   " CONTASS  uses fingerprint data to plot the most probable\n"
	   " positions of buried clones in a chosen region.  The\n"
	   " canonical clones are left undisturbed, and are highlighted\n"
	   " in the plot.  This routine is a useful aid in the selection\n"
	   " of clones for injection.\n") ;
  gprintf (stdout,
	   " KEY  provides search facilities for clones, genes, and\n"
	   " remarks.  Keywords can include degenerate characters,\n"
	   " specified by symbols in curly brackets (I:integer, L:letter,\n"
	   " A:any character), e.g. C{II}A1 searches for C followed by\n"
	   " two integers followed by A1.  Enter the optionst as a single\n"
	   " string, e.g.:  FC searches the clones for matches of the\n"
	   " keyword with the beginning of each clone name, and displays\n"
	   " each contig only once;  R searches the remarks for matches\n"
	   " of the keyword at any point.  When in KEYmode, a default on\n"
	   " the clone name prompt causes the search to move on to the\n"
	   " next occurrence; when all have been found, the keyword\n"
	   " prompt appears again.\n"
	   "\n"
	   " UNKEY allows premature exit from KEYmode.\n"
	   "\n") ;
    return 0;
} /* cemaphelp_ */

