/*  File: rancl5.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: requests genome and insert size, number of clones and
   minimum overlap of two clones to count as a join. It then generates
   a random database and prints an analysis of the following
   characteristics: number of contigs and unattached clones,
   percentage of genome in each, mean and maximum contig size, mean
   and maximum gap size.
 * SCCS: $Id: rancl.c,v 1.1 2001/02/14 12:44:39 edgrif Exp $
 * HISTORY:
 * Last edited: Feb 21 14:38 1995 (fw)
 * Created: Wed Jun 22 16:41:56 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

extern int param1_(long int *, long int *, float *, long int *, 
		   long int *, long int *, long int *,
		   long int *, long int *, long int *,
		   long int *, long int *, long int *);
extern int ransort_(long int *, long int *,
		    long int *, long int *);

/*   programme RANCL5 */
void main (void)
{
  /* System generated locals */
  long int i__1, i__2;

  /* Local variables */
  static float occa[4];
  static long int ngap, icov;
  static float rcov;
  static long int maxconta[4], ngapa[4], ltgap, ncint[4], ncext[4], ncont, 
	    nuext[4];
  static long int ic, ig;
  static float eccova[4];
  static long int mlgapa[4], ix, iy, jclone[100000];
  static char sw[1];
  static long int maxgap, nclone, nconta[4];
  static float eucova[4];
  static long int ncontd[4], nother[4], ltcont, istart, nunatt;
  static float fvp;
  static long int maxgapa[4], ncloned, njoincc[4], mlconta[4], ltconta[4], 
     njoincu[4], maxcont, nunatta[4], nunattd[4], njoinuu[4];
 
  printf ("  Programme places clones at random on a genome\n"
	  "  up to 100000000 and computes contig parameters\n\n") ;

L1:
  printf ("  genome size = ") ;
  readinput ("", stemp, 10) ; ig = a2int (stemp) ;
  printf ("  insert size = ") ;
  readinput ("", stemp, 10) ; ic = a2int (stemp) ;

  if (ig > 100000000 || ic >= ig)
    {
      goto L1;
    }
L2:
  printf ("   minimum fraction for overlap (0.0-1.0) = ") ;
  readinput ("", stemp, 10) ; fvp = a2flt (stemp) ;
  if (fvp < 0.f || fvp > 1.f)
    {
      goto L2;
    }
  
  printf (" genome = %ld   insert = %ld   overlap = %.3f\n", ig, ic, fvp) ;
L14:
  printf ("\n\n   number of clones = ") ;
  readinput ("", stemp, 10) ; nclone = a2int (stemp) ;
  if (nclone < 1)
    {
      exit (1) ;
    }
/* compute clone positions */
  istart = 1;
  i__1 = ig - ic;
  ransort_(&istart, &nclone, jclone, &i__1);

/* compute contig parameters */
  ix = 2;
  param1_(&ic, &ig, &fvp, jclone, &nclone, &nunatt, &maxcont,
	  &ltcont, &ncont, &ngap, &ltgap, &maxgap, &icov);
  nconta[ix - 1] = ncont;
  nunatta[ix - 1] = nunatt;
  if (ncont == 0)
    {
      occa[ix - 1] = 0.f;
      mlconta[ix - 1] = 0;
      goto L24;
    }
  occa[ix - 1] = (float) (nclone - nunatt) / (float) ncont;
  mlconta[ix - 1] = ltcont / ncont;
L24:
  ltconta[ix - 1] = ltcont;
  maxconta[ix - 1] = maxcont;
  ngapa[ix - 1] = ngap;
  mlgapa[ix - 1] = ltgap / ngap;
  maxgapa[ix - 1] = maxgap;
  rcov = (float) icov * 100.f / (float) ig;
  eccova[ix - 1] = (float) ltcont * 100.f / (float) ig;
  eucova[ix - 1] = (float) (nunatt * ic) * 100.f / (float) ig;

  printf (" number of contigs=          %16ld\n"
	  " number of unattached clones=%16ld\n"
	  " mean contig occupancy =     %16.1f\n"
	  " mean contig length=         %16ld\n"
	  " maximum contig length=      %16ld\n"
	  " number of gaps=             %16ld\n"
	  " mean gap length=            %16ld\n"
	  " maximum gap length=         %16ld\n"
	  " est. genome in contigs=     %16.1f%%\n"
	  " est. genome unattached=     %16.1f%%\n"
	  " true coverage= %6.1f %%\n",
	  nconta[ix - 1], nunatta[ix - 1], occa[ix - 1], mlconta[ix - 1],
	  maxconta[ix - 1], ngapa[ix - 1], mlgapa[ix - 1], maxgapa[ix - 1],
	  eccova[ix - 1], eucova[ix - 1], rcov);

L23:
  printf (" type 'd' for differential, '*' for stop : ") ;
  readinput ("", stemp, 10) ; *sw = *stemp ;
  if (*sw == '*')
    {
      exit (1) ;
    }
  if (!(*sw == 'd' || *sw == 'D'))
    {
      goto L14;
    }

  printf ("    number of additional clones= ") ;
  readinput ("", stemp, 6) ; ncloned = a2int (stemp) ;
/* set differential parameters to zero */
  if (ncloned < 1)
    {
      exit (1) ;
    }
  ix = 2;
  nunattd[ix - 1] = 0;
  ncontd[ix - 1] = 0;
  njoincc[ix - 1] = 0;
  njoincu[ix - 1] = 0;
  njoinuu[ix - 1] = 0;
  nuext[ix - 1] = 0;
  ncext[ix - 1] = 0;
  ncint[ix - 1] = 0;
/* add clones one by one */
  nother[ix - 1] = 0;
  for (iy = 1; iy <= ncloned; ++iy)
    {
      istart = nclone + 1;
      ++nclone;
/* diagnose
   consider possible cases:-
   1     - - - - -

   2          ---------	       --------
   _	--------      --------    --------   --------
   _			   - - - - -

   _                  --------
   _            --------    --------
   3               - - - - -
   4       - - - - -
   
   _         --------
   5     -------   --------     --------
   _                     - - - - -
   
   6          ---------
   _                - - - - -
   
   7         --------   --------
   _               - - - - -
*/
      i__2 = ig - ic;
      ransort_(&istart, &nclone, jclone, &i__2);
      ix = 2;
      param1_(&ic, &ig, &fvp, jclone, &nclone, &nunatt, &maxcont,
	      &ltcont, &ncont, &ngap, &ltgap, &maxgap, &icov);
      if (nunatt == nunatta[ix - 1] + 1)
	{
/* 1 */
	  ++nunattd[ix - 1];
	}
      else if (nunatt == nunatta[ix - 1])
	{
	  if (ncont == nconta[ix - 1] - 1)
	    {
/* 2 */
	      ++njoincc[ix - 1];
	    } 
	  else if (ltcont == ltconta[ix - 1])
	    {
/* 3 */
	      ++ncint[ix - 1];
	    } 
	  else if (ltcont > ltconta[ix - 1])
	    {
/* 4 */
	      ++ncext[ix - 1];
	    } 
	  else
	    {
	      ++nother[ix - 1];
	    }
	} 
      else if (nunatt == nunatta[ix - 1] - 1)
	{
	  if (ncont == nconta[ix - 1])
	    {
/* 5 */
	      ++njoincu[ix - 1];
	    } 
	  else if (ncont == nconta[ix - 1] + 1)
	    {
/* 6 */
	      ++nuext[ix - 1];
	    } 
	  else
	    {
	      ++nother[ix - 1];
	    }
	}
      else if (nunatt == nunatta[ix - 1] - 2)
	{
/* 7 */
	  if (ncont == nconta[ix - 1] + 1)
	    {
	      ++njoinuu[ix - 1];
	    }
	} 
      else 
	{
	  ++nother[ix - 1];
	}
/*   reset absolute parameters */
      nconta[ix - 1] = ncont;
      nunatta[ix - 1] = nunatt;
      ltconta[ix - 1] = ltcont;
    }
  printf (" unattached clones           %16ld\n"
	  " new contigs                 %16ld\n"
	  " inside old contigs          %16ld\n"
	  " extending old contigs       %16ld\n"
	  " contig-contig joins         %16ld\n"
	  " contig-clone joins          %16ld\n"
	  " clone-clone joins           %16ld\n"
	  " other                       %16ld\n",
	  nunattd[ix - 1], nuext[ix - 1], ncint[ix - 1],
	  ncext[ix - 1], njoincc[ix - 1], njoincu[ix - 1],
	  njoinuu[ix - 1], nother[ix - 1]);
  goto L23;

/*   failsafe clause */
/*L22:
  printf (" unforeseen diagnosis\n") ;
  exit (1) ;
*/  
  return ;
} /* main */

int ransort_(long int *istart, long int *nclone,
	     long int *jclone, long int *isp)
{
  /* System generated locals */
  long int i__2, i__3;
  
  /* Local variables */
  static long int i, j, m, itemp;

  /* Parameter adjustments */
  --jclone;

  /* Function Body */
  for (j = *istart; j <= *nclone; ++j)
    {
      jclone[j] = (long int) (*isp * ran_()) + 1;
    }

/*   sort clones */
    m = *nclone;
L15:
  m = (m + 2) / 3;
  for (i = m + 1; i <= *nclone; ++i)
    {
      i__2 = m + 1;
      i__3 = -m;
      for (j = i; i__3 < 0 ? j >= i__2 : j <= i__2; j += i__3)
	{
	  if (jclone[j] >= jclone[j - m])
	    {
	      goto L16;
	    }
	  itemp = jclone[j];
	  jclone[j] = jclone[j - m];
	  jclone[j - m] = itemp;
	}
L16:
      ;
    }
  if (m != 1)
    {
      goto L15;
    }
  return 0;
} /* ransort_ */

int param1_(long int *ic, long int *ig, float *fvp, long int *jclone,
	    long int *nclone, long int *nunatt, long int *maxcont, long int *
	    ltcont, long int *ncont, long int *ngap, long int *ltgap, long int *
	    maxgap, long int *icov)
{
  /* Local variables */
  static long int nocc, lgap, ilap, maxspace, j, k, lcont, ispace;

  /* Parameter adjustments */
  --jclone;

  /* Function Body */
  ilap = n_int(*ic * *fvp);
  maxspace = *ic - ilap;
  k = 1;
  *nunatt = 0;
  *maxcont = 0;
  *ltcont = 0;
  *ncont = 0;
  *ngap = 2;
  *ltgap = jclone[1];
  *maxgap = *ltgap;
  *icov = *ig - jclone[1];
  for (j = 1; j <= *nclone - 1; ++j)
    {
      ispace = jclone[j + 1] - jclone[j];
      if (ispace < 0)
	{
	  fprintf (stderr, "ispace < 0\n") ;
	  exit (-1) ;
	}
      if (ispace > maxspace)
	{
	  if (ispace > *ic)
	    {
	      *icov = *icov - ispace + *ic;
	    }
	  ++(*ngap);
	  lgap = ispace - *ic;
	  *ltgap += lgap;
	  if (lgap > *maxgap)
	    {
	      *maxgap = lgap;
	    }
	  if (j == k)
	    {
	      ++(*nunatt);
	    } 
	  else
	    {
	      nocc = j - k + 1;
	      ++(*ncont);
	      lcont = jclone[j] - jclone[k] + *ic;
	      if (lcont > *maxcont)
		{
		  *maxcont = lcont;
		}
		*ltcont += lcont;
	    }
	  k = j + 1;
	}
    }
/*  end effects */
  if (k == *nclone)
    {
      ++(*nunatt);
    }
  else
    {
      nocc = *nclone - k + 1;
      ++(*ncont);
      lcont = jclone[*nclone] - jclone[k] + *ic;
      if (lcont > *maxcont)
	{
	  *maxcont = lcont;
	}
      *ltcont += lcont;
    }
  *icov = *icov - *ig + jclone[*nclone] + *ic;
  lgap = *ig - jclone[*nclone] - *ic;
  *ltgap += lgap;
  if (lgap > *maxgap)
    {
      *maxgap = lgap;
    }
  return 0;
} /* param1_ */

