/*  File: randb.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: generates random database
 * SCCS: $Id: randb.c,v 1.1 2001/02/14 12:44:40 edgrif Exp $
 * HISTORY:
 * Last edited: Feb 21 14:39 1995 (fw)
 * Created: Wed Jun 22 17:50:16 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ;			/* .nam */
extern FILE *unit_2 ;			/* .cor */
extern FILE *unit_4 ;			/* .cnt */
extern FILE *unit_10 ;			/* .stk */

/* Table of constant values */

static long int c__1 = 1;
static long int c__0 = 0;
static short int c__1000 = 1000;
static long int c__4 = 4;
static long int c__8 = 8;

extern int ranc_(long int *, long int *, long int *, char *,
		 long int *, long int *);
extern int jsort_(long int *, long int *);
extern int hotran_(long int *, long int *, long int *, 
		   long int *, long int *, char *) ;
extern int jsortc_(long int *, long int *, char *);
extern int hotsite_(long int *, long int *, long int *, 
		    long int *, long int *, long int *) ;
extern int ran1_(long int *, long int *, 
		 long int *, long int *, long int *);
extern int rnam_(long int *, long int *, long int *, char *);

/*   programme RANDB */
/*   generates fictitious database */
void main (void)
{
  /* System generated locals */
  long int i__1 ;

  /* Local variables */
  static BOOL cont;
  static char namea[8*50000];
  static long int nhot, ipos, istl, istr, iphd, ilena[50000],
                  iabsl, iabsr, iposa[50000], icont, jsite[50000], nsite,
                  lhsit, nhsit, ia, ib, ic, ig, iq, jclone[50000],
                  iconta[50000], nclone, hsitdir[BANDMAX];
  
  printf ("  insert size (kb) = ") ;
  readinput ("", stemp, 10) ; ic = a2int (stemp) ;

  printf ("  genome size (kb) = ") ;
  readinput ("", stemp, 10) ; ig = a2int (stemp) ;

  printf ("\n  insert size = %3ldkb     genome size = %7ldkb\n", ic, ig) ;

  printf ("   number of enzyme sites = ") ;
  readinput ("", stemp, 10) ; nsite = a2int (stemp) ;

  printf ("   number of evenly distributed clones = ") ;
  readinput ("", stemp, 10) ; nclone = a2int (stemp) ;

/*   compute site positions */
    ran1_(&c__1, &nsite, jsite, &ig, &c__0);
    jsort_(&nsite, jsite);

/*   compute random clone positions */
    ranc_(&c__1, &nclone, jclone, namea, &ig, &ic);

/*   compute hot clone positions */
L18:
  printf ("   number of hot sites (def=compute database) = ") ;
  readinput ("", stemp, 10) ; nhsit = a2int (stemp) ;
  if (nhsit == 0)
    {
      goto L19;
    }

  printf ("   length of hot site (kb) = ") ;
  readinput ("", stemp, 10) ; lhsit = a2int (stemp) ;

  printf ("   number of hot clones = ") ;
  readinput ("", stemp, 10) ; nhot = a2int (stemp) ;

  hotsite_(&nhsit, &lhsit, hsitdir, &iphd, &ig, &ic);
  hotran_(&nhot, &iphd, hsitdir, &nclone, jclone, namea);
  goto L18;

/*   find clone positions and lengths */
L19:
  jsortc_(&nclone, jclone, namea);
  istl = 1;
  istr = 1;
/* loop for each clone */
  for (ia = 1; ia <= nclone; ++ia)
    {
      iabsl = jclone[ia - 1];
      iabsr = iabsl + ic;
/* step downwards */
      for (ib = istl; ib >= 1; --ib)
	{
/* found a site < left end */
	  if (iabsl > jsite[ib - 1])
	    {
/* step upwards */
		for (iq = ib; iq <= nsite; ++iq)
		  {
/* found the site >= left end */
		    if (iabsl <= jsite[iq - 1])
		      {
			iposa[ia - 1] = iq;
			istl = iq;
			goto L121;
		      }
		  }
/* no site >= left end */
		iposa[ia - 1] = nsite + 1;
		istl = nsite;
		goto L121;
	    }
	}
/* no site < left end */
      iposa[ia - 1] = 1;
      istl = 1;
/* step downwards */
L121:
      for (ib = istr; ib >= 1; --ib)
	{
/* found a site < right end */
	  if (iabsr > jsite[ib - 1])
	    {
/* step upwards */
	      for (iq = ib; iq <= nsite; ++iq)
		{
/* found the site >= right end */
		  if (iabsr <= jsite[iq - 1])
		    {
		      ilena[ia - 1] = iq - iposa[ia - 1];
		      istr = iq;
		      goto L122;
		    }
		}
/* no site >= right end */
	      ilena[ia - 1] = nsite + 1 - iposa[ia - 1];
	      istr = nsite;
	      goto L122;
	    }
	}
/* no site < right end */
      ilena[ia - 1] = 1 - iposa[ia - 1];
      istr = 1;
/*   allow two bands per site */
L122:
      ;
    }
    for (ia = 1; ia <= nclone; ++ia) 
      {
	ilena[ia - 1] <<= 1;
	iposa[ia - 1] <<= 1;
/*   define contigs */
      }
  cont = FALSE;
  icont = 0;
  ipos = iposa[0];
  for (ia = 2; ia <= nclone; ++ia)
    {
/* overlap */
      if (iposa[ia - 2] + ilena[ia - 2] - iposa[ia - 1] >=
	  (ilena[ia - 2] + ilena[ia - 1]) / 4)
	{
/* continue old contig */
	  if (cont)
	    {
	      iposa[ia - 2] -= ipos;
	      iconta[ia - 2] = icont;
/* new contig */
	    } 
	  else
	    {
	      cont = TRUE ;
	      ++icont;
	      ipos = iposa[ia - 2];
	      iposa[ia - 2] = 0;
	      iconta[ia - 2] = icont;
	    }
/* no overlap */
	}
      else
	{
/* end contig */
	  if (cont)
	    {
	      cont = FALSE;
	      iposa[ia - 2] -= ipos;
	      iconta[ia - 2] = icont;
/* continue unattached */
	    }
	  else
	    {
	      iposa[ia - 2] = 0;
	    }
	}
/*   write database to disc */
    }

  unit_1 = FILopen ("ran", "nam", "r+") ;
  unit_2 = FILopen ("ran", "cor", "r+") ;
  unit_4 = FILopen ("ran", "cnt", "r+") ;
  unit_10= FILopen ("ran", "stk", "r+") ;

  seek_1 (1) ;
  fwrite (&nclone, sizeof(long int), 1, unit_1);
  fwrite (&c__0, sizeof(long int), 1, unit_1);

  for (ia = 1; ia <= nclone; ++ia)
    {
      seek_1 (ia + 1) ;
      fwrite (namea + ((ia - 1) << 3), sizeof(char), 8, unit_1);
      fwrite (&c__0, sizeof(long int), 1, unit_1);
      fwrite (&c__0, sizeof(long int), 1, unit_1);
      fwrite (&c__0, sizeof(long int), 1, unit_1);
      fwrite (&iconta[ia - 1], sizeof(long int), 1, unit_1);

      seek_4 (ia + 1) ;
      fwrite (&iposa[ia - 1], sizeof(long int), 1, unit_4);
      fwrite (&ilena[ia - 1], sizeof(long int), 1, unit_4);
      fwrite (&c__0, sizeof(long int), 1, unit_4);
      fwrite (&c__0, sizeof(long int), 1, unit_4);
    }
  rewind (unit_2) ;
  fwrite (&c__1000, sizeof(short int), 1, unit_2);
  seek_10 (1) ;
  i__1 = icont + 1;
  fwrite (&i__1, sizeof(long int), 1, unit_10);


  fclose (unit_1) ;
  fclose (unit_2) ;
  fclose (unit_4) ;
  fclose (unit_10) ;
  return ;
} /* MAIN__ */

/*   places clones at random through genome */
int ran1_(long int *istart, long int *nclone, long int *jclone, 
	  long int *ig, long int *isp)
{
  /* Local variables */
  static long int j;

  /* Parameter adjustments */
  --jclone;
  
    /* Function Body */
    for (j = *istart; j <= *nclone; ++j)
      {
L12:
	jclone[j] = (long int) (*ig * ran_()) + 1;
	if (jclone[j] > *ig - *isp)
	  {
	    goto L12;
	  }
    }
  return 0;
} /* ran1_ */

/*   places clones at random through genome, naming them in turn */
int ranc_(long int *istart, long int *nclone, long int *jclone, 
	char *namea, long int *ig, long int *isp)
{
  /* Local variables */
  static char name[9];
  static long int j;

  /* Parameter adjustments */
  namea -= 8;
  --jclone;

  /* Function Body */
  strcpy (name, "ran") ;
  for (j = *istart; j <= *nclone; ++j)
    {
      jclone[j] = (long int) ((*ig - *isp) * ran_()) + 1;
      rnam_(&c__4, &c__8, &j, name);
      strncpy(namea + (j << 3), name, 8);
    }
  return 0;
} /* ranc_ */

int rnam_(long int *i1, long int *i2, long int *num, char *name)
{
  /* Local variables */
  static long int iq;

  for (iq = *i1; iq <= *i2; ++iq)
    {
      name[iq - 1] = (char) (*num / pow_ii(10, 8 - iq) % 10 + 48);
    }
  return 0;
} /* rnam_ */

/*   sort clones */
/* Subroutine */ int jsort_(long int *nclone, long int *jclone)
{
  /* Local variables */
  static long int i, j, m, itemp;

  /* Parameter adjustments */
  --jclone;
  
  /* Function Body */
  m = *nclone;
L15:
  m = (m + 2) / 3;
  for (i = m + 1; i <= *nclone; ++i)
    {
      for (j = i; -m < 0 ? j >= m + 1 : j <= m + 1; j += -m) 
	{
	  if (jclone[j] >= jclone[j - m])
	    {
	      goto L16;
	    }
	  itemp = jclone[j];
	  jclone[j] = jclone[j - m];
	  jclone[j - m] = itemp;
	}
L16:
      ;
    }
  if (m != 1)
    {
      goto L15;
    }
  return 0;
} /* jsort_ */

/*   sort clones */
int jsortc_(long int *nclone, long int *jclone, char *namea)
{
  /* Local variables */
  static long int i, j, m;
  static char tname[9];
  static long int itemp;
  
  /* Parameter adjustments */
  namea -= 8;
  --jclone;
  
  /* Function Body */
  m = *nclone;
L15:
  m = (m + 2) / 3;
  for (i = m + 1; i <= *nclone; ++i)
    {
      for (j = i; -m < 0 ? j >= m + 1 : j <= m + 1; j += -m)
	{
	  if (jclone[j] >= jclone[j - m])
	    {
	      goto L16;
	    }
	  itemp = jclone[j];
	  jclone[j] = jclone[j - m];
	  jclone[j - m] = itemp;
	  strncpy(tname, namea + (j << 3), 8);
	  strncpy(namea + (j << 3), namea + ((j - m) << 3), 8);
	  strncpy(namea + ((j - m) << 3), tname, 8);
	}
L16:
      ;
    }
  if (m != 1)
    {
      goto L15;
    }
  return 0;
} /* jsortc_ */

/*   selects random hot sites (NHSIT, each of length LHSIT) */
int hotsite_(long int *nhsit, long int *lhsit, long int *hsitdir,
	     long int *iphd, long int *ig, long int *ic)
{
  /* Local variables */
  static long int k, ifrom, iq, isp;

  /* Parameter adjustments */
  --hsitdir;

  /* Function Body */
  *iphd = 0;
  isp = *lhsit + *ic;
  for (iq = 1; iq <= *nhsit; ++iq)
    {
      ifrom = (long int) ((*ig - isp) * ran_()) + 1;
      for (k = 0; k <= *lhsit - 1; ++k)
	{
	  ++(*iphd);
	  hsitdir[*iphd] = ifrom + k;
	}
    }
  return 0;
} /* hotsite_ */

/*   places random clones in hot sites */
int hotran_(long int *nhot, long int *iphd, long int *hsitdir, 
	    long int *nclone, long int *jclone, char *namea)
{
  /* Local variables */
  static char name[9];
  static long int k, id;
  
    /* Parameter adjustments */
  namea -= 8;
  --jclone;
  --hsitdir;

  /* Function Body */
  strcpy (name, "ran") ;
  for (k = 1; k <= *nhot; ++k)
    {
      id = (long int) (*iphd * ran_()) + 1;
      ++(*nclone);
      jclone[*nclone] = hsitdir[id];
      rnam_(&c__4, &c__8, nclone, name);
      strncpy(namea + (*nclone << 3), name, 8);
    }
  return 0;
} /* hotran_ */

