/*  File: rjps.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: handle .rej and .pss files
 * SCCS: $Id: rjps.c,v 1.1 2001/02/14 12:44:40 edgrif Exp $
 * HISTORY:
 * Last edited: Jun 15 12:56 1994 (fw)
 * Created: Thu May 19 17:09:11 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_14 ;		/* here: .rej, .pss */

/* Common Block Declarations */

extern long int nbanda[30000], lbanda[30000], igela[30000], iconta[30000],
       iposa[30000], ilena[30000], icana[30000], irema[30000];

extern  char namea[240000], chrom[65];

/* Table of constant values */

static long int c__1 = 1;
static long int c__0 = 0;

/*   writes to and reads rejection(.rej) and possible(.pss) files */
/*   does mode switching, then passes to RJPS1 */
/*   sw(2)=R  imode=2    rejected */
/*   sw(2)=P  imode=3    possible */
/*   sw(1)=q   query (does entry exist for given clone pair?) */
/*   sw(1)=s   similar (does similar clone pair exist?) */
/*   sw(1)=g   read (get partner(s) for given clone) */
/*   sw(1)=w   write */
/*   sw(1)=d   delete	!done later now! */
/*   sw(1)=t   type file */
int rejpss_(char *sw, long int *iln1, long int *iln2,
	    long int *ilna, char *projectbase, long int *nclone)
{
  /* Local variables */
  static char mode[9], filename[128];
  static long int nrej, npss, imode;
  static long int rej1[30000], rej2[30000], pss1[30000], pss2[30000];
  static BOOL rejdat, pssdat;

  /* Parameter adjustments */
  --ilna;
  --sw;
  
  /* Function Body */
  if (sw[2] == 'R')
    {
      strcpy(mode, "rejected");
      imode = 2;
      sprintf (filename, "%s.rej", projectbase) ;
      if (! rejdat)
	{
	  rejdat = TRUE;
	  rddat_(filename, &nrej, rej1, rej2);
	}
      rjps1_(sw + 1, iln1, iln2, &ilna[1], &imode, mode, &nrej, rej1, rej2, 
	     filename, nclone);
    }
  else if (sw[2] == 'P')
    {
      strcpy(mode, "possible");
      imode = 3;
      sprintf (filename, "%s.pss", projectbase) ;
      if (! pssdat)
	{
	  pssdat = TRUE;
	  rddat_(filename, &npss, pss1, pss2);
	}
      rjps1_(sw + 1, iln1, iln2, &ilna[1], &imode, mode, &npss, pss1, pss2, 
	     filename, nclone);
    }
  return 0;
} /* rejpss_ */

/*   does operation switching */
int rjps1_(char *op, long int *iln1, long int *iln2, long int *
	   ilna, long int *imode, char *mode, long int *ntot,
	   long int *tot1, long int *tot2, char *filename,
	   long int *nclone)
{
  /* Parameter adjustments */
  --tot2;
  --tot1;
  --ilna;
  
  /* Function Body */
  if (*op == 'w' || *op == 'd')
    {
/*   open file */
      if (*ntot == -9)
	{
	  goto L997;
	}
      if (!(unit_14 = FILopen (filename, "", "r")))
	goto L997 ;
      if (*op == 'w')
	{
	  wrtot_(iln1, imode, mode, ntot, &tot1[1], &tot2[1], nclone);
	}
      else
	{
	  dltot_(iln1, iln2, ntot, &tot1[1], &tot2[1]);
	}
      goto L999;
    }
  else if (*op == 't')
    {
      tytot_(ntot, &tot1[1], &tot2[1]);
      return 0;
    }
  else if (*op == 'q')
    {
      if (*ntot == -9) 
	{
	  return 0;
	}
      qutot_(iln1, iln2, ntot, &tot1[1], &tot2[1]);
      return 0;
    } 
  else if (*op == 'g')
    {
      if (*ntot == -9)
	{
	  *iln2 = 0;
	  return 0;
	}
      rdtot_(iln1, iln2, &ilna[1], ntot, &tot1[1], &tot2[1]);
      return 0;
    }
L997:
  gprintf (stdout, "  %-8s match file not available;\n", mode) ;
  
  waitForMore (RETURN_KEY) ;
L999:
  fclose (unit_14) ;
  return 0;
} /* rjps1_ */

/*   writes to unit 14 (.rej or .pss) */
int wrtot_(long int *iln1, long int *imode, char *mode, 
	   long int *ntot, long int *tot1, long int *tot2,
	   long int *nclone)
{
  /* Local variables */
  static long int ilna[100];
  static char name1[9], name2[9];
  static long int ilna1[100], ilna2[100], itemp;
  static long int iq;
  static long int ibl;
  static char ans[1];
  static long int iln2;
  static BOOL del;
  
/*   reset delete switch */
/*   look for empty record */
    /* Parameter adjustments */
  --tot2;
  --tot1;

  /* Function Body */
  del = FALSE;
  for (ibl = 2; ibl <= *ntot; ++ibl)
    {
      if (tot1[ibl] == 0)
	{
	  goto L10;
	}
/*   if no empty record, enlarge file */
    }
  ibl = *ntot + 1;
L10:
  strtcpy (ntemp1, namea + ((*iln1 - 1) << 3), 8) ;
  gprintf (stdout, "   type D for delete, L for list\n"
	  "  first member of %-8s match (def  %s) = ",
	  mode, ntemp1);
  readinput ("", name1, 8) ;

  if (strcasecmp(name1, "l") == 0)
    {
      tytot_(ntot, &tot1[1], &tot2[1]);
      goto L10;
    }
  else if (strcasecmp(name1, "d") == 0)
    {
      del = TRUE;
      gprintf (stdout, "   switched to delete mode\n") ;
      goto L10;
    } 
  else if (strlength(name1, 8) > 0)
    {	/* if name1 not empty */
      getline1_(name1, iln1, nclone);
      if (*iln1 == 0)
	{
	  goto L10;
	}
    }
L11:
  gprintf (stdout, "  second member of %-8s match"
	  " (L for list, def to escape) = ", mode) ;
  readinput ("", name2, 8) ;
  if (strlength(name2, 8) == 0)
    {
      return 0;
    }
  if (strcasecmp(name2, "l") == 0)
    {
      rdtot_(iln1, &iln2, ilna, ntot, &tot1[1], &tot2[1]);
      for (iq = 1; iq <= iln2; ++iq)
	{
	  strtcpy (ntemp1, namea + ((ilna[iq - 1] - 1) << 3), 8) ;
	  gprintf (stdout, "%s\n", ntemp1);
	}
      goto L11;
    }
  getline1_(name2, &iln2, nclone);
/*   allow delete */
    if (iln2 == 0)
      {
	goto L11;
      }
  if (del)
    {
      dltot_(iln1, &iln2, ntot, &tot1[1], &tot2[1]);
      gprintf (stdout, "  %2d entry deleted\n", iln2);
      goto L12;
    }
/*   protect against identical clones or clones in same contig */
  if (*iln1 == iln2)
    {
      gprintf (stdout, "   same clone\n");
      goto L12;
    }
  else if (iconta[*iln1 - 1] == iconta[iln2 - 1])
    {
      gprintf (stdout, "   clones in same contig\n") ;
      goto L12;
    }
/*   protect against double entry */
    itemp = *iln1;
    qutot_(&itemp, &iln2, ntot, &tot1[1], &tot2[1]);
    if (itemp == -99)
      {
	gprintf (stdout, "   this pair already recorded\n");
	goto L12;
    }
/*   protect against similar matches (between same pair of contigs) */
  ilna1[0] = *iln1;
  ilna2[0] = iln2;
  sitot_(&itemp, ilna1, ilna2, ntot, &tot1[1], &tot2[1]);
  if (itemp != 0)
    {
      gprintf (stdout, "  already have record for this pair of contigs:-\n") ;
      for (iq = 1; iq <= itemp; ++iq)
	{
	  strtcpy (ntemp1, namea + ((ilna1[iq - 1] - 1) << 3), 8) ;
	  strtcpy (ntemp2, namea + ((ilna2[iq - 1] - 1) << 3), 8) ;
	  gprintf (stdout, "  %s    %s\n", ntemp1, ntemp2); 
	}
      gprintf (stdout, "  do you want to continue (Y/N) ? ");
      readinput ("", stemp, 2) ; *ans = *stemp ;
      if (*ans != 'Y' || *ans == 'y')
	{
	  return 0;
	}
    }
  tot1[ibl] = *iln1;
  tot2[ibl] = iln2;
  seek_14 (ibl) ;
  fwrite (&(*iln1), sizeof(long int), 1, unit_14);
  fwrite (&iln2, sizeof(long int), 1, unit_14);
  fwrt_(unit_14);
  
  strtcpy (ntemp1, namea + ((*iln1 - 1) << 3), 8) ;
  strtcpy (ntemp2, namea + ((iln2 - 1) << 3), 8) ;
  gprintf (stdout, "  %-8s match between %-8s and %-8s has been recorded\n",
	   mode, ntemp1, ntemp2);
  wrflag_(iln1, imode, &c__1);
  wrflag_(&iln2, imode, &c__1);
  if (ibl > *ntot)
    {
      *ntot = ibl;
      seek_14 (1) ;
      fwrite (&(*ntot), sizeof(long int), 1, unit_14);
      fwrt_(unit_14);
    }
L12:
  waitForMore (RETURN_KEY) ;

  return 0;
} /* wrtot_ */

int rddat_(char *filename, long int *ntot, long int *tot1, 
	   long int *tot2)
{
  /* Local variables */
  static long int iq;
  
    /* Parameter adjustments */
  --tot2;
  --tot1;
  
/*   try to open file */
  if (!(unit_14 = FILopen (filename, "", "r")))
    goto L87 ;

/*   read data */
  fread (&(*ntot), sizeof(long int), 1, unit_14);
  if (ferror(unit_14)) goto L87 ;
  for (iq = 2; iq <= *ntot && !feof(unit_14); ++iq)
    {
      seek_14 (iq) ;
      fread (&tot1[iq], sizeof(long int), 1, unit_14);
      fread (&tot2[iq], sizeof(long int), 1, unit_14);
      if (ferror(unit_14)) goto L86;
    }
  fclose (unit_14) ;
  return 0;
L86:
  gprintf (stdout, "  error reading match file at line %d    ntot = %d\n",
	  iq, *ntot);
L87:
  *ntot = -9;
  return 0;
} /* rddat_ */

/*   queries presence of iln1/iln2 pair; affirmative is -99 in iln1 */
int qutot_(long int *iln1, long int *iln2, long int *ntot, 
	   long int *tot1, long int *tot2)
{
  /* Local variables */
  static long int iq;
  
  /* Parameter adjustments */
  --tot2;
  --tot1;
  
  /* Function Body */
  for (iq = 2; iq <= *ntot; ++iq)
    {
      if ((tot1[iq] == *iln1 && tot2[iq] == *iln2) ||
	  (tot1[iq] == *iln2 && tot2[iq] == *iln1))
	{
	  *iln1 = -99;
	  return 0;
	}
    }
  return 0;
} /* qutot_ */

/*   reads list of clones paired with iln1 into ilna */
int rdtot_(long int *iln1, long int *iln2, long int *ilna, 
	   long int *ntot, long int *tot1, long int *tot2)
{
  /* Local variables */
  static long int iq;
  
  /* Parameter adjustments */
  --tot2;
  --tot1;
  --ilna;
  
  /* Function Body */
  *iln2 = 0;
  for (iq = 2; iq <= *ntot; ++iq)
    {
      if (tot1[iq] == *iln1)
	{
	  ++(*iln2);
	  ilna[*iln2] = tot2[iq];
	}
      else if (tot2[iq] == *iln1)
	{
	  ++(*iln2);
	  ilna[*iln2] = tot1[iq];
	}
    }
  return 0;
} /* rdtot_ */

/*   deletes iln1/iln2 entries from file */
/*   returns number of entries in iln2 */
int dltot_(long int *iln1, long int *iln2, long int *ntot, 
	   long int *tot1, long int *tot2)
{
  /* Local variables */
  static long int inum;
  static long int iq;
  
  /* Parameter adjustments */
  --tot2;
  --tot1;
  
  /* Function Body */
  inum = 0;
  for (iq = 2; iq <= *ntot; ++iq)
    {
      if ((tot1[iq] == *iln1 && tot2[iq] == *iln2) || 
	  (tot1[iq] == *iln2 && tot2[iq] == *iln1))
	{
	  tot1[iq] = 0;
	  tot2[iq] = 0;
	  seek_14 (iq) ;
	  fwrite (&c__0, sizeof(long int), 1, unit_14);
	  fwrite (&c__0, sizeof(long int), 1, unit_14);
	  fwrt_(unit_14);
	  ++inum;
	}
    }
  *iln2 = inum;
  return 0;
} /* dltot_ */

#ifndef NO_GRAPHICS
/*   displays PSS links; ihnd=-1 left,ihnd=1 right */
int dislnk_(long int *iy, long int *ihnd, long int *ipx, 
	    long int *ipy, char *filename, long int *nclone)
{
  /* System generated locals */
  static char equiv_0[13];
#define tname4 (equiv_0)
#define tname4b (equiv_0)
  
  /* Local variables */
  static long int ilna[100];
  static long int ic, iq, i;
  static long int iln1, iln2;

  for (ic = 2; ic <= *nclone + 1; ++ic)
    {
      if (ic == *iy || abs(icana[ic - 1]) % 10000000 == *iy)
	{
	  iln1 = ic;
	  rejpss_("gP", &iln1, &iln2, ilna, filename, nclone);
	  if (*ihnd == -1)
	    {
	      strtcpy(tname4 + 8, "  <?", 4);
	      for (iq = 1; iq <= iln2; ++iq)
		{
		  strncpy(tname4, namea + ((ilna[iq - 1] - 1) << 3), 8);
		  for (i = strlength (tname4, 8) ; i < 8; ++i)
		    tname4[i] = ' ' ; /* fill up rest of tname4 */
		  *ipy += 20;
		  movevt640_(*ipx - 156, *ipy);
		  textvt640_(tname4b, 12);
		}
	    }
	  else if (*ihnd == 1)
	    {
	      strncpy(tname4, "?>  ", 4);
	      for (iq = 1; iq <= iln2; ++iq)
		{
		  strtcpy(tname4 + 4, namea + ((ilna[iq - 1] - 1) << 3), 8);
		  *ipy += 20;
		  movevt640_(*ipx + 26, *ipy);
		  textvt640_(tname4b, 12);
		}
	    }
	}
    }
  return 0;
} /* dislnk_ */

#undef tname4b
#undef tname4

#endif /* ! NO_GRAPHICS */


int tytot_(long int *ntot, long int *tot1, long int *tot2)
{
  /* Local variables */
  static long int iq, iln1, iln2;
  
  /* Parameter adjustments */
  --tot2;
  --tot1;
  
  /* Function Body */
  for (iq = 2; iq <= *ntot; ++iq)
    {
      iln1 = tot1[iq];
      iln2 = tot2[iq];
      if (iln1 == 0 || iln2 == 0)
	{
	  goto L10;
	}
      strtcpy (ntemp1, namea + ((iln1 - 1) << 3), 8) ;
      strtcpy (ntemp2, namea + ((iln2 - 1) << 3), 8) ;
      gprintf (stdout, "  %s  %s\n", ntemp1, ntemp2) ;
L10:
	;
    }
  return 0;
} /* tytot_ */

/*   lists matches between same pair of contigs */
int sitot_(long int *ipnt, long int *ilna1, long int *ilna2, 
	   long int *ntot, long int *tot1, long int *tot2)
{
  /* Local variables */
  static long int icont1, icont2, itemp1, itemp2, iq, ic1, ic2;
  
    /* Parameter adjustments */
  --tot2;
  --tot1;
  --ilna2;
  --ilna1;
  
  /* Function Body */
  ic1 = ilna1[1];
  ic2 = ilna2[1];
  *ipnt = 0;
  icont1 = iconta[ic1 - 1];
  icont2 = iconta[ic2 - 1];
  for (iq = 2; iq <= *ntot; ++iq)
    {
      itemp1 = tot1[iq];
      itemp2 = tot2[iq];
      if (itemp1 == 0 || itemp2 == 0)
	{
	  goto L10;
	}
      if (iconta[itemp1 - 1] == icont1 &&
	  iconta[itemp2 - 1] == icont2)
	{
	  ++(*ipnt);
	  ilna1[*ipnt] = itemp1;
	  ilna2[*ipnt] = itemp2;
	}
      else if (iconta[itemp1 - 1] == icont2 &&
	       iconta[itemp2 - 1] == icont1)
	{
	  ++(*ipnt);
	  ilna1[*ipnt] = itemp2;
	  ilna2[*ipnt] = itemp1;
	}
L10:
      ;
    }
  return 0;
} /* sitot_ */

