/*  File: rmvec.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description:
        this program removes all vector bands from the database

	**WARING** files are lost when program is interrupted
	during execution, should be done on a copy of the database.
 * SCCS: $Id: rmvec.c,v 1.1 2001/02/14 12:44:40 edgrif Exp $
 * HISTORY:
 * Last edited: Nov 27 22:06 1995 (rd)
 * Created: 
 *-------------------------------------------------------------------
 it uses the following approach :

 open files for reading
 for each clone do
   read data for a single clone from nam/cor/int/cnt
   remove vectors and write changed data to the ..a[] arrays.
 close files

 open files for writing
 write all data from the ..a[]-arrays colne by clone to the files
 close files and print stats.
 */

#include "co9.h"

/*****************************************************************/

extern FILE *unit_1 ; /* nam */
extern FILE *unit_2; /* cor */
extern FILE *unit_4 ; /* cnt */

#ifdef USE_INTS
  extern FILE *unit_53; /* int */
#endif

/* Common Block Declarations */

/* csec CB */
short int coorda[BANDMAX] ;
#ifdef USE_INTS
  short int inta[BANDMAX];
#endif

/* cint CB */
long int nbanda[30000], lbanda[30000], igela[30000], iconta[30000],
         iposa[30000], ilena[30000], icana[30000], irema[30000];

/* chars CB */
char namea[240000], chrom[65];


/*****************************************************************/
static int vecNum, vecBands[200], vecTol1, vecTol2 ;
static BOOL getVectorBands (int *vecBands, int *vecNum) ;
static void markVectorBands (short int *band, int nbandl, int *lostNum) ;


void main(int argc, char *argv[])
{
  char inproject[128]="" ;
  long int nclone, nbands ;
  char tname[9] ;
  long int nband, lband, igel, icont ;
  long int ipos, ilen, ican, irem ;
  short int band[200], coord ;
  int bandIndex, bandPointer, i, j, lostNum, totalLost=0, totalCount=0 ;
#ifdef USE_INTS
  BOOL HAVE_INTS ;
  short int intvals[200] ;
#endif

  if (!getVectorBands (vecBands, &vecNum)) exit (-1) ;

  printf ("***************************************************\n"
	  "*                    rmvec                        *\n"
	  "*                   -------                       *\n"
	  "*  **WARNING**  This program alters  **WARNING**  *\n"
	  "*      database files of the input project.       *\n"
	  "*                                                 *\n"
	  "*  Input Project : all data is copied to output   *\n"
	  "*      project, losing all vector bands.          *\n"
	  "*                                                 *\n"
	  "*  nam/cor/int/cnt files are modified             *\n"
	  "***************************************************\n\n") ;
   
  printf ("   input project name = ") ;
  readinput ("", inproject, 128) ;
  if (strlength(inproject, 128) == 0) { printf ("User Interrupt !\n") ; exit (1) ;}
  
  if (!(unit_1 = FILopen (inproject, "nam", "r"))) exit (-1) ;
  if (!(unit_2 = FILopen (inproject, "cor", "r"))) exit (-1) ;
  if (!(unit_4 = FILopen (inproject, "cnt", "r"))) exit (-1) ;

#ifdef USE_INTS
  if ((unit_53 = FILopen (inproject, "int", "R")))
    HAVE_INTS = TRUE ;
  else
    HAVE_INTS = FALSE ;
#endif

   /* get info on input project */
  seek_1 (1) ;
  fread (&nclone, sizeof(long int), 1, unit_1);
  fread (&nbands, sizeof(long int), 1, unit_1);
  printf ("   input database %10s : number of clones = %6ld, "
	  "number of bands = %8ld\n\n", inproject, nclone, nbands) ;


  /* now get each clone in turn and create arrays of nam/cor/int/cnt data */

  bandPointer = 1 ;		/* place to start coorda[]-array and .cor-file */
  
  for (i = 2; i <= nclone + 1; ++i)
    {
      /* get nam record */
      seek_1 (i) ;
      fread (tname, sizeof(char), 8, unit_1) ;
      fread (&nband, sizeof(long int), 1, unit_1) ;
      fread (&lband, sizeof(long int), 1, unit_1) ;
      fread (&igel, sizeof(long int), 1, unit_1) ;
      fread (&icont, sizeof(long int), 1, unit_1) ;
      
      /* get cnt record */
      fread (&ipos, sizeof(long int), 1, unit_4) ;
      fread (&ilen, sizeof(long int), 1, unit_4) ;
      fread (&ican, sizeof(long int), 1, unit_4) ;
      fread (&irem, sizeof(long int), 1, unit_4) ;
     
      /* get band data */
      fseek (unit_2, (lband-1) * sizeof(short int), SEEK_SET) ;
      fread(&band, sizeof(short int), nband, unit_2) ;

#ifdef USE_INTS
      if (HAVE_INTS)
	{
	  /* get intensity data */
	  fseek (unit_53, (lband-1) * sizeof(short int), SEEK_SET) ;
	  fread(&intvals, sizeof(short int), nband, unit_53) ;
	}
#endif

      /* now clone data is complete */

      lostNum = 0 ;

      markVectorBands (band, nband, &lostNum) ;
      if (lostNum > 0)
	{
	  totalLost += lostNum ;
	  totalCount += 1 ;
	}

      tname[9] = '\0' ;

      /* collect data in ..a[] arrays */

      /* 1. collect all "good" bands */
      lband = bandPointer ;	/* save current pointer position */

      for (j = 0; j < nband; ++j)
	{
	  coord = band[j] ;
	  if (coord >= 0)	/* we don't copy vector bands to the coorda[]-array */
	    {
	      coorda[bandPointer] = coord ;
#ifdef USE_INTS
	      if (HAVE_INTS)
		inta[bandPointer] = intvals[j] ;
#endif
	      ++bandPointer ;
	    }
	}
      /* bandPointer is now the lband for the next clone */

      /* shorten clone */
      nband = nband - lostNum ;
      ilen = nband - lostNum ;

      /* 2. collect .nam record */
      memcpy (namea + (i << 3), tname, 8) ;
      nbanda[i] = nband ;
      lbanda[i] = lband ;
      igela[i] = igel ;
      iconta[i] = icont ;

      /* 3. collect .cnt record */
      iposa[i] = ipos ;
      ilena[i] = ilen ;
      icana[i] = ican ;
      irema[i] = irem ;
    }
  fclose (unit_1) ;
  fclose (unit_2) ;
  fclose (unit_4) ;

#ifdef USE_INTS
  if (HAVE_INTS)
    fclose (unit_53) ;
#endif

  nbands -= totalLost ;		/* cor file got shorter */

  /* open all files for writing this will destroy all old contents */

  if (!(unit_1 = FILopen (inproject, "nam", "w"))) exit (-1) ;
  if (!(unit_2 = FILopen (inproject, "cor", "w"))) exit (-1) ;
  if (!(unit_4 = FILopen (inproject, "cnt", "w"))) exit (-1) ;

#ifdef USE_INTS
  if (HAVE_INTS)
    if (!(unit_53 = FILopen (inproject, "int", "w"))) exit (-1) ;
#endif

  /* write new file header for .nam file */
  seek_1 (1) ;
  fwrite (&nclone, sizeof(long int), 1, unit_1);
  fwrite (&nbands, sizeof(long int), 1, unit_1);
 
  /* write the content of the ...a[]-arrays to the files */
  for (i = 2; i <= nclone+1; ++i) /* i in terms of record number in file */
    {
      /* assemble clone data */
      memcpy (tname, namea + (i << 3), 8) ;
      nband = nbanda[i] ;
      lband = lbanda[i] ;
      igel  = igela[i] ;
      icont = iconta[i] ;

      ipos = iposa[i] ;
      ilen = ilena[i] ;
      ican = icana[i] ;
      irem = irema[i] ;

      bandIndex = 0 ;
      for (j = lband; j < lband + nband; ++j)
	{
	  band[bandIndex] = coorda[j] ;
#ifdef USE_INTS
	  if (HAVE_INTS)
	    intvals[bandIndex] = inta[j] ;
#endif
	  ++bandIndex ;
	}

      /* write bands */
      fseek (unit_2, (lband-1) * sizeof(short int), SEEK_SET) ;
      fwrite (&band, sizeof(short int), nband, unit_2);

#ifdef USE_INTS
      if (HAVE_INTS)
	{
	  /* write intensities */
	  fseek (unit_53, (lband-1) * sizeof(short int), SEEK_SET) ;
	  fwrite (&intvals, sizeof(short int), nband, unit_53);
	}
#endif

      /* write nam record */
      seek_1 (i) ;
      fwrite (tname, sizeof(char), 8, unit_1) ;
      fwrite (&nband, sizeof(long int), 1, unit_1) ;
      fwrite (&lband, sizeof(long int), 1, unit_1) ;
      fwrite (&igel,  sizeof(long int), 1, unit_1) ;
      fwrite (&icont, sizeof(long int), 1, unit_1) ;

      /* write cnt record */
      seek_4 (i) ;
      fwrite (&ipos, sizeof(long int), 1, unit_4);
      fwrite (&ilen, sizeof(long int), 1, unit_4);
      fwrite (&ican, sizeof(long int), 1, unit_4);
      fwrite (&irem, sizeof(long int), 1, unit_4);

     }

  fclose (unit_1) ;
  fclose (unit_2) ;
  fclose (unit_4) ;

#ifdef USE_INTS
  if (HAVE_INTS)
    fclose (unit_53) ;
#endif

  printf ("Finished processing database.\n"
	  "total number of bands removed : %d\n"
	  "average number of lost bands per clone : %.2f\n"
	  "total band count shrunk from %ld to %ld\n",
	  totalLost, totalCount == 0 ? 0.0 : totalLost / totalCount,
	  nbands + totalLost, nbands) ;
}

static BOOL getVectorBands (int *vecBands, int *vecNum)
{
  FILE *fil ;
  int t1, t2 ;			/* temps for reading vector.dat */
  
  if (!(fil = FILopen ("vector", "dat", "r")))
    { *vecNum = 0 ;
      return FALSE ;
    }

  fscanf (fil, "%d,%d", &vecTol1, &vecTol2) ;

  /* skip rest of line */
  while (!feof(fil) && fgetc (fil) != '\n') ;

  while (!feof(fil))
    {
      fscanf (fil, "%d %d\n", &t1, &t2) ;

      vecBands[t2-1] = t1 ;
      ++(*vecNum) ;
    }
  fclose (fil) ;

  if (*vecNum > 0)
    return TRUE ;
  else
    return FALSE ;
}

static void sorta1 (int *dif, int vtot)
{
  BOOL TEST ;
  int iq, itemp ;

 L101:
  TEST = FALSE ;

  for (iq = 0; iq < vtot-1; ++iq)
    {
      if (dif[iq] > dif[iq+1])
	{
	  TEST = TRUE ;
	  itemp = dif[iq] ;
	  dif[iq] = dif[iq+1] ;
	  dif[iq+1] = itemp ;
	}
    }
  if (TEST) goto L101 ;

  return ;
}

static void vecpurge (int *vec, int *dif, int vtot)
{
  int veccur, difcur, iv ;

  veccur = vec[0] ;
  difcur = dif[0] ;

  for (iv = 1; iv < vtot; ++iv)
    {
      if (veccur == vec[iv])
	{
	  if (abs(difcur) > abs(dif[iv]))
	    {
	      vec[iv-1] = 0 ;
	      dif[iv-1] = 0 ;
	    }
	  else
	    {
	      vec[iv] = 0 ;
	      dif[iv] = 0 ;
	      continue ;
	    }
	}
      veccur = vec[iv] ;
      difcur = dif[iv] ;
    }
}

static void vecbnd (int nbandl, short int *band, int *vbnd, int vtot, int itol,
		    int *ioff, BOOL CORR, int *vec, int *lostNum)
{
  int istart, iv, ib, ibest, idiff, mind ;
  int dif[200] ;

  istart = 0 ;
  ibest = -1 ;			/* dummy value */

  for (iv = 0; iv < vtot; ++iv)
    {
      mind = 10000000 ;
      
      for (ib = istart; ib < nbandl; ++ib)
	{
	  idiff = band[ib] - vbnd[iv] - *ioff ;
	  if (abs(idiff) < abs(mind))
	    {
	      mind = idiff ;
	      ibest = ib ;
	    }
	  else if (abs(idiff) > abs(mind)*4)
	    {
	      istart = ibest + 1 ;
	      goto L103 ;
	    }
	}

    L103:
      if (abs(mind) < itol)
	{
	  vec[iv] = ibest ;
	  dif[iv] = mind + *ioff ;
	  if (CORR)
	    {
	      /* mark band at ibest as vector band */

	      band[ibest] = -1 ;
	      ++(*lostNum) ;
	    }
	}
      else
	{
	  vec[iv] = 0 ;
	  dif[iv] = 0 ;
	}
    }

  vecpurge (vec, dif, vtot) ;
  sorta1(dif, vtot) ;

  *ioff = dif[max(1, vtot/2)] ;

  return ;
}

static void markVectorBands (short int *band, int nbandl, int *lostNum)
{
/* function to detect vector band and mark them as deleted */
  int vec[200] ;
  int ioff ;

  if (vecNum == 0)
    return ;			/* do nothing then, if we haven't vector-data */

  ioff = 0 ;			/* use 0 for first pass */

  vecbnd(nbandl, band, vecBands, vecNum, vecTol1, &ioff, FALSE, vec, lostNum) ;
  vecbnd(nbandl, band, vecBands, vecNum, vecTol2, &ioff, TRUE,  vec, lostNum) ;
  
}



