/*  File: vt640.c
 *  Author: Friedemann "F" Wobus (fw@sanger.ac.uk)
 *  Copyright (C) John Sulston and Friedemann Wobus, 1994
 *-------------------------------------------------------------------
 * This file is part of the ContigC physical mapping database package,
 * 	written by John Sulston (jes@sanger.ac.uk)
 *	ported to UNIX-C by Friedemann Wobus (fw@sanger.ac.uk)
 *
 * Description: interface from fortran vt640 functions to functions
 from the ACEDB graphpackage
 * SCCS: $Id: vt640.c,v 1.1 2001/02/14 12:44:41 edgrif Exp $
 * HISTORY:
 * Last edited: Apr 26 16:18 1995 (fw)
 * Created: Thu May 19 17:09:27 1994 (fw)
 *-------------------------------------------------------------------
 */

#include "co9.h"
#include <stdarg.h>
/*****************************************************************/

extern Graph graph ;
extern int cursBox ;
extern float wChar, hChar ;
extern float winWidth, winHeight ; /* determined by graphWhere in main() */
static float pLine = 0, pRow = 0 ; /* current text position */
static long int xpos, ypos ;	   /* current possition of graph-"cursor" */
static Array allTexts = 0 ;
static int ctb = 0 ;		/* Current TextBox pos in allText-array*/
static Array allGraphs = 0 ;	/* array of all graphical boxes */
static int globalColor = BLACK ; /* global color setting */
static BOOL INVERT = FALSE ;

void selectvt640_ (void) {;} void selectvt100_ (void) {;}

void waitForMore (int key)
{
  char text[50] ;
  int box ;
  
  if (key == RETURN_KEY)
    strcpy (text, " hit RETURN to continue ") ;
  else if (key == SPACE_KEY)
    strcpy (text, " more - press SPACE ") ;

  messbeep () ;
  box = graphBoxStart () ;
  graphText (text, 0, 59*hChar) ;
  graphBoxEnd () ;
  graphBoxDraw (box, WHITE, BLACK) ;
  
  waitForKey (key) ;
  graphBoxClear (box) ;
}

int waitForKey (int key)
{
  int ret_key ;

  if (key)			/* then wait for this key */
    {
    again:
      graphRegister (KEYBOARD, graphLoopReturn) ;
      if (graphLoop(TRUE) != key) goto again ;
      graphRegister (KEYBOARD, 0) ;
      return 0 ;
    }
  else				/* return the key */
    {
      graphRegister (KEYBOARD, graphLoopReturn) ;
      ret_key = graphLoop(TRUE) ;
      graphRegister (KEYBOARD, 0) ;
      return ret_key ;
    }
}

void clearvt640_ (void)		/* clears everything */
{
  graphClear () ; graphRedraw () ;

  cursBox = 0 ;
  if (allTexts) arrayReCreate (allTexts, 20, int) ;
  if (allGraphs) arrayReCreate (allGraphs, 100, int)  ;

  graphTextFormat (FIXED_WIDTH) ;
  xpos = 0 ; ypos = 0 ;
  pRow = 0 ; pLine = 0 ;
}

int initvt640_ (void)		/* clears only graphics */
{
  int k, ret_val = 0 ;

  if (allGraphs) 
    {
      for (k = 0; k < arrayMax (allGraphs); ++k)
	graphBoxClear (arr(allGraphs, k, int)) ;
      arrayReCreate (allGraphs, 100, int) ;
      ret_val = 1 ;
    }
  if (cursBox) {graphBoxClear (cursBox) ; cursBox = 0 ;}
  xpos = 0 ; ypos = 0 ;
  return ret_val ;
}
/*
   Display the cross-hairs and wait for the user to position them. The
   crosshairs are driven by the 4 cursor keys, and the coordinates of
   the intersection are returned to the computer when a punctuation key
   or space key is pressed. The ASCII key code is returned in ICODE.
   */
void xhairvt640_ (long int *x, long int *y, long int *code)
{
  int key ;
  int hBox, vBox ;
  hBox = graphBoxStart () ;
  graphLine (0, MAPY(*y), winWidth, MAPY(*y)) ;
  graphBoxEnd () ;

  vBox = graphBoxStart () ;
  graphLine (MAPX(*x), 0, MAPX(*x), winHeight) ;
  graphBoxEnd () ;
  graphBoxDraw (hBox, BLUE, TRANSPARENT) ;
  graphBoxDraw (vBox, BLUE, TRANSPARENT) ;
  while (TRUE)
    {
      key = waitForKey(0) ;
      if (key == LEFT_KEY)
	{
	  (*x)-=8 ;
	  graphBoxShift (vBox, MAPX(*x), 0) ;
	}
      else if (key == RIGHT_KEY)
	{
	  (*x)+=8 ;
	  graphBoxShift (vBox, MAPX(*x), 0) ;
	}
      else if (key == DOWN_KEY)
	{
	  (*y)-=8 ;
	  graphBoxShift (hBox, 0, MAPY(*y)) ;
	}
      else if (key == UP_KEY)
	{
	  (*y)+=8 ;
	  graphBoxShift (hBox, 0, MAPY(*y)) ;
	}
      else
	break ;
    }
  graphBoxClear (hBox) ; graphBoxClear (vBox) ;
  graphRedraw () ;
  *code = key ;
}

void movevt640_ (long int x, long int y)
{ xpos = x ; ypos = y ;
}
void drawvt640_ (long int x , long int y)
{
  int k ;

  if (!allGraphs) allGraphs = arrayCreate (10, int) ;
  k = arrayMax(allGraphs) ;
  array(allGraphs, k, int) = graphBoxStart () ;

  graphLine (MAPX(xpos), MAPY(ypos), MAPX(x), MAPY(y)) ;

  graphBoxEnd () ;		/* current box in array */
  if (INVERT)
    graphBoxDraw (arr(allGraphs, k, int), WHITE, globalColor) ;
  else
    graphBoxDraw (arr(allGraphs, k, int), globalColor, TRANSPARENT) ;

  xpos = x ; ypos = y ;
}

/* only be used for rectangle to empasize clone */
void rectvt640_(long int x, long int height)
{
  int k ;

  if (!allGraphs) allGraphs = arrayCreate (10, int) ;
  k = arrayMax(allGraphs) ;
  array(allGraphs, k, int) = graphBoxStart () ;

  graphFillRectangle (MAPX(xpos), MAPY(ypos),
		      MAPX(1)+MAPX(x), MAPY(ypos)-MAPX(height)) ;

  graphBoxEnd () ;		/* current box in array */
  if (INVERT)
    graphBoxDraw (arr(allGraphs, k, int), WHITE, globalColor) ;
  else
    graphBoxDraw (arr(allGraphs, k, int), globalColor, TRANSPARENT) ;
}
void textvt640_ (char *s, long int len)
{
  int k ;
  char stemp[100] ;

  if (strlength(s, len) == 0) return ;

  if (!allGraphs) allGraphs = arrayCreate (10, int) ;
  k = arrayMax(allGraphs) ;
  array(allGraphs, k, int) = graphBoxStart () ;

  strtcpy (stemp, s, strlength(s, len)) ;
  graphText (stemp, MAPX(xpos), MAPY(ypos + 6)) ;

  graphBoxEnd () ;		/* current box in array */
  if (INVERT)
    graphBoxDraw (arr(allGraphs, k, int), WHITE, globalColor) ;
  else
    graphBoxDraw (arr(allGraphs, k, int), globalColor, TRANSPARENT) ;
}

void pointvt640_ (long int x, long int y)
{
  int k ;

  if (!allGraphs) allGraphs = arrayCreate (10, int) ;
  k = arrayMax(allGraphs) ;
  array(allGraphs, k, int) = graphBoxStart () ;

  graphPoint (MAPX(x), MAPY(y)) ;

  graphBoxEnd () ;		/* current box in array */
  if (INVERT)
    graphBoxDraw (arr(allGraphs, k, int), WHITE, globalColor) ;
  else
    graphBoxDraw (arr(allGraphs, k, int), globalColor, TRANSPARENT) ;
}

void colorvt640_(int color)
{ 
  if (color > 100)
    {
      INVERT = TRUE ;
      globalColor = color - 100 ;
    }
  else
    {
      INVERT = FALSE ;
      globalColor = color ;
    }
}

void movevt100_ (int x, int y)
{
  pRow = x * wChar ;
  pLine = y * hChar ;
}

int clearvt100_ (void)
{
  int k ;
  /* If there are no graph objects drawn, then it does
     a proper clear screen - and return 1.
     If there are graph objects, then we only clear the text
     parts and leave the graphs on screen - and return 0.
     */

  if ((allGraphs && arrayMax (allGraphs) == 0) || !allGraphs)
    {				/* no graphics : */
      clearvt640_() ;		/* so clear it the quick way */
      return 1 ;		/* only used for wrapText() */
    }
  else
    {
      if (allTexts) 
	{
	  for (k = 0; k < arrayMax (allTexts); ++k)
	    graphBoxClear (arr(allTexts, k, int)) ;
	  arrayReCreate (allTexts, 20, int) ;
	}
      pRow = 0 ; pLine = 0 ;
      return 0 ;
    }
}

void wrapText (BOOL OPEN_BOX)
{
  if ((pLine + 2*hChar) > winHeight)
    {
      if (OPEN_BOX)
	{
	  graphBoxEnd () ;
	  graphBoxDraw (arr(allTexts, ctb, int), BLACK, TRANSPARENT) ;

	  waitForMore (SPACE_KEY) ;
	}

      clearvt100_() ;
      

      if (OPEN_BOX)		/* start new box */
	{
	  ctb = arrayMax(allTexts) ;
	  array(allTexts, ctb, int) = graphBoxStart () ;
	}
    }
  else
    {
      pRow = 0; pLine += hChar;
    }
}

static char messbuf[4096], textbuf[4096] ;
void gprintf (FILE *fil, const char *format, ...)
{ va_list args ;
  int i ;

  if (fil == stderr)
    { 
      messbeep() ;
      colorvt640_(RED+100) ;
      gprintf (stdout, " ERROR ") ;
      colorvt640_(BLACK) ;
      ++pRow ;
      /*
      strcpy (textbuf, "!!! Error : ") ;
      strcat (textbuf, format) ;
      strcpy ((char *)format, textbuf) ;
      */
      fil = stdout ;
    }

  va_start (args,format) ;
    vsprintf (messbuf,format,args) ;
  va_end (args) ;

  if (fil != stdout)
    {
      fprintf (fil, messbuf) ;	/* fprintf required */
      fflush (fil) ;
    }
  else		/* print everything on graphscreen */
    {
      if (!allTexts)
	allTexts = arrayCreate (10, int) ;
      ctb = arrayMax(allTexts) ;
      messalloccheck () ;
      array(allTexts, ctb, int) = graphBoxStart () ;
      messalloccheck () ;
      for (i = 0; i < strlen (messbuf); ++i)
	{
	  if (messbuf[i] == '\n')
	    {
	      wrapText (TRUE) ;
	    }
	  else
	    { 
	      textbuf[0] = messbuf[i]; textbuf[1] = 0 ;
	      graphText (textbuf, pRow + (wChar / 3.0), pLine) ;
	      pRow += wChar ;
	    }
	}
      graphBoxEnd () ;		/* current box in array */
      if (INVERT)
	graphBoxDraw (arr(allTexts, ctb, int), WHITE, globalColor) ;
      else
	graphBoxDraw (arr(allTexts, ctb, int), globalColor, TRANSPARENT) ;
    }
}

#ifndef NO_GRAPHICS  /* non-graphical programs find it in f2c.c */
/**************************************************************/
/*                                                            */
/* GET KEYBOARD INPUT - WRAPPER FOR MODIFIED graphTextEntry() */
/*                                                            */
/**************************************************************/
char stemp[100] ;

void readinput (char *src, char *dest, int len)
{
  char line[200] ;

  if (len == 0)			/* just wait for RETURN */
    {
      waitForKey (RETURN_KEY) ;
      return ;
    }
  strtcpy (line, src, len) ;
  graphFortranEntry (line, len, pRow, pLine, 0) ;
  gprintf (stdout, line) ;
  if (!*line)
    strcpy (line, " ") ;
  strtcpy (dest, line, len) ;
  wrapText (FALSE) ;
}
#endif

